@@ -36,6 +36,8 @@ namespace ICSharpCode.Decompiler.Ast
 		bool firstUsingDeclaration;
 		bool lastUsingDeclaration;
 		
+		public bool FoldBraces = false;
+		
 		public TextOutputFormatter(ITextOutput output)
 		{
 			if (output == null)
@@ -114,6 +116,9 @@ namespace ICSharpCode.Decompiler.Ast
 		object GetCurrentLocalDefinition()
 		{
 			AstNode node = nodeStack.Peek();
+			if (node is Identifier && node.Parent != null)
+				node = node.Parent;
+			
 			var parameterDef = node.Annotation<ParameterDefinition>();
 			if (parameterDef != null)
 				return parameterDef;
@@ -126,14 +131,11 @@ namespace ICSharpCode.Decompiler.Ast
 					//if (variable.OriginalVariable != null)
 					//    return variable.OriginalVariable;
 					return variable;
-				} else {
-
 				}
 			}
 
 			var label = node as LabelStatement;
-			if (label != null)
-			{
+			if (label != null) {
 				var method = nodeStack.Select(nd => nd.Annotation<MethodReference>()).FirstOrDefault(mr => mr != null);
 				if (method != null)
 					return method.ToString() + label.Label;
@@ -165,8 +167,6 @@ namespace ICSharpCode.Decompiler.Ast
 		
 		public void WriteToken(string token)
 		{
-			if (string.IsNullOrEmpty (token))
-				return;
 			// Attach member reference to token only if there's no identifier in the current node.
 			MemberReference memberRef = GetCurrentMemberReference();
 			var node = nodeStack.Peek();
@@ -185,7 +185,7 @@ namespace ICSharpCode.Decompiler.Ast
 		{
 			if (braceLevelWithinType >= 0 || nodeStack.Peek() is TypeDeclaration)
 				braceLevelWithinType++;
-			if (nodeStack.OfType<BlockStatement>().Count() <= 1) {
+			if (nodeStack.OfType<BlockStatement>().Count() <= 1 || FoldBraces) {
 				output.MarkFoldStart(defaultCollapsed: braceLevelWithinType == 1);
 			}
 			output.WriteLine();
@@ -197,7 +197,7 @@ namespace ICSharpCode.Decompiler.Ast
 		{
 			output.Unindent();
 			output.Write('}');
-			if (nodeStack.OfType<BlockStatement>().Count() <= 1)
+			if (nodeStack.OfType<BlockStatement>().Count() <= 1 || FoldBraces)
 				output.MarkFoldEnd();
 			if (braceLevelWithinType >= 0)
 				braceLevelWithinType--;
@@ -284,6 +284,9 @@ namespace ICSharpCode.Decompiler.Ast
 			nodeStack.Push(node);
 			startLocations.Push(output.Location);
 			
+			if (node is EntityDeclaration && node.Annotation<MemberReference>() != null && node.GetChildByRole(Roles.Identifier).IsNull)
+				output.WriteDefinition("", node.Annotation<MemberReference>(), false);
+			
 			MemberMapping mapping = node.Annotation<MemberMapping>();
 			if (mapping != null) {
 				parentMemberMappings.Push(currentMemberMapping);
@@ -329,15 +332,7 @@ namespace ICSharpCode.Decompiler.Ast
 		
 		private static bool IsDefinition(AstNode node)
 		{
-			return
-				node is FieldDeclaration ||
-				node is ConstructorDeclaration ||
-				node is DestructorDeclaration ||
-				node is EventDeclaration ||
-				node is DelegateDeclaration ||
-				node is OperatorDeclaration||
-				node is EntityDeclaration ||
-				node is TypeDeclaration;
+			return node is EntityDeclaration;
 		}
 	}
 }
