@@ -35,6 +35,7 @@ namespace ICSharpCode.Decompiler.ILAst
 		InlineVariables,
 		CopyPropagation,
 		YieldReturn,
+		AsyncAwait,
 		PropertyAccessInstructions,
 		SplitToMovableBlocks,
 		TypeInference,
@@ -44,7 +45,7 @@ namespace ICSharpCode.Decompiler.ILAst
 		JoinBasicBlocks,
 		SimplifyLogicNot,
 		SimplifyShiftOperators,
-		TransformDecimalCtorToConstant,
+		TypeConversionSimplifications,
 		SimplifyLdObjAndStObj,
 		SimplifyCustomShortCircuit,
 		SimplifyLiftedOperators,
@@ -107,6 +108,10 @@ namespace ICSharpCode.Decompiler.ILAst
 			
 			if (abortBeforeStep == ILAstOptimizationStep.YieldReturn) return;
 			YieldReturnDecompiler.Run(context, method);
+			AsyncDecompiler.RunStep1(context, method);
+			
+			if (abortBeforeStep == ILAstOptimizationStep.AsyncAwait) return;
+			AsyncDecompiler.RunStep2(context, method);
 			
 			if (abortBeforeStep == ILAstOptimizationStep.PropertyAccessInstructions) return;
 			IntroducePropertyAccessInstructions(method);
@@ -143,9 +148,8 @@ namespace ICSharpCode.Decompiler.ILAst
 					if (abortBeforeStep == ILAstOptimizationStep.SimplifyShiftOperators) return;
 					modified |= block.RunOptimization(SimplifyShiftOperators);
 
-					if (abortBeforeStep == ILAstOptimizationStep.TransformDecimalCtorToConstant) return;
-					modified |= block.RunOptimization(TransformDecimalCtorToConstant);
-					modified |= block.RunOptimization(SimplifyLdcI4ConvI8);
+					if (abortBeforeStep == ILAstOptimizationStep.TypeConversionSimplifications) return;
+					modified |= block.RunOptimization(TypeConversionSimplifications);
 					
 					if (abortBeforeStep == ILAstOptimizationStep.SimplifyLdObjAndStObj) return;
 					modified |= block.RunOptimization(SimplifyLdObjAndStObj);
@@ -154,7 +158,7 @@ namespace ICSharpCode.Decompiler.ILAst
 					modified |= block.RunOptimization(new SimpleControlFlow(context, method).SimplifyCustomShortCircuit);
 
 					if (abortBeforeStep == ILAstOptimizationStep.SimplifyLiftedOperators) return;
-//					modified |= block.RunOptimization(SimplifyLiftedOperators);
+					modified |= block.RunOptimization(SimplifyLiftedOperators);
 					
 					if (abortBeforeStep == ILAstOptimizationStep.TransformArrayInitializers) return;
 					modified |= block.RunOptimization(TransformArrayInitializers);
@@ -257,7 +261,7 @@ namespace ICSharpCode.Decompiler.ILAst
 		/// Ignore arguments of 'leave'
 		/// </summary>
 		/// <param name="method"></param>
-		void RemoveRedundantCode(ILBlock method)
+		internal static void RemoveRedundantCode(ILBlock method)
 		{
 			Dictionary<ILLabel, int> labelRefCount = new Dictionary<ILLabel, int>();
 			foreach (ILLabel target in method.GetSelfAndChildrenRecursive<ILExpression>(e => e.IsBranch()).SelectMany(e => e.GetBranchTargets())) {
@@ -287,7 +291,13 @@ namespace ICSharpCode.Decompiler.ILAst
 							prevExpr.ILRanges.AddRange(((ILExpression)body[i]).ILRanges);
 						// Ignore pop
 					} else {
-						newBody.Add(body[i]);
+						ILLabel label = body[i] as ILLabel;
+						if (label != null) {
+							if (labelRefCount.GetOrDefault(label) > 0)
+								newBody.Add(label);
+						} else {
+							newBody.Add(body[i]);
+						}
 					}
 				}
 				block.Body = newBody;
@@ -852,5 +862,16 @@ namespace ICSharpCode.Decompiler.ILAst
 			if (!collection.Remove(key))
 				throw new Exception("The key was not found in the dictionary");
 		}
+		
+		public static bool ContainsReferenceTo(this ILExpression expr, ILVariable v)
+		{
+			if (expr.Operand == v)
+				return true;
+			foreach (var arg in expr.Arguments) {
+				if (ContainsReferenceTo(arg, v))
+					return true;
+			}
+			return false;
+		}
 	}
 }
