@@ -2,8 +2,10 @@
 // ABPerson.cs: Implements the managed ABPerson
 //
 // Authors: Mono Team
+//          Marek Safar (marek.safar@gmail.com)
 //     
 // Copyright (C) 2009 Novell, Inc
+// Copyright (C) 2012 Xamarin Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,6 +29,7 @@
 //
 using System;
 using System.Runtime.InteropServices;
+using System.Collections.Generic;
 
 using MonoMac.CoreFoundation;
 using MonoMac.Foundation;
@@ -315,17 +318,11 @@ namespace MonoMac.AddressBook {
 		}
 	}
 
-	public static class ABPersonSocialProfile {
-		static readonly NSString URLKey;
-		static readonly NSString ServiceKey;
-		static readonly NSString UsernameKey;
-		static readonly NSString UserIdentifierKey;
-		static readonly NSString ServiceTwitter;
-		static readonly NSString ServiceGameCenter;
-		static readonly NSString ServiceFacebook;
-		static readonly NSString ServiceMyspace;
-		static readonly NSString ServiceLinkedIn;
-		static readonly NSString ServiceFlickr;
+	static class ABPersonSocialProfile {
+		public static readonly NSString URLKey;
+		public static readonly NSString ServiceKey;
+		public static readonly NSString UsernameKey;
+		public static readonly NSString UserIdentifierKey;
 
 		static ABPersonSocialProfile ()
 		{
@@ -337,17 +334,41 @@ namespace MonoMac.AddressBook {
 				ServiceKey = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceKey");
 				UsernameKey = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileUsernameKey");
 				UserIdentifierKey = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileUserIdentifierKey");
-				ServiceTwitter = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceTwitter");
-				ServiceGameCenter = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceGameCenter");
-				ServiceFacebook = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceFacebook");
-				ServiceMyspace = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceMyspace");
-				ServiceLinkedIn = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceLinkedIn");
-				ServiceFlickr = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceFlickr");
 			} finally {
 				Dlfcn.dlclose (handle);
 			}
 		}
 	}
+
+	public static class ABPersonSocialProfileService
+	{
+		public static readonly NSString Twitter;
+		public static readonly NSString GameCenter;
+		public static readonly NSString Facebook;
+		public static readonly NSString Myspace;
+		public static readonly NSString LinkedIn;
+		public static readonly NSString Flickr;
+		// Since 6.0
+		public static readonly NSString SinaWeibo;
+
+		static ABPersonSocialProfileService ()
+		{
+			var handle = Dlfcn.dlopen (Constants.AddressBookLibrary, 0);
+			if (handle == IntPtr.Zero)
+				return;
+			try {
+				Twitter = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceTwitter");
+				GameCenter = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceGameCenter");
+				Facebook = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceFacebook");
+				Myspace = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceMyspace");
+				LinkedIn = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceLinkedIn");
+				Flickr = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceFlickr");
+				SinaWeibo = Dlfcn.GetStringConstant (handle, "kABPersonSocialProfileServiceSinaWeibo");
+			} finally {
+				Dlfcn.dlclose (handle);
+			}
+		}		
+	}
 	
 	public static class ABPersonPhoneLabel {
 		public static NSString HomeFax {get; private set;}
@@ -356,6 +377,7 @@ namespace MonoMac.AddressBook {
 		public static NSString Mobile {get; private set;}
 		public static NSString Pager {get; private set;}
 		public static NSString WorkFax {get; private set;}
+		public static NSString OtherFax { get; private set; }
 
 		static ABPersonPhoneLabel ()
 		{
@@ -374,6 +396,9 @@ namespace MonoMac.AddressBook {
 				Mobile  = Dlfcn.GetStringConstant (handle, "kABPersonPhoneMobileLabel");
 				Pager   = Dlfcn.GetStringConstant (handle, "kABPersonPhonePagerLabel");
 				WorkFax = Dlfcn.GetStringConstant (handle, "kABPersonPhoneWorkFAXLabel");
+
+				// Since 5.0
+				OtherFax = Dlfcn.GetStringConstant (handle, "kABPersonPhoneOtherFAXLabel");
 			}
 			finally {
 				Dlfcn.dlclose (handle);
@@ -387,6 +412,12 @@ namespace MonoMac.AddressBook {
 		public static NSString Jabber {get; private set;}
 		public static NSString Msn {get; private set;}
 		public static NSString Yahoo {get; private set;}
+		// Since 5.0
+		public static NSString QQ {get; private set;}
+		public static NSString GoogleTalk {get; private set;}
+		public static NSString Skype {get; private set;}
+		public static NSString Facebook {get; private set;}
+		public static NSString GaduGadu {get; private set;}
 
 		static ABPersonInstantMessageService ()
 		{
@@ -404,6 +435,11 @@ namespace MonoMac.AddressBook {
 				Jabber  = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceJabber");
 				Msn     = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceMSN");
 				Yahoo   = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceYahoo");
+				QQ      = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceQQ");
+				GoogleTalk = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceGoogleTalk");
+				Skype   = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceSkype");
+				Facebook   = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceFacebook");
+				GaduGadu   = Dlfcn.GetStringConstant (handle, "kABPersonInstantMessageServiceGaduGadu");
 			}
 			finally {
 				Dlfcn.dlclose (handle);
@@ -486,8 +522,8 @@ namespace MonoMac.AddressBook {
 				Child     = Dlfcn.GetStringConstant (handle, "kABPersonChildLabel");
 				Father    = Dlfcn.GetStringConstant (handle, "kABPersonFatherLabel");
 				Friend    = Dlfcn.GetStringConstant (handle, "kABPersonFriendLabel");
-				Manager   = Dlfcn.GetStringConstant (handle, "kABPersonMotherLabel");
-				Mother    = Dlfcn.GetStringConstant (handle, "kABPersonManagerLabel");
+				Manager   = Dlfcn.GetStringConstant (handle, "kABPersonManagerLabel");
+				Mother    = Dlfcn.GetStringConstant (handle, "kABPersonMotherLabel");
 				Parent    = Dlfcn.GetStringConstant (handle, "kABPersonParentLabel");
 				Partner   = Dlfcn.GetStringConstant (handle, "kABPersonPartnerLabel");
 				Sister    = Dlfcn.GetStringConstant (handle, "kABPersonSisterLabel");
@@ -530,14 +566,33 @@ namespace MonoMac.AddressBook {
 		extern static IntPtr ABPersonCreate ();
 
 		public ABPerson ()
-			: base (ABPersonCreate (), null)
+			: base (ABPersonCreate (), true)
 		{
 			InitConstants.Init ();
 		}
 
-		internal ABPerson (IntPtr handle, ABAddressBook addressbook)
-			: base (CFObject.CFRetain (handle), addressbook)
+		[DllImport (Constants.AddressBookLibrary)]
+		extern static IntPtr ABPersonCreateInSource (IntPtr source);
+
+		[Since (4,0)]
+		public ABPerson (ABRecord source)
+			: base (IntPtr.Zero, true)
 		{
+			if (source == null)
+				throw new ArgumentNullException ("source");
+
+			Handle = ABPersonCreateInSource (source.Handle);
+		}
+
+		internal ABPerson (IntPtr handle, bool owns)
+			: base (handle, owns)
+		{
+		}
+
+		internal ABPerson (IntPtr handle, ABAddressBook addressbook)
+			: base (handle, false)
+		{
+			AddressBook = addressbook;
 		}
 
 		int IComparable.CompareTo (object o)
@@ -679,6 +734,20 @@ namespace MonoMac.AddressBook {
 			set {SetValue (ABPersonPropertyId.Department, value);}
 		}
 
+		[DllImport (Constants.AddressBookLibrary)]
+		extern static IntPtr ABPersonCopySource (IntPtr group);
+
+		[Since (4,0)]
+		public ABRecord Source {
+			get {
+				var h = ABPersonCopySource (Handle);
+				if (h == IntPtr.Zero)
+					return null;
+
+				return FromHandle (h, null);
+			}
+		}
+
 		internal static string ToString (IntPtr value)
 		{
 			if (value == IntPtr.Zero)
@@ -730,11 +799,29 @@ namespace MonoMac.AddressBook {
 			set {SetValue (ABPersonPropertyId.ModificationDate, value);}
 		}
 
+		[Advice ("Use GetAllAddresses")]
 		public ABMultiValue<NSDictionary> GetAddresses ()
 		{
 			return CreateDictionaryMultiValue (CopyValue (ABPersonPropertyId.Address));
 		}
 
+		public ABMultiValue<PersonAddress> GetAllAddresses ()
+		{
+			return CreateDictionaryMultiValue<PersonAddress> (CopyValue (ABPersonPropertyId.Address), l => new PersonAddress (l));
+		}
+
+		// Obsolete
+		public void SetAddresses (ABMultiValue<NSDictionary> value)
+		{
+			SetValue (ABPersonPropertyId.Address, value == null ? IntPtr.Zero : value.Handle);
+		}
+
+		public void SetAddresses (ABMultiValue<PersonAddress> addresses)
+		{
+			SetValue (ABPersonPropertyId.Address, addresses == null ? IntPtr.Zero : addresses.Handle);
+		}
+
+		// Obsolete
 		static ABMultiValue<NSDictionary> CreateDictionaryMultiValue (IntPtr handle)
 		{
 			if (handle == IntPtr.Zero)
@@ -742,9 +829,14 @@ namespace MonoMac.AddressBook {
 			return new ABMultiValue<NSDictionary> (handle);
 		}
 
-		public void SetAddresses (ABMultiValue<NSDictionary> value)
+		static ABMultiValue<T> CreateDictionaryMultiValue<T> (IntPtr handle, Func<NSDictionary, T> factory) where T : DictionaryContainer
 		{
-			SetValue (ABPersonPropertyId.Address, value == null ? IntPtr.Zero : value.Handle);
+			if (handle == IntPtr.Zero)
+				return null;
+
+			return new ABMultiValue<T> (handle,
+				l => factory ((NSDictionary) (object) Runtime.GetNSObject (l)),
+				l => l.Dictionary.Handle);
 		}
 
 		public ABMultiValue<NSDate> GetDates ()
@@ -779,25 +871,49 @@ namespace MonoMac.AddressBook {
 			SetValue (ABPersonPropertyId.Phone, value == null ? IntPtr.Zero : value.Handle);
 		}
 
+		[Advice ("Use GetInstantMessageServices")]
 		public ABMultiValue<NSDictionary> GetInstantMessages ()
 		{
 			return CreateDictionaryMultiValue (CopyValue (ABPersonPropertyId.InstantMessage));
 		}
 
+		public ABMultiValue<InstantMessageService> GetInstantMessageServices ()
+		{
+			return CreateDictionaryMultiValue<InstantMessageService> (CopyValue (ABPersonPropertyId.InstantMessage), l => new InstantMessageService (l));
+		}
+
+		// Obsolete
 		public void SetInstantMessages (ABMultiValue<NSDictionary> value)
 		{
 			SetValue (ABPersonPropertyId.InstantMessage, value == null ? IntPtr.Zero : value.Handle);
 		}
 
+		public void SetInstantMessages (ABMultiValue<InstantMessageService> services)
+		{
+			SetValue (ABPersonPropertyId.InstantMessage, services == null ? IntPtr.Zero : services.Handle);
+		}
+
+		[Advice ("Use GetSocialProfiles")]
 		public ABMultiValue<NSDictionary> GetSocialProfile ()
 		{
 			return CreateDictionaryMultiValue (CopyValue (ABPersonPropertyId.SocialProfile));
 		}
+
+		public ABMultiValue<SocialProfile> GetSocialProfiles ()
+		{
+			return CreateDictionaryMultiValue<SocialProfile> (CopyValue (ABPersonPropertyId.SocialProfile), l => new SocialProfile (l));
+		}
 		
+		// Obsolete
 		public void SetSocialProfile (ABMultiValue<NSDictionary> value)
 		{
 			SetValue (ABPersonPropertyId.SocialProfile, value == null ? IntPtr.Zero : value.Handle);
 		}
+
+		public void SetSocialProfile (ABMultiValue<SocialProfile> profiles)
+		{
+			SetValue (ABPersonPropertyId.SocialProfile, profiles == null ? IntPtr.Zero : profiles.Handle);
+		}
 		
 		public ABMultiValue<string> GetUrls ()
 		{
@@ -852,14 +968,218 @@ namespace MonoMac.AddressBook {
 			throw new ArgumentException ("Invalid property value: " + property);
 		}
 
+		[DllImport (Constants.AddressBookLibrary)]
+		extern static IntPtr ABPersonCopyArrayOfAllLinkedPeople (IntPtr person);
+
+		[Since (4,0)]
+		public ABPerson[] GetLinkedPeople ()
+		{
+			var linked = ABPersonCopyArrayOfAllLinkedPeople (Handle);
+			return NSArray.ArrayFromHandle (linked, l => new ABPerson (l, null));
+		}
+
 		[DllImport (Constants.AddressBookLibrary)]
 		extern static IntPtr ABPersonCopyImageDataWithFormat (IntPtr handle, ABPersonImageFormat format);
 		
 		[Since (4,1)]
-		NSData CopyImage (ABPersonImageFormat format)
+		public NSData GetImage (ABPersonImageFormat format)
 		{
-			return new NSData (ABPersonCopyImageDataWithFormat (Handle, format));
+			return (NSData) Runtime.GetNSObject (ABPersonCopyImageDataWithFormat (Handle, format));
+		}
+
+		[DllImport (Constants.AddressBookLibrary)]
+		extern static IntPtr ABPersonCreateVCardRepresentationWithPeople (IntPtr people);
+
+		[Since (5,0)]
+		public static NSData GetVCards (params ABPerson[] people)
+		{
+			if (people == null)
+				throw new ArgumentNullException ("people");
+
+			var ptrs = new IntPtr [people.Length];
+			for (int i = 0; i < people.Length; ++i) {
+				ptrs[i] = people[i].Handle;
+			}
+
+			var ptr = ABPersonCreateVCardRepresentationWithPeople (CFArray.Create (ptrs));
+			return new NSData (ptr, true);
+		}
+
+		[DllImport (Constants.AddressBookLibrary)]
+		extern static IntPtr ABPersonCreatePeopleInSourceWithVCardRepresentation (IntPtr source, IntPtr vCardData);
+
+		[Since (5,0)]
+		public static ABPerson[] CreateFromVCard (ABRecord source, NSData vCardData)
+		{
+			if (vCardData == null)
+				throw new ArgumentNullException ("vCardData");
+
+			// TODO: SIGSEGV when source is not null
+			var res = ABPersonCreatePeopleInSourceWithVCardRepresentation (source == null ? IntPtr.Zero : source.Handle,
+				vCardData.Handle);
+
+			return NSArray.ArrayFromHandle (res, l => new ABPerson (l, null));
+		}
+	}
+
+	public class SocialProfile : DictionaryContainer
+	{
+		public SocialProfile ()
+		{
+		}
+
+		public SocialProfile (NSDictionary dictionary)
+			: base (dictionary)
+		{
+		}
+
+		public string ServiceName {
+			get {
+				return GetStringValue (ABPersonSocialProfile.ServiceKey);
+			}
+			set {
+				SetStringValue (ABPersonSocialProfile.ServiceKey, value);
+			}			
+		}
+
+		// NSString from ABPersonSocialProfileService
+		public NSString Service {
+			set {
+				SetStringValue (ABPersonSocialProfile.ServiceKey, value);
+			}
+		}		
+
+		public string Username {
+			get {
+				return GetStringValue (ABPersonSocialProfile.UsernameKey);
+			}
+			set {
+				SetStringValue (ABPersonSocialProfile.UsernameKey, value);
+			}
+		}
+
+		public string UserIdentifier {
+			get {
+				return GetStringValue (ABPersonSocialProfile.UserIdentifierKey);
+			}
+			set {
+				SetStringValue (ABPersonSocialProfile.UserIdentifierKey, value);
+			}
+		}
+
+		public string Url {
+			get {
+				return GetStringValue (ABPersonSocialProfile.URLKey);
+			}
+			set {
+				SetStringValue (ABPersonSocialProfile.URLKey, value);
+			}
+		}
+	}
+
+	public class InstantMessageService : DictionaryContainer
+	{
+		public InstantMessageService ()
+		{
+		}
+
+		public InstantMessageService (NSDictionary dictionary)
+			: base (dictionary)
+		{
+		}
+
+		public string ServiceName {
+			get {
+				// TODO: It does not return ABPersonInstantMessageService value. Underlying
+				// value is custom string, it coould be MT bug because this makes
+				// ABPersonInstantMessageService constants useless
+				return GetStringValue (ABPersonInstantMessageKey.Service);
+			}
+			set {
+				SetStringValue (ABPersonInstantMessageKey.Service, value);
+			}
+		}
+
+		// NSString from ABPersonInstantMessageService
+		public NSString Service {
+			set {
+				SetStringValue (ABPersonInstantMessageKey.Service, value);
+			}
+		}
+
+		public string Username {
+			get {
+				return GetStringValue (ABPersonInstantMessageKey.Username);
+			}
+			set {
+				SetStringValue (ABPersonInstantMessageKey.Username, value);
+			}
+		}
+	}
+
+	public class PersonAddress : DictionaryContainer
+	{
+		public PersonAddress ()
+		{
+		}
+
+		public PersonAddress (NSDictionary dictionary)
+			: base (dictionary)
+		{
+		}
+
+		public string City {
+			get {
+				return GetStringValue (ABPersonAddressKey.City);
+			}
+			set {
+				SetStringValue (ABPersonAddressKey.City, value);
+			}
+		}
+
+		public string Country {
+			get {
+				return GetStringValue (ABPersonAddressKey.Country);
+			}
+			set {
+				SetStringValue (ABPersonAddressKey.Country, value);
+			}
+		}
+
+		public string CountryCode {
+			get {
+				return GetStringValue (ABPersonAddressKey.CountryCode);
+			}
+			set {
+				SetStringValue (ABPersonAddressKey.CountryCode, value);
+			}
+		}
+
+		public string State {
+			get {
+				return GetStringValue (ABPersonAddressKey.State);
+			}
+			set {
+				SetStringValue (ABPersonAddressKey.State, value);
+			}
+		}
+
+		public string Street {
+			get {
+				return GetStringValue (ABPersonAddressKey.Street);
+			}
+			set {
+				SetStringValue (ABPersonAddressKey.Street, value);
+			}
+		}
+
+		public string Zip {
+			get {
+				return GetStringValue (ABPersonAddressKey.Zip);
+			}
+			set {
+				SetStringValue (ABPersonAddressKey.Zip, value);
+			}
 		}
 	}
 }
-
