@@ -1,10 +1,12 @@
 // 
-// AudioSessions.cs:
+// AudioSession.cs: AudioSession bindings
 //
 // Authors:
 //    Miguel de Icaza (miguel@novell.com)
+//    Marek Safar (marek.safar@gmail.com)
 //     
 // Copyright 2009 Novell, Inc
+// Copyright 2011, 2012 Xamarin Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -36,10 +38,6 @@ using MonoMac.ObjCRuntime;
 using MonoMac.Foundation;
 
 using OSStatus = System.Int32;
-using AudioQueueParameterID = System.UInt32;
-using AudioQueueParameterValue = System.Single;
-using AudioQueueRef = System.IntPtr;
-using AudioQueueTimelineRef = System.IntPtr;
 
 namespace MonoMac.AudioToolbox {
 
@@ -75,7 +73,7 @@ namespace MonoMac.AudioToolbox {
 				return "This operation requries AudioSession.Category to be explicitly set";
 				
 			}
-			return String.Format ("Unknown error code: 0x{0:x}", k);
+			return String.Format ("Unknown error code: {0}", k);
 		}
 		
 		internal AudioSessionException (int k) : base (Lookup (k))
@@ -85,6 +83,18 @@ namespace MonoMac.AudioToolbox {
 
 		public AudioSessionErrors ErrorCode { get; private set; }
 	}
+
+	public class AccessoryInfo
+	{
+		internal AccessoryInfo (int id, string description)
+		{
+			ID = id;
+			Description = description;
+		}
+
+		public int ID { get; private set; }
+		public string Description { get; private set; }
+	}
 	
 	public class AudioSessionPropertyEventArgs :EventArgs {
 		public AudioSessionPropertyEventArgs (AudioSessionProperty prop, int size, IntPtr data)
@@ -97,15 +107,89 @@ namespace MonoMac.AudioToolbox {
 		public int Size  { get; set; }
 		public IntPtr Data { get; set; }
 	}
+
+	public class AudioSessionRouteChangeEventArgs : EventArgs {
+		static IntPtr route_change_key, previous_route_key, current_route_key;
+
+		static AudioSessionRouteChangeEventArgs ()
+		{
+			var lib = Dlfcn.dlopen (Constants.AudioToolboxLibrary, 0);
+			route_change_key = Dlfcn.GetIntPtr (lib, "kAudioSession_RouteChangeKey_Reason");
+			previous_route_key = Dlfcn.GetIntPtr (lib, "kAudioSession_AudioRouteChangeKey_PreviousRouteDescription");
+			current_route_key = Dlfcn.GetIntPtr (lib, "kAudioSession_AudioRouteChangeKey_CurrentRouteDescription");
+
+			Dlfcn.dlclose (lib);
+		}
+
+		public NSDictionary Dictionary { get; private set; }
+		
+		public AudioSessionRouteChangeEventArgs (IntPtr dictHandle)
+		{
+			Dictionary = new NSDictionary (dictHandle);
+		}
+		
+		public AudioSessionRouteChangeReason Reason {
+			get {
+				using (var num = new NSNumber (Dictionary.LowlevelObjectForKey (route_change_key))){
+					return (AudioSessionRouteChangeReason) num.Int32Value;
+				}
+			}
+		}
+
+		NSArray Extract (IntPtr key, NSString secondKey)
+		{
+			var dictH = Dictionary.LowlevelObjectForKey (key);
+			if (dictH == IntPtr.Zero)
+				return null;
+
+//			Console.WriteLine ("Extracting from {2} {0} and getting {1}", new NSString (key), new NSDictionary (dictH).Description, Dictionary.Description);
+			// Description dictionary, indexed by the second key, the result is an array
+			using (var descDict = new NSDictionary (dictH)){
+				var sdict = descDict.LowlevelObjectForKey (secondKey.Handle);
+				if (sdict == IntPtr.Zero)
+					return null;
+
+				return new NSArray (sdict);
+			}
+		}
+
+		public AudioSessionInputRouteKind PreviousInputRoute {
+			get {
+				using (var array = Extract (previous_route_key, AudioSession.AudioRouteKey_Inputs))
+					return AudioSession.GetInputRoute (array);
+			}
+		}
+
+		public AudioSessionOutputRouteKind [] PreviousOutputRoutes {
+			get {
+				using (var array = Extract (previous_route_key, AudioSession.AudioRouteKey_Outputs))
+					return AudioSession.GetOutputRoutes (array);
+			}
+		}
+
+		public AudioSessionInputRouteKind CurrentInputRoute {
+			get {
+				using (var array = Extract (current_route_key, AudioSession.AudioRouteKey_Inputs))
+					return AudioSession.GetInputRoute (array);
+			}
+		}
+
+		public AudioSessionOutputRouteKind [] CurrentOutputRoutes {
+			get {
+				using (var array = Extract (current_route_key, AudioSession.AudioRouteKey_Outputs))
+					return AudioSession.GetOutputRoutes (array);
+			}
+		}
+	}
 	
 	public static class AudioSession {
 		static bool initialized;
 		public static event EventHandler Interrupted;
 		public static event EventHandler Resumed;
 
-		static NSString AudioRouteKey_Type;
-		static NSString AudioRouteKey_Inputs;
-		static NSString AudioRouteKey_Outputs;
+		internal static NSString AudioRouteKey_Type;
+		internal static NSString AudioRouteKey_Inputs;
+		internal static NSString AudioRouteKey_Outputs;
 		
 		static NSString InputRoute_LineIn;
 		static NSString InputRoute_BuiltInMic;
@@ -122,6 +206,10 @@ namespace MonoMac.AudioToolbox {
 		static NSString OutputRoute_USBAudio;
 		static NSString OutputRoute_HDMI;
 		static NSString OutputRoute_AirPlay;
+		static NSString InputSourceKey_ID;
+		static NSString InputSourceKey_Description;
+		static NSString OutputDestinationKey_ID;
+		static NSString OutputDestinationKey_Description;
 		
 		[DllImport (Constants.AudioToolboxLibrary)]
 		extern static OSStatus AudioSessionInitialize(IntPtr cfRunLoop, IntPtr cfstr_runMode, InterruptionListener listener, IntPtr userData);
@@ -162,6 +250,12 @@ namespace MonoMac.AudioToolbox {
 			OutputRoute_USBAudio = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSessionOutputRoute_USBAudio"));
 			OutputRoute_HDMI = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSessionOutputRoute_HDMI"));
 			OutputRoute_AirPlay = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSessionOutputRoute_AirPlay"));
+
+			InputSourceKey_ID = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSession_InputSourceKey_ID"));
+			InputSourceKey_Description = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSession_InputSourceKey_Description"));
+
+			OutputDestinationKey_ID = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSession_OutputDestinationKey_ID"));
+			OutputDestinationKey_Description = new NSString (Dlfcn.GetIntPtr (lib, "kAudioSession_OutputDestinationKey_Description"));
 			
 			Dlfcn.dlclose (lib);
 			
@@ -182,6 +276,7 @@ namespace MonoMac.AudioToolbox {
 
 		[DllImport (Constants.AudioToolboxLibrary)]
 		extern static OSStatus AudioSessionSetActive (int active);
+
 		public static void SetActive (bool active)
 		{
 			int k = AudioSessionSetActive (active ? 1 : 0);
@@ -189,6 +284,15 @@ namespace MonoMac.AudioToolbox {
 				throw new AudioSessionException (k);
 		}
 
+		[DllImport (Constants.AudioToolboxLibrary)]
+		extern static AudioSessionErrors AudioSessionSetActive (int active, AudioSessionActiveFlags inFlags);
+
+		[Since (4,0)]
+		public static AudioSessionErrors SetActive (bool active, AudioSessionActiveFlags flags)
+		{
+			return AudioSessionSetActive (active ? 1 : 0, flags);
+		}
+
 		[DllImport (Constants.AudioToolboxLibrary)]
 		extern static OSStatus AudioSessionGetProperty(AudioSessionProperty id, ref int size, IntPtr data);
 
@@ -248,7 +352,7 @@ namespace MonoMac.AudioToolbox {
 		static void SetInt (AudioSessionProperty property, int val)
 		{
 			unsafe {
-				int k = AudioSessionSetProperty (property, 4, (IntPtr) (&val));
+				int k = AudioSessionSetProperty (property, sizeof (int), (IntPtr) (&val));
 				if (k != 0)
 					throw new AudioSessionException (k);
 			}
@@ -297,90 +401,157 @@ namespace MonoMac.AudioToolbox {
 			}
 		}
 
+		[Obsolete ("Deprecated in iOS 5.0. Use InputRoute or OutputRoute instead")]
 		static public string AudioRoute {
 			get {
 				return CFString.FetchString ((IntPtr) GetInt (AudioSessionProperty.AudioRoute));
 			}
 		}
-		
-		static public AudioSessionInputRouteKind InputRoute {
+
+		[Since (5,0)]
+		static public AccessoryInfo[] InputSources {
 			get {
-				var arr = (NSArray) AudioRouteDescription [AudioRouteKey_Inputs];
+				using (var array = new CFArray ((IntPtr) GetInt (AudioSessionProperty.InputSources))) {
+					var res = new AccessoryInfo [array.Count];
+					for (int i = 0; i < res.Length; ++i) {
+						var dict = array.GetValue (i);
+						var id = new NSNumber (CFDictionary.GetValue (dict, InputSourceKey_ID.Handle));
+						var desc = CFString.FetchString (CFDictionary.GetValue (dict, InputSourceKey_Description.Handle));
+
+						res [i] = new AccessoryInfo ((int) id, desc);
+						id.Dispose ();
+					}
+					return res;
+				}
+			}
+		}
+
+		[Since (5,0)]
+		static public AccessoryInfo[] OutputDestinations {
+			get {
+				using (var array = new CFArray ((IntPtr) GetInt (AudioSessionProperty.OutputDestinations))) {
+					var res = new AccessoryInfo [array.Count];
+					for (int i = 0; i < res.Length; ++i) {
+						var dict = array.GetValue (i);
+						var id = new NSNumber (CFDictionary.GetValue (dict, OutputDestinationKey_ID.Handle));
+						var desc = CFString.FetchString (CFDictionary.GetValue (dict, OutputDestinationKey_Description.Handle));
+
+						res [i] = new AccessoryInfo ((int) id, desc);
+						id.Dispose ();
+					}
+					return res;
+				}
+			}
+		}
+
+		/* Could not test what sort of unique CFNumberRef value it's
+
+		[Since (5,0)]
+		static public int InputSource {
+			get {
+				return GetInt (AudioSessionProperty.InputSource);
+			}
+			set {
+				SetInt (AudioSessionProperty.InputSource, value);
+			}
+		}
+
+		[Since (5,0)]
+		static public int OutputDestination {
+			get {
+				return GetInt (AudioSessionProperty.OutputDestination);
+			}
+			set {
+				SetInt (AudioSessionProperty.OutputDestination, value);
+			}
+		}
+
+		*/
+
+		static internal AudioSessionInputRouteKind GetInputRoute (NSArray arr)
+		{
+			if (arr == null || arr.Count == 0)
+				return AudioSessionInputRouteKind.None;
+			
+			var dict = new NSDictionary (arr.ValueAt (0));
+			
+			if (dict == null || dict.Count == 0)
+				return AudioSessionInputRouteKind.None;
+			
+			var val = (NSString) dict [AudioRouteKey_Type];
+			
+			if (val == null)
+				return AudioSessionInputRouteKind.None;
+			
+			if (val == InputRoute_LineIn) {
+				return AudioSessionInputRouteKind.LineIn;
+			} else if (val == InputRoute_BuiltInMic) {
+				return AudioSessionInputRouteKind.BuiltInMic;
+			} else if (val == InputRoute_HeadsetMic) {
+				return AudioSessionInputRouteKind.HeadsetMic;
+			} else if (val == InputRoute_BluetoothHFP) {
+				return AudioSessionInputRouteKind.BluetoothHFP;
+			} else if (val == InputRoute_USBAudio) {
+				return AudioSessionInputRouteKind.USBAudio;
+			} else {
+				return (AudioSessionInputRouteKind) val.Handle;
+			}
+		}
+
+		static internal AudioSessionOutputRouteKind [] GetOutputRoutes (NSArray arr)
+		{
+			if (arr == null || arr.Count == 0)
+				return null;
+			
+			var result = new AudioSessionOutputRouteKind [arr.Count];
+			for (uint i = 0; i < arr.Count; i++) {
+				var dict = new NSDictionary ((IntPtr) arr.ValueAt (i));
 				
-				if (arr == null || arr.Count == 0)
-					return AudioSessionInputRouteKind.None;
-				
-				var dict = new NSDictionary (arr.ValueAt (0));
+				result [i] = AudioSessionOutputRouteKind.None;
 				
 				if (dict == null || dict.Count == 0)
-					return AudioSessionInputRouteKind.None;
+					continue;
 				
 				var val = (NSString) dict [AudioRouteKey_Type];
 				
 				if (val == null)
-					return AudioSessionInputRouteKind.None;
+					continue;
 				
-				if (val == InputRoute_LineIn) {
-					return AudioSessionInputRouteKind.LineIn;
-				} else if (val == InputRoute_BuiltInMic) {
-					return AudioSessionInputRouteKind.BuiltInMic;
-				} else if (val == InputRoute_HeadsetMic) {
-					return AudioSessionInputRouteKind.HeadsetMic;
-				} else if (val == InputRoute_BluetoothHFP) {
-					return AudioSessionInputRouteKind.BluetoothHFP;
-				} else if (val == InputRoute_USBAudio) {
-					return AudioSessionInputRouteKind.USBAudio;
-				} else {
-					// now what?
-					throw new Exception (); // return AudioSessionInputRouteKind.None;
-				}
+				if (val == OutputRoute_LineOut) {
+					result [i] = AudioSessionOutputRouteKind.LineOut;
+				} else if (val == OutputRoute_Headphones) {
+					result [i] = AudioSessionOutputRouteKind.Headphones;
+				} else if (val == OutputRoute_BluetoothHFP) {
+					result [i] = AudioSessionOutputRouteKind.BluetoothHFP;
+				} else if (val == OutputRoute_BluetoothA2DP) {
+					result [i] = AudioSessionOutputRouteKind.BluetoothA2DP;
+				} else if (val == OutputRoute_BuiltInReceiver) {
+					result [i] = AudioSessionOutputRouteKind.BuiltInReceiver;
+				} else if (val == OutputRoute_BuiltInSpeaker) {
+					result [i] = AudioSessionOutputRouteKind.BuiltInSpeaker;
+				} else if (val == OutputRoute_USBAudio) {
+					result [i] = AudioSessionOutputRouteKind.USBAudio;
+				} else if (val == OutputRoute_HDMI) {
+					result [i] = AudioSessionOutputRouteKind.HDMI;
+				} else if (val == OutputRoute_AirPlay) {
+					result [i] = AudioSessionOutputRouteKind.AirPlay;
+				} else
+					result [i] = (AudioSessionOutputRouteKind) val.Handle;
+			}
+			return result;
+		}
+
+		[Since (5,0)]
+		static public AudioSessionInputRouteKind InputRoute {
+			get {
+				return GetInputRoute ((NSArray) AudioRouteDescription [AudioRouteKey_Inputs]);
 			}
 		}
 		
+		[Since (5,0)]
 		static public AudioSessionOutputRouteKind [] OutputRoutes {
 			get {
-				var arr = (NSArray) AudioRouteDescription [AudioRouteKey_Outputs];
-				
-				if (arr == null || arr.Count == 0)
-					return null;
-				
-				var result = new AudioSessionOutputRouteKind [arr.Count];
-				for (uint i = 0; i < arr.Count; i++) {
-					var dict = new NSDictionary ((IntPtr) arr.ValueAt (i));
-					
-					result [i] = AudioSessionOutputRouteKind.None;
-					
-					if (dict == null || dict.Count == 0)
-						continue;
-					
-					var val = (NSString) dict [AudioRouteKey_Type];
-					
-					if (val == null)
-						continue;
-					
-					if (val == OutputRoute_LineOut) {
-						result [i] = AudioSessionOutputRouteKind.LineOut;
-					} else if (val == OutputRoute_Headphones) {
-						result [i] = AudioSessionOutputRouteKind.Headphones;
-					} else if (val == OutputRoute_BluetoothHFP) {
-						result [i] = AudioSessionOutputRouteKind.BluetoothHFP;
-					} else if (val == OutputRoute_BluetoothA2DP) {
-						result [i] = AudioSessionOutputRouteKind.BluetoothA2DP;
-					} else if (val == OutputRoute_BuiltInReceiver) {
-						result [i] = AudioSessionOutputRouteKind.BuiltInReceiver;
-					} else if (val == OutputRoute_BuiltInSpeaker) {
-						result [i] = AudioSessionOutputRouteKind.BuiltInSpeaker;
-					} else if (val == OutputRoute_USBAudio) {
-						result [i] = AudioSessionOutputRouteKind.USBAudio;
-					} else if (val == OutputRoute_HDMI) {
-						result [i] = AudioSessionOutputRouteKind.HDMI;
-					} else if (val == OutputRoute_AirPlay) {
-						result [i] = AudioSessionOutputRouteKind.AirPlay;
-					
-					}
-				}
-				
-				return result;
+				return GetOutputRoutes ((NSArray) AudioRouteDescription [AudioRouteKey_Outputs]);
 			}
 		}
 		
@@ -470,15 +641,6 @@ namespace MonoMac.AudioToolbox {
 			}
 		}
 
-		static public AudioSessionMode Mode {
-			get {
-				return (AudioSessionMode) GetInt (AudioSessionProperty.Mode);
-			}
-			set {
-				SetInt (AudioSessionProperty.Mode, (int) value);
-			}
-		}
-		
 		static public bool OverrideCategoryDefaultToSpeaker {
 			get {
 				return GetInt (AudioSessionProperty.OverrideCategoryDefaultToSpeaker) != 0;
@@ -497,6 +659,34 @@ namespace MonoMac.AudioToolbox {
 			}
 		}
 		
+		[Since (5,0)]
+		static public AudioSessionMode Mode {
+			get {
+				return (AudioSessionMode) GetInt (AudioSessionProperty.Mode);
+			}
+			set {
+				SetInt (AudioSessionProperty.Mode, (int) value);
+			}
+		}
+
+		// InputSources
+
+		[Since (5,0)]
+		static public bool InputGainAvailable {
+			get {
+				return GetInt (AudioSessionProperty.InputGainAvailable) != 0;
+			}
+		}
+
+		[Since (5,0)]
+		static public float InputGainScalar {
+			get {
+				return GetFloat (AudioSessionProperty.InputGainScalar);
+			}
+			set {
+				SetFloat (AudioSessionProperty.InputGainScalar, value);
+			}
+		}
 
 		delegate void _PropertyListener (IntPtr userData, AudioSessionProperty prop, int size, IntPtr data);
 		public delegate void PropertyListener (AudioSessionProperty prop, int size, IntPtr data);
@@ -551,5 +741,40 @@ namespace MonoMac.AudioToolbox {
 				listeners [property] = null;
 		}
 
+               class RouteChangeListener {
+                       public EventHandler<AudioSessionRouteChangeEventArgs> cback;
+                       
+                       public RouteChangeListener (EventHandler<AudioSessionRouteChangeEventArgs> cback)
+                       {
+                               this.cback = cback;
+                       }
+
+                       public void Listener (AudioSessionProperty prop, int size, IntPtr data)
+                       {
+                               cback (null, new AudioSessionRouteChangeEventArgs (data));
+                       }
+               }
+               
+               static Hashtable strongListenerHash;
+               
+               public static event EventHandler<AudioSessionRouteChangeEventArgs> AudioRouteChanged {
+                       add {
+                               if (strongListenerHash == null)
+                                       strongListenerHash = new Hashtable ();
+                               var routeChangeListener = new RouteChangeListener (value);
+                               strongListenerHash [value] = routeChangeListener;
+                               AddListener (AudioSessionProperty.AudioRouteChange, routeChangeListener.Listener);
+                       }
+
+                       remove {
+                               if (strongListenerHash == null)
+                                       return;
+                               var k = strongListenerHash [value] as RouteChangeListener;
+                               if (k != null){
+                                       RemoveListener (AudioSessionProperty.AudioRouteChange, k.Listener);
+                                       strongListenerHash.Remove (value);
+                               }
+                       }
+               }
 	}
 }
