@@ -4,6 +4,7 @@
 // Authors: Mono Team
 //     
 // Copyright 2009 Novell, Inc
+// Copyright 2011, 2012 Xamarin Inc
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -192,11 +193,12 @@ namespace MonoMac.CoreGraphics {
 		[DllImport (Constants.CoreGraphicsLibrary)]
 		extern static void CGPathAddLineToPoint(IntPtr path, IntPtr m, float x, float y);
 
-		[Obsolete("Use AddLineToPoint instead")]
+		[Advice ("Use AddLineToPoint instead")] // Bad name
 		public void CGPathAddLineToPoint (float x, float y)
 		{
 			AddLineToPoint (x, y);
 		}
+
 		public void AddLineToPoint (float x, float y)
 		{
 			CGPathAddLineToPoint (handle, IntPtr.Zero, x, y);
@@ -207,7 +209,7 @@ namespace MonoMac.CoreGraphics {
 			CGPathAddLineToPoint (handle, IntPtr.Zero, point.X, point.Y);
 		}
 		
-		[Obsolete("Use AddLineToPoint instead")]
+		[Advice ("Use AddLineToPoint instead")] // Bad name
 		public void CGPathAddLineToPoint (CGAffineTransform transform, float x, float y)
 		{
 			AddLineToPoint (transform, x, y);
@@ -335,6 +337,12 @@ namespace MonoMac.CoreGraphics {
 
 		[DllImport (Constants.CoreGraphicsLibrary)]
 		extern static void CGPathAddEllipseInRect(IntPtr path, ref CGAffineTransform m, RectangleF rect);
+		public void AddEllipseInRect (CGAffineTransform m, RectangleF rect)
+		{
+			CGPathAddEllipseInRect (handle, ref m, rect);
+		}
+		
+		[Obsolete ("Use AddEllipseInRect instead")]
 		public void AddElipseInRect (CGAffineTransform m, RectangleF rect)
 		{
 			CGPathAddEllipseInRect (handle, ref m, rect);
