@@ -59,7 +59,8 @@ namespace MonoMac.CoreMidi {
         	SetupFormatErr = -10840,
         	WrongThread = -10841,
         	ObjectNotFound = -10842,
-        	IDNotUnique = -10843
+        	IDNotUnique = -10843,
+		NotPermitted = -10844
 	}
 
 	public enum MidiNetworkConnectionPolicy {
@@ -168,8 +169,7 @@ namespace MonoMac.CoreMidi {
 	}
 	
 	public class MidiObject : INativeObject, IDisposable {
-                internal static IntPtr midiLibrary = Dlfcn.dlopen (Constants.CoreMidiLibrary, 0);
-                internal IntPtr handle;
+		internal IntPtr handle;
 		internal bool owns;
 
 #if !COREBUILD
@@ -220,6 +220,11 @@ namespace MonoMac.CoreMidi {
 
 		static MidiObject ()
 		{
+#if MONOMAC
+			var midiLibrary = Dlfcn.dlopen (Constants.CoreMidiLibrary, 0);
+#else
+			var midiLibrary = Libraries.CoreMidi.Handle;
+#endif
 			kMIDIPropertyAdvanceScheduleTimeMuSec = Dlfcn.GetIntPtr (midiLibrary, "kMIDIPropertyAdvanceScheduleTimeMuSec");
 			kMIDIPropertyCanRoute = Dlfcn.GetIntPtr (midiLibrary, "kMIDIPropertyCanRoute");
 			kMIDIPropertyConnectionUniqueID = Dlfcn.GetIntPtr (midiLibrary, "kMIDIPropertyConnectionUniqueID");
@@ -709,7 +714,10 @@ namespace MonoMac.CoreMidi {
 	delegate void MidiReadProc (IntPtr packetList, IntPtr context, IntPtr srcPtr);
 	
 	public class MidiPort : MidiObject {
-		
+		#if !MONOMAC
+		static bool isDevice = MonoTouch.ObjCRuntime.Runtime.Arch == MonoTouch.ObjCRuntime.Arch.DEVICE;
+		#endif
+
 		[DllImport (Constants.CoreMidiLibrary)]
 		extern static int MIDIInputPortCreate (IntPtr client, IntPtr portName, MidiReadProc readProc, IntPtr context, out IntPtr midiPort);
 
@@ -763,9 +771,15 @@ namespace MonoMac.CoreMidi {
 			int p = 4;
 			var packets = new MidiPacket [npackets];
 			for (int i = 0; i < npackets; i++){
-				ushort len = (ushort) Marshal.ReadInt16 (p, 8);
-				packets [i] = new MidiPacket (Marshal.ReadInt64 (packetList, p), len, (IntPtr)((long)packetList + 10));
-				p += 10 + len;
+				ushort len = (ushort) Marshal.ReadInt16 (packetList, p+8);
+				packets [i] = new MidiPacket (Marshal.ReadInt64 (packetList, p), len, (IntPtr)((long)packetList + p + 10));
+#if !MONOMAC
+				// MIDIPacket must be 4-byte aligned for ARM
+				if (isDevice)
+					p += (((10 + len) + 3) & ~3);
+				else
+#endif
+					p += 10 + len;
 			}
 			return packets;
 		}
