@@ -1,5 +1,6 @@
 //
 // MonoPInvokeCallbackAttribute.cs: necessary for AOT ports of Mono
+// Copyright 2011, 2012 Xamarin Inc
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -25,7 +26,7 @@ using System;
 namespace MonoMac {
 
 	[AttributeUsage (AttributeTargets.Delegate)]
-	class MonoNativeFunctionWrapperAttribute : Attribute {
+	public class MonoNativeFunctionWrapperAttribute : Attribute {
 	}
 }
 
