@@ -1,3 +1,4 @@
+// Copyright 2011, 2012 Xamarin Inc
 
 namespace MonoMac.Security {
 
@@ -25,6 +26,20 @@ namespace MonoMac.Security {
 		PKCS1MD2  = 0x8000,
 		PKCS1MD5  = 0x8001,
 		PKCS1SHA1 = 0x8002,
+		PKCS1SHA224 = 0x8003,
+		PKCS1SHA256 = 0x8004,
+		PKCS1SHA384 = 0x8005,
+		PKCS1SHA512 = 0x8006,
 	}
 	
-}
+	public enum SecTrustResult {
+		Invalid,
+		Proceed,
+		Confirm,
+		Deny,
+		Unspecified,
+		RecoverableTrustFailure,
+		FatalTrustFailure,
+		ResultOtherError,
+	}
+}
\ No newline at end of file
