@@ -22,6 +22,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 using System;
+using MonoMac.ObjCRuntime;
 
 namespace MonoMac.AppKit {
 
@@ -328,7 +329,10 @@ namespace MonoMac.AppKit {
 		Swipe = 31,
 		Rotate = 18,
 		BeginGesture = 19,
-		EndGesture = 20
+		EndGesture = 20,
+
+		SmartMagnify = 32,
+		QuickLook = 33
 	}
 
 	[Flags]
@@ -830,7 +834,8 @@ namespace MonoMac.AppKit {
 		Save, SaveAs, SaveTo,
 		Autosave = 3,	/* Deprecated name in Lion */
 		Elsewhere = 3,	/* New Lion name */
-		InPlace = 4	/* New in Lion */
+		InPlace = 4,	/* New in Lion */
+		AutoSaveAs = 5	/* New in Mountain Lion */
 	}
 
 #endregion
@@ -1356,13 +1361,23 @@ namespace MonoMac.AppKit {
 	}
 
 	public enum NSOpenGLContextParameter {
-		SwapRectangle = 200,
-		SwapRectangleEnable = 201,
-		RasterizationEnable = 221,
+		[Obsolete] SwapRectangle = 200,
+		[Obsolete] SwapRectangleEnable = 201,
+		[Obsolete] RasterizationEnable = 221,
+		[Obsolete] StateValidation = 301,
+		[Obsolete] SurfaceSurfaceVolatile = 306,
+
 		SwapInterval = 222,
 		SurfaceOrder = 235,
 		SurfaceOpacity = 236,
-		StateValidation = 301,
+
+		[Lion] SurfaceBackingSize = 304,
+		[Lion] ReclaimResources = 308,
+		[Lion] CurrentRendererID = 309,
+		[Lion] GpuVertexProcessing = 310,
+		[Lion] GpuFragmentProcessing = 311,
+		[Lion] HasDrawable = 314,
+		[Lion] MpsSwapsInFlight = 315
 	}
 	
 	public enum NSSurfaceOrder {
@@ -1373,6 +1388,7 @@ namespace MonoMac.AppKit {
 	public enum NSOpenGLPixelFormatAttribute {
 		AllRenderers       =   1,
 		DoubleBuffer       =   5,
+		[Lion] TrippleBuffer = 3,
 		Stereo             =   6,
 		AuxBuffers         =   7,
 		ColorSize          =   8,
@@ -1396,20 +1412,29 @@ namespace MonoMac.AppKit {
 		NoRecovery         =  72,
 		Accelerated        =  73,
 		ClosestPolicy      =  74,
-		Robust             =  75,
 		BackingStore       =  76,
-		MPSafe             =  78,
 		Window             =  80,
-		MultiScreen        =  81,
 		Compliant          =  83,
 		ScreenMask         =  84,
 		PixelBuffer        =  90,
 		RemotePixelBuffer  =  91,
 		AllowOfflineRenderers = 96,
 		AcceleratedCompute =  97,
+
+		// Specify the profile
+		[Lion] OpenGLProfile = 99,
 		VirtualScreenCount = 128,
+
+		[Obsolete] Robust  =  75,
+		[Obsolete] MPSafe  =  78,
+		[Obsolete] MultiScreen =  81
 	}
-    
+
+	public enum NSOpenGLProfile {
+		VersionLegacy   = 0x1000, // Legacy
+		Version3_2Core  = 0x3200  // 3.2 or better
+	}
+	
 	public enum NSAlertButtonReturn {
 		First = 1000,
 		Second = 1001,
@@ -1419,8 +1444,9 @@ namespace MonoMac.AppKit {
 	public enum NSOpenGLGlobalOption {
 		FormatCacheSize = 501,
 		ClearFormatCache = 502,
-		RetainRenderers  = 503,
-		ResetLibrary     = 504
+		RetainRenderers = 503,
+		[Lion] UseBuildCache = 506,
+		[Obsolete] ResetLibrary = 504
 	}
 
 	public enum NSGLTextureTarget {
@@ -1469,9 +1495,9 @@ namespace MonoMac.AppKit {
 	}
 
 	public static class NSFileTypeForHFSTypeCode {
-		public static string ComputerIcon = "root";
-		public static string DesktopIcon = "desk";
-		public static string FinderIcon = "FNDR";
+		public static readonly string ComputerIcon = "root";
+		public static readonly string DesktopIcon = "desk";
+		public static readonly string FinderIcon = "FNDR";
 	}
 	
 	// These constants specify the possible states of a drawer.
@@ -1592,6 +1618,8 @@ namespace MonoMac.AppKit {
 	}
 
 	public enum NSLayoutFormatOptions {
+		None = 0,
+
 		AlignAllLeft = (1 << NSLayoutAttribute.Left),
 		AlignAllRight = (1 << NSLayoutAttribute.Right),
 		AlignAllTop = (1 << NSLayoutAttribute.Top),
@@ -1665,6 +1693,7 @@ namespace MonoMac.AppKit {
 		Default = 0, None = 2, DocumentWindow, UtilityWindow, AlertPanel
 	}
 
+	[Lion]
 	public enum NSTextFinderAction {
 		ShowFindInterface = 1,
 		NextMatch = 2,
@@ -1680,4 +1709,60 @@ namespace MonoMac.AppKit {
 		ShowReplaceInterface = 12,
 		HideReplaceInterface = 13
 	}
-} 
+
+	[Flags]
+	public enum NSFontPanelMode {
+		FaceMask = 1 << 0,
+		SizeMask = 1 << 1,
+		CollectionMask = 1 << 2,
+		UnderlineEffectMask = 1<<8,
+		StrikethroughEffectMask = 1<<9,
+		TextColorEffectMask = 1<< 10,
+		DocumentColorEffectMask = 1<<11,
+		ShadowEffectMask = 1<<12,
+		AllEffectsMask = 0XFFF00,
+		StandardMask = 0xFFFF,
+		AllModesMask = unchecked( (int)0xFFFFFFFF )
+	}
+
+	[Flags]
+	public enum NSFontCollectionVisibility {
+		Process = 1 << 0,
+		User = 1 << 1,
+		Computer = 1 << 2,
+	}
+
+	public enum NSSharingContentScope {
+		Item,
+		Partial,
+		Full
+	}
+
+	public enum NSSharingServiceName {
+		PostOnFacebook,
+		PostOnTwitter,
+		PostOnSinaWeibo,
+		ComposeEmail,
+		ComposeMessage,
+		SendViaAirDrop,
+		AddToSafariReadingList,
+		AddToIPhoto,
+		AddToAperture,
+		UseAsTwitterProfileImage,
+		UseAsDesktopPicture,
+		PostImageOnFlickr,
+		PostVideoOnVimeo,
+		PostVideoOnYouku,
+		PostVideoOnTudou
+	}
+
+	[Flags]
+	public enum NSTypesetterControlCharacterAction {
+		ZeroAdvancement = 1 << 0,
+		Whitespace = 1 << 1,
+		HorizontalTab = 1 << 2,
+		LineBreak = 1 << 3,
+		ParagraphBreak = 1 << 4,
+		ContainerBreak = 1 << 5,
+	}
+}
