@@ -25,6 +25,7 @@
 // Class.cs
 //
 // Copyright 2009 Novell, Inc
+// Copyright 2013 Xamarin Inc
 //
 using System;
 using System.Reflection;
@@ -35,13 +36,12 @@ using MonoMac.Foundation;
 
 namespace MonoMac.ObjCRuntime {
 	 public class Class : INativeObject {
-#if OBJECT_REF_TRACKING
-		static NativeMethodBuilder release_builder = new NativeMethodBuilder (typeof (NSObject).GetMethod ("NativeRelease", BindingFlags.NonPublic | BindingFlags.Instance));
-		static NativeMethodBuilder retain_builder = new NativeMethodBuilder (typeof (NSObject).GetMethod ("NativeRetain", BindingFlags.NonPublic | BindingFlags.Instance));
-#endif
+		public static bool ThrowOnInitFailure = true;
+
 		static Dictionary <IntPtr, Type> type_map = new Dictionary <IntPtr, Type> ();
-		static List <Type> custom_types = new List <Type> ();
+		static Dictionary <Type, Type> custom_types = new Dictionary <Type, Type> ();
 		static List <Delegate> method_wrappers = new List <Delegate> ();
+		static object lock_obj = new object ();
 
 		internal IntPtr handle;
 
@@ -49,7 +49,7 @@ namespace MonoMac.ObjCRuntime {
 			this.handle = objc_getClass (name);
 
 			if (this.handle == IntPtr.Zero)
-				throw new ArgumentException ("name is an unknown class", name);
+				throw new ArgumentException (String.Format ("name {0} is an unknown class", name), "name");
 		}
 
 		public Class (Type type) {
@@ -78,128 +78,167 @@ namespace MonoMac.ObjCRuntime {
 				return Marshal.PtrToStringAuto (ptr);
 			}
 		}
+		
+		internal static string GetName (IntPtr @class)
+		{
+			return Marshal.PtrToStringAuto (class_getName (@class));
+		}
 
 		public static IntPtr GetHandle (string name) {
 			return objc_getClass (name);
 		}
-
-		public static bool IsCustomType (Type type) {
-			return custom_types.Contains (type);
+		
+		public static IntPtr GetHandle (Type type) {
+			RegisterAttribute attr = (RegisterAttribute) Attribute.GetCustomAttribute (type, typeof (RegisterAttribute), false);
+			string name = attr == null ? type.FullName : attr.Name ?? type.FullName;
+			bool is_wrapper = attr == null ? false : attr.IsWrapper;
+			var handle = objc_getClass (name);
+			
+			if (handle == IntPtr.Zero)
+				handle = Class.Register (type, name, is_wrapper);
+			
+			return handle;
 		}
 
-		internal static Type Lookup (IntPtr klass) {
+		public static bool IsCustomType (Type type) {
+			lock (lock_obj)
+				return custom_types.ContainsKey (type);
+		}
+
+		internal static Type Lookup (IntPtr klass)
+		{
+			return Lookup (klass, true);
+		}
+
+		internal static Type Lookup (IntPtr klass, bool throw_on_error) {
 			// FAST PATH
-			Type type;
-			if (type_map.TryGetValue (klass, out type))
-				return type;
-
-			// TODO:  When we type walk we currently populate the type map
-			// from the walk point with the target, we should gather some 
-			// stats here, and see how many times there is a intermediate class
-			// and see if we should populate them in the map as well
-			IntPtr orig_klass = klass;
-
-			do {
-				IntPtr kls = class_getSuperclass (klass);
-
-				if (type_map.TryGetValue (kls, out type)) {
-					type_map [orig_klass] = type;
+			lock (lock_obj) {
+				Type type;
+				if (type_map.TryGetValue (klass, out type))
 					return type;
-				}
 
-				klass = kls;
-			} while (true);
+				// TODO:  When we type walk we currently populate the type map
+				// from the walk point with the target, we should gather some 
+				// stats here, and see how many times there is a intermediate class
+				// and see if we should populate them in the map as well
+				IntPtr orig_klass = klass;
+
+				do {
+					IntPtr kls = class_getSuperclass (klass);
+
+					if (type_map.TryGetValue (kls, out type)) {
+						type_map [orig_klass] = type;
+						return type;
+					}
+
+					if (kls == IntPtr.Zero) {
+						if (!throw_on_error)
+							return null;
+
+						var message = "Could not find a valid superclass for type " + new Class (orig_klass).Name 
+							+ ". Did you forget to register the bindings at " + typeof(Class).FullName
+							+ ".Register() or call NSApplication.Init()?";
+						throw new ArgumentException (message);
+					}
+
+					klass = kls;
+				} while (true);
+			}
 		}
 
 		internal static IntPtr Register (Type type) { 
 			RegisterAttribute attr = (RegisterAttribute) Attribute.GetCustomAttribute (type, typeof (RegisterAttribute), false);
 			string name = attr == null ? type.FullName : attr.Name ?? type.FullName;
-			return Class.Register (type, name);
+			bool is_wrapper = attr == null ? false : attr.IsWrapper;
+			return Register (type, name, is_wrapper);
 		}
 
-		internal unsafe static IntPtr Register (Type type, string name) {
+		static IntPtr Register (Type type, string name, bool is_wrapper) {
 			IntPtr parent = IntPtr.Zero;
 			IntPtr handle = IntPtr.Zero;
 
 			handle = objc_getClass (name);
 
-			if (handle != IntPtr.Zero) {
-				if (!type_map.ContainsKey (handle)) {
-					type_map [handle] = type;
+			lock (lock_obj) {
+				if (handle != IntPtr.Zero) {
+					if (!type_map.ContainsKey (handle)) {
+						type_map [handle] = type;
+					}
+					return handle;
 				}
+
+				if (objc_getProtocol (name) != IntPtr.Zero)
+					throw new ArgumentException ("Attempting to register a class named: " + name + " which is a valid protocol");
+				
+				if (is_wrapper)
+					return IntPtr.Zero;
+
+				Type parent_type = type.BaseType;
+				string parent_name = null;
+				while (Attribute.IsDefined (parent_type, typeof (ModelAttribute), false))
+					parent_type = parent_type.BaseType;
+				RegisterAttribute parent_attr = (RegisterAttribute)Attribute.GetCustomAttribute (parent_type, typeof(RegisterAttribute), false);
+				parent_name = parent_attr == null ? parent_type.FullName : parent_attr.Name ?? parent_type.FullName;
+				parent = objc_getClass (parent_name);
+				if (parent == IntPtr.Zero && parent_type.Assembly != NSObject.MonoMacAssembly) {
+					bool parent_is_wrapper = parent_attr == null ? false : parent_attr.IsWrapper;
+					// Its possible as we scan that we might be derived from a type that isn't reigstered yet.
+					Register (parent_type, parent_name, parent_is_wrapper);
+					parent = objc_getClass (parent_name);
+				}
+				if (parent == IntPtr.Zero) {
+					// This spams mtouch, we need a way to differentiate from mtouch's (ab)use
+					// Console.WriteLine ("CRITICAL WARNING: Falling back to NSObject for type {0} reported as {1}", type, parent_type);
+					parent = objc_getClass ("NSObject");
+				}
+				handle = objc_allocateClassPair (parent, name, IntPtr.Zero);
+
+				foreach (PropertyInfo prop in type.GetProperties (BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance)) {
+					ConnectAttribute cattr = (ConnectAttribute)Attribute.GetCustomAttribute (prop, typeof(ConnectAttribute));
+					if (cattr != null) {
+						string ivar_name = cattr.Name ?? prop.Name;
+						class_addIvar (handle, ivar_name, (IntPtr)Marshal.SizeOf (typeof(IntPtr)), (ushort)Math.Log (Marshal.SizeOf (typeof(IntPtr)), 2), "@");
+					}
+
+					RegisterProperty (prop, type, handle);
+				}
+	
+				NSObject.OverrideRetainAndRelease (handle);
+
+				foreach (MethodInfo minfo in type.GetMethods (BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static))
+					RegisterMethod (minfo, type, handle);
+
+				ConstructorInfo default_ctor = type.GetConstructor (Type.EmptyTypes);
+				if (default_ctor != null) {
+					NativeConstructorBuilder builder = new NativeConstructorBuilder (default_ctor);
+
+					class_addMethod (handle, builder.Selector, builder.Delegate, builder.Signature);
+					method_wrappers.Add (builder.Delegate);
+#if DEBUG
+					Console.WriteLine ("[CTOR] Registering {0}[0x{1:x}|{2}] on {3} -> ({4})", "init", (int) builder.Selector, builder.Signature, type, default_ctor);
+#endif
+				}
+
+				foreach (ConstructorInfo cinfo in type.GetConstructors (BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)) {
+					ExportAttribute ea = (ExportAttribute)Attribute.GetCustomAttribute (cinfo, typeof(ExportAttribute));
+					if (ea == null)
+						continue;
+					NativeConstructorBuilder builder = new NativeConstructorBuilder (cinfo);
+
+					class_addMethod (handle, builder.Selector, builder.Delegate, builder.Signature);
+					method_wrappers.Add (builder.Delegate);
+#if DEBUG
+					Console.WriteLine ("[CTOR] Registering {0}[0x{1:x}|{2}] on {3} -> ({4})", ea.Selector, (int) builder.Selector, builder.Signature, type, cinfo);
+#endif
+				}
+
+				objc_registerClassPair (handle);
+
+				type_map [handle] = type;
+				custom_types.Add (type, type);
+
 				return handle;
 			}
-
-			if (objc_getProtocol (name) != IntPtr.Zero)
-				throw new ArgumentException ("Attempting to register a class named: " + name + " which is a valid protocol");
-
-			Type parent_type = type.BaseType;
-			string parent_name = null;
-			while (Attribute.IsDefined (parent_type, typeof (ModelAttribute), false))
-				parent_type = parent_type.BaseType;
-			RegisterAttribute parent_attr = (RegisterAttribute) Attribute.GetCustomAttribute (parent_type, typeof (RegisterAttribute), false);
-			parent_name = parent_attr == null ? parent_type.FullName : parent_attr.Name ?? parent_type.FullName;
-			parent = objc_getClass (parent_name);
-			if (parent == IntPtr.Zero && parent_type.Assembly != NSObject.MonoMacAssembly) {
-				// Its possible as we scan that we might be derived from a type that isn't reigstered yet.
-				Class.Register (parent_type, parent_name);
-				parent = objc_getClass (parent_name);
-			}
-			if (parent == IntPtr.Zero) {
-				// This spams mtouch, we need a way to differentiate from mtouch's (ab)use
-				// Console.WriteLine ("CRITICAL WARNING: Falling back to NSObject for type {0} reported as {1}", type, parent_type);
-				parent = objc_getClass ("NSObject");
-			}
-			handle = objc_allocateClassPair (parent, name, IntPtr.Zero);
-
-			foreach (PropertyInfo prop in type.GetProperties (BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance)) {
-				ConnectAttribute cattr = (ConnectAttribute) Attribute.GetCustomAttribute (prop, typeof (ConnectAttribute));
-				if (cattr != null) {
-					string ivar_name = cattr.Name ?? prop.Name;
-					class_addIvar (handle, ivar_name, (IntPtr) Marshal.SizeOf (typeof (IntPtr)), (ushort) Math.Log (Marshal.SizeOf (typeof (IntPtr)), 2), "@");
-				}
-
-				RegisterProperty (prop, type, handle);
-			}
-	
-#if OBJECT_REF_TRACKING
-			class_addMethod (handle, release_builder.Selector, release_builder.Delegate, release_builder.Signature);
-			class_addMethod (handle, retain_builder.Selector, retain_builder.Delegate, retain_builder.Signature);
-#endif
-
-			foreach (MethodInfo minfo in type.GetMethods (BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static))
-				RegisterMethod (minfo, type, handle);
-
-			ConstructorInfo default_ctor = type.GetConstructor (Type.EmptyTypes);
-			if (default_ctor != null) {
-				NativeConstructorBuilder builder = new NativeConstructorBuilder (default_ctor);
-
-				class_addMethod (handle, builder.Selector, builder.Delegate, builder.Signature);
-				method_wrappers.Add (builder.Delegate);
-#if DEBUG
-				Console.WriteLine ("[CTOR] Registering {0}[0x{1:x}|{2}] on {3} -> ({4})", "init", (int) builder.Selector, builder.Signature, type, default_ctor);
-#endif
-			}
-
-			foreach (ConstructorInfo cinfo in type.GetConstructors (BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)) {
-				ExportAttribute ea = (ExportAttribute) Attribute.GetCustomAttribute (cinfo, typeof (ExportAttribute));
-				if (ea == null)
-					continue;
-				NativeConstructorBuilder builder = new NativeConstructorBuilder (cinfo);
-
-				class_addMethod (handle, builder.Selector, builder.Delegate, builder.Signature);
-				method_wrappers.Add (builder.Delegate);
-#if DEBUG
-				Console.WriteLine ("[CTOR] Registering {0}[0x{1:x}|{2}] on {3} -> ({4})", ea.Selector, (int) builder.Selector, builder.Signature, type, cinfo);
-#endif
-			}
-
-			objc_registerClassPair (handle);
-
-			type_map [handle] = type;
-			custom_types.Add (type);
-
-			return handle;
 		}
 
 		// FIXME: This doesn't properly handle virtual properties yet
@@ -208,12 +247,32 @@ namespace MonoMac.ObjCRuntime {
 			if (ea == null)
 				return;
 			
+			if (prop.PropertyType.IsGenericType || prop.PropertyType.IsGenericTypeDefinition)
+				throw new ArgumentException (string.Format ("Cannot export the property '{0}.{1}': it is generic.", prop.DeclaringType.FullName, prop.Name));
+
 			var m = prop.GetGetMethod (true);
 			if (m != null)
 				RegisterMethod (m, ea.ToGetter (prop), type, handle);
 			m = prop.GetSetMethod (true);
 			if (m != null)
 				RegisterMethod (m, ea.ToSetter (prop), type, handle);
+				
+			// http://developer.apple.com/library/mac/#documentation/Cocoa/Conceptual/ObjCRuntimeGuide/Articles/ocrtPropertyIntrospection.html
+			int count = 0;
+			var props = new objc_attribute_prop [3];
+			props [count++] = new objc_attribute_prop { name = "T", value = TypeConverter.ToNative (prop.PropertyType) };
+			switch (ea.ArgumentSemantic) {
+			case ArgumentSemantic.Copy:
+				props [count++] = new objc_attribute_prop { name = "C", value = "" };
+				break;
+			case ArgumentSemantic.Retain:
+				props [count++] = new objc_attribute_prop { name = "&", value = "" };
+				break;
+			}
+			props [count++] = new objc_attribute_prop { name = "V", value = ea.Selector };
+			
+			class_addProperty (handle, ea.Selector, props, count);
+			
 		}
 
 		private unsafe static void RegisterMethod (MethodInfo minfo, Type type, IntPtr handle) {
@@ -271,13 +330,16 @@ namespace MonoMac.ObjCRuntime {
 			foreach (var field in t.GetFields (BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance)) {
 				if (field.FieldType == typeof (double) || field.FieldType == typeof (float))
 					return true;
+				if (field.FieldType == t)
+					continue;
 				if (TypeContainsFloatingPoint (field.FieldType))
 					return true;
 			}
 
 			return false;
 		}
-		
+
+		[MonoNativeFunctionWrapper]
 		delegate int getFrameLengthDelegate (IntPtr @this, IntPtr sel);
 		static getFrameLengthDelegate getFrameLength = Selector.GetFrameLength;
 		static IntPtr getFrameLengthPtr = Marshal.GetFunctionPointerForDelegate (getFrameLength);
@@ -388,7 +450,8 @@ retl    $0x4                   */  0xc2, 0x04, 0x00,
 			NativeMethodBuilder builder = new NativeMethodBuilder (minfo, type, ea);
 
 			class_addMethod (minfo.IsStatic ? ((objc_class *) handle)->isa : handle, builder.Selector, GetFunctionPointer (minfo, builder.Delegate), builder.Signature);
-			method_wrappers.Add (builder.Delegate);
+			lock (lock_obj)
+				method_wrappers.Add (builder.Delegate);
 #if DEBUG
 			Console.WriteLine ("[METHOD] Registering {0}[0x{1:x}|{2}] on {3} -> ({4})", ea.Selector, (int) builder.Selector, builder.Signature, type, minfo);
 #endif
@@ -410,14 +473,47 @@ retl    $0x4                   */  0xc2, 0x04, 0x00,
 		[DllImport ("/usr/lib/libobjc.dylib")]
 		extern static bool class_addIvar (IntPtr cls, string name, IntPtr size, ushort alignment, string types);
 		[DllImport ("/usr/lib/libobjc.dylib")]
-		extern static bool class_addMethod (IntPtr cls, IntPtr name, Delegate imp, string types);
+		internal extern static bool class_addMethod (IntPtr cls, IntPtr name, Delegate imp, string types);
 		[DllImport ("/usr/lib/libobjc.dylib")]
-		extern static bool class_addMethod (IntPtr cls, IntPtr name, IntPtr imp, string types);
+		internal extern static bool class_addMethod (IntPtr cls, IntPtr name, IntPtr imp, string types);
 		[DllImport ("/usr/lib/libobjc.dylib")]
 		extern static IntPtr class_getName (IntPtr cls);
 		[DllImport ("/usr/lib/libobjc.dylib")]
-		extern static IntPtr class_getSuperclass (IntPtr cls);
+		internal extern static IntPtr class_getSuperclass (IntPtr cls);
+		[DllImport ("/usr/lib/libobjc.dylib")]
+		internal extern static IntPtr class_getMethodImplementation (IntPtr cls, IntPtr sel);
+		[DllImport ("/usr/lib/libobjc.dylib")]
+		internal extern static IntPtr class_getInstanceVariable (IntPtr cls, string name);
 
+		[MonoNativeFunctionWrapper]
+		delegate IntPtr addPropertyDelegate (IntPtr cls, string name, objc_attribute_prop [] attributes, int count);
+		static addPropertyDelegate addProperty;
+		static bool addPropertyInitialized;
+
+		static IntPtr class_addProperty (IntPtr cls, string name, objc_attribute_prop [] attributes, int count)
+		{
+			if (!addPropertyInitialized) {
+				var handle = Dlfcn.dlopen (Constants.ObjectiveCLibrary, 0);
+				try {
+					var fptr = Dlfcn.dlsym (handle, "class_addProperty");
+					if (fptr != IntPtr.Zero)
+						addProperty = (addPropertyDelegate) Marshal.GetDelegateForFunctionPointer (fptr, typeof (addPropertyDelegate));
+				} finally {
+					Dlfcn.dlclose (handle);
+				}
+				addPropertyInitialized = true;
+			}
+			if (addProperty == null)
+				return IntPtr.Zero;
+			return addProperty (cls, name, attributes, count);
+		}
+
+		[StructLayout (LayoutKind.Sequential, CharSet=CharSet.Ansi)]
+		private struct objc_attribute_prop {
+			[MarshalAs (UnmanagedType.LPStr)] internal string name;
+			[MarshalAs (UnmanagedType.LPStr)] internal string value;
+		}
+		
 		internal struct objc_class {
 			internal IntPtr isa;
 		}
