@@ -42,6 +42,7 @@ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
 using System;
+using System.Collections.Generic;
 using System.Text;
 using NGit;
 using NGit.Api;
@@ -259,7 +260,7 @@ namespace NGit.Api
 			// change third line in topic branch
 			WriteTrashFile(FILE1, "1\n2\n3\ntopic\n");
 			git.Add().AddFilepattern(FILE1).Call();
-			git.Commit().SetMessage("change file1 in topic").Call();
+			RevCommit origHead = git.Commit().SetMessage("change file1 in topic").Call();
 			RebaseResult res = git.Rebase().SetUpstream("refs/heads/master").Call();
 			NUnit.Framework.Assert.AreEqual(RebaseResult.Status.OK, res.GetStatus());
 			CheckFile(theFile, "1master\n2\n3\ntopic\n");
@@ -267,6 +268,7 @@ namespace NGit.Api
 			NUnit.Framework.Assert.AreEqual("refs/heads/topic", db.GetFullBranch());
 			NUnit.Framework.Assert.AreEqual(lastMasterChange, new RevWalk(db).ParseCommit(db.
 				Resolve(Constants.HEAD)).GetParent(0));
+			NUnit.Framework.Assert.AreEqual(origHead, db.ReadOrigHead());
 		}
 
 		/// <exception cref="System.Exception"></exception>
@@ -367,7 +369,7 @@ namespace NGit.Api
 			RebaseResult res = git.Rebase().SetUpstream("refs/heads/master").Call();
 			NUnit.Framework.Assert.AreEqual(RebaseResult.Status.STOPPED, res.GetStatus());
 			NUnit.Framework.Assert.AreEqual(conflicting, res.GetCurrentCommit());
-			CheckFile(FILE1, "<<<<<<< OURS\n1master\n=======\n1topic\n>>>>>>> THEIRS\n2\n3\ntopic4"
+			CheckFile(FILE1, "<<<<<<< Upstream, based on master\n1master\n=======\n1topic", ">>>>>>> e0d1dea change file1 in topic\n2\n3\ntopic4"
 				);
 			NUnit.Framework.Assert.AreEqual(RepositoryState.REBASING_INTERACTIVE, db.GetRepositoryState
 				());
@@ -677,8 +679,8 @@ namespace NGit.Api
 			NUnit.Framework.Assert.AreEqual(RebaseResult.Status.STOPPED, res.GetStatus());
 			res = git.Rebase().SetOperation(RebaseCommand.Operation.SKIP).Call();
 			// TODO is this correct? It is what the command line returns
-			CheckFile(FILE1, "1master\n2\n<<<<<<< OURS\n3master\n=======\n3topic\n>>>>>>> THEIRS\n4\n5topic"
-				);
+			CheckFile(FILE1, "1master\n2\n<<<<<<< Upstream, based on master\n3master\n=======\n3topic"
+				, ">>>>>>> 5afc8df change file1 in topic again\n4\n5topic");
 			NUnit.Framework.Assert.AreEqual(RebaseResult.Status.STOPPED, res.GetStatus());
 		}
 
@@ -872,7 +874,7 @@ namespace NGit.Api
 			string[] lines = convertedAuthor.Split("\n");
 			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_NAME='Author name'", lines[0]);
 			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_EMAIL='a.mail@some.com'", lines[1]);
-			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_DATE='123456789 -0100'", lines[2]);
+			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_DATE='@123456789 -0100'", lines[2]);
 			PersonIdent parsedIdent = git.Rebase().ParseAuthor(Sharpen.Runtime.GetBytesForString
 				(convertedAuthor, "UTF-8"));
 			NUnit.Framework.Assert.AreEqual(ident.GetName(), parsedIdent.GetName());
@@ -888,7 +890,7 @@ namespace NGit.Api
 			lines = convertedAuthor.Split("\n");
 			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_NAME='Author name'", lines[0]);
 			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_EMAIL='a.mail@some.com'", lines[1]);
-			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_DATE='123456789 +0930'", lines[2]);
+			NUnit.Framework.Assert.AreEqual("GIT_AUTHOR_DATE='@123456789 +0930'", lines[2]);
 			parsedIdent = git.Rebase().ParseAuthor(Sharpen.Runtime.GetBytesForString(convertedAuthor
 				, "UTF-8"));
 			NUnit.Framework.Assert.AreEqual(ident.GetName(), parsedIdent.GetName());
@@ -1256,11 +1258,41 @@ namespace NGit.Api
 			NUnit.Framework.Assert.AreEqual(RepositoryState.SAFE, db.GetRepositoryState());
 		}
 
+		/// <exception cref="System.Exception"></exception>
+		[NUnit.Framework.Test]
+		public virtual void TestRebaseWithUncommittedDelete()
+		{
+			// create file0 + file1, add and commit
+			FilePath file0 = WriteTrashFile("file0", "file0");
+			WriteTrashFile(FILE1, "file1");
+			git.Add().AddFilepattern("file0").AddFilepattern(FILE1).Call();
+			RevCommit commit = git.Commit().SetMessage("commit1").Call();
+			// create topic branch
+			CreateBranch(commit, "refs/heads/topic");
+			// still on master / modify file1, add and commit
+			WriteTrashFile(FILE1, "modified file1");
+			git.Add().AddFilepattern(FILE1).Call();
+			git.Commit().SetMessage("commit2").Call();
+			// checkout topic branch / delete file0 and add to index
+			CheckoutBranch("refs/heads/topic");
+			git.Rm().AddFilepattern("file0").Call();
+			// do not commit
+			// rebase
+			RebaseResult result = git.Rebase().SetUpstream("refs/heads/master").Call();
+			NUnit.Framework.Assert.AreEqual(RebaseResult.Status.FAST_FORWARD, result.GetStatus
+				());
+			NUnit.Framework.Assert.IsFalse(file0.Exists(), "File should still be deleted");
+			// index should only have updated file1
+			NUnit.Framework.Assert.AreEqual("[file1, mode:100644, content:modified file1]", IndexState
+				(CONTENT));
+			NUnit.Framework.Assert.AreEqual(RepositoryState.SAFE, db.GetRepositoryState());
+		}
+
 		/// <exception cref="System.IO.IOException"></exception>
 		private int CountPicks()
 		{
 			int count = 0;
-			FilePath todoFile = new FilePath(db.Directory, "rebase-merge/git-rebase-todo");
+			FilePath todoFile = GetTodoFile();
 			BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(
 				todoFile), "UTF-8"));
 			try
@@ -1365,5 +1397,26 @@ namespace NGit.Api
 			NUnit.Framework.Assert.AreEqual(RepositoryState.SAFE, git.GetRepository().GetRepositoryState
 				());
 		}
+
+		/// <exception cref="System.IO.IOException"></exception>
+		[NUnit.Framework.Test]
+		public virtual void TestRebaseShouldBeAbleToHandleEmptyLinesInRebaseTodoFile()
+		{
+			string emptyLine = "\n";
+			string todo = "pick 1111111 Commit 1\n" + emptyLine + "pick 2222222 Commit 2\n" +
+				 emptyLine + "# Comment line at end\n";
+			Write(GetTodoFile(), todo);
+			RebaseCommand rebaseCommand = git.Rebase();
+			IList<RebaseCommand.Step> steps = rebaseCommand.LoadSteps();
+			NUnit.Framework.Assert.AreEqual(2, steps.Count);
+			NUnit.Framework.Assert.AreEqual("1111111", steps[0].commit.Name);
+			NUnit.Framework.Assert.AreEqual("2222222", steps[1].commit.Name);
+		}
+
+		private FilePath GetTodoFile()
+		{
+			FilePath todoFile = new FilePath(db.Directory, "rebase-merge/git-rebase-todo");
+			return todoFile;
+		}
 	}
 }
