@@ -41,6 +41,7 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+using System;
 using System.Collections.Generic;
 using System.IO;
 using System.Text;
@@ -102,6 +103,8 @@ namespace NGit.Api
 
 		private static readonly string ONTO = "onto";
 
+		private static readonly string ONTO_NAME = "onto-name";
+
 		private static readonly string PATCH = "patch";
 
 		private static readonly string REBASE_HEAD = "head";
@@ -119,6 +122,8 @@ namespace NGit.Api
 
 		private RevCommit upstreamCommit;
 
+		private string upstreamCommitName;
+
 		private ProgressMonitor monitor = NullProgressMonitor.INSTANCE;
 
 		private readonly RevWalk walk;
@@ -141,10 +146,14 @@ namespace NGit.Api
 		/// this method twice on an instance.
 		/// </summary>
 		/// <returns>an object describing the result of this command</returns>
-		/// <exception cref="NGit.Api.Errors.NoHeadException"></exception>
-		/// <exception cref="NGit.Api.Errors.RefNotFoundException"></exception>
-		/// <exception cref="NGit.Api.Errors.JGitInternalException"></exception>
-		/// <exception cref="NGit.Api.Errors.GitAPIException"></exception>
+		/// <exception cref="NGit.Api.Errors.GitAPIException">NGit.Api.Errors.GitAPIException
+		/// 	</exception>
+		/// <exception cref="NGit.Api.Errors.WrongRepositoryStateException">NGit.Api.Errors.WrongRepositoryStateException
+		/// 	</exception>
+		/// <exception cref="NGit.Api.Errors.NoHeadException">NGit.Api.Errors.NoHeadException
+		/// 	</exception>
+		/// <exception cref="NGit.Api.Errors.RefNotFoundException">NGit.Api.Errors.RefNotFoundException
+		/// 	</exception>
 		public override RebaseResult Call()
 		{
 			RevCommit newHead = null;
@@ -172,8 +181,18 @@ namespace NGit.Api
 					case RebaseCommand.Operation.CONTINUE:
 					{
 						// fall through
-						string upstreamCommitName = ReadFile(rebaseDir, ONTO);
-						this.upstreamCommit = walk.ParseCommit(repo.Resolve(upstreamCommitName));
+						string upstreamCommitId = ReadFile(rebaseDir, ONTO);
+						try
+						{
+							upstreamCommitName = ReadFile(rebaseDir, ONTO_NAME);
+						}
+						catch (FileNotFoundException)
+						{
+							// Fall back to commit ID if file doesn't exist (e.g. rebase
+							// was started by C Git)
+							upstreamCommitName = upstreamCommitId;
+						}
+						this.upstreamCommit = walk.ParseCommit(repo.Resolve(upstreamCommitId));
 						break;
 					}
 
@@ -238,8 +257,9 @@ namespace NGit.Api
 							// TODO if the content of this commit is already merged
 							// here we should skip this step in order to avoid
 							// confusing pseudo-changed
+							string ourCommitName = GetOurCommitName();
 							CherryPickResult cherryPickResult = new Git(repo).CherryPick().Include(commitToPick
-								).Call();
+								).SetOurCommitName(ourCommitName).Call();
 							switch (cherryPickResult.GetStatus())
 							{
 								case CherryPickResult.CherryPickStatus.FAILED:
@@ -292,6 +312,15 @@ namespace NGit.Api
 			}
 		}
 
+		private string GetOurCommitName()
+		{
+			// If onto is different from upstream, this should say "onto", but
+			// RebaseCommand doesn't support a different "onto" at the moment.
+			string ourCommitName = "Upstream, based on " + Repository.ShortenRefName(upstreamCommitName
+				);
+			return ourCommitName;
+		}
+
 		/// <exception cref="System.IO.IOException"></exception>
 		private void UpdateHead(string headName, RevCommit newHead)
 		{
@@ -336,7 +365,6 @@ namespace NGit.Api
 
 		/// <exception cref="System.IO.IOException"></exception>
 		/// <exception cref="NGit.Api.Errors.NoHeadException"></exception>
-		/// <exception cref="NGit.Api.Errors.JGitInternalException"></exception>
 		private RevCommit CheckoutCurrentHead()
 		{
 			ObjectId headTree = repo.Resolve(Constants.HEAD + "^{tree}");
@@ -462,6 +490,8 @@ namespace NGit.Api
 			// representation for date and timezone
 			sb.Append(GIT_AUTHOR_DATE);
 			sb.Append("='");
+			sb.Append("@");
+			// @ for time in seconds since 1970
 			string externalString = author.ToExternalString();
 			sb.Append(Sharpen.Runtime.Substring(externalString, externalString.LastIndexOf('>'
 				) + 2));
@@ -563,10 +593,8 @@ namespace NGit.Api
 			}
 		}
 
-		/// <exception cref="NGit.Api.Errors.RefNotFoundException"></exception>
 		/// <exception cref="System.IO.IOException"></exception>
-		/// <exception cref="NGit.Api.Errors.NoHeadException"></exception>
-		/// <exception cref="NGit.Api.Errors.JGitInternalException"></exception>
+		/// <exception cref="NGit.Api.Errors.GitAPIException"></exception>
 		private RebaseResult InitFilesAndRewind()
 		{
 			// we need to store everything into files so that we can implement
@@ -629,10 +657,11 @@ namespace NGit.Api
 			Sharpen.Collections.Reverse(cherryPickList);
 			// create the folder for the meta information
 			FileUtils.Mkdir(rebaseDir);
-			CreateFile(repo.Directory, Constants.ORIG_HEAD, headId.Name);
+			repo.WriteOrigHead(headId);
 			CreateFile(rebaseDir, REBASE_HEAD, headId.Name);
 			CreateFile(rebaseDir, HEAD_NAME, headName);
 			CreateFile(rebaseDir, ONTO, upstreamCommit.Name);
+			CreateFile(rebaseDir, ONTO_NAME, upstreamCommitName);
 			CreateFile(rebaseDir, INTERACTIVE, string.Empty);
 			BufferedWriter fw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream
 				(new FilePath(rebaseDir, GIT_REBASE_TODO)), Constants.CHARACTER_ENCODING));
@@ -683,9 +712,9 @@ namespace NGit.Api
 		/// 	</summary>
 		/// <param name="newCommit"></param>
 		/// <returns>the new head, or null</returns>
-		/// <exception cref="NGit.Api.Errors.RefNotFoundException">NGit.Api.Errors.RefNotFoundException
-		/// 	</exception>
 		/// <exception cref="System.IO.IOException">System.IO.IOException</exception>
+		/// <exception cref="NGit.Api.Errors.GitAPIException">NGit.Api.Errors.GitAPIException
+		/// 	</exception>
 		public virtual RevCommit TryFastForward(RevCommit newCommit)
 		{
 			Ref head = repo.GetRef(Constants.HEAD);
@@ -718,7 +747,7 @@ namespace NGit.Api
 		}
 
 		/// <exception cref="System.IO.IOException"></exception>
-		/// <exception cref="NGit.Api.Errors.JGitInternalException"></exception>
+		/// <exception cref="NGit.Api.Errors.GitAPIException"></exception>
 		private RevCommit TryFastForward(string headName, RevCommit oldCommit, RevCommit 
 			newCommit)
 		{
@@ -788,7 +817,8 @@ namespace NGit.Api
 			if (this.operation != RebaseCommand.Operation.BEGIN)
 			{
 				// these operations are only possible while in a rebasing state
-				if (repo.GetRepositoryState() != RepositoryState.REBASING_INTERACTIVE)
+				if (s != RepositoryState.REBASING_INTERACTIVE && s != RepositoryState.REBASING &&
+					 s != RepositoryState.REBASING_REBASING && s != RepositoryState.REBASING_MERGE)
 				{
 					throw new WrongRepositoryStateException(MessageFormat.Format(JGitText.Get().wrongRepositoryState
 						, repo.GetRepositoryState().Name()));
@@ -835,7 +865,8 @@ namespace NGit.Api
 		{
 			try
 			{
-				string commitId = ReadFile(repo.Directory, Constants.ORIG_HEAD);
+				ObjectId origHead = repo.ReadOrigHead();
+				string commitId = origHead != null ? origHead.Name : null;
 				monitor.BeginTask(MessageFormat.Format(JGitText.Get().abortingRebase, commitId), 
 					ProgressMonitor.UNKNOWN);
 				DirCacheCheckout dco;
@@ -945,7 +976,7 @@ namespace NGit.Api
 		}
 
 		/// <exception cref="System.IO.IOException"></exception>
-		private IList<RebaseCommand.Step> LoadSteps()
+		internal virtual IList<RebaseCommand.Step> LoadSteps()
 		{
 			byte[] buf = IOUtil.ReadFully(new FilePath(rebaseDir, GIT_REBASE_TODO));
 			int ptr = 0;
@@ -955,7 +986,7 @@ namespace NGit.Api
 			{
 				tokenBegin = ptr;
 				ptr = RawParseUtils.NextLF(buf, ptr);
-				int nextSpace = 0;
+				int nextSpace = RawParseUtils.Next(buf, tokenBegin, ' ');
 				int tokenCount = 0;
 				RebaseCommand.Step current = null;
 				while (tokenCount < 3 && nextSpace < ptr)
@@ -964,7 +995,6 @@ namespace NGit.Api
 					{
 						case 0:
 						{
-							nextSpace = RawParseUtils.Next(buf, tokenBegin, ' ');
 							string actionToken = Sharpen.Runtime.GetStringForBytes(buf, tokenBegin, nextSpace
 								 - tokenBegin - 1);
 							tokenBegin = nextSpace;
@@ -1023,6 +1053,7 @@ namespace NGit.Api
 		public virtual NGit.Api.RebaseCommand SetUpstream(RevCommit upstream)
 		{
 			this.upstreamCommit = upstream;
+			this.upstreamCommitName = upstream.Name;
 			return this;
 		}
 
@@ -1036,6 +1067,7 @@ namespace NGit.Api
 			try
 			{
 				this.upstreamCommit = walk.ParseCommit(upstream);
+				this.upstreamCommitName = upstream.Name;
 			}
 			catch (IOException e)
 			{
@@ -1063,6 +1095,7 @@ namespace NGit.Api
 						, upstream));
 				}
 				upstreamCommit = walk.ParseCommit(repo.Resolve(upstream));
+				upstreamCommitName = upstream;
 				return this;
 			}
 			catch (IOException ioe)
@@ -1071,6 +1104,31 @@ namespace NGit.Api
 			}
 		}
 
+		/// <summary>Optionally override the name of the upstream.</summary>
+		/// <remarks>
+		/// Optionally override the name of the upstream. If this is used, it has to
+		/// come after any
+		/// <see cref="SetUpstream(NGit.Revwalk.RevCommit)">SetUpstream(NGit.Revwalk.RevCommit)
+		/// 	</see>
+		/// call.
+		/// </remarks>
+		/// <param name="upstreamName">the name which will be used to refer to upstream in conflicts
+		/// 	</param>
+		/// <returns>
+		/// 
+		/// <code>this</code>
+		/// </returns>
+		public virtual NGit.Api.RebaseCommand SetUpstreamName(string upstreamName)
+		{
+			if (upstreamCommit == null)
+			{
+				throw new InvalidOperationException("setUpstreamName must be called after setUpstream."
+					);
+			}
+			this.upstreamCommitName = upstreamName;
+			return this;
+		}
+
 		/// <param name="operation">the operation to perform</param>
 		/// <returns>
 		/// 
@@ -1108,6 +1166,11 @@ namespace NGit.Api
 				return this.token;
 			}
 
+			public override string ToString()
+			{
+				return "Action[" + token + "]";
+			}
+
 			internal static RebaseCommand.Action Parse(string token)
 			{
 				if (token.Equals("pick") || token.Equals("p"))
@@ -1131,6 +1194,13 @@ namespace NGit.Api
 			{
 				this.action = action;
 			}
+
+			public override string ToString()
+			{
+				return "Step[" + action + ", " + ((commit == null) ? "null" : commit.ToString ()) + ", " + ((
+					shortMessage == null) ? "null" : Sharpen.Runtime.GetStringForBytes(shortMessage)
+					) + "]";
+			}
 		}
 
 		internal virtual PersonIdent ParseAuthor(byte[] raw)
@@ -1161,7 +1231,17 @@ namespace NGit.Api
 			string email = keyValueMap.Get(GIT_AUTHOR_EMAIL);
 			string time = keyValueMap.Get(GIT_AUTHOR_DATE);
 			// the time is saved as <seconds since 1970> <timezone offset>
-			long when = long.Parse(Sharpen.Runtime.Substring(time, 0, time.IndexOf(' '))) * 1000;
+			int timeStart = 0;
+			if (time.StartsWith("@"))
+			{
+				timeStart = 1;
+			}
+			else
+			{
+				timeStart = 0;
+			}
+			long when = long.Parse(Sharpen.Runtime.Substring(time, timeStart, time.IndexOf(' '
+				))) * 1000;
 			string tzOffsetString = Sharpen.Runtime.Substring(time, time.IndexOf(' ') + 1);
 			int multiplier = -1;
 			if (tzOffsetString[0] == '+')
