@@ -139,7 +139,7 @@ namespace NGit.Dircache
 
 		/// <exception cref="System.IO.IOException"></exception>
 		internal DirCacheEntry(byte[] sharedInfo, MutableInteger infoAt, InputStream @in, 
-			MessageDigest md)
+			MessageDigest md, int smudge_s, int smudge_ns)
 		{
 			// private static final int P_CTIME_NSEC = 4;
 			// private static final int P_MTIME_NSEC = 12;
@@ -214,6 +214,10 @@ namespace NGit.Dircache
 				IOUtil.SkipFully(@in, padLen);
 				md.Update(nullpad, 0, padLen);
 			}
+			if (MightBeRacilyClean(smudge_s, smudge_ns))
+			{
+				SmudgeRacilyClean();
+			}
 		}
 
 		/// <summary>Create an empty entry at stage 0.</summary>
@@ -506,6 +510,14 @@ namespace NGit.Dircache
 			}
 		}
 
+		/// <summary>Returns whether this entry is in the fully-merged stage (0).</summary>
+		/// <remarks>Returns whether this entry is in the fully-merged stage (0).</remarks>
+		/// <returns>true if this entry is merged</returns>
+		public virtual bool IsMerged()
+		{
+			return Stage == STAGE_0;
+		}
+
 		/// <summary>
 		/// Obtain the raw
 		/// <see cref="NGit.FileMode">NGit.FileMode</see>
@@ -840,7 +852,7 @@ namespace NGit.Dircache
 					{
 						// Tree's never have a backslash in them, not even on Windows
 						// but even there we regard it as an invalid path
-						if ("Windows".Equals(SystemReader.GetInstance().GetProperty("os.name")))
+						if (SystemReader.GetInstance().IsWindows())
 						{
 							return false;
 						}
