@@ -138,6 +138,8 @@ namespace NGit.Internal
 
 		public string cannotCreateHEAD;
 
+		public string cannotCreateIndexfile;
+
 		public string cannotDeleteCheckedOutBranch;
 
 		public string cannotDeleteFile;
@@ -168,6 +170,8 @@ namespace NGit.Internal
 
 		public string cannotOpenService;
 
+		public string cannotParseDate;
+
 		public string cannotParseGitURIish;
 
 		public string cannotPullOnARepoWithState;
@@ -228,6 +232,8 @@ namespace NGit.Internal
 
 		public string commitOnRepoWithoutHEADCurrentlyNotSupported;
 
+		public string commitAmendOnInitialNotPossible;
+
 		public string compressingObjects;
 
 		public string connectionFailed;
@@ -470,6 +476,8 @@ namespace NGit.Internal
 
 		public string funnyRefname;
 
+		public string gitmodulesNotFound;
+
 		public string headRequiredToStash;
 
 		public string hoursAgo;
@@ -506,6 +514,8 @@ namespace NGit.Internal
 
 		public string inMemoryBufferLimitExceeded;
 
+		public string inputStreamMustSupportMark;
+
 		public string integerValueOutOfRange;
 
 		public string internalRevisionError;
@@ -750,10 +760,14 @@ namespace NGit.Internal
 
 		public string packObjectCountMismatch;
 
+		public string packRefs;
+
 		public string packTooLargeForIndexVersion1;
 
 		public string packWriterStatistics;
 
+		public string panicCantRenameIndexFile;
+
 		public string patchApplyException;
 
 		public string patchFormatException;
@@ -774,6 +788,10 @@ namespace NGit.Internal
 
 		public string propertyIsAlreadyNonNull;
 
+		public string pruneLoosePackedObjects;
+
+		public string pruneLooseUnreferencedObjects;
+
 		public string pullOnRepoWithoutHEADCurrentlyNotSupported;
 
 		public string pullTaskName;
@@ -798,6 +816,8 @@ namespace NGit.Internal
 
 		public string refAlreadyExists;
 
+		public string refAlreadyExists1;
+
 		public string reflogEntryNotFound;
 
 		public string refNotResolved;
@@ -908,6 +928,8 @@ namespace NGit.Internal
 
 		public string sourceRefNotSpecifiedForRefspec;
 
+		public string squashCommitNotUpdatingHEAD;
+
 		public string staleRevFlagsOn;
 
 		public string startingReadStageWithoutWrittenRequestDataPendingIsNotSupported;
@@ -960,6 +982,8 @@ namespace NGit.Internal
 
 		public string transportExceptionReadRef;
 
+		public string transportNeedsRepository;
+
 		public string transportProtoAmazonS3;
 
 		public string transportProtoBundleFile;
