@@ -257,11 +257,22 @@ namespace NGit.Storage.File
 			Ref @ref = null;
 			foreach (string prefix in SEARCH_PATH)
 			{
-				@ref = ReadRef(prefix + needle, packed);
-				if (@ref != null)
+				try
 				{
-					@ref = Resolve(@ref, 0, null, null, packed);
-					break;
+					@ref = ReadRef(prefix + needle, packed);
+					if (@ref != null)
+					{
+						@ref = Resolve(@ref, 0, null, null, packed);
+						break;
+					}
+				}
+				catch (IOException e)
+				{
+					if (!(!needle.Contains("/") && string.Empty.Equals(prefix) && e.InnerException is
+						 InvalidObjectIdException))
+					{
+						throw;
+					}
 				}
 			}
 			FireRefsChanged();
@@ -696,6 +707,146 @@ namespace NGit.Storage.File
 			FireRefsChanged();
 		}
 
+		/// <summary>Adds a set of refs to the set of packed-refs.</summary>
+		/// <remarks>
+		/// Adds a set of refs to the set of packed-refs. Only non-symbolic refs are
+		/// added. If a ref with the given name already existed in packed-refs it is
+		/// updated with the new value. Each loose ref which was added to the
+		/// packed-ref file is deleted. If a given ref can't be locked it will not be
+		/// added to the pack file.
+		/// </remarks>
+		/// <param name="refs">the refs to be added. Must be fully qualified.</param>
+		/// <exception cref="System.IO.IOException">System.IO.IOException</exception>
+		public virtual void Pack(IList<string> refs)
+		{
+			if (refs.Count == 0)
+			{
+				return;
+			}
+			FS fs = parent.FileSystem;
+			// Lock the packed refs file and read the content
+			LockFile lck = new LockFile(packedRefsFile, fs);
+			if (!lck.Lock())
+			{
+				throw new IOException(MessageFormat.Format(JGitText.Get().cannotLock, packedRefsFile
+					));
+			}
+			try
+			{
+				RefDirectory.PackedRefList packed = GetPackedRefs();
+				RefList<Ref> cur = ReadPackedRefs();
+				// Iterate over all refs to be packed
+				foreach (string refName in refs)
+				{
+					Ref @ref = ReadRef(refName, cur);
+					if (@ref.IsSymbolic())
+					{
+						continue;
+					}
+					// can't pack symbolic refs
+					// Add/Update it to packed-refs
+					int idx = cur.Find(refName);
+					if (idx >= 0)
+					{
+						cur = cur.Set(idx, PeeledPackedRef(@ref));
+					}
+					else
+					{
+						cur = cur.Add(idx, PeeledPackedRef(@ref));
+					}
+				}
+				// The new content for packed-refs is collected. Persist it.
+				CommitPackedRefs(lck, cur, packed);
+				// Now delete the loose refs which are now packed
+				foreach (string refName_1 in refs)
+				{
+					// Lock the loose ref
+					FilePath refFile = FileFor(refName_1);
+					if (!refFile.Exists())
+					{
+						continue;
+					}
+					LockFile rLck = new LockFile(refFile, parent.FileSystem);
+					if (!rLck.Lock())
+					{
+						continue;
+					}
+					try
+					{
+						RefDirectory.LooseRef currentLooseRef = ScanRef(null, refName_1);
+						if (currentLooseRef == null || currentLooseRef.IsSymbolic())
+						{
+							continue;
+						}
+						Ref packedRef = cur.Get(refName_1);
+						ObjectId clr_oid = currentLooseRef.GetObjectId();
+						if (clr_oid != null && clr_oid.Equals(packedRef.GetObjectId()))
+						{
+							RefList<RefDirectory.LooseRef> curLoose;
+							RefList<RefDirectory.LooseRef> newLoose;
+							do
+							{
+								curLoose = looseRefs.Get();
+								int idx = curLoose.Find(refName_1);
+								if (idx < 0)
+								{
+									break;
+								}
+								newLoose = curLoose.Remove(idx);
+							}
+							while (!looseRefs.CompareAndSet(curLoose, newLoose));
+							int levels = LevelsIn(refName_1) - 2;
+							Delete(FileFor(refName_1), levels);
+						}
+					}
+					finally
+					{
+						rLck.Unlock();
+					}
+				}
+			}
+			finally
+			{
+				// Don't fire refsChanged. The refs have not change, only their
+				// storage.
+				lck.Unlock();
+			}
+		}
+
+		/// <summary>Make sure a ref is peeled and has the Storage PACKED.</summary>
+		/// <remarks>
+		/// Make sure a ref is peeled and has the Storage PACKED. If the given ref
+		/// has this attributes simply return it. Otherwise create a new peeled
+		/// <see cref="NGit.ObjectIdRef">NGit.ObjectIdRef</see>
+		/// where Storage is set to PACKED.
+		/// </remarks>
+		/// <param name="f"></param>
+		/// <returns>a ref for Storage PACKED having the same name, id, peeledId as f</returns>
+		/// <exception cref="NGit.Errors.MissingObjectException">NGit.Errors.MissingObjectException
+		/// 	</exception>
+		/// <exception cref="System.IO.IOException">System.IO.IOException</exception>
+		private Ref PeeledPackedRef(Ref f)
+		{
+			if (f.GetStorage().IsPacked() && f.IsPeeled())
+			{
+				return f;
+			}
+			if (!f.IsPeeled())
+			{
+				f = Peel(f);
+			}
+			if (f.GetPeeledObjectId() != null)
+			{
+				return new ObjectIdRef.PeeledTag(RefStorage.PACKED, f.GetName(), f.GetObjectId(), 
+					f.GetPeeledObjectId());
+			}
+			else
+			{
+				return new ObjectIdRef.PeeledNonTag(RefStorage.PACKED, f.GetName(), f.GetObjectId
+					());
+			}
+		}
+
 		/// <exception cref="System.IO.IOException"></exception>
 		internal virtual void Log(RefUpdate update, string msg, bool deref)
 		{
@@ -762,7 +913,7 @@ namespace NGit.Storage.File
 				return curList;
 			}
 			RefDirectory.PackedRefList newList = ReadPackedRefs();
-			if (packedRefs.CompareAndSet(curList, newList))
+			if (packedRefs.CompareAndSet(curList, newList) && !curList.id.Equals(newList.id))
 			{
 				modCnt.IncrementAndGet();
 			}
@@ -774,10 +925,11 @@ namespace NGit.Storage.File
 		{
 			FileSnapshot snapshot = FileSnapshot.Save(packedRefsFile);
 			BufferedReader br;
+			MessageDigest digest = Constants.NewMessageDigest();
 			try
 			{
-				br = new BufferedReader(new InputStreamReader(new FileInputStream(packedRefsFile)
-					, Constants.CHARSET));
+				br = new BufferedReader(new InputStreamReader(new DigestInputStream(new FileInputStream
+					(packedRefsFile), digest), Constants.CHARSET));
 			}
 			catch (FileNotFoundException)
 			{
@@ -786,7 +938,8 @@ namespace NGit.Storage.File
 			}
 			try
 			{
-				return new RefDirectory.PackedRefList(ParsePackedRefs(br), snapshot);
+				return new RefDirectory.PackedRefList(ParsePackedRefs(br), snapshot, ObjectId.FromRaw
+					(digest.Digest()));
 			}
 			finally
 			{
@@ -862,12 +1015,12 @@ namespace NGit.Storage.File
 		private void CommitPackedRefs(LockFile lck, RefList<Ref> refs, RefDirectory.PackedRefList
 			 oldPackedList)
 		{
-			new _RefWriter_707(this, lck, oldPackedList, refs, refs).WritePackedRefs();
+			new _RefWriter_826(this, lck, oldPackedList, refs, refs).WritePackedRefs();
 		}
 
-		private sealed class _RefWriter_707 : RefWriter
+		private sealed class _RefWriter_826 : RefWriter
 		{
-			public _RefWriter_707(RefDirectory _enclosing, LockFile lck, RefDirectory.PackedRefList
+			public _RefWriter_826(RefDirectory _enclosing, LockFile lck, RefDirectory.PackedRefList
 				 oldPackedList, RefList<Ref> refs, RefList<Ref> baseArg1) : base(baseArg1)
 			{
 				this._enclosing = _enclosing;
@@ -905,8 +1058,9 @@ namespace NGit.Storage.File
 					throw new ObjectWritingException(MessageFormat.Format(JGitText.Get().unableToWrite
 						, name));
 				}
+				byte[] digest = Constants.NewMessageDigest().Digest(content);
 				this._enclosing.packedRefs.CompareAndSet(oldPackedList, new RefDirectory.PackedRefList
-					(refs, lck.GetCommitSnapshot()));
+					(refs, lck.GetCommitSnapshot(), ObjectId.FromRaw(digest)));
 			}
 
 			private readonly RefDirectory _enclosing;
@@ -1041,15 +1195,16 @@ namespace NGit.Storage.File
 					return @ref;
 				}
 			}
-			catch (ArgumentException)
+			catch (ArgumentException notRef)
 			{
 				while (0 < n && char.IsWhiteSpace((char)buf[n - 1]))
 				{
 					n--;
 				}
 				string content = RawParseUtils.Decode(buf, 0, n);
-				throw new IOException(MessageFormat.Format(JGitText.Get().notARef, name, content)
-					);
+				IOException ioException = new IOException(MessageFormat.Format(JGitText.Get().notARef
+					, name, content), notRef);
+				throw ioException;
 			}
 			return new RefDirectory.LooseUnpeeled(otherSnapshot, name, id);
 		}
@@ -1142,13 +1297,16 @@ namespace NGit.Storage.File
 		private class PackedRefList : RefList<Ref>
 		{
 			internal static readonly RefDirectory.PackedRefList NO_PACKED_REFS = new RefDirectory.PackedRefList
-				(RefList.EmptyList(), FileSnapshot.MISSING_FILE);
+				(RefList.EmptyList(), FileSnapshot.MISSING_FILE, ObjectId.ZeroId);
 
 			internal readonly FileSnapshot snapshot;
 
-			internal PackedRefList(RefList<Ref> src, FileSnapshot s) : base(src)
+			internal readonly ObjectId id;
+
+			internal PackedRefList(RefList<Ref> src, FileSnapshot s, ObjectId i) : base(src)
 			{
 				snapshot = s;
+				id = i;
 			}
 		}
 
