@@ -122,12 +122,12 @@ namespace NGit.Transport
 
 		private bool allowThin;
 
-		private bool checkObjectCollisions;
-
 		private bool needBaseObjectIds;
 
 		private bool checkEofAfterPackFooter;
 
+		private bool expectDataAfterPackFooter;
+
 		private long objectCount;
 
 		private PackedObjectInfo[] entries;
@@ -195,7 +195,6 @@ namespace NGit.Transport
 			objectDigest = Constants.NewMessageDigest();
 			tempObjectId = new MutableObjectId();
 			packDigest = Constants.NewMessageDigest();
-			checkObjectCollisions = true;
 		}
 
 		/// <returns>true if a thin pack (missing base objects) is permitted.</returns>
@@ -217,36 +216,6 @@ namespace NGit.Transport
 			allowThin = allow;
 		}
 
-		/// <returns>if true received objects are verified to prevent collisions.</returns>
-		public virtual bool IsCheckObjectCollisions()
-		{
-			return checkObjectCollisions;
-		}
-
-		/// <summary>Enable checking for collisions with existing objects.</summary>
-		/// <remarks>
-		/// Enable checking for collisions with existing objects.
-		/// <p>
-		/// By default PackParser looks for each received object in the repository.
-		/// If the object already exists, the existing object is compared
-		/// byte-for-byte with the newly received copy to ensure they are identical.
-		/// The receive is aborted with an exception if any byte differs. This check
-		/// is necessary to prevent an evil attacker from supplying a replacement
-		/// object into this repository in the event that a discovery enabling SHA-1
-		/// collisions is made.
-		/// <p>
-		/// This check may be very costly to perform, and some repositories may have
-		/// other ways to segregate newly received object data. The check is enabled
-		/// by default, but can be explicitly disabled if the implementation can
-		/// provide the same guarantee, or is willing to accept the risks associated
-		/// with bypassing the check.
-		/// </remarks>
-		/// <param name="check">true to enable collision checking (strongly encouraged).</param>
-		public virtual void SetCheckObjectCollisions(bool check)
-		{
-			checkObjectCollisions = check;
-		}
-
 		/// <summary>Configure this index pack instance to keep track of new objects.</summary>
 		/// <remarks>
 		/// Configure this index pack instance to keep track of new objects.
@@ -318,6 +287,22 @@ namespace NGit.Transport
 			checkEofAfterPackFooter = b;
 		}
 
+		/// <returns>true if there is data expected after the pack footer.</returns>
+		public virtual bool IsExpectDataAfterPackFooter()
+		{
+			return expectDataAfterPackFooter;
+		}
+
+		/// <param name="e">
+		/// true if there is additional data in InputStream after pack.
+		/// This requires the InputStream to support the mark and reset
+		/// functions.
+		/// </param>
+		public virtual void SetExpectDataAfterPackFooter(bool e)
+		{
+			expectDataAfterPackFooter = e;
+		}
+
 		/// <returns>the new objects that were sent by the user</returns>
 		public virtual ObjectIdSubclassMap<ObjectId> GetNewObjectIds()
 		{
@@ -558,13 +543,13 @@ namespace NGit.Transport
 						if (!IsAllowThin())
 						{
 							throw new IOException(MessageFormat.Format(JGitText.Get().packHasUnresolvedDeltas
-								, (objectCount - entryCount)));
+								, Sharpen.Extensions.ValueOf(objectCount - entryCount)));
 						}
 						ResolveDeltasWithExternalBases(resolving);
 						if (entryCount < objectCount)
 						{
 							throw new IOException(MessageFormat.Format(JGitText.Get().packHasUnresolvedDeltas
-								, (objectCount - entryCount)));
+								, Sharpen.Extensions.ValueOf(objectCount - entryCount)));
 						}
 					}
 					resolving.EndTask();
@@ -639,14 +624,14 @@ namespace NGit.Transport
 
 				default:
 				{
-					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, info
-						.type));
+					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, Sharpen.Extensions.ValueOf
+						(info.type)));
 				}
 			}
 			if (!CheckCRC(oe.GetCRC()))
 			{
 				throw new IOException(MessageFormat.Format(JGitText.Get().corruptionDetectedReReadingAt
-					, oe.GetOffset()));
+					, Sharpen.Extensions.ValueOf(oe.GetOffset())));
 			}
 			ResolveDeltas(visit.Next(), info.type, info, progress);
 		}
@@ -669,8 +654,8 @@ namespace NGit.Transport
 
 					default:
 					{
-						throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, info
-							.type));
+						throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, Sharpen.Extensions.ValueOf
+							(info.type)));
 					}
 				}
 				byte[] delta = InflateAndReturn(PackParser.Source.DATABASE, info.size);
@@ -680,7 +665,7 @@ namespace NGit.Transport
 				if (!CheckCRC(visit.delta.crc))
 				{
 					throw new IOException(MessageFormat.Format(JGitText.Get().corruptionDetectedReReadingAt
-						, visit.delta.position));
+						, Sharpen.Extensions.ValueOf(visit.delta.position)));
 				}
 				objectDigest.Update(Constants.EncodedTypeString(type));
 				objectDigest.Update(unchecked((byte)' '));
@@ -724,8 +709,8 @@ namespace NGit.Transport
 
 					default:
 					{
-						throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, typeCode
-							));
+						throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, Sharpen.Extensions.ValueOf
+							(typeCode)));
 					}
 				}
 			}
@@ -803,8 +788,8 @@ namespace NGit.Transport
 
 				default:
 				{
-					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, info
-						.type));
+					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, Sharpen.Extensions.ValueOf
+						(info.type)));
 				}
 			}
 			return info;
@@ -937,6 +922,14 @@ namespace NGit.Transport
 		/// <exception cref="System.IO.IOException"></exception>
 		private void ReadPackHeader()
 		{
+			if (expectDataAfterPackFooter)
+			{
+				if (!@in.MarkSupported())
+				{
+					throw new IOException(JGitText.Get().inputStreamMustSupportMark);
+				}
+				@in.Mark(buf.Length);
+			}
 			int hdrln = Constants.PACK_SIGNATURE.Length + 4 + 4;
 			int p = Fill(PackParser.Source.INPUT, hdrln);
 			for (int k = 0; k < Constants.PACK_SIGNATURE.Length; k++)
@@ -950,7 +943,7 @@ namespace NGit.Transport
 			if (vers != 2 && vers != 3)
 			{
 				throw new IOException(MessageFormat.Format(JGitText.Get().unsupportedPackVersion, 
-					vers));
+					Sharpen.Extensions.ValueOf(vers)));
 			}
 			objectCount = NB.DecodeUInt32(buf, p + 8);
 			Use(hdrln);
@@ -966,12 +959,7 @@ namespace NGit.Transport
 			byte[] srcHash = new byte[20];
 			System.Array.Copy(buf, c, srcHash, 0, 20);
 			Use(20);
-			// The input stream should be at EOF at this point. We do not support
-			// yielding back any remaining buffered data after the pack footer, so
-			// protocols that embed a pack stream are required to either end their
-			// stream with the pack, or embed the pack with a framing system like
-			// the SideBandInputStream does.
-			if (bAvail != 0)
+			if (bAvail != 0 && !expectDataAfterPackFooter)
 			{
 				throw new CorruptObjectException(MessageFormat.Format(JGitText.Get().expectedEOFReceived
 					, "\\x" + Sharpen.Extensions.ToHexString(buf[bOffset] & unchecked((int)(0xff))))
@@ -986,6 +974,14 @@ namespace NGit.Transport
 						, "\\x" + Sharpen.Extensions.ToHexString(eof)));
 				}
 			}
+			else
+			{
+				if (bAvail > 0 && expectDataAfterPackFooter)
+				{
+					@in.Reset();
+					IOUtil.SkipFully(@in, bOffset);
+				}
+			}
 			if (!Arrays.Equals(actHash, srcHash))
 			{
 				throw new CorruptObjectException(JGitText.Get().corruptObjectPackfileChecksumIncorrect
@@ -1081,8 +1077,8 @@ namespace NGit.Transport
 
 				default:
 				{
-					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, typeCode
-						));
+					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, Sharpen.Extensions.ValueOf
+						(typeCode)));
 				}
 			}
 		}
@@ -1113,7 +1109,7 @@ namespace NGit.Transport
 				}
 				inf.Close();
 				tempObjectId.FromRaw(objectDigest.Digest(), 0);
-				checkContentLater = IsCheckObjectCollisions() && readCurs.Has(tempObjectId);
+				checkContentLater = readCurs.Has(tempObjectId);
 				data = null;
 			}
 			else
@@ -1152,21 +1148,18 @@ namespace NGit.Transport
 						.TypeString(type), id.Name, e.Message));
 				}
 			}
-			if (IsCheckObjectCollisions())
+			try
 			{
-				try
-				{
-					ObjectLoader ldr = readCurs.Open(id, type);
-					byte[] existingData = ldr.GetCachedBytes(data.Length);
-					if (!Arrays.Equals(data, existingData))
-					{
-						throw new IOException(MessageFormat.Format(JGitText.Get().collisionOn, id.Name));
-					}
-				}
-				catch (MissingObjectException)
+				ObjectLoader ldr = readCurs.Open(id, type);
+				byte[] existingData = ldr.GetCachedBytes(data.Length);
+				if (!Arrays.Equals(data, existingData))
 				{
+					throw new IOException(MessageFormat.Format(JGitText.Get().collisionOn, id.Name));
 				}
 			}
+			catch (MissingObjectException)
+			{
+			}
 		}
 
 		// This is OK, we don't have a copy of the object locally
@@ -1183,8 +1176,8 @@ namespace NGit.Transport
 				info = OpenDatabase(obj, info);
 				if (info.type != Constants.OBJ_BLOB)
 				{
-					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, info
-						.type));
+					throw new IOException(MessageFormat.Format(JGitText.Get().unknownObjectType, Sharpen.Extensions.ValueOf
+						(info.type)));
 				}
 				ObjectStream cur = readCurs.Open(obj, info.type).OpenStream();
 				try
@@ -1321,9 +1314,22 @@ namespace NGit.Transport
 		{
 			packDigest.Update(buf, 0, bOffset);
 			OnStoreStream(buf, 0, bOffset);
-			if (bAvail > 0)
+			if (expectDataAfterPackFooter)
 			{
-				System.Array.Copy(buf, bOffset, buf, 0, bAvail);
+				if (bAvail > 0)
+				{
+					@in.Reset();
+					IOUtil.SkipFully(@in, bOffset);
+					bAvail = 0;
+				}
+				@in.Mark(buf.Length);
+			}
+			else
+			{
+				if (bAvail > 0)
+				{
+					System.Array.Copy(buf, bOffset, buf, 0, bAvail);
+				}
 			}
 			bBase += bOffset;
 			bOffset = 0;
