@@ -78,8 +78,8 @@ namespace NGit.Transport
 			PUSH
 		}
 
-		private static readonly IList<WeakReference<TransportProtocol>> protocols = new CopyOnWriteArrayList
-			<WeakReference<TransportProtocol>>();
+		private static readonly IList<JavaWeakReference<TransportProtocol>> protocols = new 
+			CopyOnWriteArrayList<JavaWeakReference<TransportProtocol>>();
 
 		static Transport()
 		{
@@ -94,8 +94,8 @@ namespace NGit.Transport
 			Register(TransportGitSsh.PROTO_SSH);
 //			RegisterByService();
 		}
-
-/*		private static void RegisterByService()
+		/*
+		private static void RegisterByService()
 		{
 			ClassLoader ldr = Sharpen.Thread.CurrentThread().GetContextClassLoader();
 			if (ldr == null)
@@ -238,7 +238,7 @@ namespace NGit.Transport
 		/// <param name="proto">the protocol definition. Must not be null.</param>
 		public static void Register(TransportProtocol proto)
 		{
-			protocols.Add(0, new WeakReference<TransportProtocol>(proto));
+			protocols.Add(0, new JavaWeakReference<TransportProtocol>(proto));
 		}
 
 		/// <summary>Unregister a TransportProtocol instance.</summary>
@@ -256,7 +256,7 @@ namespace NGit.Transport
 		/// <param name="proto">the exact object previously given to register.</param>
 		public static void Unregister(TransportProtocol proto)
 		{
-			foreach (WeakReference<TransportProtocol> @ref in protocols)
+			foreach (JavaWeakReference<TransportProtocol> @ref in protocols)
 			{
 				TransportProtocol refProto = @ref.Get();
 				if (refProto == null || refProto == proto)
@@ -273,7 +273,7 @@ namespace NGit.Transport
 		{
 			int cnt = protocols.Count;
 			IList<TransportProtocol> res = new AList<TransportProtocol>(cnt);
-			foreach (WeakReference<TransportProtocol> @ref in protocols)
+			foreach (JavaWeakReference<TransportProtocol> @ref in protocols)
 			{
 				TransportProtocol proto = @ref.Get();
 				if (proto != null)
@@ -610,7 +610,7 @@ namespace NGit.Transport
 		public static NGit.Transport.Transport Open(Repository local, URIish uri, string 
 			remoteName)
 		{
-			foreach (WeakReference<TransportProtocol> @ref in protocols)
+			foreach (JavaWeakReference<TransportProtocol> @ref in protocols)
 			{
 				TransportProtocol proto = @ref.Get();
 				if (proto == null)
@@ -627,6 +627,31 @@ namespace NGit.Transport
 				, uri));
 		}
 
+		/// <summary>Open a new transport with no local repository.</summary>
+		/// <remarks>Open a new transport with no local repository.</remarks>
+		/// <param name="uri"></param>
+		/// <returns>new Transport instance</returns>
+		/// <exception cref="System.NotSupportedException">System.NotSupportedException</exception>
+		/// <exception cref="NGit.Errors.TransportException">NGit.Errors.TransportException</exception>
+		public static NGit.Transport.Transport Open(URIish uri)
+		{
+			foreach (JavaWeakReference<TransportProtocol> @ref in protocols)
+			{
+				TransportProtocol proto = @ref.Get();
+				if (proto == null)
+				{
+					protocols.Remove(@ref);
+					continue;
+				}
+				if (proto.CanHandle(uri, null, null))
+				{
+					return proto.Open(uri);
+				}
+			}
+			throw new NGit.Errors.NotSupportedException(MessageFormat.Format(JGitText.Get().URINotSupported
+				, uri));
+		}
+
 		/// <summary>
 		/// Convert push remote refs update specification from
 		/// <see cref="RefSpec">RefSpec</see>
@@ -868,6 +893,17 @@ namespace NGit.Transport
 			this.credentialsProvider = CredentialsProvider.GetDefault();
 		}
 
+		/// <summary>Create a minimal transport instance not tied to a single repository.</summary>
+		/// <remarks>Create a minimal transport instance not tied to a single repository.</remarks>
+		/// <param name="uri"></param>
+		protected internal Transport(URIish uri)
+		{
+			this.uri = uri;
+			this.local = null;
+			this.checkFetchedObjects = true;
+			this.credentialsProvider = CredentialsProvider.GetDefault();
+		}
+
 		/// <summary>Get the URI this transport connects to.</summary>
 		/// <remarks>
 		/// Get the URI this transport connects to.
