@@ -781,6 +781,7 @@ namespace NGit.Treewalk
 		/// start of name component part within
 		/// <see cref="GetEntryPathBuffer()">GetEntryPathBuffer()</see>
 		/// </returns>
+		/// <since>2.0</since>
 		public virtual int GetNameOffset()
 		{
 			return pathOffset;
@@ -804,5 +805,10 @@ namespace NGit.Treewalk
 		{
 			System.Array.Copy(path, pathOffset, buffer, offset, pathLen - pathOffset);
 		}
+
+		public override string ToString()
+		{
+			return GetType().Name + "[" + EntryPathString + "]";
+		}
 	}
 }
