@@ -32,38 +32,38 @@ using System.Collections.Generic;
 using System.Diagnostics;
 
 using MonoDevelop.Core;
-using MonoDevelop.Projects;
 using MonoDevelop.Ide.CodeCompletion;
 using MonoDevelop.Ide.Gui.Content;
 
 using MonoDevelop.AspNet;
 using MonoDevelop.AspNet.Parser;
-using MonoDevelop.AspNet.Parser.Dom;
-using MonoDevelop.Html;
 using MonoDevelop.DesignerSupport;
 
-//I initially aliased this as SE, which (unintentionally) looked a little odd with the XDOM types :-)
 using S = MonoDevelop.Xml.StateEngine; 
 using MonoDevelop.AspNet.StateEngine;
-using System.Text;
 using System.Text.RegularExpressions;
 using ICSharpCode.NRefactory.TypeSystem;
 using MonoDevelop.Ide.TypeSystem;
-using ICSharpCode.NRefactory.CSharp;
 using ICSharpCode.NRefactory;
-using ICSharpCode.NRefactory.Completion;
 
 namespace MonoDevelop.AspNet.Gui
 {
 	public class AspNetEditorExtension : BaseHtmlEditorExtension
 	{
+		static readonly Regex DocTypeRegex = new Regex (@"(?:PUBLIC|public)\s+""(?<fpi>[^""]*)""\s+""(?<uri>[^""]*)""");
+
 		AspNetParsedDocument aspDoc;
 		AspNetAppProject project;
 		DocumentReferenceManager refman;
+
+		ILanguageCompletionBuilder documentBuilder;
+		LocalDocumentInfo localDocumentInfo;
+		DocumentInfo documentInfo;
+
+		ICompletionWidget defaultCompletionWidget;
+		MonoDevelop.Ide.Gui.Document defaultDocument;
 		
-		bool HasDoc { get { return aspDoc != null; } }
-		 
-		Regex DocTypeRegex = new Regex (@"(?:PUBLIC|public)\s+""(?<fpi>[^""]*)""\s+""(?<uri>[^""]*)""");
+		bool HasDoc { get { return aspDoc != null; } } 
 		
 		#region Setup and teardown
 		
@@ -95,24 +95,20 @@ namespace MonoDevelop.AspNet.Gui
 			documentBuilder = HasDoc ? LanguageCompletionBuilderService.GetBuilder (aspDoc.Info.Language) : null;
 			
 			if (documentBuilder != null) {
-				var usings = refman.GetUsings ();
-				documentInfo = new DocumentInfo (document.Compilation, aspDoc, usings, refman.GetDoms ());
+				documentInfo = new DocumentInfo (document.Compilation, aspDoc, refman.GetUsings (), refman.GetDoms ());
 				documentInfo.ParsedDocument = documentBuilder.BuildDocument (documentInfo, Editor);
 				documentInfo.CodeBesideClass = CreateCodeBesideClass (documentInfo, refman);
-/*				var domWrapper = new AspProjectDomWrapper (documentInfo);
-				if (localDocumentInfo != null)
-					localDocumentInfo.HiddenDocument.HiddenContext = domWrapper;*/
 			}
 		}
 		
 		static IUnresolvedTypeDefinition CreateCodeBesideClass (DocumentInfo info, DocumentReferenceManager refman)
 		{
-			var v = new MemberListVisitor (refman);
-			info.AspNetDocument.RootNode.AcceptVisit (v);
+			var memberList = new MemberListBuilder (refman, info.AspNetDocument.XDocument);
+			memberList.Build ();
 			var t = new ICSharpCode.NRefactory.TypeSystem.Implementation.DefaultUnresolvedTypeDefinition (info.ClassName);
 			var dom = refman.TypeCtx.Compilation;
-			var baseType = dom.LookupType (info.BaseType);
-			foreach (var m in CodeBehind.GetDesignerMembers (v.Members.Values, baseType, null)) {
+			var baseType = ReflectionHelper.ParseReflectionName (info.BaseType).Resolve (dom);
+			foreach (var m in CodeBehind.GetDesignerMembers (memberList.Members.Values, baseType, null)) {
 				t.Members.Add (new ICSharpCode.NRefactory.TypeSystem.Implementation.DefaultUnresolvedField (t, m.Name) {
 					Accessibility = Accessibility.Protected,
 					ReturnType = m.Type.ToTypeReference ()
@@ -120,12 +116,7 @@ namespace MonoDevelop.AspNet.Gui
 			}
 			return t;
 		}
-		
-		ILanguageCompletionBuilder documentBuilder;
-		LocalDocumentInfo localDocumentInfo;
-		DocumentInfo documentInfo;
-		
-		
+
 		protected override ICompletionDataList HandleCodeCompletion (CodeCompletionContext completionContext,
 		                                                            bool forced, ref int triggerWordLength)
 		{
@@ -144,7 +135,8 @@ namespace MonoDevelop.AspNet.Gui
 					return DirectiveCompletion.GetDirectives (aspDoc.Type);
 				}
 				return null;
-			} else if (Tracker.Engine.CurrentState is S.XmlNameState && Tracker.Engine.CurrentState.Parent is AspNetDirectiveState) {
+			}
+			if (Tracker.Engine.CurrentState is S.XmlNameState && Tracker.Engine.CurrentState.Parent is AspNetDirectiveState) {
 				var directive = Tracker.Engine.Nodes.Peek () as AspNetDirective;
 				if (HasDoc && directive != null && directive.Region.BeginLine == completionContext.TriggerLine &&
 				    directive.Region.BeginColumn + 4 == completionContext.TriggerLineOffset && char.IsLetter (currentChar))
@@ -177,8 +169,7 @@ namespace MonoDevelop.AspNet.Gui
 			if (Tracker.Engine.CurrentState is HtmlScriptBodyState) {
 				var el = Tracker.Engine.Nodes.Peek () as S.XElement;
 				if (el != null) {
-					var att = GetHtmlAtt (el, "runat");
-					if (att != null && "server".Equals (att.Value, StringComparison.OrdinalIgnoreCase)) {
+					if (el.IsRunatServer ()) {
 						if (documentBuilder != null) {
 							// TODO: C# completion
 						}
@@ -203,9 +194,7 @@ namespace MonoDevelop.AspNet.Gui
 		//case insensitive, no prefix
 		static S.XAttribute GetHtmlAtt (S.XElement el, string name)
 		{
-			return el.Attributes
-				.Where (a => a.IsNamed && !a.Name.HasPrefix && a.Name.Name.Equals (name, StringComparison.OrdinalIgnoreCase))
-				.FirstOrDefault ();
+			return el.Attributes.FirstOrDefault (a => a.IsNamed && !a.Name.HasPrefix && a.Name.Name.Equals (name, StringComparison.OrdinalIgnoreCase));
 		}
 		
 		public void InitializeCodeCompletion (char ch)
@@ -218,11 +207,7 @@ namespace MonoDevelop.AspNet.Gui
 			if (ch != '\0')
 				sourceText += ch;
 			string textAfterCaret = Document.Editor.GetTextBetween (caretOffset, Math.Min (Document.Editor.Length, Math.Max (caretOffset, Tracker.Engine.Position + Tracker.Engine.CurrentStateLength - 2)));
-			
-			var loc = new MonoDevelop.AspNet.Parser.Internal.Location ();
-			var docLoc = Document.Editor.Document.OffsetToLocation (start);
-			loc.EndLine = loc.BeginLine = docLoc.Line;
-			loc.EndColumn = loc.BeginColumn = docLoc.Column;
+
 			if (documentBuilder == null){
 				localDocumentInfo = null;
 				return;
@@ -254,10 +239,7 @@ namespace MonoDevelop.AspNet.Gui
 			}
 			return base.CodeCompletionCommand (completionContext);
 		}
-		
-		ICompletionWidget defaultCompletionWidget;
-		MonoDevelop.Ide.Gui.Document defaultDocument;
-//		AspProjectDomWrapper domWrapper;
+
 		public override void Initialize ()
 		{
 			base.Initialize ();
@@ -266,8 +248,35 @@ namespace MonoDevelop.AspNet.Gui
 			defaultDocument.Editor.Caret.PositionChanged += delegate {
 				OnCompletionContextChanged (CompletionWidget, EventArgs.Empty);
 			};
+			defaultDocument.Saved += AsyncUpdateDesignerFile;
+		}
+
+		void AsyncUpdateDesignerFile (object sender, EventArgs e)
+		{
+			if (project == null)
+				return;
+
+			var file = project.GetProjectFile (FileName);
+			if (file == null)
+				return;
+
+			var designerFile = CodeBehind.GetDesignerFile (file);
+			if (designerFile == null)
+				return;
+
+			System.Threading.ThreadPool.QueueUserWorkItem (r => {
+				using (var monitor = MonoDevelop.Ide.IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (
+					GettextCatalog.GetString ("Updating ASP.NET Designer File..."), null, false))
+				{
+					var writer = CodeBehindWriter.CreateForProject (monitor, project);
+					var result = CodeBehind.UpdateDesignerFile (writer, project, file, designerFile);
+					//don't worry about reporting error's here for now
+					//if the user wants to see errors, they can compile
+					if (!result.Failed)
+						writer.WriteOpenFiles ();
+				}
+			});
 		}
-		
 		
 		public override ICompletionDataList HandleCodeCompletion (
 		    CodeCompletionContext completionContext, char completionChar, ref int triggerWordLength)
@@ -302,18 +311,18 @@ namespace MonoDevelop.AspNet.Gui
 		
 		public override bool GetParameterCompletionCommandOffset (out int cpos)
 		{
-			if (Tracker.Engine.CurrentState is AspNetExpressionState && documentBuilder != null && localDocumentInfo != null) {
+			/*if (Tracker.Engine.CurrentState is AspNetExpressionState && documentBuilder != null && localDocumentInfo != null) {
 				var result = documentBuilder.GetParameterCompletionCommandOffset (defaultDocument, documentInfo, localDocumentInfo, out cpos);
 				return result;
-			}
+			}*/
 			return base.GetParameterCompletionCommandOffset (out cpos);
 		}
 		
-		public override IParameterDataProvider HandleParameterCompletion (CodeCompletionContext completionContext, char completionChar)
+		public override ParameterDataProvider HandleParameterCompletion (CodeCompletionContext completionContext, char completionChar)
 		{
-			if (Tracker.Engine.CurrentState is AspNetExpressionState && documentBuilder != null && localDocumentInfo != null) {
+/*			if (Tracker.Engine.CurrentState is AspNetExpressionState && documentBuilder != null && localDocumentInfo != null) {
 				return documentBuilder.HandleParameterCompletion (defaultDocument, completionContext, documentInfo, localDocumentInfo, completionChar);
-			}
+			}*/
 			
 			return base.HandleParameterCompletion (completionContext, completionChar);
 		}
@@ -344,7 +353,8 @@ namespace MonoDevelop.AspNet.Gui
 			if (!HasDoc) {
 				AddAspBeginExpressions (list);
 				string aspPrefix = "asp:";
-				foreach (var cls in WebTypeContext.ListSystemControlClasses (TypeSystemService.GetCompilation (project).LookupType ("System.Web.UI", "Control"), project))
+				var type = ReflectionHelper.ParseReflectionName ("System.Web.UI.Control").Resolve (TypeSystemService.GetCompilation (project));
+				foreach (var cls in WebTypeContext.ListSystemControlClasses (type, project))
 					list.Add (new AspTagCompletionData (aspPrefix, cls));
 				
 				base.GetElementCompletions (list);
@@ -399,16 +409,10 @@ namespace MonoDevelop.AspNet.Gui
 			
 			//children of properties
 			if (childrenAsProperties && (!parentName.HasPrefix || defaultProp != null)) {
-				if (controlClass.GetProjectContent () == null) {
-					LoggingService.LogWarning ("IType {0} does not have a SourceProjectDom", controlClass);
-					return;
-				}
-				
 				string propName = defaultProp ?? parentName.Name;
 				IProperty property =
 					controlClass.GetProperties ()
-						.Where (x => string.Compare (propName, x.Name, StringComparison.OrdinalIgnoreCase) == 0)
-						.FirstOrDefault ();
+						.FirstOrDefault (x => string.Equals (propName, x.Name, StringComparison.OrdinalIgnoreCase));
 				
 				if (property == null)
 					return;
@@ -450,12 +454,12 @@ namespace MonoDevelop.AspNet.Gui
 				}
 				
 				string addStr = "Add";
-				IMethod meth = collectionType.GetMethods ()
-					.Where (m => m.Parameters.Count == 1 && m.Name == addStr).FirstOrDefault ();
+				IMethod meth = collectionType.GetMethods ().FirstOrDefault (m => m.Parameters.Count == 1 && m.Name == addStr);
 				
 				if (meth != null) {
 					IType argType = meth.Parameters [0].Type;
-					if (argType != null && argType.IsBaseType (argType.GetDefinition ().Compilation.LookupType ("System.Web.UI", "Control"))) {
+					var type = ReflectionHelper.ParseReflectionName ("System.Web.UI.Control").Resolve (argType.GetDefinition ().Compilation);
+					if (argType != null && argType.IsBaseType (type)) {
 						list.AddRange (refman.GetControlCompletionData (argType));
 						return;
 					}
@@ -467,13 +471,9 @@ namespace MonoDevelop.AspNet.Gui
 			
 			//properties as children of controls
 			if (parentName.HasPrefix && childrenAsProperties) {
-				if (controlClass.GetProjectContent () == null) {
-					LoggingService.LogWarning ("IType {0} does not have a SourceProjectDom", controlClass);
-				}
-				
 				foreach (IProperty prop in GetUniqueMembers<IProperty> (controlClass.GetProperties ()))
 					if (GetPersistenceMode (prop) != System.Web.UI.PersistenceMode.Attribute)
-						list.Add (prop.Name, prop.GetStockIcon (), AmbienceService.GetDocumentationSummary (prop));
+						list.Add (prop.Name, prop.GetStockIcon (), AmbienceService.GetSummaryMarkup (prop));
 				return;
 			}
 		}
@@ -515,8 +515,7 @@ namespace MonoDevelop.AspNet.Gui
 			var list = base.GetAttributeValueCompletions (ob, att) ?? new CompletionDataList ();
 			if (ob is S.XElement) {
 				if (ob.Name.HasPrefix) {
-					S.XAttribute idAtt = ob.Attributes[new S.XName ("id")];
-					string id = idAtt == null? null : idAtt.Value;
+					string id = ob.GetId ();
 					if (string.IsNullOrEmpty (id) || string.IsNullOrEmpty (id.Trim ()))
 						id = null;
 					AddAspAttributeValueCompletionData (list, ob.Name, att.Name, id);
@@ -562,7 +561,7 @@ namespace MonoDevelop.AspNet.Gui
 			if (HasDoc && !string.IsNullOrEmpty (aspDoc.Info.InheritedClass)) {
 				projectDatabase = TypeSystemService.GetCompilation (project);
 				if (projectDatabase != null)
-					codeBehindClass = projectDatabase.FindType (aspDoc.Info.InheritedClass);
+					codeBehindClass = ReflectionHelper.ParseReflectionName (aspDoc.Info.InheritedClass).Resolve (projectDatabase);
 			}
 		}
 		
@@ -581,7 +580,7 @@ namespace MonoDevelop.AspNet.Gui
 				list.Add ("%$", "md-literal", GettextCatalog.GetString ("ASP.NET resource expression"));
 			}
 			
-			//valid on 2.0+ runtime only
+			//valid on 4.0+ runtime only
 			if (ProjClrVersion != ClrVersion.Net_4_0) {
 				list.Add ("%:", "md-literal", GettextCatalog.GetString ("ASP.NET HTML encoded expression"));
 			}
@@ -601,7 +600,7 @@ namespace MonoDevelop.AspNet.Gui
 			
 			IType controlClass = refman.GetControlType (name.Prefix, name.Name);
 			if (controlClass == null) {
-				controlClass = database.LookupType ("System.Web.UI.WebControls", "WebControl");
+				controlClass = ReflectionHelper.ParseReflectionName ("System.Web.UI.WebControls.WebControl").Resolve (database);
 				if (controlClass == null) {
 					LoggingService.LogWarning ("Could not obtain IType for System.Web.UI.WebControls.WebControl");
 					return;
@@ -618,13 +617,13 @@ namespace MonoDevelop.AspNet.Gui
 			foreach (var prop in GetUniqueMembers<IProperty> (controlClass.GetProperties ()))
 				if (prop.Accessibility == Accessibility.Public && (existingAtts == null || !existingAtts.ContainsKey (prop.Name)))
 				if (GetPersistenceMode (prop) == System.Web.UI.PersistenceMode.Attribute)
-					list.Add (prop.Name, prop.GetStockIcon (), AmbienceService.GetDocumentationSummary (prop));
+						list.Add (prop.Name, prop.GetStockIcon (), AmbienceService.GetSummaryMarkup (prop));
 			
 			//similarly add events
 			foreach (var eve in GetUniqueMembers<IEvent> (controlClass.GetEvents ())) {
 				string eveName = "On" + eve.Name;
 				if (eve.Accessibility == Accessibility.Public && (existingAtts == null || !existingAtts.ContainsKey (eveName)))
-					list.Add (eveName, eve.GetStockIcon (), AmbienceService.GetDocumentationSummary (eve));
+					list.Add (eveName, eve.GetStockIcon (), AmbienceService.GetSummaryMarkup (eve));
 			}
 		}
 		
@@ -639,7 +638,7 @@ namespace MonoDevelop.AspNet.Gui
 				LoggingService.LogWarning ("Could not obtain IType for {0}", tagName.FullName);
 				
 				var database = WebTypeContext.GetSystemWebDom (project);
-				controlClass = database.LookupType ("System.Web.UI.WebControls", "WebControl");
+				controlClass = ReflectionHelper.ParseReflectionName ("System.Web.UI.WebControls.WebControl").Resolve (database);
 
 				if (controlClass == null) {
 					LoggingService.LogWarning ("Could not obtain IType for System.Web.UI.WebControls.WebControl");
@@ -653,7 +652,7 @@ namespace MonoDevelop.AspNet.Gui
 			GetCodeBehind (out codeBehindClass, out projectDatabase);
 			
 			//if it's an event, suggest compatible methods 
-			if (codeBehindClass != null && attName.Name.StartsWith ("On")) {
+			if (codeBehindClass != null && attName.Name.StartsWith ("On", StringComparison.Ordinal)) {
 				string eventName = attName.Name.Substring (2);
 				
 				foreach (IEvent ev in controlClass.GetEvents ()) {
@@ -725,7 +724,7 @@ namespace MonoDevelop.AspNet.Gui
 				if (retCls != null && retCls.Kind == TypeKind.Enum) {
 					foreach (var enumVal in retCls.GetFields ())
 						if (enumVal.IsPublic && enumVal.IsStatic)
-							list.Add (enumVal.Name, "md-literal", AmbienceService.GetDocumentationSummary (enumVal));
+							list.Add (enumVal.Name, "md-literal", AmbienceService.GetSummaryMarkup (enumVal));
 					return;
 				}
 			}
@@ -764,8 +763,7 @@ namespace MonoDevelop.AspNet.Gui
 					
 					return (System.Web.UI.PersistenceMode) expr.ConstantValue;
 				}
-				else if (att.AttributeType.ReflectionName == "System.Web.UI.TemplateContainerAttribute")
-				{
+				if (att.AttributeType.ReflectionName == "System.Web.UI.TemplateContainerAttribute") {
 					return System.Web.UI.PersistenceMode.InnerProperty;
 				}
 			}
@@ -778,8 +776,12 @@ namespace MonoDevelop.AspNet.Gui
 			defaultProperty = "";
 			
 			IAttribute att = GetAttributes (type, "System.Web.UI.ParseChildrenAttribute").FirstOrDefault ();
+
+			if (att == null)
+				return childrenAsProperties;
+
 			var posArgs = att.PositionalArguments;
-			if (att == null || posArgs.Count == 0)
+			if (posArgs.Count == 0)
 				return childrenAsProperties;
 			
 			if (posArgs.Count > 0) {
@@ -842,11 +844,6 @@ namespace MonoDevelop.AspNet.Gui
 					yield return att;
 			}
 			
-			if (type.GetProjectContent () == null) {
-				LoggingService.LogWarning ("IType {0} has null SourceProjectDom", type);
-				yield break;
-			}
-			
 			foreach (IType t2 in type.GetAllBaseTypes ()) {
 				foreach (IAttribute att in t2.GetDefinition ().Attributes)
 					if (att.AttributeType.ReflectionName == attName)
@@ -855,93 +852,5 @@ namespace MonoDevelop.AspNet.Gui
 		}
 		
 		#endregion
-		
-		#region Document outline
-		
-		protected override void RefillOutlineStore (ParsedDocument doc, Gtk.TreeStore store)
-		{
-			ParentNode p = ((AspNetParsedDocument)doc).RootNode;
-//			Gtk.TreeIter iter = outlineTreeStore.AppendValues (System.IO.Path.GetFileName (CU.Document.FilePath), p);
-			BuildTreeChildren (store, Gtk.TreeIter.Zero, p);
-		}
-		
-		protected override void InitializeOutlineColumns (MonoDevelop.Ide.Gui.Components.PadTreeView outlineTree)
-		{
-			outlineTree.TextRenderer.Xpad = 0;
-			outlineTree.TextRenderer.Ypad = 0;
-			outlineTree.AppendColumn ("Node", outlineTree.TextRenderer, new Gtk.TreeCellDataFunc (outlineTreeDataFunc));
-		}
-		
-		protected override void OutlineSelectionChanged (object selection)
-		{
-			SelectNode ((Node)selection);
-		}
-		
-		static void BuildTreeChildren (Gtk.TreeStore store, Gtk.TreeIter parent, ParentNode p)
-		{
-			foreach (Node n in p) {
-				if (!(n is TagNode || n is DirectiveNode || n is ExpressionNode))
-					continue;
-				Gtk.TreeIter childIter;
-				if (!parent.Equals (Gtk.TreeIter.Zero))
-					
-					childIter = store.AppendValues (parent, n);
-				else
-					childIter = store.AppendValues (n);
-				ParentNode pChild = n as ParentNode;
-				if (pChild != null)
-					BuildTreeChildren (store, childIter, pChild);
-			}
-		}
-		
-		void outlineTreeDataFunc (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
-		{
-			Gtk.CellRendererText txtRenderer = (Gtk.CellRendererText) cell;
-			Node n = (Node) model.GetValue (iter, 0);
-			string name = null;
-			if (n is TagNode) {
-				TagNode tn = (TagNode) n;
-				name = tn.TagName;
-				string att = null;
-				if (tn.Attributes != null) {
-					att = tn.Attributes["id"] as string;
-				}
-				if (att != null)
-					name = "<" + name + "#" + att + ">";
-				else
-					name = "<" + name + ">";
-			} else if (n is DirectiveNode) {
-				DirectiveNode dn = (DirectiveNode) n;
-				name = "<%@ " + dn.Name + " %>";
-			} else if (n is ExpressionNode) {
-				ExpressionNode en = (ExpressionNode) n;
-				string expr = en.Expression;
-				if (string.IsNullOrEmpty (expr)) {
-					name = "<% %>";
-				} else {
-					if (expr.Length > 10)
-						expr = expr.Substring (0, 10) + "...";
-					name = "<% " + expr + "%>";
-				}
-			}
-			if (name != null)
-				txtRenderer.Text = name;
-		}
-		
-		void SelectNode (Node n)
-		{
-			ILocation start = n.Location, end;
-			TagNode tn = n as TagNode;
-			if (tn != null && tn.EndLocation != null)
-				end = tn.EndLocation;
-			else
-				end = start;
-			
-			//FIXME: why is this offset necessary?
-			int offset = n is TagNode? 1 : 0;
-			EditorSelect (new DomRegion (start.BeginLine, start.BeginColumn + offset, end.EndLine, end.EndColumn + offset));
-		}
-		#endregion
 	}
-		
 }
