@@ -25,27 +25,18 @@
 // THE SOFTWARE.
 
 using System;
-using System.Linq;
 using System.Collections.Generic;
-using System.Diagnostics;
-using MonoDevelop.Core;
-using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
-using MonoDevelop.Ide.CodeCompletion;
-using MonoDevelop.Ide.Gui.Content;
-using MonoDevelop.AspNet;
 using MonoDevelop.AspNet.Parser;
-using MonoDevelop.AspNet.Parser.Dom;
-using MonoDevelop.Html;
-using MonoDevelop.DesignerSupport;
 using S = MonoDevelop.Xml.StateEngine;
 using MonoDevelop.AspNet.StateEngine;
-using System.Text;
 using Mono.TextEditor;
+using MonoDevelop.Ide.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem;
+using MonoDevelop.Ide.CodeCompletion;
 
 namespace MonoDevelop.AspNet.Gui
 {
+	
 	/// <summary>
 	/// Embedded local region completion information for each keystroke
 	/// </summary>
@@ -55,7 +46,7 @@ namespace MonoDevelop.AspNet.Gui
 		public ParsedDocument ParsedLocalDocument { get; set; }
 		public int CaretPosition { get; set; }
 		public int OriginalCaretPosition { get; set; }
-		public MonoDevelop.Ide.Gui.Document HiddenDocument { get; set; }
+		public HiddenDocument HiddenDocument { get; set; }
 		
 		public List<OffsetInfo> OffsetInfos = new List<OffsetInfo> ();
 		
@@ -95,29 +86,23 @@ namespace MonoDevelop.AspNet.Gui
 	/// </summary>
 	public class DocumentInfo
 	{
-		public DocumentInfo (ProjectDom dom, AspNetParsedDocument aspNetParsedDocument, IEnumerable<string> imports,
-		                     IList<ProjectDom> references)
+		public DocumentInfo (ICompilation dom, AspNetParsedDocument aspNetParsedDocument, IEnumerable<string> imports,
+		                     IList<ICompilation> references)
 		{
 			this.Dom = dom;
 			this.AspNetDocument = aspNetParsedDocument;
 			this.Imports = imports;
 			this.References = references;
-			ScriptBlocks = new List<TagNode> ();
-			Expressions = new List<ExpressionNode> ();
-			aspNetParsedDocument.RootNode.AcceptVisit (new ExpressionCollector (this));
+			BuildExpressionAndScriptsLists ();
 		}
 		
-		IType codeBesideClass;
-		
-		public ProjectDom Dom { get; private set; }
+		public ICompilation Dom { get; private set; }
 		public AspNetParsedDocument AspNetDocument { get; private set; }
 		public ParsedDocument ParsedDocument { get; set; }
-		public List<ExpressionNode> Expressions { get; private set; }
-		public List<TagNode> ScriptBlocks { get; private set; }
-		public IList<ProjectDom> References { get; set; }
+		public IList<ICompilation> References { get; set; }
 		public IEnumerable<string> Imports { get; private set; }
 		
-		public IType CodeBesideClass { get; set; }
+		public IUnresolvedTypeDefinition CodeBesideClass { get; set; }
 		
 		public string BaseType {
 			get {
@@ -146,26 +131,32 @@ namespace MonoDevelop.AspNet.Gui
 			throw new InvalidOperationException (string.Format ("Unexpected filetype '{0}'", type));
 		}
 		
-		class ExpressionCollector : Visitor
+		#region parsing for expression and runat="server" script tags
+		
+		public List<S.XNode> XExpressions { get; private set; }
+		public List<S.XElement> XScriptBlocks { get; private set; }
+		
+		void BuildExpressionAndScriptsLists ()
 		{
-			DocumentInfo parent;
+			XExpressions = new List<S.XNode> ();
+			XScriptBlocks = new List<S.XElement> ();
 			
-			public ExpressionCollector (DocumentInfo parent)
-			{
-				this.parent = parent;
-			}
-			
-			public override void Visit (TagNode node)
-			{
-				if (node.TagName == "script" && (string)node.Attributes["runat"] == "server")
-					parent.ScriptBlocks.Add (node);
-			}
-			
-			public override void Visit (ExpressionNode node)
-			{
-				parent.Expressions.Add (node);
+			foreach (S.XNode node in AspNetDocument.XDocument.AllDescendentNodes) {
+				if (node is AspNetRenderExpression || node is AspNetHtmlEncodedExpression || node is AspNetRenderBlock) {
+					XExpressions.Add (node);
+					continue;
+				}
+				S.XElement el = node as S.XElement;
+				if (el == null) {
+					continue;
+				}
+				if (el.IsServerScriptTag ()) {
+					XScriptBlocks.Add (el);	
+				}
 			}
 		}
+		
+		#endregion
 	}
 	
 	/// <summary>
@@ -183,7 +174,7 @@ namespace MonoDevelop.AspNet.Gui
 		
 		ICompletionDataList HandlePopupCompletion (MonoDevelop.Ide.Gui.Document realDocument, DocumentInfo info, LocalDocumentInfo localInfo);
 		ICompletionDataList HandleCompletion (MonoDevelop.Ide.Gui.Document realDocument, CodeCompletionContext completionContext, DocumentInfo info, LocalDocumentInfo localInfo, char currentChar, ref int triggerWordLength);
-		IParameterDataProvider HandleParameterCompletion (MonoDevelop.Ide.Gui.Document realDocument, CodeCompletionContext completionContext, DocumentInfo info, LocalDocumentInfo localInfo, char completionChar);
+		ParameterDataProvider HandleParameterCompletion (MonoDevelop.Ide.Gui.Document realDocument, CodeCompletionContext completionContext, DocumentInfo info, LocalDocumentInfo localInfo, char completionChar);
 		bool GetParameterCompletionCommandOffset (MonoDevelop.Ide.Gui.Document realDocument, DocumentInfo info, LocalDocumentInfo localInfo, out int cpos);
 	}
 	
@@ -220,4 +211,18 @@ namespace MonoDevelop.AspNet.Gui
 			return null;
 		}
 	}
+	
+	public class HiddenDocument : MonoDevelop.Ide.Gui.Document
+	{
+		internal ParsedDocument HiddenParsedDocument;
+		public override ParsedDocument ParsedDocument {
+			get {
+				return HiddenParsedDocument;
+			}
+		}
+		
+		public HiddenDocument (MonoDevelop.Ide.Gui.IWorkbenchWindow window) : base(window)
+		{
+		}
+	}
 }
