@@ -37,21 +37,21 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Deployment;
 using MonoDevelop.Core.Assemblies;
 
 using MonoDevelop.AspNet.Parser;
-using MonoDevelop.AspNet.Parser.Dom;
 using MonoDevelop.AspNet.Deployment;
 using MonoDevelop.AspNet.Gui;
-using MonoDevelop.Projects.Dom.Parser;
+using ICSharpCode.NRefactory.TypeSystem;
+using MonoDevelop.Ide.TypeSystem;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.AspNet
 {
 	[DataInclude (typeof(AspNetAppProjectConfiguration))]
-	public class AspNetAppProject : DotNetProject
+	public class AspNetAppProject : DotNetAssemblyProject
 	{
 		[ItemProperty("XspParameters", IsExternal=true)]
 		protected XspParameters xspParameters = new XspParameters ();
@@ -128,12 +128,6 @@ namespace MonoDevelop.AspNet
 			}
 		}
 		
-		public override bool SupportsFramework (TargetFramework framework)
-		{
-			//only support 1.1, 2.0, 3.5, 4.0 etc, not monotouch, moonlight and so on
-			return framework.IsCompatibleWithFramework ("1.1") && base.SupportsFramework (framework);
-		}
-		
 		#endregion
 		
 		#region constructors
@@ -196,7 +190,6 @@ namespace MonoDevelop.AspNet
 				}
 			}
 			
-			BuildResult ret;
 			if (needsCompile)
 				return base.DoBuild (monitor, configuration);
 			else
@@ -213,6 +206,7 @@ namespace MonoDevelop.AspNet
 				TargetRuntime = TargetRuntime,
 				TargetFramework = TargetFramework,
 				UserAssemblyPaths = this.GetUserAssemblyPaths (config),
+				EnvironmentVariables = configuration.EnvironmentVariables,
 			};
 		}
 		
@@ -232,8 +226,22 @@ namespace MonoDevelop.AspNet
 			
 			IConsole console = null;
 			var operationMonitor = new AggregatedOperationMonitor (monitor);
+
+			bool isXsp = true; //FIXME: fix this when it might not be true - should delegate to the ExecutionHandler
 			
 			try {
+				//HACK: check XSP exists first, because error UX is cleaner w/o displaying a blank console pad.
+				if (isXsp) {
+					try {
+						AspNetExecutionHandler.GetXspPath ((AspNetExecutionCommand)cmd);
+					} catch (UserException ex) {
+						MessageService.ShowError (
+							GettextCatalog.GetString ("Could not launch ASP.NET web server"),
+						    ex.Message);
+						throw;
+					}
+				}
+
 				if (configuration.ExternalConsole)
 					console = context.ExternalConsoleFactory.CreateConsole (!configuration.PauseConsoleOutput);
 				else
@@ -241,7 +249,6 @@ namespace MonoDevelop.AspNet
 				
 				string url = String.Format ("http://{0}:{1}", this.XspParameters.Address, this.XspParameters.Port);
 				
-				bool isXsp = true; //FIXME: fix this when it might not be true
 				
 				if (isXsp) {
 					console = new XspBrowserLauncherConsole (console, delegate {
@@ -262,6 +269,9 @@ namespace MonoDevelop.AspNet
 				monitor.Log.WriteLine ("The web server exited with code: {0}", op.ExitCode);
 				
 			} catch (Exception ex) {
+				if (!(ex is UserException)) {
+					LoggingService.LogError ("Could not launch ASP.NET web server.", ex);
+				}
 				monitor.ReportError ("Could not launch web server.", ex);
 			} finally {
 				operationMonitor.Dispose ();
@@ -286,45 +296,44 @@ namespace MonoDevelop.AspNet
 			string extension = System.IO.Path.GetExtension (fileName);
 			if (extension == null)
 				return WebSubtype.None;
-			extension = extension.ToLower ().TrimStart ('.');
+			extension = extension.ToUpperInvariant ().TrimStart ('.');
 			
 			//NOTE: No way to identify WebSubtype.Code from here
 			//use the instance method for that
-			switch (extension)
-			{
-			case "aspx":
+			switch (extension) {
+			case "ASPX":
 				return WebSubtype.WebForm;
-			case "master":
+			case "MASTER":
 				return WebSubtype.MasterPage;
-			case "ashx":
+			case "ASHX":
 				return WebSubtype.WebHandler;
-			case "ascx":
+			case "ASCX":
 				return WebSubtype.WebControl;
-			case "asmx":
+			case "ASMX":
 				return WebSubtype.WebService;
-			case "asax":
+			case "ASAX":
 				return WebSubtype.Global;
-			case "gif":
-			case "png":
-			case "jpg":
+			case "GIF":
+			case "PNG":
+			case "JPG":
 				return WebSubtype.WebImage;
-			case "skin":
+			case "SKIN":
 				return WebSubtype.WebSkin;
-			case "config":
+			case "CONFIG":
 				return WebSubtype.Config;
-			case "browser":
+			case "BROWSER":
 				return WebSubtype.BrowserDefinition;
-			case "axd":
+			case "AXD":
 				return WebSubtype.Axd;
-			case "sitemap":
+			case "SITEMAP":
 				return WebSubtype.Sitemap;
-			case "css":
+			case "CSS":
 				return WebSubtype.Css;
-			case "xhtml":
-			case "html":
-			case "htm":
+			case "XHTML":
+			case "HTML":
+			case "HTM":
 				return WebSubtype.Html;
-			case "js":
+			case "JS":
 				return WebSubtype.JavaScript;
 			default:
 				return WebSubtype.None;
@@ -337,7 +346,7 @@ namespace MonoDevelop.AspNet
 		
 		#endregion
 		
-		public ProjectDom ResolveAssemblyDom (string assemblyName)
+		public ICompilation ResolveAssemblyDom (string assemblyName)
 		{
 			var parsed = SystemAssemblyService.ParseAssemblyName (assemblyName);
 			if (string.IsNullOrEmpty (parsed.Name))
@@ -346,23 +355,23 @@ namespace MonoDevelop.AspNet
 			var dllName = parsed.Name + ".dll";
 			
 			foreach (var reference in References) {
-				if (reference.ReferenceType == ReferenceType.Gac || reference.ReferenceType == ReferenceType.Assembly) {
+				if (reference.ReferenceType == ReferenceType.Package || reference.ReferenceType == ReferenceType.Assembly) {
 					foreach (string refPath in reference.GetReferencedFileNames (null))
 						if (Path.GetFileName (refPath) == dllName)
-							return ProjectDomService.GetAssemblyDom (TargetRuntime, refPath);
+							return new ICSharpCode.NRefactory.TypeSystem.Implementation.SimpleCompilation (TypeSystemService.LoadAssemblyContext (TargetRuntime, TargetFramework, refPath));
 				} else if (reference.ReferenceType == ReferenceType.Project && parsed.Name == reference.Reference) {
 					var p = ParentSolution.FindProjectByName (reference.Reference) as DotNetProject;
 					if (p == null) {
 						LoggingService.LogWarning ("Project '{0}' referenced from '{1}' could not be found", reference.Reference, this.Name);
 						continue;
 					}
-					return ProjectDomService.GetProjectDom (p);
+					return TypeSystemService.GetCompilation (p);
 				}
 			}
 			
 			string path = GetAssemblyPath (assemblyName);
 			if (path != null)
-				return ProjectDomService.GetAssemblyDom (TargetRuntime, path);
+				return new ICSharpCode.NRefactory.TypeSystem.Implementation.SimpleCompilation (TypeSystemService.LoadAssemblyContext (TargetRuntime, TargetFramework, path));
 			return null;
 		}
 		
@@ -478,7 +487,7 @@ namespace MonoDevelop.AspNet
 			if (webConfig == null || !File.Exists (webConfig.FilePath))
 				return;
 			
-			var textFile = DesignerSupport.OpenDocumentFileProvider.Instance.GetEditableTextFile (webConfig.FilePath);
+			var textFile = MonoDevelop.Ide.TextFileProvider.Instance.GetEditableTextFile (webConfig.FilePath);
 			//use textfile API because it's write safe (writes out to another file then moves)
 			if (textFile == null)
 				textFile = MonoDevelop.Projects.Text.TextFile.ReadFile (webConfig.FilePath);
@@ -624,11 +633,20 @@ namespace MonoDevelop.AspNet
 				base.OnFileAddedToProject (e);
 				return;
 			}
+
+			bool webConfigChange = false;
+			List<string> filesToAdd = new List<string> ();
 			
-			IEnumerable<string> filesToAdd = MonoDevelop.DesignerSupport.CodeBehind.GuessDependencies
-				(this, e.ProjectFile, groupedExtensions);
+			foreach (ProjectFileEventInfo fargs in e) {
+				IEnumerable<string> files = MonoDevelop.DesignerSupport.CodeBehind.GuessDependencies
+					(this, fargs.ProjectFile, groupedExtensions);
+				if (files != null)
+					filesToAdd.AddRange (files);
+				if (IsWebConfig (fargs.ProjectFile.FilePath))
+					webConfigChange = true;
+			}
 			
-			if (IsWebConfig (e.ProjectFile.FilePath))
+			if (webConfigChange)
 				UpdateWebConfigRefs ();
 			
 			//let the base fire the event before we add files
@@ -636,11 +654,9 @@ namespace MonoDevelop.AspNet
 			base.OnFileAddedToProject (e);
 			
 			//make sure that the parent and child files are in the project
-			if (filesToAdd != null) {
-				foreach (string file in filesToAdd) {
-					//NOTE: this only adds files if they are not already in the project
-					AddFile (file);
-				}
+			foreach (string file in filesToAdd) {
+				//NOTE: this only adds files if they are not already in the project
+				AddFile (file);
 			}
 		}
 		
@@ -688,9 +704,9 @@ namespace MonoDevelop.AspNet
 		{
 			string typeName = GetCodebehindTypeName (fileName);
 			if (typeName != null) {
-				var dom = ProjectDomService.GetProjectDom (this);
+				var dom = TypeSystemService.GetCompilation (this);
 				if (dom != null)
-					return dom.GetType (typeName, true);
+					return ReflectionHelper.ParseReflectionName (typeName).Resolve (dom);
 			}
 			return null;
 		}
@@ -710,7 +726,7 @@ namespace MonoDevelop.AspNet
 			protected override string GenerateInfo (string filename)
 			{
 				try {
-					var doc = ProjectDomService.Parse (Project, filename, null) as AspNetParsedDocument;
+					var doc = TypeSystemService.ParseFile (filename, DesktopService.GetMimeTypeForUri (filename), File.ReadAllText (filename)) as AspNetParsedDocument;
 					if (doc != null && !string.IsNullOrEmpty (doc.Info.InheritedClass))
 						return doc.Info.InheritedClass;
 				} catch (Exception ex) {
