@@ -386,7 +386,7 @@ namespace CBinding
 			}
 			
 			string linker_args = string.Format ("-o \"{0}\" {1} {2} {3}",
-			    outputName, objectFiles, args.ToString (), pkgargs);
+			    outputName, objectFiles, pkgargs, args.ToString ());
 			
 			monitor.BeginTask (GettextCatalog.GetString ("Generating binary \"{0}\" from object files", Path.GetFileName (outputName)), 1);
 			
@@ -464,7 +464,7 @@ namespace CBinding
 			}
 			
 			string linker_args = string.Format ("-shared -o \"{0}\" {1} {2} {3}",
-			    outputName, objectFiles, args.ToString (), pkgargs);
+			    outputName, objectFiles, pkgargs, args.ToString ());
 			
 			monitor.BeginTask (GettextCatalog.GetString ("Generating shared object \"{0}\" from object files", Path.GetFileName (outputName)), 1);
 			
@@ -665,7 +665,7 @@ namespace CBinding
 		    @"^\s*(?<file>.*):(?<line>\d*):\s*(?<level>.*)\s*:\s(?<message>.*)",
 		    RegexOptions.Compiled | RegexOptions.ExplicitCapture);
 		private static Regex linkerRegex = new Regex (
-		    @"^\s*(?<file>[^:]*):(?<line>\d*):\s*(?<message>[^:]*)",
+		    @"^\s*(?<file>[^:]*):(?<line>\d*):\s*(?<message>.*)",
 		    RegexOptions.Compiled | RegexOptions.ExplicitCapture);
 		
 		private CompilerError CreateErrorFromErrorString (string errorString, TextReader reader)
