@@ -202,6 +202,9 @@ namespace CBinding
 		
 		public override IEnumerable<SolutionItem> GetReferencedItems (ConfigurationSelector configuration)
 		{
+			foreach (var p in base.GetReferencedItems (configuration))
+				yield return p;
+
 			List<string> project_names = new List<string> ();
 			
 			foreach (Package p in Packages) {
@@ -362,6 +365,7 @@ namespace CBinding
 				
 				monitor.Log.WriteLine ("The operation exited with code: {0}", op.ExitCode);
 			} catch (Exception ex) {
+				LoggingService.LogError (string.Format ("Cannot execute \"{0}\"", conf.Output), ex);
 				monitor.ReportError ("Cannot execute \"" + conf.Output + "\"", ex);
 			} finally {			
 				operationMonitor.Dispose ();			
