@@ -38,7 +38,7 @@ using ICSharpCode.NRefactory.TypeSystem;
 
 namespace MonoDevelop.CSharp.Completion
 {
-	public class EventCreationCompletionData : CompletionData
+	class EventCreationCompletionData : CompletionData
 	{
 		string parameterList;
 		IUnresolvedMember callingMember;
@@ -46,7 +46,13 @@ namespace MonoDevelop.CSharp.Completion
 		int initialOffset;
 		public bool AddSemicolon = true;
 		TextEditorData editor;
-		
+
+		public override TooltipInformation CreateTooltipInformation (bool smartWrap)
+		{
+			var tooltipInfo = new TooltipInformation ();
+			return tooltipInfo;
+		}
+
 		public EventCreationCompletionData (CSharpCompletionTextEditorExtension ext, string varName, IType delegateType, IEvent evt, string parameterList, IUnresolvedMember callingMember, IUnresolvedTypeDefinition declaringType) : base (null)
 		{
 			if (string.IsNullOrEmpty (varName)) {
@@ -63,7 +69,7 @@ namespace MonoDevelop.CSharp.Completion
 					}
 				}
 			}
-			this.editor        = ext.textEditorData;
+			this.editor        = ext.TextEditorData;
 			this.parameterList = parameterList;
 			this.callingMember = callingMember;
 			this.Icon          = "md-newmethod";
@@ -76,7 +82,7 @@ namespace MonoDevelop.CSharp.Completion
 			editor.Replace (initialOffset, editor.Caret.Offset - initialOffset, this.DisplayText + (AddSemicolon ? ";" : ""));
 			
 			// Search opening bracket of member
-			int pos = callingMember != null ? editor.Document.LocationToOffset (callingMember.BodyRegion.BeginLine, callingMember.BodyRegion.BeginColumn) : initialOffset;
+			int pos = callingMember != null && !callingMember.BodyRegion.Begin.IsEmpty ? editor.Document.LocationToOffset (callingMember.BodyRegion.BeginLine, callingMember.BodyRegion.BeginColumn) : initialOffset;
 			while (pos < editor.Document.TextLength && editor.Document.GetCharAt (pos) != '{') {
 				pos++;
 			}
@@ -87,13 +93,14 @@ namespace MonoDevelop.CSharp.Completion
 			pos = Math.Max (0, Math.Min (pos, editor.Document.TextLength - 1));
 			
 			// Insert new event handler after closing bracket
-			string indent = editor.Document.GetLine (callingMember.Region.BeginLine).GetIndentation (editor.Document);
+			var line = callingMember != null ? editor.Document.GetLine (callingMember.Region.BeginLine) : editor.Document.GetLineByOffset (initialOffset);
+			string indent = line.GetIndentation (editor.Document);
 			
 			StringBuilder sb = new StringBuilder ();
 			sb.Append (editor.EolMarker);
 			sb.Append (editor.EolMarker);
 			sb.Append (indent);
-			if (callingMember.IsStatic)
+			if (callingMember != null && callingMember.IsStatic)
 				sb.Append ("static ");
 			sb.Append ("void ");
 			int pos2 = sb.Length;
@@ -121,8 +128,8 @@ namespace MonoDevelop.CSharp.Completion
 			link.AddLink (new TextSegment (pos - initialOffset + pos2, this.DisplayText.Length));
 			links.Add (link);
 			
-			CompletionTextLinkMode tle = new CompletionTextLinkMode (editor.Parent, initialOffset, links);
-			tle.TriggerCodeCompletion = false;
+			var tle = new TextLinkEditMode (editor.Parent, initialOffset, links);
+			tle.TextLinkMode = TextLinkMode.EditIdentifier;
 			tle.SetCaretPosition = true;
 			tle.SelectPrimaryLink = true;
 			tle.OldMode = editor.CurrentMode;
