@@ -29,6 +29,8 @@ using MonoDevelop.Ide.CodeFormatting;
 using MonoDevelop.Projects.Text;
 using System.Xml;
 using MonoDevelop.Ide.Gui.Dialogs;
+using System;
+using MonoDevelop.Ide.Gui.Content;
 
 namespace MonoDevelop.CSharp.Formatting
 {
@@ -36,6 +38,12 @@ namespace MonoDevelop.CSharp.Formatting
 	{
 		CSharpFormattingPolicyPanelWidget panel;
 		
+		static CSharpFormattingPolicyPanel ()
+		{
+			// ensure that custom text editor shemes are loaded.
+			MonoDevelop.SourceEditor.SourceEditorDisplayBinding.InitSourceEditor ();
+		}
+		
 		public override Widget CreatePanelWidget ()
 		{
 			return panel = new CSharpFormattingPolicyPanelWidget ();
@@ -43,9 +51,13 @@ namespace MonoDevelop.CSharp.Formatting
 		
 		protected override void LoadFrom (CSharpFormattingPolicy policy)
 		{
-			panel.Policy = policy.Clone ();
+			panel.SetPolicy (policy.Clone (), GetCurrentOtherPolicy<TextStylePolicy> ());
+		}
+
+		public override void PanelSelected ()
+		{
+			panel.SetPolicy (GetCurrentOtherPolicy<TextStylePolicy> ());
 		}
-		
 		
 		protected override CSharpFormattingPolicy GetPolicy ()
 		{
