@@ -37,11 +37,11 @@ namespace MonoDevelop.CSharp.Formatting
 	{
 		Mono.TextEditor.TextEditor texteditor = new Mono.TextEditor.TextEditor ();
 		CSharpFormattingPolicy profile;
-		Gtk.TreeStore indentOptions, bacePositionOptions, newLineOptions, whiteSpaceOptions;
+		Gtk.TreeStore indentOptions, bacePositionOptions, newLineOptions, whiteSpaceOptions, wrappingOptions;
 		
 		static Dictionary<Wrapping, string> arrayInitializerTranslationDictionary = new Dictionary<Wrapping, string> ();
 		static Dictionary<BraceStyle, string> braceStyleTranslationDictionary = new Dictionary<BraceStyle, string> ();
-		static Dictionary<BraceForcement, string> braceForcementTranslationDictionary = new Dictionary<BraceForcement, string> ();
+		//static Dictionary<BraceForcement, string> braceForcementTranslationDictionary = new Dictionary<BraceForcement, string> ();
 		static Dictionary<PropertyFormatting, string> propertyFormattingTranslationDictionary = new Dictionary<PropertyFormatting, string> ();
 		static Dictionary<NewLinePlacement, string>  newLinePlacementTranslationDictionary = new Dictionary<NewLinePlacement, string> ();
 		
@@ -55,10 +55,6 @@ namespace MonoDevelop.CSharp.Formatting
 			braceStyleTranslationDictionary [BraceStyle.NextLineShifted2] = GettextCatalog.GetString ("Next line shifted2");
 			braceStyleTranslationDictionary [BraceStyle.BannerStyle] = GettextCatalog.GetString ("Banner style");
 			
-			braceForcementTranslationDictionary [BraceForcement.DoNotChange] = GettextCatalog.GetString ("Do not change");
-			braceForcementTranslationDictionary [BraceForcement.AddBraces] = GettextCatalog.GetString ("Add braces");
-			braceForcementTranslationDictionary [BraceForcement.RemoveBraces] = GettextCatalog.GetString ("Remove braces");
-			
 			propertyFormattingTranslationDictionary [PropertyFormatting.AllowOneLine] = GettextCatalog.GetString ("Allow one line");
 			propertyFormattingTranslationDictionary [PropertyFormatting.ForceOneLine] = GettextCatalog.GetString ("Force one line");
 			propertyFormattingTranslationDictionary [PropertyFormatting.ForceNewLine] = GettextCatalog.GetString ("Force new line");
@@ -77,8 +73,8 @@ namespace MonoDevelop.CSharp.Formatting
 		{
 			if (value is BraceStyle)
 				return braceStyleTranslationDictionary [(BraceStyle)value];
-			if (value is BraceForcement) 
-				return braceForcementTranslationDictionary [(BraceForcement)value];
+//			if (value is BraceForcement) 
+//				return braceForcementTranslationDictionary [(BraceForcement)value];
 			if (value is PropertyFormatting)
 				return propertyFormattingTranslationDictionary [(PropertyFormatting)value];
 			if (value is Wrapping)
@@ -92,8 +88,8 @@ namespace MonoDevelop.CSharp.Formatting
 		{
 			if (propertyType == typeof(BraceStyle))
 				return braceStyleTranslationDictionary.First (p => p.Value == newText).Key;
-			if (propertyType == typeof(BraceForcement)) 
-				return braceForcementTranslationDictionary.First (p => p.Value == newText).Key;
+//			if (propertyType == typeof(BraceForcement)) 
+//				return braceForcementTranslationDictionary.First (p => p.Value == newText).Key;
 			if (propertyType == typeof(PropertyFormatting)) 
 				return propertyFormattingTranslationDictionary.First (p => p.Value == newText).Key;
 			if (propertyType == typeof(Wrapping))
@@ -219,7 +215,69 @@ namespace MonoDevelop.CSharp.Formatting
 			}
 		}
 	}";
-		
+		const string longMethodCall = @"class ClassDeclaration { 
+		public void Test (int test)
+		{
+			LongMethodCallInSameLine (""Hello"", 1, test);
+			LongMethodCallInMultiple (
+""Hello"", 
+		1, 
+		test);
+
+		LongMethodCallInMultipleCase2 (""Hello"", 
+		                               1, 
+		                               test);
+	}
+}";
+		const string longMethodDeclaration = @"class ClassDeclaration { 
+	public void LongMethodCallInSameLine (int test, string foo, double bar)
+	{
+	}
+	public void LongMethodCallInMultiple (
+int test,
+string foo,
+double bar)
+	{
+	}
+	public void LongMethodCallInMultipleCase2 (int test,
+string foo,
+double bar)
+	{
+	}
+}";
+		const string longIndexerDeclaration = @"class ClassDeclaration { 
+	public int this [int test, string foo, double bar]
+	{
+		get {}
+	}
+	public int this [
+int test,
+string foo,
+double bar]
+	{
+		get {}
+	}
+	public int this [int test,
+string foo,
+double bar]
+	{
+		get {}
+	}
+}";
+		const string longIndexer = @"class ClassDeclaration { 
+		public void Test (int test)
+		{
+			this [""Hello"", 1, test] = 0;
+			this [
+""Hello"", 
+		1, 
+		test] = 0;
+
+		this [""Hello"", 
+		                               1, 
+		                               test] = 0;
+	}
+}";
 		const string simpleWhile = @"class ClassDeclaration { 
 		public void Test ()
 		{
@@ -390,6 +448,7 @@ namespace TestSpace {
 			comboboxCategories.AppendText (GettextCatalog.GetString ("Indentation"));
 			comboboxCategories.AppendText (GettextCatalog.GetString ("Braces"));
 			comboboxCategories.AppendText (GettextCatalog.GetString ("Blank lines"));
+			comboboxCategories.AppendText (GettextCatalog.GetString ("Wrapping"));
 			comboboxCategories.AppendText (GettextCatalog.GetString ("White Space"));
 			comboboxCategories.AppendText (GettextCatalog.GetString ("New Lines"));
 			comboboxCategories.Changed += delegate(object sender, EventArgs e) {
@@ -404,7 +463,6 @@ namespace TestSpace {
 			texteditor.Options.ShowFoldMargin = false;
 			texteditor.Options.ShowIconMargin = false;
 			texteditor.Options.ShowLineNumberMargin = false;
-			texteditor.Options.ShowInvalidLines = false;
 			texteditor.Document.ReadOnly = true;
 			texteditor.Document.MimeType = CSharpFormatter.MimeType;
 			scrolledwindow.Child = texteditor;
@@ -547,61 +605,27 @@ namespace TestSpace {
 			AddOption (bacePositionOptions, category, "EventRemoveBraceStyle", GettextCatalog.GetString ("Remove declaration"), eventExample);
 			AddOption (bacePositionOptions, category, "AllowEventRemoveBlockInline", GettextCatalog.GetString ("Allow one line remove"), eventExample);
 			
-			category = AddOption (bacePositionOptions, null, GettextCatalog.GetString ("Brace forcement"), null);
-			AddOption (bacePositionOptions, category, "IfElseBraceForcement", GettextCatalog.GetString ("'if...else' statement"), @"class ClassDeclaration { 
-	public void Test ()
-		{
-			if (true) {
-				Console.WriteLine (""Hello World!"");
-			}
-			if (true)
-				Console.WriteLine (""Hello World!"");
-		}
-	}");
-			AddOption (bacePositionOptions, category, "ForBraceForcement", GettextCatalog.GetString ("'for' statement"), @"class ClassDeclaration { 
-		public void Test ()
-		{
-			for (int i = 0; i < 10; i++) {
-				Console.WriteLine (""Hello World "" + i);
-			}
-			for (int i = 0; i < 10; i++)
-				Console.WriteLine (""Hello World "" + i);
-		}
-	}");
-			AddOption (bacePositionOptions, category, "WhileBraceForcement", GettextCatalog.GetString ("'while' statement"), @"class ClassDeclaration { 
-		public void Test ()
-		{
-			int i = 0;
-			while (i++ < 10) {
-				Console.WriteLine (""Hello World "" + i);
-			}
-			while (i++ < 20)
-				Console.WriteLine (""Hello World "" + i);
-		}
-	}");
-			AddOption (bacePositionOptions, category, "UsingBraceForcement", GettextCatalog.GetString ("'using' statement"), simpleUsingStatement);
-			AddOption (bacePositionOptions, category, "FixedBraceForcement", GettextCatalog.GetString ("'fixed' statement"), simpleFixedStatement);
 			treeviewBracePositions.ExpandAll ();
 			#endregion
 			
 			#region New line options
 			newLineOptions = new Gtk.TreeStore (typeof(string), typeof(string), typeof(string), typeof(bool), typeof(bool));
-			
+
 			column = new TreeViewColumn ();
 			// pixbuf column
 			column.PackStart (pixbufCellRenderer, false);
 			column.SetCellDataFunc (pixbufCellRenderer, RenderIcon);
-			
+
 			// text column
 			cellRendererText.Ypad = 1;
 			column.PackStart (cellRendererText, true);
 			column.SetAttributes (cellRendererText, "text", 1);
-			
+
 			treeviewNewLines.Model = newLineOptions;
 			treeviewNewLines.HeadersVisible = false;
 			treeviewNewLines.Selection.Changed += TreeSelectionChanged;
 			treeviewNewLines.AppendColumn (column);
-			
+
 			column = new TreeViewColumn ();
 			cellRendererCombo = new CellRendererCombo ();
 			cellRendererCombo.Ypad = 1;
@@ -615,7 +639,7 @@ namespace TestSpace {
 			column.PackStart (cellRendererCombo, false);
 			column.SetAttributes (cellRendererCombo, "visible", comboVisibleColumn);
 			column.SetCellDataFunc (cellRendererCombo, ComboboxDataFunc);
-			
+
 			cellRendererToggle = new CellRendererToggle ();
 			cellRendererToggle.Activatable = !profile.IsBuiltIn;
 			cellRendererToggle.Ypad = 1;
@@ -623,9 +647,9 @@ namespace TestSpace {
 			column.PackStart (cellRendererToggle, false);
 			column.SetAttributes (cellRendererToggle, "visible", toggleVisibleColumn);
 			column.SetCellDataFunc (cellRendererToggle, ToggleDataFunc);
-			
+
 			treeviewNewLines.AppendColumn (column);
-			
+
 			AddOption (newLineOptions, "ElseNewLinePlacement", GettextCatalog.GetString ("Place 'else' on new line"), simpleIf);
 			AddOption (newLineOptions, "ElseIfNewLinePlacement", GettextCatalog.GetString ("Place 'else if' on new line"), simpleIf);
 			AddOption (newLineOptions, "CatchNewLinePlacement", GettextCatalog.GetString ("Place 'catch' on new line"), simpleCatch);
@@ -635,6 +659,76 @@ namespace TestSpace {
 			treeviewNewLines.ExpandAll ();
 			#endregion
 			
+			#region Wrapping options
+			wrappingOptions = new Gtk.TreeStore (typeof(string), typeof(string), typeof(string), typeof(bool), typeof(bool));
+
+			column = new TreeViewColumn ();
+			// pixbuf column
+			column.PackStart (pixbufCellRenderer, false);
+			column.SetCellDataFunc (pixbufCellRenderer, RenderIcon);
+
+			// text column
+			cellRendererText.Ypad = 1;
+			column.PackStart (cellRendererText, true);
+			column.SetAttributes (cellRendererText, "text", 1);
+
+
+			treeviewWrappingCategory.Model = wrappingOptions;
+			treeviewWrappingCategory.HeadersVisible = false;
+			treeviewWrappingCategory.Selection.Changed += TreeSelectionChanged;
+			treeviewWrappingCategory.AppendColumn (column);
+
+			column = new TreeViewColumn ();
+			cellRendererCombo = new CellRendererCombo ();
+			cellRendererCombo.Ypad = 1;
+			cellRendererCombo.Mode = CellRendererMode.Editable;
+			cellRendererCombo.TextColumn = 1;
+			cellRendererCombo.Model = comboBoxStore;
+			cellRendererCombo.HasEntry = false;
+			cellRendererCombo.Editable = !profile.IsBuiltIn;
+			cellRendererCombo.Edited += new ComboboxEditedHandler (this, wrappingOptions).ComboboxEdited;
+
+			column.PackStart (cellRendererCombo, false);
+			column.SetAttributes (cellRendererCombo, "visible", comboVisibleColumn);
+			column.SetCellDataFunc (cellRendererCombo, ComboboxDataFunc);
+
+			cellRendererToggle = new CellRendererToggle ();
+			cellRendererToggle.Activatable = !profile.IsBuiltIn;
+			cellRendererToggle.Ypad = 1;
+			cellRendererToggle.Toggled += new CellRendererToggledHandler (this, treeviewNewLines, wrappingOptions).Toggled;
+			column.PackStart (cellRendererToggle, false);
+			column.SetAttributes (cellRendererToggle, "visible", toggleVisibleColumn);
+			column.SetCellDataFunc (cellRendererToggle, ToggleDataFunc);
+
+			treeviewWrappingCategory.AppendColumn (column);
+
+			category = AddOption (wrappingOptions, null, GettextCatalog.GetString ("Method declarations"), null);
+			AddOption (wrappingOptions, category, "MethodDeclarationParameterWrapping", GettextCatalog.GetString ("Parameters"), longMethodDeclaration);
+			AddOption (wrappingOptions, category, "NewLineAferMethodDeclarationOpenParentheses", GettextCatalog.GetString ("New line after open parentheses"), longMethodDeclaration);
+			AddOption (wrappingOptions, category, "MethodDeclarationClosingParenthesesOnNewLine", GettextCatalog.GetString ("New line before closing parentheses"), longMethodDeclaration);
+			AddOption (wrappingOptions, category, "AlignToFirstMethodDeclarationParameter", GettextCatalog.GetString ("Align to first parameter"), longMethodDeclaration);
+
+			category = AddOption (wrappingOptions, null, GettextCatalog.GetString ("Method calls"), null);
+			AddOption (wrappingOptions, category, "MethodCallArgumentWrapping", GettextCatalog.GetString ("Arguments"), longMethodCall);
+			AddOption (wrappingOptions, category, "NewLineAferMethodCallOpenParentheses", GettextCatalog.GetString ("New line after open parentheses"), longMethodCall);
+			AddOption (wrappingOptions, category, "MethodCallClosingParenthesesOnNewLine", GettextCatalog.GetString ("New line before closing parentheses"), longMethodCall);
+			AddOption (wrappingOptions, category, "AlignToFirstMethodCallArgument", GettextCatalog.GetString ("Align to first argument"), longMethodCall);
+
+			category = AddOption (wrappingOptions, null, GettextCatalog.GetString ("Indexer declarations"), null);
+			AddOption (wrappingOptions, category, "IndexerDeclarationParameterWrapping", GettextCatalog.GetString ("Parameters"), longIndexerDeclaration);
+			AddOption (wrappingOptions, category, "NewLineAferIndexerDeclarationOpenBracket", GettextCatalog.GetString ("New line after open parentheses"), longIndexerDeclaration);
+			AddOption (wrappingOptions, category, "IndexerDeclarationClosingBracketOnNewLine", GettextCatalog.GetString ("New line before closing parentheses"), longIndexerDeclaration);
+			AddOption (wrappingOptions, category, "AlignToFirstIndexerDeclarationParameter", GettextCatalog.GetString ("Align to first parameter"), longIndexerDeclaration);
+
+			category = AddOption (wrappingOptions, null, GettextCatalog.GetString ("Indexer usage"), null);
+			AddOption (wrappingOptions, category, "IndexerArgumentWrapping", GettextCatalog.GetString ("Arguments"), longIndexer);
+			AddOption (wrappingOptions, category, "NewLineAferIndexerOpenBracket", GettextCatalog.GetString ("New line after open bracket"), longIndexer);
+			AddOption (wrappingOptions, category, "IndexerClosingBracketOnNewLine", GettextCatalog.GetString ("New line before closing bracket"), longIndexer);
+			AddOption (wrappingOptions, category, "AlignToFirstIndexerArgument", GettextCatalog.GetString ("Align to first parameter"), longIndexer);
+
+			treeviewWrappingCategory.ExpandAll ();
+			#endregion
+
 			#region White space options
 			whiteSpaceOptions = new Gtk.TreeStore (typeof (string), typeof (string), typeof (string), typeof(bool), typeof(bool));
 			
