@@ -27,51 +27,66 @@
 //
 using System;
 using System.Collections.Generic;
-using System.Diagnostics;
-using System.Text;
-using System.Xml;
 using MonoDevelop.Core;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 
 using MonoDevelop.Projects;
 using MonoDevelop.Ide.CodeCompletion;
 
 using MonoDevelop.CSharp.Formatting;
-using MonoDevelop.CSharp.Parser;
 using Mono.TextEditor;
 using MonoDevelop.Ide.CodeTemplates;
-using MonoDevelop.CSharp.Resolver;
-using ICSharpCode.NRefactory.TypeSystem;
-using ICSharpCode.NRefactory.CSharp;
-using MonoDevelop.Ide.TypeSystem;
-using ICSharpCode.NRefactory;
 using MonoDevelop.SourceEditor;
 using ICSharpCode.NRefactory.CSharp.Completion;
+using ICSharpCode.NRefactory.Editor;
+using System.Linq;
+using System.Text;
 
 namespace MonoDevelop.CSharp.Formatting
 {
-	public class CSharpTextEditorIndentation : TextEditorExtension
+	public class CSharpTextEditorIndentation : TextEditorExtension, ITextPasteHandler
 	{
 		DocumentStateTracker<CSharpIndentEngine> stateTracker;
 		int cursorPositionBeforeKeyPress;
-		TextEditorData textEditorData;
-		CSharpFormattingPolicy policy;
-		TextStylePolicy textStylePolicy;
+		TextEditorData textEditorData {
+			get {
+				return document.Editor;
+			}
+		}
+
+		IEnumerable<string> types = MonoDevelop.Ide.DesktopService.GetMimeTypeInheritanceChain (CSharpFormatter.MimeType);
+
+		CSharpFormattingPolicy Policy {
+			get {
+				if (Document != null && Document.Project != null && Document.Project.Policies != null) {
+					return base.Document.Project.Policies.Get<CSharpFormattingPolicy> (types);
+				}
+				return MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<CSharpFormattingPolicy> (types);
+			}
+		}
+
+		TextStylePolicy TextStylePolicy {
+			get {
+				if (Document != null && Document.Project != null && Document.Project.Policies != null) {
+					return base.Document.Project.Policies.Get<TextStylePolicy> (types);
+				}
+				return MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<TextStylePolicy> (types);
+			}
+		}
 
 		char lastCharInserted;
 
 		static CSharpTextEditorIndentation ()
 		{
 			CompletionWindowManager.WordCompleted += delegate(object sender,CodeCompletionContextEventArgs e) {
-				IExtensibleTextEditor editor = e.Widget as IExtensibleTextEditor;
+				var editor = e.Widget as IExtensibleTextEditor;
 				if (editor == null)
 					return;
-				ITextEditorExtension textEditorExtension = editor.Extension;
+				var textEditorExtension = editor.Extension;
 				while (textEditorExtension != null && !(textEditorExtension is CSharpTextEditorIndentation)) {
 					textEditorExtension = textEditorExtension.Next;
 				}
-				CSharpTextEditorIndentation extension = textEditorExtension as CSharpTextEditorIndentation;
+				var extension = textEditorExtension as CSharpTextEditorIndentation;
 				if (extension == null)
 					return;
 				extension.stateTracker.UpdateEngine ();
@@ -80,44 +95,55 @@ namespace MonoDevelop.CSharp.Formatting
 			};
 		}
 
+		bool IsPreprocessorDirective (DocumentLine documentLine)
+		{
+			int o = documentLine.Offset;
+			for (int i = 0; i < documentLine.Length; i++) {
+				char ch = Editor.GetCharAt (o++);
+				if (ch == '#')
+					return true;
+				if (!char.IsWhiteSpace (ch)) {
+					return false;
+				}
+			}
+			return false;
+		}
 
 		void HandleTextPaste (int insertionOffset, string text, int insertedChars)
 		{
-			// Trim blank spaces on text paste, see: Bug 511 - Trim blank spaces when copy-pasting
-			if (OnTheFlyFormatting) {
-				int i = insertionOffset + insertedChars;
-				bool foundNonWsFollowUp = false;
+			if (document.Editor.Options.IndentStyle == IndentStyle.None ||
+			    document.Editor.Options.IndentStyle == IndentStyle.Auto)
+				return;
 
-				var line = Document.Editor.GetLineByOffset (i);
-				if (line != null) {
-					for (int j = 0; j < line.Offset + line.Length; j++) {
-						var ch = Document.Editor.GetCharAt (j);
-						if (ch != ' ' && ch != '\t') {
-							foundNonWsFollowUp = true;
-							break;
+			// Just correct the start line of the paste operation - the text is already indented.
+			var curLine = Editor.GetLineByOffset (insertionOffset);
+			var curLineOffset = curLine.Offset;
+			stateTracker.UpdateEngine (curLineOffset);
+			if (!stateTracker.Engine.IsInsideOrdinaryCommentOrString) {
+				// The Indent engine doesn't really handle pre processor directives very well.
+				if (IsPreprocessorDirective (curLine)) {
+					Editor.Replace (curLineOffset, curLine.Length, stateTracker.Engine.NewLineIndent + Editor.GetTextAt (curLine).TrimStart ());
+				} else {
+					int pos = curLineOffset;
+					string curIndent = curLine.GetIndentation (textEditorData.Document);
+					int nlwsp = curIndent.Length;
+
+					if (!stateTracker.Engine.LineBeganInsideMultiLineComment || (nlwsp < curLine.LengthIncludingDelimiter && textEditorData.Document.GetCharAt (curLineOffset + nlwsp) == '*')) {
+						// Possibly replace the indent
+						stateTracker.UpdateEngine (curLineOffset + curLine.Length);
+						string newIndent = stateTracker.Engine.ThisLineIndent;
+						if (newIndent != curIndent) {
+							if (CompletionWindowManager.IsVisible) {
+								if (pos < CompletionWindowManager.CodeCompletionContext.TriggerOffset)
+									CompletionWindowManager.CodeCompletionContext.TriggerOffset -= nlwsp;
+							}
+							textEditorData.Replace (pos, nlwsp, newIndent);
+							textEditorData.Document.CommitLineUpdate (textEditorData.Caret.Line);
 						}
 					}
 				}
-
-				if (!foundNonWsFollowUp) {
-					while (i > insertionOffset) {
-						char ch = Document.Editor.GetCharAt (i - 1);
-						if (ch != ' ' && ch != '\t') 
-							break;
-						i--;
-					}
-					int delta = insertionOffset + insertedChars - i;
-					if (delta > 0) {
-						Editor.Caret.Offset -= delta;
-						Editor.Remove (insertionOffset + insertedChars - delta, delta);
-					}
-				}
-			}
-			var documentLine = Editor.GetLineByOffset (insertionOffset + insertedChars);
-			while (documentLine != null && insertionOffset < documentLine.EndOffset) {
-				DoReSmartIndent (documentLine.Offset);
-				documentLine = documentLine.PreviousLine;
 			}
+			textEditorData.FixVirtualIndentation ();
 		} 
 
 		public static bool OnTheFlyFormatting {
@@ -132,64 +158,293 @@ namespace MonoDevelop.CSharp.Formatting
 		void RunFormatter (DocumentLocation location)
 		{
 			if (OnTheFlyFormatting && textEditorData != null && !(textEditorData.CurrentMode is TextLinkEditMode) && !(textEditorData.CurrentMode is InsertionCursorEditMode)) {
-
 				OnTheFlyFormatter.Format (Document, location);
 			}
 		}
 
-		public CSharpTextEditorIndentation ()
-		{
-			IEnumerable<string> types = MonoDevelop.Ide.DesktopService.GetMimeTypeInheritanceChain (CSharpFormatter.MimeType);
-			policy = MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<CSharpFormattingPolicy> (types);
-			textStylePolicy = MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<TextStylePolicy> (types);
-		}
-
 		public override void Initialize ()
 		{
 			base.Initialize ();
 
-			IEnumerable<string> types = MonoDevelop.Ide.DesktopService.GetMimeTypeInheritanceChain (CSharpFormatter.MimeType);
-			if (base.Document.Project != null && base.Document.Project.Policies != null) {
-				policy = base.Document.Project.Policies.Get<CSharpFormattingPolicy> (types);
-				textStylePolicy = base.Document.Project.Policies.Get<TextStylePolicy> (types);
-			}
 
-			textEditorData = Document.Editor;
 			if (textEditorData != null) {
-				textEditorData.Options.Changed += delegate {
-					var project = base.Document.Project;
-					if (project != null) {
-						policy = project.Policies.Get<CSharpFormattingPolicy> (types);
-						textStylePolicy = project.Policies.Get<TextStylePolicy> (types);
-					}
-					textEditorData.IndentationTracker = new IndentVirtualSpaceManager (
-						textEditorData,
-						new DocumentStateTracker<CSharpIndentEngine> (new CSharpIndentEngine (policy, textStylePolicy), textEditorData)
-					);
-				};
+				textEditorData.Options.Changed += HandleTextOptionsChanged;
 				textEditorData.IndentationTracker = new IndentVirtualSpaceManager (
 					textEditorData,
-					new DocumentStateTracker<CSharpIndentEngine> (new CSharpIndentEngine (policy, textStylePolicy), textEditorData)
+					new DocumentStateTracker<CSharpIndentEngine> (new CSharpIndentEngine (Policy, TextStylePolicy), textEditorData)
 				);
+				textEditorData.Document.TextReplacing += HandleTextReplacing;
+				textEditorData.Document.TextReplaced += HandleTextReplaced;;
+				textEditorData.TextPasteHandler = this;
+				textEditorData.Paste += HandleTextPaste;
 			}
 
 			InitTracker ();
-//			Document.Editor.Paste += HandleTextPaste;
 		}
 
-		/*		void TextCut (object sender, ReplaceEventArgs e)
+		void HandleTextOptionsChanged (object sender, EventArgs e)
 		{
-			if (!string.IsNullOrEmpty (e.Value) || e.Count == 0)
-				return;
-			RunFormatterAt (e.Offset);
-		}*/
+			textEditorData.IndentationTracker = new IndentVirtualSpaceManager (
+				textEditorData,
+				new DocumentStateTracker<CSharpIndentEngine> (new CSharpIndentEngine (Policy, TextStylePolicy), textEditorData)
+				);
 
+		}
+
+		public override void Dispose ()
+		{
+			if (textEditorData != null) {
+				textEditorData.TextPasteHandler = null;
+				textEditorData.Paste -= HandleTextPaste;
+				textEditorData.Options.Changed -= HandleTextOptionsChanged;
+				textEditorData.IndentationTracker = null;
+				textEditorData.Document.TextReplacing -= HandleTextReplacing;
+				textEditorData.Document.TextReplaced -= HandleTextReplaced;;
+			}
+			base.Dispose ();
+		}
+
+		bool wasInVerbatimString;
+
+		void HandleTextReplaced (object sender, DocumentChangeEventArgs e)
+		{
+			if (e.RemovalLength != 1)
+				return;
+			stateTracker.UpdateEngine (Math.Min (textEditorData.Document.TextLength, e.Offset + e.InsertionLength + 1));
+			if (wasInVerbatimString && !stateTracker.Engine.IsInsideVerbatimString) {
+				textEditorData.Document.TextReplacing -= HandleTextReplacing;
+				textEditorData.Document.TextReplaced -= HandleTextReplaced;;
+				ConvertVerbatimStringToNormal (textEditorData, e.Offset + e.InsertionLength + 1);
+				textEditorData.Document.TextReplacing += HandleTextReplacing;
+				textEditorData.Document.TextReplaced += HandleTextReplaced;;
+			}
+		}
+
+		void HandleTextReplacing (object sender, DocumentChangeEventArgs e)
+		{
+			var o = e.Offset + e.RemovalLength;
+			if (o < 0 || o + 1 > textEditorData.Length || e.RemovalLength != 1 || textEditorData.Document.IsInUndo) {
+				wasInVerbatimString = false;
+				return;
+			}
+			if (textEditorData.GetCharAt (o) != '"')
+				return;
+			stateTracker.UpdateEngine (o + 1);
+			wasInVerbatimString = stateTracker.Engine.IsInsideVerbatimString;
+		}
+
+		#region ITextPasteHandler implementation
+		enum CopySource : byte
+		{
+			Text = 0,
+			StringLiteral = 1,
+			VerbatimString = 2
+		}
+
+		byte[] ITextPasteHandler.GetCopyData (TextSegment segment)
+		{
+			stateTracker.UpdateEngine (segment.Offset);
+			if (stateTracker.Engine.IsInsideStringLiteral)
+				return new [] { (byte)CopySource.StringLiteral };
+			if (stateTracker.Engine.IsInsideVerbatimString)
+				return new [] { (byte)CopySource.VerbatimString };
+			return null;
+		}
+		
+		static string ConvertFromString (string nonVerbatimStringContent)
+		{
+			var result = new StringBuilder ();
+			for (int i = 0; i < nonVerbatimStringContent.Length; i++) {
+				var ch = nonVerbatimStringContent [i];
+				switch (ch) {
+				case '\\':
+					i++;
+					switch (nonVerbatimStringContent [i]) {
+					case '\\':
+						result.Append ('\\');
+						break;
+					case 'r':
+						result.Append ('\r');
+						break;
+					case 'n':
+						result.Append ('\n');
+						break;
+					case 't':
+						result.Append ('\t');
+						break;
+					case '"':
+						result.Append ('"');
+						break;
+					}
+					break;
+				default:
+					result.Append (ch);
+					break;
+				}
+			}
+			return result.ToString ();
+		}
+		
+		static string ConvertFromVerbatimString (string verbatimStringContent)
+		{
+			var result = new StringBuilder ();
+			for (int i = 0; i < verbatimStringContent.Length; i++) {
+				var ch = verbatimStringContent [i];
+
+				switch (ch) {
+				case '"':
+					if (i + 1 < verbatimStringContent.Length && verbatimStringContent [i + 1] == '"') {
+						result.Append ("\"");
+						i++;
+					}
+					break;
+				default:
+					result.Append (ch);
+					break;
+				}
+			}
+			return result.ToString ();
+		}
+
+		static string ConvertToStringLiteral (string text)
+		{
+			var result = new StringBuilder ();
+			foreach (var ch in text) {
+				switch (ch) {
+				case '\t':
+					result.Append ("\\t");
+					break;
+				case '"':
+					result.Append ("\\\"");
+					break;
+				case '\n':
+					result.Append ("\\n");
+					break;
+				case '\r':
+					result.Append ("\\r");
+					break;
+				case '\\':
+					result.Append ("\\\\");
+					break;
+				default:
+					result.Append (ch);
+					break;
+				}
+			}
+			return result.ToString ();
+		}
+
+		static string ConvertToVerbatimLiteral (string text)
+		{
+			var result = new StringBuilder ();
+			foreach (var ch in text) {
+				switch (ch) {
+					case '"':
+					result.Append ("\"\"");
+					break;
+					default:
+					result.Append (ch);
+					break;
+				}
+			}
+			return result.ToString ();
+		}
+
+		string ITextPasteHandler.FormatPlainText (int insertionOffset, string text, byte[] copyData)
+		{
+			if (document.Editor.Options.IndentStyle == IndentStyle.None ||
+			    document.Editor.Options.IndentStyle == IndentStyle.Auto)
+				return text;
+
+			if (copyData != null && copyData.Length == 1) {
+				CopySource src = (CopySource)copyData [0];
+				switch (src) {
+				case CopySource.VerbatimString:
+					text = ConvertFromVerbatimString (text);
+					break;
+				case CopySource.StringLiteral:
+					text = ConvertFromString (text);
+					break;
+				}
+			}
+
+			stateTracker.UpdateEngine (insertionOffset);
+			var engine = stateTracker.Engine.Clone () as CSharpIndentEngine;
+
+			var result = new StringBuilder ();
+
+			if (engine.IsInsideStringLiteral)
+				return ConvertToStringLiteral (text);
+
+			if (engine.IsInsideVerbatimString)
+				return ConvertToVerbatimLiteral (text);
+
+			bool inNewLine = false;
+			foreach (var ch in text) {
+				if (!engine.IsInsideOrdinaryCommentOrString) {
+					if (inNewLine && (ch == ' ' || ch == '\t')) {
+						engine.Push (ch);
+						continue;
+					}
+				}
+
+				if (inNewLine && ch != '\n' && ch != '\r') {
+					if (!engine.IsInsideOrdinaryCommentOrString) {
+						if (ch != '#')
+							engine.Push (ch);
+						result.Append (engine.ThisLineIndent);
+						if (ch == '#')
+							engine.Push (ch);
+					}
+					inNewLine = false;
+				} else {
+					engine.Push (ch);
+				}
+				result.Append (ch);
+				if (ch == '\n' || ch == '\r')
+					inNewLine = true;
+			}
+			return result.ToString ();
+		}
+		#endregion
+
+		void ConvertNormalToVerbatimString (TextEditorData textEditorData, int offset)
+		{
+			var endOffset = offset;
+			while (endOffset < textEditorData.Length) {
+				char ch = textEditorData.GetCharAt (endOffset);
+				if (ch == '\\' && (endOffset + 1 < textEditorData.Length && textEditorData.GetCharAt (endOffset + 1) == '"'))  {
+					endOffset += 2;
+					continue;
+				}
+				if (ch == '"')
+					break;
+				endOffset++;
+			}
+			textEditorData.Replace (offset, endOffset - offset, ConvertToVerbatimLiteral (ConvertFromString (textEditorData.GetTextAt (offset, endOffset - offset))));
+		}
+
+		void ConvertVerbatimStringToNormal (TextEditorData textEditorData, int offset)
+		{
+			var endOffset = offset;
+			while (endOffset < textEditorData.Length) {
+				char ch = textEditorData.GetCharAt (endOffset);
+				if (ch == '"' && (endOffset + 1 < textEditorData.Length && textEditorData.GetCharAt (endOffset + 1) == '"'))  {
+					endOffset += 2;
+					continue;
+				}
+				if (ch == '"')
+					break;
+				endOffset++;
+			}
+			textEditorData.Replace (offset, endOffset - offset, ConvertToStringLiteral (ConvertFromVerbatimString (textEditorData.GetTextAt (offset, endOffset - offset))));
+		}
 
 		#region Sharing the tracker
 
 		void InitTracker ()
 		{
-			stateTracker = new DocumentStateTracker<CSharpIndentEngine> (new CSharpIndentEngine (policy, textStylePolicy), textEditorData);
+			stateTracker = new DocumentStateTracker<CSharpIndentEngine> (new CSharpIndentEngine (Policy, TextStylePolicy), textEditorData);
 		}
 
 		internal DocumentStateTracker<CSharpIndentEngine> StateTracker { get { return stateTracker; } }
@@ -208,6 +463,33 @@ namespace MonoDevelop.CSharp.Formatting
 
 		int lastInsertedSemicolon = -1;
 
+		void CheckXmlCommentCloseTag (char keyChar)
+		{
+			if (keyChar == '>' && stateTracker.Engine.IsInsideDocLineComment) {
+				var location = Editor.Caret.Location;
+				string lineText = Editor.GetLineText (Editor.Caret.Line);
+				int startIndex = Math.Min (location.Column - 2, lineText.Length - 1);
+				while (startIndex >= 0 && lineText [startIndex] != '<') {
+					--startIndex;
+					if (lineText [startIndex] == '/') {
+						// already closed.
+						startIndex = -1;
+						break;
+					}
+				}
+				if (startIndex >= 0) {
+					int endIndex = startIndex + 1;
+					while (endIndex <= location.Column -1 && endIndex < lineText.Length && Char.IsLetter (lineText [endIndex])) {
+						endIndex++;
+					}
+					string tag = endIndex - startIndex > 0 ? lineText.Substring (startIndex + 1, endIndex - startIndex - 1) : null;
+					if (!string.IsNullOrEmpty (tag) && CSharpCompletionEngine.CommentTags.Any (t => t == tag)) {
+						Editor.Document.Insert (Editor.Caret.Offset, "</" + tag + ">", AnchorMovementType.BeforeInsertion);
+					}
+				}
+			}
+		}
+
 		public override bool KeyPress (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
 		{
 			bool skipFormatting = StateTracker.Engine.IsInsideOrdinaryCommentOrString ||
@@ -228,16 +510,21 @@ namespace MonoDevelop.CSharp.Formatting
 				bool retval = base.KeyPress (key, keyChar, modifier);
 				DocumentLine curLine = textEditorData.Document.GetLine (textEditorData.Caret.Line);
 				string text = textEditorData.Document.GetTextAt (curLine);
-				if (text.EndsWith (";") || text.Trim ().StartsWith ("for"))
-					return retval;
+				if (!(text.EndsWith (";", StringComparison.Ordinal) || text.Trim ().StartsWith ("for", StringComparison.Ordinal))) {
+					int guessedOffset;
 
-				int guessedOffset = GuessSemicolonInsertionOffset (textEditorData, curLine);
-				if (guessedOffset != textEditorData.Caret.Offset) {
-					using (var undo = textEditorData.OpenUndoGroup ()) {
-						textEditorData.Remove (textEditorData.Caret.Offset - 1, 1);
-						textEditorData.Caret.Offset = guessedOffset;
-						lastInsertedSemicolon = textEditorData.Caret.Offset + 1;
-						retval = base.KeyPress (key, keyChar, modifier);
+					if (GuessSemicolonInsertionOffset (textEditorData, curLine, textEditorData.Caret.Offset, out guessedOffset)) {
+						using (var undo = textEditorData.OpenUndoGroup ()) {
+							textEditorData.Remove (textEditorData.Caret.Offset - 1, 1);
+							textEditorData.Caret.Offset = guessedOffset;
+							lastInsertedSemicolon = textEditorData.Caret.Offset + 1;
+							retval = base.KeyPress (key, keyChar, modifier);
+						}
+					}
+				}
+				using (var undo = textEditorData.OpenUndoGroup ()) {
+					if (OnTheFlyFormatting && textEditorData != null && !(textEditorData.CurrentMode is TextLinkEditMode) && !(textEditorData.CurrentMode is InsertionCursorEditMode)) {
+						OnTheFlyFormatter.FormatStatmentAt (Document, textEditorData.Caret.Location);
 					}
 				}
 				return retval;
@@ -245,7 +532,7 @@ namespace MonoDevelop.CSharp.Formatting
 			
 			if (key == Gdk.Key.Tab) {
 				stateTracker.UpdateEngine ();
-				if (stateTracker.Engine.IsInsideStringLiteral) {
+				if (stateTracker.Engine.IsInsideStringLiteral && !textEditorData.IsSomethingSelected) {
 					var lexer = new CSharpCompletionEngineBase.MiniLexer (textEditorData.Document.GetTextAt (0, textEditorData.Caret.Offset));
 					lexer.Parse ();
 					if (lexer.IsInString) {
@@ -271,7 +558,7 @@ namespace MonoDevelop.CSharp.Formatting
 					textEditorData.Document.CommitLineUpdate (textEditorData.Caret.Line);
 				} else if (cursor >= 1) {
 					if (textEditorData.Caret.Column > 1) {
-						int delta = cursor - this.cursorPositionBeforeKeyPress;
+						int delta = cursor - cursorPositionBeforeKeyPress;
 						if (delta < 2 && delta > 0) {
 							textEditorData.Remove (cursor - delta, delta);
 							textEditorData.Caret.Offset = cursor - delta;
@@ -284,6 +571,18 @@ namespace MonoDevelop.CSharp.Formatting
 				return false;
 			}
 
+			stateTracker.UpdateEngine ();
+			if (!stateTracker.Engine.IsInsideOrdinaryCommentOrString) {
+				if (keyChar == '@') {
+					var retval = base.KeyPress (key, keyChar, modifier);
+					int cursor = textEditorData.Caret.Offset;
+					if (cursor < textEditorData.Length && textEditorData.GetCharAt (cursor) == '"')
+						ConvertNormalToVerbatimString (textEditorData, cursor + 1);
+					return retval;
+				}
+			}
+
+
 			//do the smart indent
 			if (textEditorData.Options.IndentStyle == IndentStyle.Smart || textEditorData.Options.IndentStyle == IndentStyle.Virtual) {
 				bool retval;
@@ -298,17 +597,20 @@ namespace MonoDevelop.CSharp.Formatting
 				using (var undo = textEditorData.OpenUndoGroup ()) {
 					DoPreInsertionSmartIndent (key);
 				}
+
+				bool automaticReindent;
+				// need to be outside of an undo group - otherwise it interferes with other text editor extension
+				// esp. the documentation insertion undo steps.
 				retval = base.KeyPress (key, keyChar, modifier);
+				//handle inserted characters
+				if (textEditorData.Caret.Offset <= 0 || textEditorData.IsSomethingSelected)
+					return retval;
+				
+				lastCharInserted = TranslateKeyCharForIndenter (key, keyChar, textEditorData.GetCharAt (textEditorData.Caret.Offset - 1));
+				if (lastCharInserted == '\0')
+					return retval;
 
 				using (var undo = textEditorData.OpenUndoGroup ()) {
-					//handle inserted characters
-					if (textEditorData.Caret.Offset <= 0 || textEditorData.IsSomethingSelected)
-						return retval;
-
-					lastCharInserted = TranslateKeyCharForIndenter (key, keyChar, textEditorData.GetCharAt (textEditorData.Caret.Offset - 1));
-					if (lastCharInserted == '\0')
-						return retval;
-
 					stateTracker.UpdateEngine ();
 
 					if (key == Gdk.Key.Return && modifier == Gdk.ModifierType.ControlMask) {
@@ -322,21 +624,36 @@ namespace MonoDevelop.CSharp.Formatting
 					//inserted rather than just updating the stack due to moving around
 
 					stateTracker.UpdateEngine ();
-					bool automaticReindent = (stateTracker.Engine.NeedsReindent && lastCharInserted != '\0');
-					if (reIndent || automaticReindent)
+					automaticReindent = (stateTracker.Engine.NeedsReindent && lastCharInserted != '\0');
+					if (key == Gdk.Key.Return && (reIndent || automaticReindent))
 						DoReSmartIndent ();
-					if (!skipFormatting && keyChar == '}')
-						RunFormatter (new DocumentLocation (textEditorData.Caret.Location.Line, textEditorData.Caret.Location.Column));
+				}
+
+				if (key != Gdk.Key.Return && (reIndent || automaticReindent)) {
+					using (var undo = textEditorData.OpenUndoGroup ()) {
+						DoReSmartIndent ();
+					}
+				}
+				if (!skipFormatting) {
+					if (keyChar == ';' || keyChar == '}') {
+						using (var undo = textEditorData.OpenUndoGroup ()) {
+							if (OnTheFlyFormatting && textEditorData != null && !(textEditorData.CurrentMode is TextLinkEditMode) && !(textEditorData.CurrentMode is InsertionCursorEditMode)) {
+								OnTheFlyFormatter.FormatStatmentAt (Document, textEditorData.Caret.Location);
+							}
+						}
+					}
 				}
 
 				stateTracker.UpdateEngine ();
 				lastCharInserted = '\0';
+				CheckXmlCommentCloseTag (keyChar);
 				return retval;
 			}
 
 			if (textEditorData.Options.IndentStyle == IndentStyle.Auto && DefaultSourceEditorOptions.Instance.TabIsReindent && key == Gdk.Key.Tab) {
 				bool retval = base.KeyPress (key, keyChar, modifier);
 				DoReSmartIndent ();
+				CheckXmlCommentCloseTag (keyChar);
 				return retval;
 			}
 
@@ -344,27 +661,64 @@ namespace MonoDevelop.CSharp.Formatting
 			//and calls HandleCodeCompletion etc to handles completion
 			var result = base.KeyPress (key, keyChar, modifier);
 
+			CheckXmlCommentCloseTag (keyChar);
+
 			if (!skipFormatting && keyChar == '}')
 				RunFormatter (new DocumentLocation (textEditorData.Caret.Location.Line, textEditorData.Caret.Location.Column));
 			return result;
 		}
 
-		static int GuessSemicolonInsertionOffset (TextEditorData data, DocumentLine curLine)
+		static bool IsSemicolonalreadyPlaced (TextEditorData data, int caretOffset)
 		{
-			int offset = data.Caret.Offset;
-			int lastNonWsOffset = offset;
+			for (int pos2 = caretOffset - 1; pos2 --> 0;) {
+				var ch2 = data.Document.GetCharAt (pos2);
+				if (ch2 == ';') {
+					return true;
+				}
+				if (!char.IsWhiteSpace (ch2))
+					return false;
+			}
+			return false;
+		}
 
-			int max = curLine.Offset + curLine.Length;
+		public static bool GuessSemicolonInsertionOffset (TextEditorData data, IDocumentLine curLine, int caretOffset, out int outOffset)
+		{
+			int lastNonWsOffset = caretOffset;
+			char lastNonWsChar = '\0';
+			outOffset = caretOffset;
+			int max = curLine.EndOffset;
+	//		if (caretOffset - 2 >= curLine.Offset && data.Document.GetCharAt (caretOffset - 2) == ')' && !IsSemicolonalreadyPlaced (data, caretOffset))
+	//			return false;
 
-			// if the line ends with ';' the line end is not the correct place for a new semicolon.
-			if (curLine.Length > 0 && data.Document.GetCharAt (max - 1) == ';')
-				return offset;
+			int end = caretOffset;
+			while (end > 1 && char.IsWhiteSpace (data.GetCharAt (end)))
+				end--;
+			int end2 = end;
+			while (end2 > 1 && char.IsLetter(data.GetCharAt (end2 - 1)))
+				end2--;
+			if (end != end2) {
+				string token = data.GetTextBetween (end2, end + 1);
+				// guess property context
+				if (token == "get" || token == "set")
+					return false;
+			}
 
 			bool isInString = false , isInChar= false , isVerbatimString= false;
 			bool isInLineComment = false , isInBlockComment= false;
-			for (int pos = offset; pos < max; pos++) {
+			bool firstChar = true;
+			for (int pos = caretOffset; pos < max; pos++) {
+				if (pos == caretOffset) {
+					if (isInString || isInChar || isVerbatimString || isInLineComment || isInBlockComment) {
+						outOffset = pos;
+						return true;
+					}
+				}
 				char ch = data.Document.GetCharAt (pos);
 				switch (ch) {
+				case '}':
+					if (firstChar && !IsSemicolonalreadyPlaced (data, caretOffset))
+						return false;
+					break;
 				case '/':
 					if (isInBlockComment) {
 						if (pos > 0 && data.Document.GetCharAt (pos - 1) == '*') 
@@ -372,12 +726,12 @@ namespace MonoDevelop.CSharp.Formatting
 					} else if (!isInString && !isInChar && pos + 1 < max) {
 						char nextChar = data.Document.GetCharAt (pos + 1);
 						if (nextChar == '/') {
-							isInLineComment = true;
-							return lastNonWsOffset;
+							outOffset = lastNonWsOffset;
+							return true;
 						}
 						if (!isInLineComment && nextChar == '*') {
-							isInBlockComment = true;
-							return lastNonWsOffset;
+							outOffset = lastNonWsOffset;
+							return true;
 						}
 					}
 					break;
@@ -407,12 +761,17 @@ namespace MonoDevelop.CSharp.Formatting
 						isInChar = !isInChar;
 					break;
 				}
-				if (!char.IsWhiteSpace (ch))
+				if (!char.IsWhiteSpace (ch)) {
+					firstChar = false;
 					lastNonWsOffset = pos;
+					lastNonWsChar = ch;
+				}
 			}
-
-			return lastNonWsOffset;
-
+			// if the line ends with ';' the line end is not the correct place for a new semicolon.
+			if (lastNonWsChar == ';')
+				return false;
+			outOffset = lastNonWsOffset;
+			return true;
 		}
 
 		static char TranslateKeyCharForIndenter (Gdk.Key key, char keyChar, char docChar)
@@ -435,7 +794,7 @@ namespace MonoDevelop.CSharp.Formatting
 		// removes "\s*\+\s*" patterns (used for special behaviour inside strings)
 		void HandleStringConcatinationDeletion (int start, int end)
 		{
-			if (start < 0 || end >= textEditorData.Length)
+			if (start < 0 || end >= textEditorData.Length || textEditorData.IsSomethingSelected)
 				return;
 			char ch = textEditorData.GetCharAt (start);
 			if (ch == '"') {
@@ -515,29 +874,31 @@ namespace MonoDevelop.CSharp.Formatting
 				//xml doc comments
 				//check previous line was a doc comment
 				//check there's a following line?
-				if (trimmedPreviousLine.StartsWith ("///")) {
-					if (textEditorData.GetTextAt (line.Offset, line.Length).TrimStart ().StartsWith ("///"))
+				if (trimmedPreviousLine.StartsWith ("///", StringComparison.Ordinal)) {
+					if (textEditorData.GetTextAt (line.Offset, line.Length).TrimStart ().StartsWith ("///", StringComparison.Ordinal))
 						return false;
 					//check that the newline command actually inserted a newline
 					textEditorData.EnsureCaretIsNotVirtual ();
-					int insertionPoint = line.Offset + line.GetIndentation (textEditorData.Document).Length;
-					string nextLine = textEditorData.Document.GetTextAt (textEditorData.Document.GetLine (lineNumber + 1)).TrimStart ();
+					var nextLineSegment = textEditorData.Document.GetLine (lineNumber + 1);
+					string nextLine = nextLineSegment != null ? textEditorData.Document.GetTextAt (nextLineSegment).TrimStart () : "";
 
-					if (trimmedPreviousLine.Length > "///".Length || nextLine.StartsWith ("///")) {
-						textEditorData.Insert (insertionPoint, "/// ");
+					if (trimmedPreviousLine.Length > "///".Length || nextLine.StartsWith ("///", StringComparison.Ordinal)) {
+						var insertionPoint = textEditorData.Caret.Offset;
+						int inserted = textEditorData.Insert (insertionPoint, "/// ");
+						textEditorData.Caret.Offset = insertionPoint + inserted;
 						return true;
 					}
 					//multi-line comments
 				} else if (stateTracker.Engine.IsInsideMultiLineComment) {
-					if (textEditorData.GetTextAt (line.Offset, line.Length).TrimStart ().StartsWith ("*"))
+					if (textEditorData.GetTextAt (line.Offset, line.Length).TrimStart ().StartsWith ("*", StringComparison.Ordinal))
 						return false;
 					textEditorData.EnsureCaretIsNotVirtual ();
 					string commentPrefix = string.Empty;
-					if (trimmedPreviousLine.StartsWith ("* ")) {
+					if (trimmedPreviousLine.StartsWith ("* ", StringComparison.Ordinal)) {
 						commentPrefix = "* ";
-					} else if (trimmedPreviousLine.StartsWith ("/**") || trimmedPreviousLine.StartsWith ("/*")) {
+					} else if (trimmedPreviousLine.StartsWith ("/**", StringComparison.Ordinal) || trimmedPreviousLine.StartsWith ("/*", StringComparison.Ordinal)) {
 						commentPrefix = " * ";
-					} else if (trimmedPreviousLine.StartsWith ("*")) {
+					} else if (trimmedPreviousLine.StartsWith ("*", StringComparison.Ordinal)) {
 						commentPrefix = "*";
 					}
 
@@ -555,7 +916,7 @@ namespace MonoDevelop.CSharp.Formatting
 					textEditorData.Insert (prevLine.Offset + prevLine.Length, "\" +");
 
 					int indentSize = line.GetIndentation (textEditorData.Document).Length;
-					var insertedText = prevLine.GetIndentation (textEditorData.Document) + (trimmedPreviousLine.StartsWith ("\"") ? "" : "\t") + "\"";
+					var insertedText = prevLine.GetIndentation (textEditorData.Document) + (trimmedPreviousLine.StartsWith ("\"", StringComparison.Ordinal) ? "" : "\t") + "\"";
 					textEditorData.Replace (line.Offset, indentSize, insertedText);
 					return true;
 				}
@@ -571,22 +932,22 @@ namespace MonoDevelop.CSharp.Formatting
 
 		void DoReSmartIndent (int cursor)
 		{
-			string newIndent = string.Empty;
+			if (stateTracker.Engine.LineBeganInsideVerbatimString || stateTracker.Engine.LineBeganInsideMultiLineComment)
+				return;
 			DocumentLine line = textEditorData.Document.GetLineByOffset (cursor);
 //			stateTracker.UpdateEngine (line.Offset);
 			// Get context to the end of the line w/o changing the main engine's state
-			CSharpIndentEngine ctx = (CSharpIndentEngine)stateTracker.Engine.Clone ();
+			var ctx = (CSharpIndentEngine)stateTracker.Engine.Clone ();
 			for (int max = cursor; max < line.EndOffset; max++) {
 				ctx.Push (textEditorData.Document.GetCharAt (max));
 			}
-			
 			int pos = line.Offset;
 			string curIndent = line.GetIndentation (textEditorData.Document);
 			int nlwsp = curIndent.Length;
 			int offset = cursor > pos + nlwsp ? cursor - (pos + nlwsp) : 0;
 			if (!stateTracker.Engine.LineBeganInsideMultiLineComment || (nlwsp < line.LengthIncludingDelimiter && textEditorData.Document.GetCharAt (line.Offset + nlwsp) == '*')) {
 				// Possibly replace the indent
-				newIndent = ctx.ThisLineIndent;
+				string newIndent = ctx.ThisLineIndent;
 				int newIndentLength = newIndent.Length;
 				if (newIndent != curIndent) {
 					if (CompletionWindowManager.IsVisible) {
@@ -598,6 +959,7 @@ namespace MonoDevelop.CSharp.Formatting
 					textEditorData.Document.CommitLineUpdate (textEditorData.Caret.Line);
 					// Engine state is now invalid
 					stateTracker.ResetEngineToPosition (pos);
+					CompletionWindowManager.HideWindow ();
 				}
 				pos += newIndentLength;
 			} else {
