@@ -35,6 +35,10 @@ namespace MonoDevelop.CSharp.Project
 {
 	partial class CodeGenerationPanelWidget : Gtk.Bin 
 	{
+		const int DEBUG_FULL = 0;
+		const int DEBUG_PDB_ONLY = 1;
+		const int DEBUG_NONE = 2;
+
 		DotNetProjectConfiguration configuration;
 		CSharpCompilerParameters compilerParameters = null;
 		
@@ -49,7 +53,6 @@ namespace MonoDevelop.CSharp.Project
 			compilerParameters = (CSharpCompilerParameters) configuration.CompilationParameters;
 			
 			symbolsEntry.Text                          = compilerParameters.DefineSymbols;
-			generateDebugInformationCheckButton.Active = configuration.DebugMode;
 			generateXmlOutputCheckButton.Active        = compilerParameters.GenerateXmlDocumentation;
 			enableOptimizationCheckButton.Active       = compilerParameters.Optimize;
 			generateOverflowChecksCheckButton.Active   = compilerParameters.GenerateOverflowChecks;
@@ -60,6 +63,14 @@ namespace MonoDevelop.CSharp.Project
 			
 			int i = CSharpLanguageBinding.SupportedPlatforms.IndexOf (compilerParameters.PlatformTarget);
 			comboPlatforms.Active = i != -1 ? i : 0;
+
+			if (!configuration.DebugMode || string.Equals ("none", compilerParameters.DebugType, StringComparison.OrdinalIgnoreCase)) {
+				comboDebug.Active = DEBUG_NONE;
+			} else if (string.Equals ("pdbonly", compilerParameters.DebugType, StringComparison.OrdinalIgnoreCase)) {
+				comboDebug.Active = DEBUG_PDB_ONLY;
+			} else {
+				comboDebug.Active = DEBUG_FULL;
+			}
 		}
 
 		public void Store ()
@@ -68,7 +79,6 @@ namespace MonoDevelop.CSharp.Project
 				throw new ApplicationException ("Code generation panel wasn't loaded !");
 			
 			compilerParameters.DefineSymbols            = symbolsEntry.Text;
-			configuration.DebugMode                     = generateDebugInformationCheckButton.Active;
 			compilerParameters.GenerateXmlDocumentation = generateXmlOutputCheckButton.Active;
 			compilerParameters.Optimize                 = enableOptimizationCheckButton.Active;
 			compilerParameters.GenerateOverflowChecks   = generateOverflowChecksCheckButton.Active;
@@ -77,6 +87,25 @@ namespace MonoDevelop.CSharp.Project
 			compilerParameters.AdditionalArguments      = additionalArgsEntry.Text;
 			compilerParameters.NoWarnings               = ignoreWarningsEntry.Text;
 			compilerParameters.PlatformTarget           = CSharpLanguageBinding.SupportedPlatforms [comboPlatforms.Active];
+
+			switch (comboDebug.Active) {
+			case DEBUG_FULL:
+				configuration.DebugMode = true;
+				if (!string.Equals (compilerParameters.DebugType, "full", StringComparison.OrdinalIgnoreCase)) {
+					compilerParameters.DebugType = "";
+				}
+				break;
+			case DEBUG_PDB_ONLY:
+				configuration.DebugMode = true;
+				compilerParameters.DebugType = "pdbonly";
+				break;
+			case DEBUG_NONE:
+				configuration.DebugMode = false;
+				if (!string.Equals (compilerParameters.DebugType, "none", StringComparison.OrdinalIgnoreCase)) {
+					compilerParameters.DebugType = "";
+				}
+				break;
+			}
 		}
 	}
 	
