@@ -97,11 +97,15 @@ namespace MonoDevelop.CSharp.Project
 			iconEntry.Path = projectParameters.Win32Icon;
 			iconEntry.DefaultPath = project.BaseDirectory;
 			allowUnsafeCodeCheckButton.Active = compilerParameters.UnsafeCode;
-			
+			noStdLibCheckButton.Active = compilerParameters.NoStdLib;
+
 			ListStore langVerStore = new ListStore (typeof (string));
 			langVerStore.AppendValues (GettextCatalog.GetString ("Default"));
 			langVerStore.AppendValues ("ISO-1");
 			langVerStore.AppendValues ("ISO-2");
+			langVerStore.AppendValues ("Version 3");
+			langVerStore.AppendValues ("Version 4");
+			langVerStore.AppendValues ("Version 5");
 			langVerCombo.Model = langVerStore;
 			langVerCombo.Active = (int) compilerParameters.LangVersion;
 		}
@@ -180,6 +184,7 @@ namespace MonoDevelop.CSharp.Project
 			foreach (DotNetProjectConfiguration configuration in configs) {
 				CSharpCompilerParameters compilerParameters = (CSharpCompilerParameters) configuration.CompilationParameters; 
 				compilerParameters.UnsafeCode = allowUnsafeCodeCheckButton.Active;
+				compilerParameters.NoStdLib = noStdLibCheckButton.Active;
 				compilerParameters.LangVersion = langVersion;
 			}
 		}
