@@ -5,6 +5,8 @@ namespace MonoDevelop.Deployment.Linux
 {
 	
 	
+	[System.ComponentModel.Category("widget")]
+	[System.ComponentModel.ToolboxItem(true)]
 	public partial class DesktopPanelWidget : Gtk.Bin
 	{
 		
