@@ -17,9 +17,12 @@ namespace MonoDevelop.Autotools
 			get { return string.Empty; }
 		}
 
-		public bool SupportsSolutionItem (SolutionFolder parentCombine, SolutionItem entry)
+		public FeatureSupportLevel GetSupportLevel (SolutionFolder parentCombine, SolutionItem entry)
 		{
-			return entry is Project;
+			if (entry is Project)
+				return FeatureSupportLevel.SupportedByDefault;
+			else
+				return FeatureSupportLevel.NotSupported;
 		}
 		
 		public Widget CreateFeatureEditor (SolutionFolder parentCombine, SolutionItem entry)
@@ -36,10 +39,5 @@ namespace MonoDevelop.Autotools
 		{
 			return null;
 		}
-		
-		public bool IsEnabled (SolutionFolder parentCombine, SolutionItem entry) 
-		{
-			return false;
-		}
 	}
 }
