@@ -1,12 +1,7 @@
-using System;
-using System.IO;
-
-using MonoDevelop.Core;
-using MonoDevelop.Projects.Gui.Dialogs;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Projects;
 
 using Gtk;
+using MonoDevelop.Ide.Gui.Dialogs;
+using MonoDevelop.Projects;
 
 namespace MonoDevelop.Autotools
 {
@@ -27,7 +22,7 @@ namespace MonoDevelop.Autotools
 			if (data != null) {
 				tmpData = (MakefileData) data.Clone ();
 			}
-			return (widget = new MakefileOptionPanelWidget (project, tmpData));
+			return (widget = new MakefileOptionPanelWidget (ParentDialog, project, tmpData));
 		}
 		
 		public override bool ValidateChanges ()
