@@ -99,7 +99,10 @@ namespace MonoDevelop.Autotools
 				
 				IMakefileHandler handler = AutotoolsContext.GetMakefileHandler (solution.RootFolder, mt);
 				if (handler == null)
-					throw new Exception ( GettextCatalog.GetString ("MonoDevelop does not currently support generating {0} for one (or more) child projects.", filesString) );
+					throw new Exception (string.Format (
+						"{0} does not currently support generating {1} for one (or more) child projects.",
+						filesString, BrandingService.ApplicationName
+					));
 
 				solution_name = solution.Name;
 				solution_version = AutotoolsContext.EscapeStringForAutoconf (solution.Version, true);
@@ -212,7 +215,7 @@ namespace MonoDevelop.Autotools
 			{
 				monitor.EndTask ();
 			}
-			monitor.ReportSuccess ( GettextCatalog.GetString ( "Solution was succesfully deployed" ) );
+			monitor.ReportSuccess (GettextCatalog.GetString ("Solution was successfully deployed."));
 			return true;
 		}
 
@@ -342,6 +345,7 @@ AM_CONDITIONAL(ENABLE_{3}, test x$enable_{2} = xyes)",
 				if (PlatformID.Unix != Environment.OSVersion.Platform)
 					tmpmf = tmpmf.Replace("\\","/");
 
+				AutotoolsContext.CheckSpaces (tmpmf);
 				configFiles.Append(FileService.NormalizeRelativePath (tmpmf));
 				configFiles.Append("\n");
 			}
