@@ -39,9 +39,9 @@ namespace MonoDevelop.Debugger.Soft
 {
 	public class SoftDebuggerEngine: IDebuggerEngine
 	{
-		public SoftDebuggerEngine ()
+		static SoftDebuggerEngine ()
 		{
-			EnsureSdbLoggingService ();
+			Mono.Debugging.Soft.LoggingService.CustomLogger = new MDLogger ();
 		}
 		
 		public bool CanDebugCommand (ExecutionCommand cmd)
@@ -72,15 +72,14 @@ namespace MonoDevelop.Debugger.Soft
 				WorkingDirectory = cmd.WorkingDirectory,
 			};
 			
-			string error;
-			dsi.UserAssemblyNames = GetAssemblyNames (cmd.UserAssemblyPaths, out error);
-			dsi.LogMessage = error;
+			SetUserAssemblyNames (dsi, cmd.UserAssemblyPaths);
 			
 			foreach (KeyValuePair<string,string> var in cmd.EnvironmentVariables)
 				dsi.EnvironmentVariables [var.Key] = var.Value;
 			
 			var varsCopy = new Dictionary<string, string> (cmd.EnvironmentVariables);
-			dsi.ExternalConsoleLauncher = delegate (System.Diagnostics.ProcessStartInfo info) {
+			var startArgs = (SoftDebuggerLaunchArgs) dsi.StartArgs;
+			startArgs.ExternalConsoleLauncher = delegate (System.Diagnostics.ProcessStartInfo info) {
 				IProcessAsyncOperation oper;
 				oper = Runtime.ProcessService.StartConsoleProcess (info.FileName, info.Arguments, info.WorkingDirectory,
 					varsCopy, ExternalConsoleFactory.Instance.CreateConsole (dsi.CloseExternalConsoleOnExit), null);
@@ -100,51 +99,40 @@ namespace MonoDevelop.Debugger.Soft
 			return new SoftDebuggerSession ();
 		}
 		
-		public DebuggerFeatures SupportedFeatures {
-			get {
-				return DebuggerFeatures.Breakpoints | 
-					   DebuggerFeatures.Pause | 
-					   DebuggerFeatures.Stepping | 
-					   DebuggerFeatures.DebugFile |
-					   DebuggerFeatures.ConditionalBreakpoints |
-					   DebuggerFeatures.Tracepoints |
-					   DebuggerFeatures.Catchpoints;
-			}
-		}
-		
-		public static List<AssemblyName> GetAssemblyNames (IList<string> files, out string error)
+		public static void SetUserAssemblyNames (SoftDebuggerStartInfo dsi, IList<string> files)
 		{
-			error = null;
 			if (files == null || files.Count == 0)
-				return null;
+				return;
 			
+			var pathMap = new Dictionary<string, string> ();
 			var names = new List<AssemblyName> ();
+			
 			foreach (var file in files) {
 				if (!File.Exists (file)) {
-					error = GettextCatalog.GetString ("User assembly '{0}' is missing. " +
+					dsi.LogMessage = GettextCatalog.GetString ("User assembly '{0}' is missing. " +
 						"Debugger will now debug all code, not just user code.", file);
-					return null;
+					return;
 				}
+				
 				try {
-					var asm = Mono.Cecil.AssemblyFactory.GetAssemblyManifest (file);
+					var asm = Mono.Cecil.AssemblyDefinition.ReadAssembly (file);
 					if (string.IsNullOrEmpty (asm.Name.Name))
 						throw new InvalidOperationException ("Assembly has no assembly name");
-					names.Add (new AssemblyName (asm.Name.FullName));
+					
+					AssemblyName name = new AssemblyName (asm.Name.FullName);
+					if (!pathMap.ContainsKey (asm.Name.FullName))
+						pathMap.Add (asm.Name.FullName, file);
+					names.Add (name);
 				} catch (Exception ex) {
-					error = GettextCatalog.GetString ("Could not get assembly name for user assembly '{0}'. " +
+					dsi.LogMessage = GettextCatalog.GetString ("Could not get assembly name for user assembly '{0}'. " +
 						"Debugger will now debug all code, not just user code.", file);
 					MDLS.LogError ("Error getting assembly name for user assembly '" + file + "'", ex);
-					return null;
+					return;
 				}
 			}
-			return names;
-		}
-		
-		static ICustomLogger logger;
-		internal static void EnsureSdbLoggingService ()
-		{
-			if (logger == null)
-				Mono.Debugging.Soft.LoggingService.CustomLogger = logger = new MDLogger ();
+			
+			dsi.UserAssemblyNames = names;
+			dsi.AssemblyPathMap = pathMap;
 		}
 		
 		class MDLogger : ICustomLogger
