@@ -37,16 +37,22 @@ namespace MonoDevelop.Debugger
 	public class ImmediatePad: IPadContent
 	{
 		ConsoleView view;
+		bool disposed;
 		
 		public void Initialize (IPadWindow container)
 		{
 			view = new ConsoleView ();
 			view.ConsoleInput += OnViewConsoleInput;
-			Pango.FontDescription font = Pango.FontDescription.FromString (DesktopService.DefaultMonospaceFont);
-			font.Size = (font.Size * 8) / 10;
-			view.SetFont (font);
+			view.SetFont (IdeApp.Preferences.CustomOutputPadFont);
 			view.ShadowType = Gtk.ShadowType.None;
 			view.ShowAll ();
+
+			IdeApp.Preferences.CustomOutputPadFontChanged += HandleCustomOutputPadFontChanged;
+		}
+
+		void HandleCustomOutputPadFontChanged (object sender, EventArgs e)
+		{
+			view.SetFont (IdeApp.Preferences.CustomOutputPadFont);
 		}
 
 		void OnViewConsoleInput (object sender, ConsoleInputEventArgs e)
@@ -57,24 +63,28 @@ namespace MonoDevelop.Debugger
 				view.WriteOutput ("The expression can't be evaluated while the application is running.");
 			} else {
 				EvaluationOptions ops = EvaluationOptions.DefaultOptions;
+				var frame = DebuggingService.CurrentFrame;
+				string expression = e.Text;
+
 				ops.AllowMethodEvaluation = true;
 				ops.AllowToStringCalls = true;
 				ops.AllowTargetInvoke = true;
 				ops.EvaluationTimeout = 20000;
 				ops.EllipsizeStrings = false;
-				var ff = DebuggingService.CurrentFrame;
-				string tt = e.Text;
-				ValidationResult vres = ff.ValidateExpression (tt, ops);
+
+				var vres = frame.ValidateExpression (expression, ops);
 				if (!vres) {
 					view.WriteOutput (vres.Message);
 					view.Prompt (true);
 					return;
 				}
-				ObjectValue val = DebuggingService.CurrentFrame.GetExpressionValue (e.Text, ops);
+
+				var val = frame.GetExpressionValue (expression, ops);
 				if (val.IsEvaluating) {
 					WaitForCompleted (val);
 					return;
 				}
+
 				PrintValue (val);
 			}
 			view.Prompt (true);
@@ -131,6 +141,10 @@ namespace MonoDevelop.Debugger
 		
 		public void Dispose ()
 		{
+			if (!disposed) {
+				IdeApp.Preferences.CustomOutputPadFontChanged -= HandleCustomOutputPadFontChanged;
+				disposed = true;
+			}
 		}
 	}
 }
