@@ -26,8 +26,10 @@
 //
 
 using System;
-using System.Collections.Generic;
 using System.Text;
+using System.Collections.Generic;
+using System.Threading.Tasks;
+
 using Gtk;
 using Mono.Debugging.Client;
 using MonoDevelop.Components;
@@ -50,12 +52,11 @@ namespace MonoDevelop.Debugger
 		List<ObjectValue> values = new List<ObjectValue> ();
 		Dictionary<ObjectValue,TreeRowReference> nodes = new Dictionary<ObjectValue, TreeRowReference> ();
 		Dictionary<string,ObjectValue> cachedValues = new Dictionary<string,ObjectValue> ();
+		Dictionary<ObjectValue, Task> expandTasks = new Dictionary<ObjectValue, Task> ();
 		TreeStore store;
 		TreeViewState state;
 		string createMsg;
-		bool allowAdding;
-		bool allowEditing;
-		bool allowExpanding = true;
+		bool restoringState = false;
 		bool compact;
 		StackFrame frame;
 		bool disposed;
@@ -94,16 +95,15 @@ namespace MonoDevelop.Debugger
 		const int ValueCol = 1;
 		const int TypeCol = 2;
 		const int ObjectCol = 3;
-		const int ExpandedCol = 4;
-		const int NameEditableCol = 5;
-		const int ValueEditableCol = 6;
-		const int IconCol = 7;
-		const int NameColorCol = 8;
-		const int ValueColorCol = 9;
-		const int ValueButtonVisibleCol = 10;
-		const int PinIconCol = 11;
-		const int LiveUpdateIconCol = 12;
-		const int ViewerButtonVisibleCol = 13;
+		const int NameEditableCol = 4;
+		const int ValueEditableCol = 5;
+		const int IconCol = 6;
+		const int NameColorCol = 7;
+		const int ValueColorCol = 8;
+		const int ValueButtonVisibleCol = 9;
+		const int PinIconCol = 10;
+		const int LiveUpdateIconCol = 11;
+		const int ViewerButtonVisibleCol = 12;
 		
 		public event EventHandler StartEditing;
 		public event EventHandler EndEditing;
@@ -128,9 +128,10 @@ namespace MonoDevelop.Debugger
 		
 		public ObjectValueTreeView ()
 		{
-			store = new TreeStore (typeof(string), typeof(string), typeof(string), typeof(ObjectValue), typeof(bool), typeof(bool), typeof(bool), typeof(string), typeof(string), typeof(string), typeof(bool), typeof(string), typeof(Gdk.Pixbuf), typeof(bool));
+			store = new TreeStore (typeof(string), typeof(string), typeof(string), typeof(ObjectValue), typeof(bool), typeof(bool), typeof(string), typeof(string), typeof(string), typeof(bool), typeof(string), typeof(Gdk.Pixbuf), typeof(bool));
 			Model = store;
 			RulesHint = true;
+			EnableSearch = false;
 			Selection.Mode = Gtk.SelectionMode.Multiple;
 			ResetColumnSizes ();
 			
@@ -233,7 +234,13 @@ namespace MonoDevelop.Debugger
 			crtValue.EditingStarted -= OnValueEditing;
 			crtValue.Edited -= OnValueEdited;
 			crtValue.EditingCanceled -= OnEditingCancelled;
-			
+
+			if (expandTasks.Count > 0) {
+				var tasks = new Task[expandTasks.Count];
+				expandTasks.Values.CopyTo (tasks, 0);
+				Task.WaitAll (tasks);
+			}
+
 			base.OnDestroyed ();
 			disposed = true;
 		}
@@ -249,39 +256,42 @@ namespace MonoDevelop.Debugger
 			base.OnShown ();
 			AdjustColumnSizes ();
 		}
-		
-		
+
+		protected override void OnRealized ()
+		{
+			base.OnRealized ();
+			AdjustColumnSizes ();
+		}
+
 		void OnColumnWidthChanged (object o, GLib.NotifyArgs args)
 		{
 			if (!columnSizesUpdating && allowStoreColumnSizes) {
 				StoreColumnSizes ();
 			}
 		}
-		
-		
+
 		void AdjustColumnSizes ()
 		{
-			if (!IsRealized || !Visible || Allocation.Width == 0 || columnSizesUpdating || compact)
+			if (!Visible || Allocation.Width <= 0 || columnSizesUpdating || compact)
 				return;
 			
 			columnSizesUpdating = true;
 			
 			double width = (double) Allocation.Width;
 			
-			int texp = (int) (width * expColWidth);
+			int texp = Math.Max ((int) (width * expColWidth), 1);
 			if (texp != expCol.FixedWidth) {
 				expCol.FixedWidth = texp;
 			}
 			
-			int ttype = 0;
 			if (typeCol.Visible) {
-				ttype = (int) (width * typeColWidth);
+				int ttype = Math.Max ((int) (width * typeColWidth), 1);
 				if (ttype != typeCol.FixedWidth) {
 					typeCol.FixedWidth = ttype;
 				}
 			}
 			
-			int tval = (int) (width * valueColWidth);
+			int tval = Math.Max ((int) (width * valueColWidth), 1);
 
 			if (tval != valueCol.FixedWidth) {
 				valueCol.FixedWidth = tval;
@@ -328,9 +338,12 @@ namespace MonoDevelop.Debugger
 		
 		public void LoadState ()
 		{
+			restoringState = true;
 			state.Load ();
+			restoringState = false;
 		}
-		
+
+		bool allowAdding;
 		public bool AllowAdding {
 			get {
 				return allowAdding;
@@ -340,7 +353,8 @@ namespace MonoDevelop.Debugger
 				Refresh ();
 			}
 		}
-		
+
+		bool allowEditing;
 		public bool AllowEditing {
 			get {
 				return allowEditing;
@@ -357,12 +371,18 @@ namespace MonoDevelop.Debugger
 		}
 		
 		public bool RootPinAlwaysVisible { get; set; }
-		
+
+		bool allowExpanding = true;
 		public bool AllowExpanding {
-			get { return this.allowExpanding; }
-			set { this.allowExpanding = value; }
+			get { return allowExpanding; }
+			set { allowExpanding = value; }
+		}
+
+		bool CanQueryDebugger {
+			get {
+				return DebuggingService.IsConnected && DebuggingService.IsPaused;
+			}
 		}
-		
 		
 		public PinnedWatch PinnedWatch { get; set; }
 		
@@ -437,6 +457,16 @@ namespace MonoDevelop.Debugger
 			values.Remove (value);
 			Refresh ();
 		}
+
+		public void ReplaceValue (ObjectValue old, ObjectValue @new)
+		{
+			int idx = values.IndexOf (old);
+			if (idx == -1)
+				return;
+
+			values [idx] = @new;
+			Refresh ();
+		}
 		
 		public void ClearValues ()
 		{
@@ -465,9 +495,12 @@ namespace MonoDevelop.Debugger
 			foreach (ObjectValue val in new List<ObjectValue> (nodes.Keys))
 				UnregisterValue (val);
 			nodes.Clear ();
-			
-			state.Save ();
-			
+
+			if (IsRealized)
+				ScrollToPoint (0, 0);
+
+			SaveState ();
+
 			CleanPinIcon ();
 			store.Clear ();
 			
@@ -487,58 +520,62 @@ namespace MonoDevelop.Debugger
 						showExpanders = true;
 				}
 			}
-			
-			ShowExpanders = showExpanders;
+
+			if (showExpanders)
+				ShowExpanders = true;
 			
 			if (AllowAdding)
 				store.AppendValues (createMsg, "", "", null, true, true, null, disabledColor, disabledColor);
-			
-			state.Load ();
+
+			LoadState ();
 		}
 		
-		void RefreshRow (TreeIter it)
+		void RefreshRow (TreeIter iter)
 		{
-			ObjectValue val = (ObjectValue) store.GetValue (it, ObjectCol);
+			ObjectValue val = (ObjectValue) store.GetValue (iter, ObjectCol);
 			UnregisterValue (val);
 			
-			RemoveChildren (it);
+			RemoveChildren (iter);
 			TreeIter parent;
-			if (!store.IterParent (out parent, it))
+			if (!store.IterParent (out parent, iter))
 				parent = TreeIter.Zero;
-			
-			EvaluationOptions ops = frame.DebuggerSession.Options.EvaluationOptions.Clone ();
-			ops.AllowMethodEvaluation = true;
-			ops.AllowToStringCalls = true;
-			ops.AllowTargetInvoke = true;
-			ops.EllipsizeStrings = false;
-			
-			string oldName = val.Name;
-			val.Refresh (ops);
-			
-			// Don't update the name for the values entered by the user
-			if (store.IterDepth (it) == 0)
-				val.Name = oldName;
-			
-			SetValues (parent, it, val.Name, val);
-			RegisterValue (val, it);
+
+			if (CanQueryDebugger && frame != null) {
+				EvaluationOptions ops = frame.DebuggerSession.Options.EvaluationOptions.Clone ();
+				ops.AllowMethodEvaluation = true;
+				ops.AllowToStringCalls = true;
+				ops.AllowTargetInvoke = true;
+				ops.EllipsizeStrings = false;
+
+				string oldName = val.Name;
+				val.Refresh (ops);
+
+				// Don't update the name for the values entered by the user
+				if (store.IterDepth (iter) == 0)
+					val.Name = oldName;
+			}
+
+			SetValues (parent, iter, val.Name, val);
+			RegisterValue (val, iter);
 		}
 		
-		void RemoveChildren (TreeIter it)
+		void RemoveChildren (TreeIter iter)
 		{
-			TreeIter cit;
-			while (store.IterChildren (out cit, it)) {
-				ObjectValue val = (ObjectValue) store.GetValue (cit, ObjectCol);
+			TreeIter citer;
+
+			while (store.IterChildren (out citer, iter)) {
+				ObjectValue val = (ObjectValue) store.GetValue (citer, ObjectCol);
 				if (val != null)
 					UnregisterValue (val);
-				RemoveChildren (cit);
-				store.Remove (ref cit);
+				RemoveChildren (citer);
+				store.Remove (ref citer);
 			}
 		}
 
-		void RegisterValue (ObjectValue val, TreeIter it)
+		void RegisterValue (ObjectValue val, TreeIter iter)
 		{
 			if (val.IsEvaluating) {
-				nodes [val] = new TreeRowReference (store, store.GetPath (it));
+				nodes [val] = new TreeRowReference (store, store.GetPath (iter));
 				val.ValueChanged += OnValueUpdated;
 			}
 		}
@@ -591,7 +628,7 @@ namespace MonoDevelop.Debugger
 		{
 			TreeRowReference row;
 			
-			if (!nodes.TryGetValue (val, out row)) {
+			if (!nodes.TryGetValue (val, out row) || !row.Valid ()) {
 				it = TreeIter.Zero;
 				return false;
 			}
@@ -661,9 +698,14 @@ namespace MonoDevelop.Debugger
 			oldValues.TryGetValue (valPath, out oldValue);
 			
 			if (val.IsUnknown) {
-				strval = GettextCatalog.GetString ("The name '{0}' does not exist in the current context.", val.Name);
-				nameColor = disabledColor;
-				canEdit = false;
+				if (frame != null) {
+					strval = GettextCatalog.GetString ("The name '{0}' does not exist in the current context.", val.Name);
+					nameColor = disabledColor;
+					canEdit = false;
+				} else {
+					canEdit = !val.IsReadOnly;
+					strval = string.Empty;
+				}
 			}
 			else if (val.IsError) {
 				strval = val.Value;
@@ -690,7 +732,7 @@ namespace MonoDevelop.Debugger
 				canEdit = false;
 			}
 			else {
-				canEdit = val.IsPrimitive && !val.IsReadOnly && allowEditing;
+				canEdit = val.IsPrimitive && !val.IsReadOnly;
 				strval = val.DisplayValue ?? "(null)";
 				if (oldValue != null && strval != oldValue)
 					nameColor = valueColor = modifiedColor;
@@ -699,16 +741,16 @@ namespace MonoDevelop.Debugger
 			strval = strval.Replace (Environment.NewLine, " ");
 			
 			bool showViewerButton = DebuggingService.HasValueVisualizers (val);
-			
+
+			bool hasChildren = val.HasChildren;
 			string icon = GetIcon (val.Flags);
 
 			store.SetValue (it, NameCol, name);
 			store.SetValue (it, ValueCol, strval);
 			store.SetValue (it, TypeCol, val.TypeName);
 			store.SetValue (it, ObjectCol, val);
-			store.SetValue (it, ExpandedCol, !val.HasChildren);
-			store.SetValue (it, NameEditableCol, !hasParent && allowAdding);
-			store.SetValue (it, ValueEditableCol, canEdit);
+			store.SetValue (it, NameEditableCol, !hasParent && AllowAdding);
+			store.SetValue (it, ValueEditableCol, canEdit && AllowEditing);
 			store.SetValue (it, IconCol, icon);
 			store.SetValue (it, NameColorCol, nameColor);
 			store.SetValue (it, ValueColorCol, valueColor);
@@ -725,9 +767,9 @@ namespace MonoDevelop.Debugger
 			if (RootPinAlwaysVisible && (!hasParent && PinnedWatch ==null && AllowPinning))
 				store.SetValue (it, PinIconCol, "md-pin-up");
 			
-			if (val.HasChildren) {
+			if (hasChildren) {
 				// Add dummy node
-				it = store.AppendValues (it, "", "", "", null, true);
+				store.AppendValues (it, GettextCatalog.GetString ("Loading..."), "", "", null, true);
 				if (!ShowExpanders)
 					ShowExpanders = true;
 			}
@@ -742,20 +784,23 @@ namespace MonoDevelop.Debugger
 			string stic = (flags & ObjectValueFlags.Global) != 0 ? "static-" : string.Empty;
 			
 			switch (flags & ObjectValueFlags.OriginMask) {
-				case ObjectValueFlags.Property: source = "property"; break;
-				case ObjectValueFlags.Type: source = "class"; stic = string.Empty; break;
-				case ObjectValueFlags.Literal: return "md-literal";
-				case ObjectValueFlags.Namespace: return "md-name-space";
-				case ObjectValueFlags.Group: return "md-open-resource-folder";
-				default: source = "field"; break;
+			case ObjectValueFlags.Property: source = "property"; break;
+			case ObjectValueFlags.Type: source = "class"; stic = string.Empty; break;
+			case ObjectValueFlags.Literal: return "md-literal";
+			case ObjectValueFlags.Namespace: return "md-name-space";
+			case ObjectValueFlags.Group: return "md-open-resource-folder";
+			case ObjectValueFlags.Field: source = "field"; break;
+			case ObjectValueFlags.Variable: return "md-variable";
+			default: return "md-empty";
 			}
+
 			string access;
 			switch (flags & ObjectValueFlags.AccessMask) {
-				case ObjectValueFlags.Private: access = "private-"; break;
-				case ObjectValueFlags.Internal: access = "internal-"; break;
-				case ObjectValueFlags.InternalProtected:
-				case ObjectValueFlags.Protected: access = "protected-"; break;
-				default: access = string.Empty; break;
+			case ObjectValueFlags.Private: access = "private-"; break;
+			case ObjectValueFlags.Internal: access = "internal-"; break;
+			case ObjectValueFlags.InternalProtected:
+			case ObjectValueFlags.Protected: access = "protected-"; break;
+			default: access = string.Empty; break;
 			}
 			
 			return "md-" + access + stic + source;
@@ -763,16 +808,18 @@ namespace MonoDevelop.Debugger
 		
 		protected override bool OnTestExpandRow (TreeIter iter, TreePath path)
 		{
-			if (!allowExpanding)
-				return true;
-			
-			if (GetRowExpanded (path))
-				return true;
-			
-			TreeIter parent;
-			if (store.IterParent (out parent, iter)) {
-				if (!GetRowExpanded (store.GetPath (parent)))
+			if (!restoringState) {
+				if (!allowExpanding)
 					return true;
+
+				if (GetRowExpanded (path))
+					return true;
+
+				TreeIter parent;
+				if (store.IterParent (out parent, iter)) {
+					if (!GetRowExpanded (store.GetPath (parent)))
+						return true;
+				}
 			}
 			
 			return base.OnTestExpandRow (iter, path);
@@ -780,37 +827,71 @@ namespace MonoDevelop.Debugger
 		
 		protected override void OnRowCollapsed (TreeIter iter, TreePath path)
 		{
-			store.SetValue (iter, ExpandedCol, false);
 			base.OnRowCollapsed (iter, path);
+
 			if (compact)
 				ColumnsAutosize ();
+
+			ScrollToCell (path, expCol, true, 0f, 0f);
+		}
+
+		static Task<ObjectValue[]> GetChildrenAsync (ObjectValue value)
+		{
+			return Task.Factory.StartNew<ObjectValue[]> (delegate (object arg) {
+				try {
+					return ((ObjectValue) arg).GetAllChildren ();
+				} catch (Exception ex) {
+					// Note: this should only happen if someone breaks ObjectValue.GetAllChildren()
+					LoggingService.LogError ("Failed to get ObjectValue children.", ex);
+					return new ObjectValue[0];
+				}
+			}, value);
+		}
+
+		void AddChildrenAsync (ObjectValue value, TreePathReference row)
+		{
+			Task task;
+
+			if (expandTasks.TryGetValue (value, out task))
+				return;
+
+			task = GetChildrenAsync (value).ContinueWith (t => {
+				TreeIter iter, it;
+
+				if (row.IsValid && store.GetIter (out iter, row.Path) && store.IterChildren (out it, iter)) {
+					foreach (var child in t.Result) {
+						SetValues (iter, it, null, child);
+						RegisterValue (child, it);
+						it = store.InsertNodeAfter (it);
+					}
+
+					store.Remove (ref it);
+
+					if (compact)
+						ColumnsAutosize ();
+				}
+
+				expandTasks.Remove (value);
+				row.Dispose ();
+			}, Xwt.Application.UITaskScheduler);
+			expandTasks.Add (value, task);
 		}
 		
 		protected override void OnRowExpanded (TreeIter iter, TreePath path)
 		{
-			store.SetValue (iter, ExpandedCol, true);
 			TreeIter it;
 			
 			if (store.IterChildren (out it, iter)) {
-				ObjectValue val = (ObjectValue) store.GetValue (it, ObjectCol);
-				if (val == null) {
-					val = (ObjectValue) store.GetValue (iter, ObjectCol);
-					bool first = true;
-					
-					foreach (ObjectValue cval in val.GetAllChildren ()) {
-						SetValues (iter, it, null, cval);
-						RegisterValue (cval, it);
-						it = store.InsertNodeAfter (it);
-					}
-					
-					store.Remove (ref it);
+				ObjectValue value = (ObjectValue) store.GetValue (it, ObjectCol);
+				if (value == null) {
+					value = (ObjectValue) store.GetValue (iter, ObjectCol);
+					AddChildrenAsync (value, new TreePathReference (store, store.GetPath (iter)));
 				}
 			}
 			
 			base.OnRowExpanded (iter, path);
-			
-			if (compact)
-				ColumnsAutosize ();
+
+			ScrollToCell (path, expCol, true, 0f, 0f);
 		}
 		
 		string GetIterPath (TreeIter iter)
@@ -853,6 +934,9 @@ namespace MonoDevelop.Debugger
 					return;
 				
 				int i = valueNames.IndexOf (exp);
+				if (i == -1)
+					return;
+
 				if (args.NewText.Length != 0)
 					valueNames [i] = args.NewText;
 				else
@@ -928,41 +1012,66 @@ namespace MonoDevelop.Debugger
 			editing = true;
 			editEntry = (Gtk.Entry) args.Editable;
 			editEntry.KeyPressEvent += OnEditKeyPress;
+			editEntry.KeyReleaseEvent += HandleChanged;
 			if (StartEditing != null)
 				StartEditing (this, EventArgs.Empty);
 		}
-		
+
+		void HandleChanged (object sender, EventArgs e)
+		{
+			if (!wasHandled) {
+				string text = ctx == null ? editEntry.Text : editEntry.Text.Substring (Math.Max (0, Math.Min (ctx.TriggerOffset, editEntry.Text.Length)));
+				CompletionWindowManager.UpdateWordSelection (text);
+				CompletionWindowManager.PostProcessKeyEvent (key, keyChar, modifierState);
+				PopupCompletion ((Entry) sender);
+			}
+		}
+
 		void OnEndEditing ()
 		{
 			editing = false;
 			editEntry.KeyPressEvent -= OnEditKeyPress;
+			editEntry.KeyReleaseEvent -= HandleChanged;
+
 			CompletionWindowManager.HideWindow ();
 			currentCompletionData = null;
 			if (EndEditing != null)
 				EndEditing (this, EventArgs.Empty);
 		}
-		
+		bool wasHandled = false;
+		CodeCompletionContext ctx;
+		Gdk.Key key;
+		char keyChar;
+		Gdk.ModifierType modifierState;
+		uint keyValue;
+
 		[GLib.ConnectBeforeAttribute]
 		void OnEditKeyPress (object s, Gtk.KeyPressEventArgs args)
 		{
-			Gtk.Entry entry = (Gtk.Entry)s;
-			
+			wasHandled = false;
+			key = args.Event.Key;
+			keyChar =  (char)args.Event.Key;
+			modifierState = args.Event.State;
+			keyValue = args.Event.KeyValue;
 			if (currentCompletionData != null) {
-				bool ret = CompletionWindowManager.PreProcessKeyEvent (args.Event.Key, (char)args.Event.Key, args.Event.State);
-				CompletionWindowManager.PostProcessKeyEvent (args.Event.Key, (char)args.Event.Key, args.Event.State);
-				args.RetVal = ret;
+				wasHandled  = CompletionWindowManager.PreProcessKeyEvent (key, keyChar, modifierState);
+				args.RetVal = wasHandled ;
 			}
-			
+		}
+
+		void PopupCompletion (Entry entry)
+		{
 			Gtk.Application.Invoke (delegate {
-				char c = (char)Gdk.Keyval.ToUnicode (args.Event.KeyValue);
+				char c = (char)Gdk.Keyval.ToUnicode (keyValue);
 				if (currentCompletionData == null && IsCompletionChar (c)) {
 					string exp = entry.Text.Substring (0, entry.CursorPosition);
 					currentCompletionData = GetCompletionData (exp);
 					if (currentCompletionData != null) {
 						DebugCompletionDataList dataList = new DebugCompletionDataList (currentCompletionData);
-						CodeCompletionContext ctx = ((ICompletionWidget)this).CreateCodeCompletionContext (entry.CursorPosition - currentCompletionData.ExpressionLenght);
+						ctx = ((ICompletionWidget)this).CreateCodeCompletionContext (entry.CursorPosition - currentCompletionData.ExpressionLength);
 						CompletionWindowManager.ShowWindow (null, c, dataList, this, ctx);
-					} else
+					}
+					else
 						currentCompletionData = null;
 				}
 			});
@@ -1002,30 +1111,112 @@ namespace MonoDevelop.Debugger
 				CleanPinIcon ();
 			return base.OnLeaveNotifyEvent (evnt);
 		}
+		
+		protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
+		{
+			// Ignore if editing a cell
+			if (editing)
+				return base.OnKeyPressEvent (evnt);
 
+			TreePath[] selected = Selection.GetSelectedRows ();
+			bool changed = false;
+			TreePath lastPath;
+
+			if (selected == null || selected.Length < 1)
+				return base.OnKeyPressEvent (evnt);
+
+			switch (evnt.Key) {
+			case Gdk.Key.Left:
+			case Gdk.Key.KP_Left:
+				foreach (var path in selected) {
+					lastPath = path.Copy ();
+					if (GetRowExpanded (path)) {
+						CollapseRow (path);
+						changed = true;
+					} else if (path.Up ()) {
+						Selection.UnselectPath (lastPath);
+						Selection.SelectPath (path);
+						changed = true;
+					}
+				}
+				break;
+			case Gdk.Key.Right:
+			case Gdk.Key.KP_Right:
+				foreach (var path in selected) {
+					if (!GetRowExpanded (path)) {
+						ExpandRow (path, false);
+						changed = true;
+					} else {
+						lastPath = path.Copy ();
+						path.Down ();
+						if (lastPath.Compare (path) != 0) {
+							Selection.UnselectPath (lastPath);
+							Selection.SelectPath (path);
+							changed = true;
+						}
+					}
+				}
+				break;
+			case Gdk.Key.Delete:
+			case Gdk.Key.KP_Delete:
+			case Gdk.Key.BackSpace:
+				string expression;
+				ObjectValue val;
+				TreeIter iter;
+
+				if (!AllowEditing || !AllowAdding)
+					return base.OnKeyPressEvent (evnt);
+
+				// Note: since we'll be modifying the tree, we need to make changes from bottom to top
+				Array.Sort (selected, new TreePathComparer (true));
+
+				foreach (var path in selected) {
+					if (!Model.GetIter (out iter, path))
+						continue;
+
+					val = (ObjectValue)store.GetValue (iter, ObjectCol);
+					expression = GetFullExpression (iter);
+
+					// Lookup and remove
+					if (val != null && values.Contains (val)) {
+						RemoveValue (val);
+						changed = true;
+					} else if (!string.IsNullOrEmpty (expression) && valueNames.Contains (expression)) {
+						RemoveExpression (expression);
+						changed = true;
+					}
+				}
+				break;
+			}
+
+			if (changed)
+				return true;
+
+			return base.OnKeyPressEvent (evnt);
+		}
 
 		protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
 		{
 			allowStoreColumnSizes = true;
-			bool res = base.OnButtonPressEvent (evnt);
-			TreePath path;
-			TreeViewColumn col;
-			CellRenderer cr;
+			bool retval = base.OnButtonPressEvent (evnt);
 			
 			//HACK: show context menu in release event instead of show event to work around gtk bug
 			if (evnt.TriggersContextMenu ()) {
 			//	ShowPopup (evnt);
 				return true;
 			}
+
+			TreeViewColumn col;
+			CellRenderer cr;
+			TreePath path;
 			
-			if (evnt.Button == 1 && GetCellAtPos ((int)evnt.X, (int)evnt.Y, out path, out col, out cr)) {
+			if (CanQueryDebugger && evnt.Button == 1 && GetCellAtPos ((int)evnt.X, (int)evnt.Y, out path, out col, out cr)) {
 				TreeIter it;
 				store.GetIter (out it, path);
 				if (cr == crpViewer) {
 					ObjectValue val = (ObjectValue) store.GetValue (it, ObjectCol);
 					DebuggingService.ShowValueVisualizer (val);
-				}
-				else if (!editing) {
+				} else if (!editing) {
 					if (cr == crpButton) {
 						RefreshRow (it);
 					} else if (cr == crpPin) {
@@ -1047,7 +1238,7 @@ namespace MonoDevelop.Debugger
 				}
 			}
 			
-			return res;
+			return retval;
 		}
 		
 		protected override bool OnButtonReleaseEvent (Gdk.EventButton evnt)
@@ -1073,6 +1264,20 @@ namespace MonoDevelop.Debugger
 		{
 			IdeApp.CommandService.ShowContextMenu (this, evt, menuSet, this);
 		}
+
+		[CommandUpdateHandler (EditCommands.SelectAll)]
+		protected void UpdateSelectAll (CommandInfo cmd)
+		{
+			TreeIter iter;
+
+			cmd.Enabled = store.GetIterFirst (out iter);
+		}
+
+		[CommandHandler (EditCommands.SelectAll)]
+		protected void OnSelectAll ()
+		{
+			Selection.SelectAll ();
+		}
 		
 		[CommandHandler (EditCommands.Copy)]
 		protected void OnCopy ()
@@ -1080,21 +1285,57 @@ namespace MonoDevelop.Debugger
 			TreePath[] selected = Selection.GetSelectedRows ();
 			TreeIter iter;
 			
-			if (selected == null || selected.Length != 1)
-				return;
-			
-			if (!store.GetIter (out iter, selected[0]))
+			if (selected == null || selected.Length == 0)
 				return;
 
-			object focus = IdeApp.Workbench.RootWindow.Focus;
+			if (selected.Length == 1) {
+				object focus = IdeApp.Workbench.RootWindow.Focus;
 
-			if (focus is Gtk.Editable) {
-				((Gtk.Editable) focus).CopyClipboard ();
-				return;
+				if (focus is Gtk.Editable) {
+					((Gtk.Editable) focus).CopyClipboard ();
+					return;
+				}
 			}
-			
-			string value = (string) store.GetValue (iter, ValueCol);
-			Clipboard.Get (Gdk.Selection.Clipboard).Text = value;
+
+			var values = new List<string> ();
+			var names = new List<string> ();
+			var types = new List<string> ();
+			int maxValue = 0;
+			int maxName = 0;
+
+			for (int i = 0; i < selected.Length; i++) {
+				if (!store.GetIter (out iter, selected[i]))
+					continue;
+
+				string value = (string) store.GetValue (iter, ValueCol);
+				string name = (string) store.GetValue (iter, NameCol);
+				string type = (string) store.GetValue (iter, TypeCol);
+
+				maxValue = Math.Max (maxValue, value.Length);
+				maxName = Math.Max (maxName, name.Length);
+
+				values.Add (value);
+				names.Add (name);
+				types.Add (type);
+			}
+
+			var str = new StringBuilder ();
+			for (int i = 0; i < values.Count; i++) {
+				if (i > 0)
+					str.AppendLine ();
+
+				str.Append (names[i]);
+				if (names[i].Length < maxName)
+					str.Append (new string (' ', maxName - names[i].Length));
+				str.Append ('\t');
+				str.Append (values[i]);
+				if (values[i].Length < maxValue)
+					str.Append (new string (' ', maxValue - values[i].Length));
+				str.Append ('\t');
+				str.Append (types[i]);
+			}
+
+			Clipboard.Get (Gdk.Selection.Clipboard).Text = str.ToString ();
 		}
 		
 		[CommandHandler (EditCommands.Delete)]
@@ -1121,15 +1362,17 @@ namespace MonoDevelop.Debugger
 				return;
 			}
 			
-			if (!allowAdding) {
+			if (!AllowAdding) {
 				cinfo.Visible = false;
 				return;
 			}
+
 			TreePath[] sel = Selection.GetSelectedRows ();
 			if (sel.Length == 0) {
 				cinfo.Enabled = false;
 				return;
 			}
+
 			foreach (TreePath tp in sel) {
 				if (tp.Depth > 1) {
 					cinfo.Enabled = false;
@@ -1171,20 +1414,26 @@ namespace MonoDevelop.Debugger
 		[CommandUpdateHandler (EditCommands.Rename)]
 		protected void OnUpdateRename (CommandInfo cinfo)
 		{
-			cinfo.Visible = allowAdding;
+			cinfo.Visible = AllowAdding;
 			cinfo.Enabled = Selection.GetSelectedRows ().Length == 1;
 		}
 		
 		protected override void OnRowActivated (TreePath path, TreeViewColumn column)
 		{
 			base.OnRowActivated (path, column);
-			TreeIter it;
-			TreePath[] sel = Selection.GetSelectedRows ();
-			if (store.GetIter (out it, sel[0])) {
-				ObjectValue val = (ObjectValue) store.GetValue (it, ObjectCol);
-				if (val != null && val.Name == DebuggingService.DebuggerSession.EvaluationOptions.CurrentExceptionTag)
-					DebuggingService.ShowExceptionCaughtDialog ();
-			}
+
+			if (!CanQueryDebugger)
+				return;
+
+			TreePath[] selected = Selection.GetSelectedRows ();
+			TreeIter iter;
+
+			if (!store.GetIter (out iter, selected[0]))
+				return;
+
+			ObjectValue val = (ObjectValue) store.GetValue (iter, ObjectCol);
+			if (val != null && val.Name == DebuggingService.DebuggerSession.EvaluationOptions.CurrentExceptionTag)
+				DebuggingService.ShowExceptionCaughtDialog ();
 		}
 		
 		
@@ -1208,13 +1457,19 @@ namespace MonoDevelop.Debugger
 		
 		string GetFullExpression (TreeIter it)
 		{
+			TreePath path = store.GetPath (it);
 			string exp = "";
-			while (store.GetPath (it).Depth != 1) {
-				ObjectValue val = (ObjectValue) store.GetValue (it, ObjectCol);
+			
+			while (path.Depth != 1) {
+				ObjectValue val = (ObjectValue)store.GetValue (it, ObjectCol);
 				exp = val.ChildSelector + exp;
-				store.IterParent (out it, it);
+				if (!store.IterParent (out it, it))
+					break;
+				path = store.GetPath (it);
 			}
+
 			string name = (string) store.GetValue (it, NameCol);
+
 			return name + exp;
 		}
 
@@ -1263,18 +1518,25 @@ namespace MonoDevelop.Debugger
 			}
 		}
 		
-		EventHandler completionContextChanged;
-		
-		event EventHandler ICompletionWidget.CompletionContextChanged {
-			add { completionContextChanged += value; }
-			remove { completionContextChanged -= value; }
+		public event EventHandler CompletionContextChanged;
+
+		protected virtual void OnCompletionContextChanged (EventArgs e)
+		{
+			EventHandler handler = this.CompletionContextChanged;
+			if (handler != null)
+				handler (this, e);
 		}
 		
 		string ICompletionWidget.GetText (int startOffset, int endOffset)
 		{
-			if (startOffset < 0) startOffset = 0;
-			if (endOffset > editEntry.Text.Length) endOffset = editEntry.Text.Length;
-			return editEntry.Text.Substring (startOffset, endOffset - startOffset);
+			string text = editEntry.Text;
+
+			if (startOffset < 0 || endOffset < 0 || startOffset > endOffset || startOffset >= text.Length)
+				return "";
+
+			int length = Math.Min (endOffset - startOffset, text.Length - startOffset);
+
+			return text.Substring (startOffset, length);
 		}
 		
 		void ICompletionWidget.Replace (int offset, int count, string text)
@@ -1307,7 +1569,7 @@ namespace MonoDevelop.Debugger
 			c.TriggerOffset = triggerOffset;
 			c.TriggerLineOffset = c.TriggerOffset;
 			c.TriggerTextHeight = editEntry.SizeRequest ().Height;
-			c.TriggerWordLength = currentCompletionData.ExpressionLenght;
+			c.TriggerWordLength = currentCompletionData.ExpressionLength;
 			
 			int x, y;
 			int tx, ty;
@@ -1398,10 +1660,10 @@ namespace MonoDevelop.Debugger
 		
 		Mono.Debugging.Client.CompletionData GetCompletionData (string exp)
 		{
-			if (frame != null)
+			if (CanQueryDebugger && frame != null)
 				return frame.GetExpressionCompletionData (exp);
-			else
-				return null;
+
+			return null;
 		}
 		
 		internal void SetCustomFont (Pango.FontDescription font)
@@ -1418,6 +1680,7 @@ namespace MonoDevelop.Debugger
 			IsSorted = false;
 			foreach (CompletionItem it in data.Items)
 				Add (new DebugCompletionData (it));
+			AutoSelect =true;
 		}
 		public bool AutoSelect { get; set; }
 		public string DefaultCompletionString {
@@ -1433,7 +1696,9 @@ namespace MonoDevelop.Debugger
 		public bool AutoCompleteEmptyMatch {
 			get { return false; }
 		}
-		
+		public bool AutoCompleteEmptyMatchOnCurlyBrace {
+			get { return false; }
+		}
 		public bool CloseOnSquareBrackets {
 			get {
 				return false;
