@@ -23,7 +23,6 @@
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
-
 using System;
 using MonoDevelop.Ide.Gui.Content;
 using Mono.TextEditor;
@@ -38,25 +37,23 @@ namespace MonoDevelop.DocFood
 {
 	public class DocFoodTextEditorExtension : TextEditorExtension
 	{
-		TextEditorData textEditorData;
-		
-		public override void Initialize ()
-		{
-			base.Initialize ();
-			textEditorData = Document.Editor;
+		TextEditorData textEditorData {
+			get {
+				return Document.Editor;
+			}
 		}
 		
 		string GenerateDocumentation (IEntity member, string indent)
 		{
 			string doc = DocumentBufferHandler.GenerateDocumentation (textEditorData, member, indent);
-			int trimStart = (Math.Min (doc.Length-1, indent.Length + "//".Length));
+			int trimStart = (Math.Min (doc.Length - 1, indent.Length + "//".Length));
 			return doc.Substring (trimStart).TrimEnd ('\n', '\r');
 		}
 		
 		string GenerateEmptyDocumentation (IEntity member, string indent)
 		{
 			string doc = DocumentBufferHandler.GenerateEmptyDocumentation (textEditorData, member, indent);
-			int trimStart = (Math.Min (doc.Length-1, indent.Length + "//".Length));
+			int trimStart = (Math.Min (doc.Length - 1, indent.Length + "//".Length));
 			return doc.Substring (trimStart).TrimEnd ('\n', '\r');
 		}
 
@@ -65,12 +62,25 @@ namespace MonoDevelop.DocFood
 			if (keyChar != '/')
 				return base.KeyPress (key, keyChar, modifier);
 			
-			DocumentLine line = textEditorData.Document.GetLine (textEditorData.Caret.Line);
+			var line = textEditorData.Document.GetLine (textEditorData.Caret.Line);
 			string text = textEditorData.Document.GetTextAt (line.Offset, line.Length);
 			
 			if (!text.EndsWith ("//"))
 				return base.KeyPress (key, keyChar, modifier);
-			
+
+			// check if there is doc comment above or below.
+			var l = line.PreviousLine;
+			while (l != null && l.Length == 0)
+				l = l.PreviousLine;
+			if (l != null && textEditorData.GetTextAt (l).TrimStart ().StartsWith ("///"))
+				return base.KeyPress (key, keyChar, modifier);
+
+			l = line.NextLine;
+			while (l != null && l.Length == 0)
+				l = l.NextLine;
+			if (l != null && textEditorData.GetTextAt (l).TrimStart ().StartsWith ("///"))
+				return base.KeyPress (key, keyChar, modifier);
+
 			var member = GetMemberToDocument ();
 			if (member == null)
 				return base.KeyPress (key, keyChar, modifier);
@@ -96,12 +106,46 @@ namespace MonoDevelop.DocFood
 			
 			using (var undo = textEditorData.OpenUndoGroup ()) {
 				insertedLength = textEditorData.Replace (offset, insertedLength, documentation);
-				textEditorData.Caret.Offset = offset + insertedLength;
+				if (SelectSummary (offset, insertedLength, documentation) == false)
+					textEditorData.Caret.Offset = offset + insertedLength;
 			}
 			return false;
 		}
 
-		
+		/// <summary>
+		/// Make the summary content selected
+		/// </summary>
+		/// <returns>
+		/// <c>true</c>, if summary was selected, <c>false</c> if summary was not found.
+		/// </returns>
+		/// <param name='offset'>
+		/// Offset in document where the documentation is inserted
+		/// </param>
+		/// <param name='documentation'>
+		/// Documentation containing the summary
+		/// </param>
+		bool SelectSummary (int offset, int insertedLength, string documentation)
+		{
+			//Adjust the line endings to what the document uses to assure correct offset within the documentation
+			if (insertedLength > documentation.Length)
+				documentation = documentation.Replace ("\n", "\r\n");
+
+			const string summaryStart = "<summary>";
+			const string summaryEnd = "</summary>";
+			int start = documentation.IndexOf (summaryStart);
+			int end = documentation.IndexOf (summaryEnd);
+			if (start < 0 || end < 0)
+				return false;
+			start += summaryStart.Length;
+			string summaryText = documentation.Substring (start, end - start).Trim (new char[] {' ', '\t', '\r', '\n', '/'});
+			start = documentation.IndexOf (summaryText, start);
+			if (start < 0)
+				return false;
+			textEditorData.Caret.Offset = offset + start;
+			textEditorData.SetSelection (offset + start, offset + start + summaryText.Length);
+			return true;
+		}
+
 		bool IsEmptyBetweenLines (int start, int end)
 		{
 			for (int i = start + 1; i < end - 1; i++) {
@@ -123,7 +167,7 @@ namespace MonoDevelop.DocFood
 			if (type == null) {
 				foreach (var t in parsedDocument.TopLevelTypeDefinitions) {
 					if (t.Region.BeginLine > textEditorData.Caret.Line) {
-						var ctx = (parsedDocument.ParsedFile as CSharpParsedFile).GetTypeResolveContext (Document.Compilation, t.Region.Begin);
+						var ctx = (parsedDocument.ParsedFile as CSharpUnresolvedFile).GetTypeResolveContext (Document.Compilation, t.Region.Begin);
 						return t.Resolve (ctx).GetDefinition ();
 					}
 				}
@@ -133,7 +177,7 @@ namespace MonoDevelop.DocFood
 			IMember result = null;
 			foreach (var member in type.Members) {
 				if (member.Region.Begin > new TextLocation (textEditorData.Caret.Line, textEditorData.Caret.Column) && (result == null || member.Region.Begin < result.Region.Begin) && IsEmptyBetweenLines (textEditorData.Caret.Line, member.Region.BeginLine)) {
-					var ctx = (parsedDocument.ParsedFile as CSharpParsedFile).GetTypeResolveContext (Document.Compilation, member.Region.Begin);
+					var ctx = (parsedDocument.ParsedFile as CSharpUnresolvedFile).GetTypeResolveContext (Document.Compilation, member.Region.Begin);
 					result = member.CreateResolved (ctx);
 				}
 			}
