@@ -49,8 +49,6 @@ namespace MonoDevelop.Gettext.Editor
 			poEditorWidget = new POEditorWidget (project);
 			catalog.DirtyChanged += delegate (object sender, EventArgs args) {
 				IsDirty = catalog.IsDirty;
-				if (sender is CatalogEntry)
-					this.poEditorWidget.UpdateEntry (sender as CatalogEntry);
 			};
 		}
 		
@@ -91,14 +89,9 @@ namespace MonoDevelop.Gettext.Editor
 			poEditorWidget.Redo ();
 		}
 		
-		void IUndoHandler.BeginAtomicUndo ()
+		IDisposable IUndoHandler.OpenUndoGroup ()
 		{
-			poEditorWidget.BeginAtomicUndo ();
-		}
-		
-		void IUndoHandler.EndAtomicUndo ()
-		{
-			poEditorWidget.EndAtomicUndo ();
+			return poEditorWidget.OpenUndoGroup ();
 		}
 		
 		bool IUndoHandler.EnableUndo {
