@@ -30,6 +30,7 @@ using System;
 using System.Collections;
 using System.IO;
 using MonoDevelop.Projects;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.GtkCore
 {
@@ -47,7 +48,7 @@ namespace MonoDevelop.GtkCore
 			ArrayList list = new ArrayList ();
 			foreach (ProjectFile file in project.Files) {
 				if (file.BuildAction == BuildAction.EmbeddedResource)
-					list.Add (new Stetic.ResourceInfo (file.ResourceId, file.Name, MonoDevelop.Core.Gui.DesktopService.GetMimeTypeForUri (file.Name)));
+					list.Add (new Stetic.ResourceInfo (file.ResourceId, file.Name, DesktopService.GetMimeTypeForUri (file.Name)));
 			}
 			return (Stetic.ResourceInfo[]) list.ToArray (typeof(Stetic.ResourceInfo));
 		}
@@ -64,7 +65,7 @@ namespace MonoDevelop.GtkCore
 		public Stetic.ResourceInfo AddResource (string fileName)
 		{
 			ProjectFile file = project.AddFile (fileName, BuildAction.EmbeddedResource);
-			MonoDevelop.Ide.Gui.IdeApp.ProjectOperations.Save (project);
+			IdeApp.ProjectOperations.Save (project);
 			return new Stetic.ResourceInfo (file.ResourceId, fileName);
 		}
 		
@@ -73,7 +74,7 @@ namespace MonoDevelop.GtkCore
 			foreach (ProjectFile file in project.Files) {
 				if (resourceName == file.ResourceId) {
 					project.Files.Remove (file);
-					MonoDevelop.Ide.Gui.IdeApp.ProjectOperations.Save (project);
+					IdeApp.ProjectOperations.Save (project);
 					return;
 				}
 			}
