@@ -225,6 +225,8 @@ namespace Stetic
 		
 		public ResizableFixed ()
 		{
+			GtkWorkarounds.FixContainerLeak (this);
+			
 			fixd = new Fixed ();
 			Add (fixd);
 			this.CanFocus = true;
@@ -235,10 +237,6 @@ namespace Stetic
 			selectionBox.Show ();
 		}
 		
-		public ResizableFixed (IntPtr p): base (p)
-		{
-		}
-		
 		protected override void OnDestroyed ()
 		{
 			if (cursorX != null) {
