@@ -206,6 +206,8 @@ namespace Mono.MHex
 				string hex = "0123456789ABCDEF";
 				int idx = hex.IndexOf (char.ToUpper (ch));
 				if (idx >= 0) {
+					if (HexEditorData.Caret.Offset >= HexEditorData.Length)
+						HexEditorData.Insert (HexEditorData.Length, 0);
 					if (HexEditorData.Caret.IsInsertMode && HexEditorData.Caret.SubPosition == 0) {
 						HexEditorData.Insert (HexEditorData.Caret.Offset, (byte)(idx * 0x10));
 					} else {
