@@ -28,17 +28,15 @@ using Mono.TextEditor;
 using MonoDevelop.SourceEditor;
 using MonoDevelop.SourceEditor.QuickTasks;
 using ICSharpCode.NRefactory.CSharp;
+using ICSharpCode.NRefactory.Refactoring;
 
 namespace MonoDevelop.AnalysisCore.Gui
 {
-	class ResultMarker : UnderlineMarker
+	class ResultMarker : UnderlineTextSegmentMarker
 	{
 		Result result;
 		
-		public ResultMarker (Result result) : base (
-				GetColor (result),
-				IsOneLine (result)? (result.Region.BeginColumn) : 0,
-				IsOneLine (result)? (result.Region.EndColumn) : 0)
+		public ResultMarker (Result result, TextSegment segment) : base ("", segment)
 		{
 			this.result = result;
 		}
@@ -56,19 +54,19 @@ namespace MonoDevelop.AnalysisCore.Gui
 		public int ColEnd   { get { return IsOneLine (result)? (result.Region.EndColumn) : 0; } }
 		public string Message { get { return result.Message; } }
 		
-		static string GetColor (Result result)
+		static Cairo.Color GetColor (TextEditor editor, Result result)
 		{
 			switch (result.Level) {
 			case Severity.None:
-				return Mono.TextEditor.Highlighting.ColorScheme.DefaultString;
+				return editor.ColorStyle.PlainText.Background;
 			case Severity.Error:
-				return Mono.TextEditor.Highlighting.ColorScheme.ErrorUnderlineString;
+				return editor.ColorStyle.UnderlineError.Color;
 			case Severity.Warning:
-				return Mono.TextEditor.Highlighting.ColorScheme.WarningUnderlineString;
+				return editor.ColorStyle.UnderlineWarning.Color;
 			case Severity.Suggestion:
-				return Mono.TextEditor.Highlighting.ColorScheme.SuggestionUnderlineString;
+				return editor.ColorStyle.UnderlineSuggestion.Color;
 			case Severity.Hint:
-				return Mono.TextEditor.Highlighting.ColorScheme.HintUnderlineString;
+				return editor.ColorStyle.UnderlineHint.Color;
 			default:
 				throw new System.ArgumentOutOfRangeException ();
 			}
@@ -76,8 +74,10 @@ namespace MonoDevelop.AnalysisCore.Gui
 		
 		public override void Draw (TextEditor editor, Cairo.Context cr, Pango.Layout layout, bool selected, int startOffset, int endOffset, double y, double startXPos, double endXPos)
 		{
-			int markerStart = LineSegment.Offset + System.Math.Max (StartCol - 1, 0);
-			int markerEnd = LineSegment.Offset + (EndCol < 1 ? LineSegment.Length : EndCol - 1);
+			if (Debugger.DebuggingService.IsDebugging)
+				return;
+			int markerStart = Segment.Offset;
+			int markerEnd = Segment.EndOffset;
 			if (markerEnd < startOffset || markerStart > endOffset) 
 				return;
 			
@@ -119,10 +119,10 @@ namespace MonoDevelop.AnalysisCore.Gui
 				return;
 			
 			double height = editor.LineHeight / 5;
-			cr.Color = ColorName == null ? Color : editor.ColorStyle.GetColorFromDefinition (ColorName);
+			cr.Color = GetColor (editor, Result);
 			if (drawOverlay) {
 				cr.Rectangle (drawFrom, y, drawTo - drawFrom, editor.LineHeight);
-				var color = editor.ColorStyle.Default.CairoBackgroundColor;
+				var color = editor.ColorStyle.PlainText.Background;
 				color.A = 0.6;
 				cr.Color = color;
 				cr.Fill ();
@@ -135,4 +135,35 @@ namespace MonoDevelop.AnalysisCore.Gui
 			}
 		}
 	}
+
+	class GrayOutMarker : ResultMarker, IChunkMarker
+	{
+		public GrayOutMarker (Result result, TextSegment segment) : base (result, segment)
+		{
+		}
+
+		public override void Draw (TextEditor editor, Cairo.Context cr, Pango.Layout layout, bool selected, int startOffset, int endOffset, double y, double startXPos, double endXPos)
+		{
+		}
+
+		#region IChunkMarker implementation
+		void IChunkMarker.ChangeForeColor (TextEditor editor, Chunk chunk, ref Cairo.Color color)
+		{
+			if (Debugger.DebuggingService.IsDebugging)
+				return;
+			int markerStart = Segment.Offset;
+			int markerEnd = Segment.EndOffset;
+			if (!(markerStart <= chunk.Offset && chunk.Offset < markerEnd)) 
+				return;
+
+			var bgc = editor.ColorStyle.PlainText.Background;
+			double alpha = 0.6;
+			color = new Cairo.Color (
+				color.R * alpha + bgc.R * (1.0 - alpha),
+				color.G * alpha + bgc.G * (1.0 - alpha),
+				color.B * alpha + bgc.B * (1.0 - alpha)
+			);
+		}
+		#endregion
+	}
 }
