@@ -29,16 +29,16 @@ using System;
 using Gtk;
 
 using MonoDevelop.Core;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.CodeGeneration;
 using System.Collections.Generic;
 using MonoDevelop.Ide;
+using MonoDevelop.Ide.ProgressMonitoring;
+using ICSharpCode.NRefactory.CSharp.Refactoring;
+using ICSharpCode.NRefactory.TypeSystem;
 
 namespace MonoDevelop.Refactoring.Rename
 {
 	public partial class RenameItemDialog : Gtk.Dialog
 	{
-		string fileName;
 		RenameRefactoring rename;
 		RefactoringOptions options;
 		
@@ -50,25 +50,28 @@ namespace MonoDevelop.Refactoring.Rename
 				options.SelectedItem = ((IMethod)options.SelectedItem).DeclaringType;
 			}
 			this.Build ();
-
 			if (options.SelectedItem is IType) {
-				IType type = (IType)options.SelectedItem;
-				if (type.DeclaringType == null) {
-					// not supported for inner types
-					this.renameFileFlag.Visible = true;
-					this.renameFileFlag.Active = true;
-					// if more than one type is in the file, only rename the file as defilt if the file name contains the type name
-					// see Bug 603938 - Renaming a Class in a file with multiple classes renames the file
-					if (options.Document.CompilationUnit != null && options.Document.CompilationUnit.Types.Count > 1) 
-						this.renameFileFlag.Active = options.Document.FileName.FileNameWithoutExtension.Contains (type.Name);
+
+				var t = (IType)options.SelectedItem;
+				if (t.Kind == TypeKind.TypeParameter) {
+					this.Title = GettextCatalog.GetString ("Rename Type Parameter");
+					entry.Text = t.Name;
+
 				} else {
-					this.renameFileFlag.Active = false;
+					var typeDefinition = (t).GetDefinition ();
+					if (typeDefinition.DeclaringType == null) {
+						// not supported for inner types
+						this.renameFileFlag.Visible = true;
+						this.renameFileFlag.Active = options.Document != null ? options.Document.FileName.FileNameWithoutExtension.Contains (typeDefinition.Name) : false;
+					} else {
+						this.renameFileFlag.Active = false;
+					}
+					if (typeDefinition.Kind == TypeKind.Interface)
+						this.Title = GettextCatalog.GetString ("Rename Interface");
+					else
+						this.Title = GettextCatalog.GetString ("Rename Class");
 				}
-				if (type.ClassType == ClassType.Interface)
-					this.Title = GettextCatalog.GetString ("Rename Interface");
-				else
-					this.Title = GettextCatalog.GetString ("Rename Class");
-				this.fileName = type.CompilationUnit.FileName;
+				//				this.fileName = type.GetDefinition ().Region.FileName;
 			} else if (options.SelectedItem is IField) {
 				this.Title = GettextCatalog.GetString ("Rename Field");
 			} else if (options.SelectedItem is IProperty) {
@@ -79,31 +82,39 @@ namespace MonoDevelop.Refactoring.Rename
 				}
 			} else if (options.SelectedItem is IEvent) {
 				this.Title = GettextCatalog.GetString ("Rename Event");
-			} else if (options.SelectedItem is IMethod) {
-				this.Title = GettextCatalog.GetString ("Rename Method");
+			} else if (options.SelectedItem is IMethod) { 
+				var m = (IMethod)options.SelectedItem;
+				if (m.IsConstructor || m.IsDestructor) {
+					this.Title = GettextCatalog.GetString ("Rename Class");
+				} else {
+					this.Title = GettextCatalog.GetString ("Rename Method");
+				}
 			} else if (options.SelectedItem is IParameter) {
 				this.Title = GettextCatalog.GetString ("Rename Parameter");
-			} else if (options.SelectedItem is LocalVariable) {
+			} else if (options.SelectedItem is IVariable) {
 				this.Title = GettextCatalog.GetString ("Rename Variable");
+			} else if (options.SelectedItem is ITypeParameter) {
+				this.Title = GettextCatalog.GetString ("Rename Type Parameter");
 			} else {
 				this.Title = GettextCatalog.GetString ("Rename Item");
 			}
 			
-			if (options.SelectedItem is IMember) {
-				IMember member = (IMember)options.SelectedItem;
-				entry.Text = member.Name;
-				if (!(member is IType) && member.DeclaringType != null)
-					this.fileName = member.DeclaringType.CompilationUnit.FileName;
-			} else if (options.SelectedItem is LocalVariable) {
-				LocalVariable lvar = (LocalVariable)options.SelectedItem;
-				entry.Text = lvar.Name;
-				this.fileName = lvar.FileName;
-			} else {
-				IParameter par = options.SelectedItem as IParameter;
-				if (par != null) {
-					entry.Text = par.Name;
-					this.fileName = par.DeclaringMember.DeclaringType.CompilationUnit.FileName;
+			if (options.SelectedItem is IEntity) {
+				var member = (IEntity)options.SelectedItem;
+				if (member.SymbolKind == SymbolKind.Constructor || member.SymbolKind == SymbolKind.Destructor) {
+					entry.Text = member.DeclaringType.Name;
+				} else {
+					entry.Text = member.Name;
 				}
+				//				fileName = member.Region.FileName;
+			} else if (options.SelectedItem is ITypeParameter) {
+				var lvar = (ITypeParameter)options.SelectedItem;
+				entry.Text = lvar.Name;
+				//				this.fileName = lvar.Region.FileName;
+			} else if (options.SelectedItem is IVariable) {
+				var lvar = (IVariable)options.SelectedItem;
+				entry.Text = lvar.Name;
+				//				this.fileName = lvar.Region.FileName;
 			}
 			entry.SelectRegion (0, -1);
 			
@@ -119,19 +130,20 @@ namespace MonoDevelop.Refactoring.Rename
 
 		bool ValidateName ()
 		{
-			INameValidator nameValidator = MonoDevelop.Projects.LanguageBindingService.GetRefactorerForFile (fileName ?? "default.cs");
-			if (nameValidator == null)
-				return true;
-			ValidationResult result = nameValidator.ValidateName (this.options.SelectedItem, entry.Text);
-			if (!result.IsValid) {
-				imageWarning.IconName = Gtk.Stock.DialogError;
-			} else if (result.HasWarning) {
-				imageWarning.IconName = Gtk.Stock.DialogWarning;
-			} else {
-				imageWarning.IconName = Gtk.Stock.Apply;
-			}
-			labelWarning.Text = result.Message;
-			return result.IsValid;
+			return true; // TODO: Name validation.
+//			var nameValidator = MonoDevelop.Projects.LanguageBindingService.GetRefactorerForFile (fileName ?? "default.cs");
+//			if (nameValidator == null)
+//				return true;
+//			ValidationResult result = nameValidator.ValidateName (this.options.SelectedItem, entry.Text);
+//			if (!result.IsValid) {
+//				imageWarning.IconName = Gtk.Stock.DialogError;
+//			} else if (result.HasWarning) {
+//				imageWarning.IconName = Gtk.Stock.DialogWarning;
+//			} else {
+//				imageWarning.IconName = Gtk.Stock.Apply;
+//			}
+//			labelWarning.Text = result.Message;
+//			return result.IsValid;
 		}
 
 		void OnEntryChanged (object sender, EventArgs e)
@@ -157,17 +169,19 @@ namespace MonoDevelop.Refactoring.Rename
 		
 		void OnOKClicked (object sender, EventArgs e)
 		{
-			List<Change> changes = rename.PerformChanges (options, Properties);
-			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetBackgroundProgressMonitor (this.Title, null);
-			RefactoringService.AcceptChanges (monitor, options.Dom, changes);
+			var properties = Properties;
 			((Widget)this).Destroy ();
+			List<Change> changes = rename.PerformChanges (options, properties);
+			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetBackgroundProgressMonitor (this.Title, null);
+			RefactoringService.AcceptChanges (monitor, changes);
 		}
 		
 		void OnPreviewClicked (object sender, EventArgs e)
 		{
-			List<Change> changes = rename.PerformChanges (options, Properties);
+			var properties = Properties;
 			((Widget)this).Destroy ();
-			MessageService.ShowCustomDialog (new RefactoringPreviewDialog (options.Dom, changes));
+			List<Change> changes = rename.PerformChanges (options, properties);
+			MessageService.ShowCustomDialog (new RefactoringPreviewDialog (changes));
 		}
 	}
 		
