@@ -61,8 +61,14 @@ namespace MonoDevelop.Refactoring.Rename
 		{
 			if (options.SelectedItem is IVariable || options.SelectedItem is IParameter)
 				return true;
+			if (options.SelectedItem is INamespace) {
+				var ns = (INamespace)options.SelectedItem;
+				return ns.Types.Any (type => !string.IsNullOrEmpty (type.Region.FileName));
+			}
 			if (options.SelectedItem is ITypeDefinition)
 				return !string.IsNullOrEmpty (((ITypeDefinition)options.SelectedItem).Region.FileName);
+			if (options.SelectedItem is IType && ((IType)options.SelectedItem).Kind == TypeKind.TypeParameter)
+				return !string.IsNullOrEmpty (((ITypeParameter)options.SelectedItem).Region.FileName);
 
 			if (options.SelectedItem is IMember) {
 				var cls = ((IMember)options.SelectedItem).DeclaringTypeDefinition;
@@ -81,7 +87,7 @@ namespace MonoDevelop.Refactoring.Rename
 				return;
 			}
 			using (var monitor = new NullProgressMonitor ()) {
-				var col = ReferenceFinder.FindReferences (entity, monitor);
+				var col = ReferenceFinder.FindReferences (entity, true, monitor);
 				
 				List<Change> result = new List<Change> ();
 				foreach (var memberRef in col) {
@@ -102,7 +108,7 @@ namespace MonoDevelop.Refactoring.Rename
 		public static void RenameVariable (IVariable variable, string newName)
 		{
 			using (var monitor = new NullProgressMonitor ()) {
-				var col = ReferenceFinder.FindReferences (variable, monitor);
+				var col = ReferenceFinder.FindReferences (variable, true, monitor);
 				
 				List<Change> result = new List<Change> ();
 				foreach (var memberRef in col) {
@@ -131,7 +137,7 @@ namespace MonoDevelop.Refactoring.Rename
 			}
 
 			using (var monitor = new NullProgressMonitor ()) {
-				var col = ReferenceFinder.FindReferences (typeParameter, monitor);
+				var col = ReferenceFinder.FindReferences (typeParameter, true, monitor);
 				
 				List<Change> result = new List<Change> ();
 				foreach (var memberRef in col) {
@@ -149,6 +155,28 @@ namespace MonoDevelop.Refactoring.Rename
 			}
 		}
 
+		public static void RenameNamespace (INamespace ns, string newName)
+		{
+			using (var monitor = new NullProgressMonitor ()) {
+				var col = ReferenceFinder.FindReferences (ns, true, monitor);
+
+				List<Change> result = new List<Change> ();
+				foreach (var memberRef in col) {
+					var change = new TextReplaceChange ();
+					change.FileName = memberRef.FileName;
+					change.Offset = memberRef.Offset;
+					change.RemovedChars = memberRef.Length;
+					change.InsertedText = newName;
+					change.Description = string.Format (GettextCatalog.GetString ("Replace '{0}' with '{1}'"), memberRef.GetName (), newName);
+					result.Add (change);
+				}
+				if (result.Count > 0) {
+					RefactoringService.AcceptChanges (monitor, result);
+				}
+			}
+		}
+
+
 		public override string GetMenuDescription (RefactoringOptions options)
 		{
 			return IdeApp.CommandService.GetCommandInfo (MonoDevelop.Ide.Commands.EditCommands.Rename).Text;
@@ -158,12 +186,12 @@ namespace MonoDevelop.Refactoring.Rename
 		{
 			if (options.SelectedItem is IVariable) {
 				var field = options.SelectedItem as IField;
-				if (field != null && field.Accessibility != Accessibility.Private) {
+				if (field != null && (field.Accessibility != Accessibility.Private || field.DeclaringTypeDefinition != null && field.DeclaringTypeDefinition.Parts.Count > 1)) {
 					MessageService.ShowCustomDialog (new RenameItemDialog (options, this));
 					return;
 				}
 
-				var col = ReferenceFinder.FindReferences (options.SelectedItem);
+				var col = ReferenceFinder.FindReferences (options.SelectedItem, true);
 				if (col == null)
 					return;
 				var data = options.Document != null ? options.GetTextEditorData () : IdeApp.Workbench.ActiveDocument.Editor;
@@ -234,7 +262,7 @@ namespace MonoDevelop.Refactoring.Rename
 			List<Change> result = new List<Change> ();
 			IEnumerable<MemberReference> col = null;
 			using (var monitor = new MessageDialogProgressMonitor (true, false, false, true)) {
-				col = ReferenceFinder.FindReferences (options.SelectedItem, monitor);
+				col = ReferenceFinder.FindReferences (options.SelectedItem, true, monitor);
 				if (col == null)
 					return result;
 					
