@@ -123,7 +123,7 @@ namespace MonoDevelop.Refactoring
 			buttonUnselectAll.Clicked += delegate { SelectAll (false); };
 
 			refactorer = IdeApp.Workspace.GetCodeRefactorer (IdeApp.ProjectOperations.CurrentSelectedSolution);
-			ambience = AmbienceService.GetAmbienceForFile (cls.CompilationUnit.FileName);
+			ambience = AmbienceService.GetAmbienceForFile (cls.GetDefinition ().Region.FileName);
 			PopulateTreeView ();
 			UpdateOKButton ();
 		}
@@ -241,8 +241,8 @@ namespace MonoDevelop.Refactoring
 		{
 			try {
 				StringBuilder code = new StringBuilder ();
-				CodeGenerator generator =  CodeGenerator.CreateGenerator (editor.Editor.Document.MimeType, editor.Editor.Options.TabsToSpaces, editor.Editor.Options.TabSize, editor.Editor.EolMarker);
-				IType declaringType = editor.CompilationUnit.GetTypeAt (cls.Location.Line, cls.Location.Column) ?? cls;
+				CodeGenerator generator =  CodeGenerator.CreateGenerator (editor.Editor.Document.MimeType, editor.Editor.TabsToSpaces, editor.Editor.Options.TabSize, editor.Editor.EolMarker);
+				IType declaringType = editor.GetType (cls.Location.Line, cls.Location.Column) ?? cls;
 
 				foreach (KeyValuePair<IType, IEnumerable<TreeIter>> kvp in GetAllClasses ()) {
 					if (code.Length > 0) {
@@ -266,8 +266,9 @@ namespace MonoDevelop.Refactoring
 					}
 				}
 				
-				InsertionCursorEditMode mode = new InsertionCursorEditMode (editor.Editor.Parent, CodeGenerationService.GetInsertionPoints (editor, this.cls));
-				ModeHelpWindow helpWindow = new ModeHelpWindow ();
+				var mode = new InsertionCursorEditMode (editor.Editor.Parent, CodeGenerationService.GetInsertionPoints (editor, this.cls));
+				var helpWindow = new ModeHelpWindow ();
+				helpWindow.Shown += (s, a) => DesktopService.RemoveWindowShadow (helpWindow);
 				helpWindow.TransientFor = IdeApp.Workbench.RootWindow;
 				helpWindow.TitleText = GettextCatalog.GetString ("<b>Override -- Targeting</b>");
 				helpWindow.Items.Add (new KeyValuePair<string, string> (GettextCatalog.GetString ("<b>Key</b>"), GettextCatalog.GetString ("<b>Behavior</b>")));
