@@ -159,6 +159,7 @@ namespace MonoDevelop.Refactoring
 			var rctx = new RefactoringOptions (null);
 			var handler = new RenameHandler (changes);
 			FileService.FileRenamed += handler.FileRename;
+			var fileNames = new HashSet<FilePath> ();
 			for (int i = 0; i < changes.Count; i++) {
 				changes[i].PerformChange (monitor, rctx);
 				var replaceChange = changes[i] as TextReplaceChange;
@@ -168,6 +169,7 @@ namespace MonoDevelop.Refactoring
 					var change = changes[j] as TextReplaceChange;
 					if (change == null)
 						continue;
+					fileNames.Add (change.FileName);
 					if (replaceChange.Offset >= 0 && change.Offset >= 0 && replaceChange.FileName == change.FileName) {
 						if (replaceChange.Offset < change.Offset) {
 							change.Offset -= replaceChange.RemovedChars;
@@ -180,6 +182,7 @@ namespace MonoDevelop.Refactoring
 					}
 				}
 			}
+			FileService.NotifyFilesChanged (fileNames);
 			FileService.FileRenamed -= handler.FileRename;
 			TextReplaceChange.FinishRefactoringOperation ();
 		}
@@ -195,13 +198,17 @@ namespace MonoDevelop.Refactoring
 				var result = new List<MonoDevelop.CodeActions.CodeAction> ();
 				try {
 					var editor = doc.Editor;
-					if (editor != null) {
-						string disabledNodes = PropertyService.Get ("ContextActions." + editor.Document.MimeType, "") ?? "";
-						foreach (var provider in contextActions.Where (fix => disabledNodes.IndexOf (fix.IdString) < 0)) {
-							try {
-								result.AddRange (provider.GetActions (doc, loc, cancellationToken));
-							} catch (Exception ex) {
-								LoggingService.LogError ("Error in context action provider " + provider.Title, ex);
+					var parsedDocument = doc.ParsedDocument;
+					if (editor != null && parsedDocument != null && parsedDocument.CreateRefactoringContext != null) {
+						var ctx = parsedDocument.CreateRefactoringContext (doc, cancellationToken);
+						if (ctx != null) {
+							string disabledNodes = PropertyService.Get ("ContextActions." + editor.Document.MimeType, "") ?? "";
+							foreach (var provider in contextActions.Where (fix => disabledNodes.IndexOf (fix.IdString) < 0)) {
+								try {
+									result.AddRange (provider.GetActions (doc, ctx, loc, cancellationToken));
+								} catch (Exception ex) {
+									LoggingService.LogError ("Error in context action provider " + provider.Title, ex);
+								}
 							}
 						}
 					}
@@ -212,21 +219,24 @@ namespace MonoDevelop.Refactoring
 			}, cancellationToken);
 		}
 
-		public static void QueueQuickFixAnalysis (MonoDevelop.Ide.Gui.Document doc, TextLocation loc, Action<List<MonoDevelop.CodeActions.CodeAction>> callback)
+		public static void QueueQuickFixAnalysis (MonoDevelop.Ide.Gui.Document doc, TextLocation loc, CancellationToken token, Action<List<MonoDevelop.CodeActions.CodeAction>> callback)
 		{
-			System.Threading.ThreadPool.QueueUserWorkItem (delegate {
+			var ext = doc.GetContent<MonoDevelop.AnalysisCore.Gui.ResultsEditorExtension> ();
+			var issues = ext != null ? ext.GetResultsAtOffset (doc.Editor.LocationToOffset (loc), token).OrderBy (r => r.Level).ToList () : new List<Result> ();
+
+			ThreadPool.QueueUserWorkItem (delegate {
 				try {
 					var result = new List<MonoDevelop.CodeActions.CodeAction> ();
-
-					var ext = doc.GetContent<MonoDevelop.AnalysisCore.Gui.ResultsEditorExtension> ();
-					if (ext != null) {
-						foreach (var r in ext.GetResultsAtOffset (doc.Editor.LocationToOffset (loc)).OrderBy (r => r.Level)) {
-							var fresult = r as FixableResult;
-							if (fresult == null)
-								continue;
-							foreach (var action in FixOperationsHandler.GetActions (doc, fresult)) {
-								result.Add (new AnalysisContextActionProvider.AnalysisCodeAction (action, r));
-							}
+					foreach (var r in issues) {
+						if (token.IsCancellationRequested)
+							return;
+						var fresult = r as FixableResult;
+						if (fresult == null)
+							continue;
+						foreach (var action in FixOperationsHandler.GetActions (doc, fresult)) {
+							result.Add (new AnalysisContextActionProvider.AnalysisCodeAction (action, r) {
+								DocumentRegion = action.DocumentRegion
+							});
 						}
 					}
 					result.AddRange (GetValidActions (doc, loc).Result);
@@ -249,9 +259,8 @@ namespace MonoDevelop.Refactoring
 				return editor.MainSelection.Start;
 
 			var line = editor.GetLine (location.Line);
-			if (line == null || location.Column >= line.Length)
+			if (line == null || location.Column > line.LengthIncludingDelimiter)
 				return location;
-
 			int offset = editor.LocationToOffset (location);
 			if (offset > 0 && !char.IsLetterOrDigit (doc.Editor.GetCharAt (offset)) && char.IsLetterOrDigit (doc.Editor.GetCharAt (offset - 1)))
 				return new DocumentLocation (location.Line, location.Column - 1);
