@@ -31,12 +31,16 @@ using System.Collections.Generic;
 using Gdk;
 using MonoDevelop.Core;
 using ICSharpCode.NRefactory;
-using ICSharpCode.NRefactory.CSharp;
+using ICSharpCode.NRefactory.Refactoring;
 
 namespace MonoDevelop.SourceEditor.QuickTasks
 {
 	public class QuickTaskOverviewMode : DrawingArea
 	{
+		const int indicatorPadding = 3;
+		bool flatStyle = Platform.IsWindows;
+		int barPadding = Platform.IsWindows? 1 : 3;
+
 		readonly QuickTaskStrip parentStrip;
 		protected readonly Adjustment vadjustment;
 		
@@ -64,7 +68,7 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 			this.parentStrip = parent;
 			Events |= EventMask.ButtonPressMask | EventMask.ButtonReleaseMask | EventMask.ButtonMotionMask | EventMask.PointerMotionMask | EventMask.LeaveNotifyMask;
 			vadjustment = this.parentStrip.VAdjustment;
-			
+
 			vadjustment.ValueChanged += RedrawOnUpdate;
 			vadjustment.Changed += RedrawOnUpdate;
 			parentStrip.TaskProviderUpdated += RedrawOnUpdate;
@@ -268,15 +272,15 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 				return new Cairo.Color (0, 0, 0);
 			switch (severity) {
 			case Severity.Error:
-				return style.ErrorUnderline;
+				return style.UnderlineError.Color;
 			case Severity.Warning:
-				return style.WarningUnderline;
+				return style.UnderlineWarning.Color;
 			case Severity.Suggestion:
-				return style.SuggestionUnderline;
+				return style.UnderlineSuggestion.Color;
 			case Severity.Hint:
-				return style.HintUnderline;
+				return style.UnderlineHint.Color;
 			case Severity.None:
-				return style.Default.CairoColor;
+				return style.PlainText.Background;
 			default:
 				throw new ArgumentOutOfRangeException ();
 			}
@@ -289,11 +293,11 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 				return new Cairo.Color (0, 0, 0);
 			switch (severity) {
 			case Severity.Error:
-				return style.ErrorUnderline;
+				return style.UnderlineError.Color;
 			case Severity.Warning:
-				return style.WarningUnderline;
+				return style.UnderlineWarning.Color;
 			default:
-				return style.SuggestionUnderline;
+				return style.UnderlineSuggestion.Color;
 			}
 		}
 		protected virtual double IndicatorHeight  {
@@ -302,7 +306,7 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 			}
 		}
 		
-		void MouseMove (double y)
+		protected virtual void MouseMove (double y)
 		{
 			if (button != 1)
 				return;
@@ -310,10 +314,10 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 			position = Math.Max (vadjustment.Lower, Math.Min (position, vadjustment.Upper - vadjustment.PageSize));
 			vadjustment.Value = position;
 		}
-		
+
 		QuickTask hoverTask = null;
 		
-		uint button;
+		protected uint button;
 
 		protected override bool OnButtonPressEvent (EventButton evnt)
 		{
@@ -339,7 +343,12 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 
 		protected void DrawIndicator (Cairo.Context cr, Severity severity)
 		{
-			cr.Rectangle (3, Allocation.Height - IndicatorHeight + 4, Allocation.Width - 6, IndicatorHeight - 6);
+			cr.Rectangle (
+				indicatorPadding + 0.5,
+				Allocation.Height - IndicatorHeight + indicatorPadding + 0.5,
+				Allocation.Width - indicatorPadding * 2,
+				IndicatorHeight - indicatorPadding * 2
+			);
 			
 			var darkColor = (HslColor)GetIndicatorColor (severity);
 			darkColor.L *= 0.5;
@@ -357,22 +366,31 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 
 		protected void DrawSearchIndicator (Cairo.Context cr)
 		{
-			var x1 = 1 + Allocation.Width / 2;
-			var y1 = Allocation.Height - IndicatorHeight / 2;
-			cr.Arc (x1, 
-				y1, 
-				(IndicatorHeight - 1) / 2, 
-				0, 
-				2 * Math.PI);
+			int diameter = Math.Min (Allocation.Width, (int)IndicatorHeight) - indicatorPadding * 2;
+			var x1 = Math.Round (Allocation.Width / 2d);
+			var y1 = Allocation.Height - Math.Floor (IndicatorHeight / 2d);
+			if (diameter % 2 == 0) {
+				x1 += 0.5;
+				y1 += 0.5;
+			}
+
+			cr.Arc (x1, y1, diameter / 2d, 0, 2 * Math.PI);
 			
-			var darkColor = (HslColor)TextEditor.ColorStyle.SearchTextBg;
+			var darkColor = (HslColor)TextEditor.ColorStyle.SearchResult.Color;
 			darkColor.L *= 0.5;
-			
-			using (var pattern = new Cairo.RadialGradient (x1, y1, Allocation.Width / 2, x1 - Allocation.Width, y1 - Allocation.Width, Allocation.Width)) {
-				pattern.AddColorStop (0, darkColor);
-				pattern.AddColorStop (1, TextEditor.ColorStyle.SearchTextMainBg);
-				cr.Pattern = pattern;
-				cr.FillPreserve ();
+
+			if (flatStyle) {
+				using (var pattern = new Cairo.SolidPattern (TextEditor.ColorStyle.SearchResultMain.Color)) {
+					cr.Pattern = pattern;
+					cr.FillPreserve ();
+				}
+			} else {
+				using (var pattern = new Cairo.RadialGradient (x1, y1, Allocation.Width / 2, x1 - Allocation.Width, y1 - Allocation.Width, Allocation.Width)) {
+					pattern.AddColorStop (0, darkColor);
+					pattern.AddColorStop (1, TextEditor.ColorStyle.SearchResultMain.Color);
+					cr.Pattern = pattern;
+					cr.FillPreserve ();
+				}
 			}
 			
 			cr.Color = darkColor;
@@ -409,7 +427,7 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 			cr.LineTo (7, y);
 			cr.LineTo (0, y + 4);
 			cr.ClosePath ();
-			cr.Color = TextEditor.ColorStyle.Default.CairoColor;
+			cr.Color = TextEditor.ColorStyle.PlainText.Foreground;
 			cr.Fill ();
 		}
 
@@ -426,10 +444,10 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 		protected Severity DrawQuickTasks (Cairo.Context cr)
 		{
 			Severity severity = Severity.None;
-
+			/*
 			foreach (var usage in AllUsages) {
 				double y = GetYPosition (usage.Line);
-				var usageColor = TextEditor.ColorStyle.Default.CairoColor;
+				var usageColor = TextEditor.ColorStyle.PlainText.Foreground;
 				usageColor.A = 0.4;
 				cr.Color = usageColor;
 				cr.MoveTo (0, y - 3);
@@ -438,12 +456,12 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 				cr.ClosePath ();
 				cr.Fill ();
 			}
-
+*/
 			foreach (var task in AllTasks) {
 				double y = GetYPosition (task.Location.Line);
 
 				cr.Color = GetBarColor (task.Severity);
-				cr.Rectangle (3 + 0.5, y - 1 + 0.5, Allocation.Width - 5, 2);
+				cr.Rectangle (barPadding, Math.Round (y) - 1, Allocation.Width - barPadding * 2, 2);
 				cr.Fill ();
 
 				switch (task.Severity) {
@@ -461,16 +479,16 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 		
 		protected void DrawLeftBorder (Cairo.Context cr)
 		{
-			cr.MoveTo (0.5, 1.5);
+			cr.MoveTo (0.5, 0);
 			cr.LineTo (0.5, Allocation.Height);
 			if (TextEditor.ColorStyle != null) {
-				var col = (HslColor)TextEditor.ColorStyle.Default.CairoBackgroundColor;
-				col.L *= 0.90;
+				var col = (HslColor)TextEditor.ColorStyle.PlainText.Background;
+				if (!flatStyle) {
+					col.L *= 0.88;
+				}
 				cr.Color = col;
-				
 			}
 			cr.Stroke ();
-			
 		}
 
 		protected override void OnSizeAllocated (Rectangle allocation)
@@ -484,22 +502,21 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 			if (vadjustment == null || vadjustment.Upper <= vadjustment.PageSize) 
 				return;
 
-			int barPadding = 3;
+			int barWidth = Allocation.Width - barPadding - barPadding;
 			var allocH = Allocation.Height - (int) IndicatorHeight;
 			var adjUpper = vadjustment.Upper;
-			var barY = allocH * vadjustment.Value / adjUpper + barPadding;
-			var barH = allocH * (vadjustment.PageSize / adjUpper) - barPadding - barPadding;
-			int barWidth = Allocation.Width - barPadding - barPadding;
+			var barY = Math.Round (allocH * vadjustment.Value / adjUpper) + barPadding;
+			const int minBarHeight = 16;
+			var barH = Math.Max (minBarHeight, Math.Round (allocH * (vadjustment.PageSize / adjUpper)) - barPadding - barPadding);
+
+			if (flatStyle) {
+				cr.Rectangle (barPadding, barY, barWidth, barH);
+			} else {
+				MonoDevelop.Components.CairoExtensions.RoundedRectangle (cr, barPadding, barY, barWidth, barH, barWidth / 2);
+			}
 			
-			MonoDevelop.Components.CairoExtensions.RoundedRectangle (cr, 
-				barPadding,
-				barY,
-				barWidth,
-				barH,
-				barWidth / 2);
-			
-			var color = (HslColor)((TextEditor.ColorStyle != null) ? TextEditor.ColorStyle.Default.CairoColor : new Cairo.Color (0, 0, 0));
-			color.L = 0.5;
+			var color = (HslColor)((TextEditor.ColorStyle != null) ? TextEditor.ColorStyle.PlainText.Foreground : new Cairo.Color (0, 0, 0));
+			color.L = flatStyle? 0.7 : 0.5;
 			var c = (Cairo.Color)color;
 			c.A = 0.6;
 			cr.Color = c;
@@ -514,8 +531,8 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 				bool isMainSelection = false;
 				if (!TextEditor.TextViewMargin.MainSearchResult.IsInvalid)
 					isMainSelection = region.Offset == TextEditor.TextViewMargin.MainSearchResult.Offset;
-				cr.Color = isMainSelection ? TextEditor.ColorStyle.SearchTextMainBg : TextEditor.ColorStyle.SearchTextBg;
-				cr.Rectangle (3 + 0.5, y - 1 + 0.5, Allocation.Width - 5, 2);
+				cr.Color = isMainSelection ? TextEditor.ColorStyle.SearchResultMain.Color : TextEditor.ColorStyle.SearchResult.Color;
+				cr.Rectangle (barPadding, Math.Round (y) - 1, Allocation.Width - barPadding * 2, 2);
 				cr.Fill ();
 			}
 		}
@@ -529,21 +546,32 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 				cr.Rectangle (0, 0, Allocation.Width, Allocation.Height);
 				
 				if (TextEditor.ColorStyle != null) {
-					var grad = new Cairo.LinearGradient (0, 0, Allocation.Width, 0);
-					var col = (HslColor)TextEditor.ColorStyle.Default.CairoBackgroundColor;
+					var col = (HslColor)TextEditor.ColorStyle.PlainText.Background;
 					col.L *= 0.95;
-					grad.AddColorStop (0, col);
-					grad.AddColorStop (0.7, TextEditor.ColorStyle.Default.CairoBackgroundColor);
-					grad.AddColorStop (1, col);
-					cr.Pattern = grad;
+					if (flatStyle) {
+						using (var pattern = new Cairo.SolidPattern (col)) {
+							cr.Pattern = pattern;
+						}
+					} else {
+						using (var grad = new Cairo.LinearGradient (0, 0, Allocation.Width, 0)) {
+							grad.AddColorStop (0, col);
+							grad.AddColorStop (0.7, TextEditor.ColorStyle.PlainText.Background);
+							grad.AddColorStop (1, col);
+							cr.Pattern = grad;
+						}
+					}
 				}
 				cr.Fill ();
-				
+
+				/*
 				cr.Color = (HslColor)Style.Dark (State);
 				cr.MoveTo (-0.5, 0.5);
 				cr.LineTo (Allocation.Width, 0.5);
-				cr.Stroke ();
-				
+
+				cr.MoveTo (-0.5, Allocation.Height - 0.5);
+				cr.LineTo (Allocation.Width, Allocation.Height - 0.5);
+				cr.Stroke ();*/
+
 				if (TextEditor == null)
 					return true;
 				
@@ -551,8 +579,10 @@ namespace MonoDevelop.SourceEditor.QuickTasks
 					DrawSearchResults (cr);
 					DrawSearchIndicator (cr);
 				} else {
-					var severity = DrawQuickTasks (cr);
-					DrawIndicator (cr, severity);
+					if (!Debugger.DebuggingService.IsDebugging) {
+						var severity = DrawQuickTasks (cr);
+						DrawIndicator (cr, severity);
+					}
 				}
 				DrawCaret (cr);
 				
