@@ -54,6 +54,7 @@ namespace MonoDevelop.SourceEditor
 		
 		SourceEditorView view;
 		ExtensionContext extensionContext;
+		Adjustment cachedHAdjustment, cachedVAdjustment;
 		
 		public ITextEditorExtension Extension {
 			get;
@@ -94,16 +95,6 @@ namespace MonoDevelop.SourceEditor
 			
 			Document.TextReplaced += HandleSkipCharsOnReplace;
 			
-			Document.TextReplaced += delegate(object sender, DocumentChangeEventArgs args) {
-				if (Extension != null) {
-					try {
-						Extension.TextChanged (args.Offset, args.Offset + Math.Max (args.RemovalLength, args.InsertionLength));
-					} catch (Exception ex) {
-						ReportExtensionError (ex);
-					}
-				}
-			};
-			
 			UpdateEditMode ();
 			this.DoPopupMenu = ShowPopup;
 		}
@@ -168,8 +159,21 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 
+		void UnregisterAdjustments ()
+		{
+			if (cachedHAdjustment != null)
+				cachedHAdjustment.ValueChanged -= HAdjustment_ValueChanged;
+			if (cachedVAdjustment != null)
+				cachedVAdjustment.ValueChanged -= VAdjustment_ValueChanged;
+			cachedHAdjustment = null;
+			cachedVAdjustment = null;
+		}
+
 		protected override void OnDestroyed ()
 		{
+			UnregisterAdjustments ();
+			resolveResult = null;
+			Extension = null;
 			ExtensionContext = null;
 			view = null;
 			base.OnDestroyed ();
@@ -181,9 +185,9 @@ namespace MonoDevelop.SourceEditor
 		
 		void OnTooltipProviderChanged (object s, ExtensionNodeEventArgs a)
 		{
-			ITooltipProvider provider;
+			TooltipProvider provider;
 			try {
-				provider = (ITooltipProvider) a.ExtensionObject;
+				provider = (TooltipProvider) a.ExtensionObject;
 			} catch (Exception e) {
 				LoggingService.LogError ("Can't create tooltip provider:"+ a.ExtensionNode, e);
 				return;
@@ -202,7 +206,7 @@ namespace MonoDevelop.SourceEditor
 		
 		protected override void OptionsChanged (object sender, EventArgs args)
 		{
-			if (view.Control != null) {
+			if (view != null && view.Control != null) {
 				if (!Options.ShowFoldMargin)
 					this.Document.ClearFoldSegments ();
 			}
@@ -232,10 +236,13 @@ namespace MonoDevelop.SourceEditor
 		
 		bool ExtensionKeyPress (Gdk.Key key, uint ch, Gdk.ModifierType state)
 		{
+			isInKeyStroke = true;
 			try {
 				return Extension.KeyPress (key, (char)ch, state);
 			} catch (Exception ex) {
 				ReportExtensionError (ex);
+			} finally {
+				isInKeyStroke = false;
 			}
 			return false;
 		}
@@ -312,9 +319,6 @@ namespace MonoDevelop.SourceEditor
 			DocumentLine line = Document.GetLine (Caret.Line);
 			if (line == null)
 				return true;
-			bool inChar = false;
-			bool inComment = false;
-			bool inString = false;
 			//			string escape = "\"";
 			var stack = line.StartSpan.Clone ();
 			var sm = Document.SyntaxMode as SyntaxMode;
@@ -323,24 +327,11 @@ namespace MonoDevelop.SourceEditor
 			foreach (Span span in stack) {
 				if (string.IsNullOrEmpty (span.Color))
 					continue;
-				if (span.Color == "string.other") {
-					inStringOrComment = inChar = inString = true;
-					break;
-				}
-				if (span.Color == "string.single" || span.Color == "string.double" || span.Color.StartsWith ("comment")) {
+				if (span.Color.StartsWith ("String", StringComparison.Ordinal) || span.Color.StartsWith ("Comment", StringComparison.Ordinal)) {
 					inStringOrComment = true;
-					inChar |= span.Color == "string.single";
-					inComment |= span.Color.StartsWith ("comment");
-					inString = !inChar && !inComment;
-					//escape = span.Escape;
 					break;
 				}
 			}
-			if (Caret.Offset > 0) {
-				char c = GetCharAt (Caret.Offset - 1);
-				if (c == '"' || c == '\'')
-					inStringOrComment = inChar = inString = true;
-			}
 
 			// insert template when space is typed (currently disabled - it's annoying).
 			bool templateInserted = false;
@@ -357,13 +348,13 @@ namespace MonoDevelop.SourceEditor
 			// special handling for escape chars inside ' and "
 			if (Caret.Offset > 0) {
 				char charBefore = Document.GetCharAt (Caret.Offset - 1);
-				if (inStringOrComment && (ch == '"' || (inChar && ch == '\'')) && charBefore == '\\')
+				if (inStringOrComment && ch == '"' && charBefore == '\\')
 					skipChar = null;
 			}
 			char insertionChar = '\0';
 			bool insertMatchingBracket = false;
 			IDisposable undoGroup = null;
-			if (skipChar == null && Options.AutoInsertMatchingBracket && braceIndex >= 0) {
+			if (skipChar == null && Options.AutoInsertMatchingBracket && braceIndex >= 0 && !IsSomethingSelected) {
 				if (!inStringOrComment) {
 					char closingBrace = closingBrackets [braceIndex];
 					char openingBrace = openBrackets [braceIndex];
@@ -383,7 +374,7 @@ namespace MonoDevelop.SourceEditor
 					}
 				} else {
 					char charBefore = Document.GetCharAt (Caret.Offset - 1);
-					if (!inString && !inComment && !inChar && ch == '"' && charBefore != '\\') {
+					if (!inStringOrComment && ch == '"' && charBefore != '\\') {
 						insertMatchingBracket = true;
 						insertionChar = '"';
 					}
@@ -394,21 +385,25 @@ namespace MonoDevelop.SourceEditor
 			if (insertMatchingBracket)
 				undoGroup = Document.OpenUndoGroup ();
 
+			var oldMode = Caret.IsInInsertMode;
 			if (skipChar != null) {
-				Caret.Offset++;
+				Caret.IsInInsertMode = false;
 				skipChars.Remove (skipChar);
-			} else {
-				if (Extension != null) {
-					if (ExtensionKeyPress (key, ch, state)) 
-						result = base.OnIMProcessedKeyPressEvent (key, ch, state);
-					if (returnBetweenBraces)
-						HitReturn ();
-				} else {
-					result = base.OnIMProcessedKeyPressEvent (key, ch, state);
-					if (returnBetweenBraces)
-						HitReturn ();
-				}
 			}
+			if (Extension != null) {
+				if (ExtensionKeyPress (key, ch, state)) 
+					result = base.OnIMProcessedKeyPressEvent (key, ch, state);
+				if (returnBetweenBraces)
+					HitReturn ();
+			} else {
+				result = base.OnIMProcessedKeyPressEvent (key, ch, state);
+				if (returnBetweenBraces)
+					HitReturn ();
+			}
+			if (skipChar != null) {
+				Caret.IsInInsertMode = oldMode;
+			}
+
 			if (insertMatchingBracket) {
 				GetTextEditorData ().EnsureCaretIsNotVirtual ();
 				int offset = Caret.Offset;
@@ -556,24 +551,21 @@ namespace MonoDevelop.SourceEditor
 		
 		void ShowPopup (Gdk.EventButton evt)
 		{
-			// Fire event that will close an open outo complete window
 			view.FireCompletionContextChanged ();
+			CompletionWindowManager.HideWindow ();
+			ParameterInformationWindowManager.HideWindow (null, view);
 			HideTooltip ();
 			const string menuPath = "/MonoDevelop/SourceEditor2/ContextMenu/Editor";
 			var ctx = ExtensionContext ?? AddinManager.AddinEngine;
 			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet (ctx, menuPath);
 			Gtk.Menu menu = IdeApp.CommandService.CreateMenu (cset);
-			
 			var imMenu = CreateInputMethodMenuItem (GettextCatalog.GetString ("_Input Methods"));
 			if (imMenu != null) {
 				menu.Append (new SeparatorMenuItem ());
 				menu.Append (imMenu);
 			}
 			
-			menu.Destroyed += delegate {
-				this.QueueDraw ();
-			};
-			
+			menu.Hidden += HandleMenuHidden; 
 			if (evt != null) {
 				GtkWorkarounds.ShowContextMenu (menu, this, evt);
 			} else {
@@ -581,6 +573,16 @@ namespace MonoDevelop.SourceEditor
 				GtkWorkarounds.ShowContextMenu (menu, this, new Gdk.Rectangle (pt.X, pt.Y, 1, (int)LineHeight));
 			}
 		}
+
+		void HandleMenuHidden (object sender, EventArgs e)
+		{	
+			var menu = sender as Gtk.Menu;
+			menu.Hidden -= HandleMenuHidden; 
+			GLib.Timeout.Add (10, delegate {
+				menu.Destroy ();
+				return false;
+			});
+		}
 		
 #region Templates
 		int FindPrevWordStart (int offset)
@@ -631,25 +633,9 @@ namespace MonoDevelop.SourceEditor
 				var result = template.InsertTemplateContents (document);
 
 				var links = result.TextLinks;
-				if (PropertyService.Get ("OnTheFlyFormatting", true)) {
-					var prettyPrinter = CodeFormatterService.GetFormatter (Document.MimeType);
-					if (prettyPrinter != null) {
-						int endOffset = result.InsertPosition + result.Code.Length;
-						//						string oldText = Document.GetTextAt (result.InsertPosition, result.Code.Length);
-						//						var policies = document.Project != null ? document.Project.Policies : null;
-						var oldVersion = Document.Version;
-						prettyPrinter.OnTheFlyFormat (document, result.InsertPosition, endOffset);
-						foreach (var textLink in links) {
-							for (int i = 0; i < textLink.Links.Count; i++) {
-								var segment = textLink.Links [i];
-								var translatedOffset = oldVersion.MoveOffsetTo (Document.Version, result.InsertPosition + segment.Offset) - result.InsertPosition;
-								textLink.Links [i] = new TextSegment (translatedOffset, segment.Length);
-							}
-						}
-					}
-				}
 
 				var tle = new TextLinkEditMode (this, result.InsertPosition, links);
+				tle.TextLinkMode = TextLinkMode.General;
 				if (tle.ShouldStartTextLinkMode) {
 					tle.OldMode = CurrentMode;
 					tle.StartMode ();
@@ -658,22 +644,35 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 
-		protected override void HAdjustmentValueChanged ()
+		protected override void OnScrollAdjustmentsSet()
 		{
-			base.HAdjustmentValueChanged ();
-			if (!isInKeyStroke) {
-				CompletionWindowManager.HideWindow ();
-				ParameterInformationWindowManager.HideWindow (null, view);
+			UnregisterAdjustments ();
+			if (HAdjustment != null) {
+				cachedHAdjustment = HAdjustment;
+				HAdjustment.ValueChanged += HAdjustment_ValueChanged;
+			}
+			if (VAdjustment != null) {
+				cachedVAdjustment = VAdjustment;
+				VAdjustment.ValueChanged += VAdjustment_ValueChanged;
 			}
 		}
-		
-		protected override void VAdjustmentValueChanged ()
+
+		void VAdjustment_ValueChanged (object sender, EventArgs e)
 		{
-			base.VAdjustmentValueChanged ();
 			CompletionWindowManager.HideWindow ();
 			ParameterInformationWindowManager.HideWindow (null, view);
 		}
-		
+
+		void HAdjustment_ValueChanged (object sender, EventArgs e)
+		{
+			if (!isInKeyStroke) {
+				CompletionWindowManager.HideWindow ();
+				ParameterInformationWindowManager.HideWindow (null, view);
+			} else {
+				CompletionWindowManager.RepositionWindow ();
+				ParameterInformationWindowManager.RepositionWindow (null, view);
+			}
+		}
 		
 #endregion
 		
@@ -983,12 +982,6 @@ namespace MonoDevelop.SourceEditor
 			StartCaretPulseAnimation ();
 		}
 		
-		[CommandHandler (MonoDevelop.SourceEditor.SourceEditorCommands.ToggleCodeFocus)]
-		internal void OnToggleCodeFocus ()
-		{
-			foldMarkerMargin.IsInCodeFocusMode = !foldMarkerMargin.IsInCodeFocusMode;
-		}
-		
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.TransposeCharacters)]
 		internal void TransposeCharacters ()
 		{
