@@ -36,13 +36,21 @@ using MonoDevelop.Core;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.Ide.CodeCompletion;
 
 namespace MonoDevelop.SourceEditor
 {
-	
-	
+	public enum EditorFontType {
+		// Default Monospace font as set in the user's GNOME font properties
+		DefaultMonospace,
+		
+		// Default Sans font as set in the user's GNOME font properties
+		DefaultSans,
+		
+		// Custom font, will need to get the FontName property for more specifics
+		UserSpecified
+	}
+
 	public interface ISourceEditorOptions : Mono.TextEditor.ITextEditorOptions
 	{
 		
@@ -54,9 +62,7 @@ namespace MonoDevelop.SourceEditor
 		bool TabIsReindent {get; }
 		bool AutoInsertMatchingBracket {get; }
 		bool EnableCodeCompletion {get; }
-		bool EnableQuickFinder {get; }
 		bool UnderlineErrors {get; }
-		IndentStyle IndentStyle {get; }
 		EditorFontType EditorFontType {get; }
 		bool UseViModes {get; }
 	}
