@@ -33,7 +33,6 @@ using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
-using MonoDevelop.Ide.Commands;
 using Document = Mono.TextEditor.TextDocument;
 using Services = MonoDevelop.Projects.Services;
 using System.Threading;
@@ -45,7 +44,8 @@ using ICSharpCode.NRefactory.TypeSystem;
 using MonoDevelop.Ide.TypeSystem;
 using Mono.TextEditor.Highlighting;
 using MonoDevelop.SourceEditor.QuickTasks;
-using ICSharpCode.NRefactory.CSharp;
+using ICSharpCode.NRefactory.Semantics;
+using ICSharpCode.NRefactory.Refactoring;
 
 namespace MonoDevelop.SourceEditor
 {
@@ -66,17 +66,17 @@ namespace MonoDevelop.SourceEditor
 //		bool canShowClassBrowser;
 		ISourceEditorOptions options {
 			get {
-				return this.textEditor.Options;
+				return textEditor.Options;
 			}
 		}
 		
-		bool isDisposed = false;
+		bool isDisposed;
 		
 		ParsedDocument parsedDocument;
 		
-		readonly MonoDevelop.SourceEditor.ExtensibleTextEditor textEditor;
-		MonoDevelop.SourceEditor.ExtensibleTextEditor splittedTextEditor;
-		MonoDevelop.SourceEditor.ExtensibleTextEditor lastActiveEditor;
+		readonly ExtensibleTextEditor textEditor;
+		ExtensibleTextEditor splittedTextEditor;
+		ExtensibleTextEditor lastActiveEditor;
 		
 		public MonoDevelop.SourceEditor.ExtensibleTextEditor TextEditor {
 			get {
@@ -84,15 +84,9 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 		
-		public TextEditorContainer TextEditorContainer {
-			get {
-				return lastActiveEditor == textEditor ? textEditorContainer : splittedTextEditorContainer;
-			}
-		}
-		
 		public Ambience Ambience {
 			get {
-				string fileName = this.view.IsUntitled ? this.view.UntitledName : this.view.ContentName;
+				string fileName = view.IsUntitled ? view.UntitledName : view.ContentName;
 				return AmbienceService.GetAmbienceForFile (fileName);
 			}
 		}
@@ -113,7 +107,7 @@ namespace MonoDevelop.SourceEditor
 			if (secondsw != null)
 				secondsw.RemoveQuickTaskProvider (provider);
 		}		
-		
+
 		
 		List<IUsageProvider> usageProvider = new List<IUsageProvider> ();
 		public void AddUsageTaskProvider (IUsageProvider provider)
@@ -162,6 +156,36 @@ namespace MonoDevelop.SourceEditor
 		public Gtk.VBox Vbox {
 			get { return this.vbox; }
 		}
+
+		public bool SearchWidgetHasFocus {
+			get {
+				if (HasAnyFocusedChild (searchAndReplaceWidget) || HasAnyFocusedChild (gotoLineNumberWidget))
+					return true;
+				return false;
+			}
+		}
+
+		static bool HasAnyFocusedChild (Widget widget)
+		{
+			// Seems that this is the only reliable way doing it on os x and linux :/
+			if (widget == null)
+				return false;
+			var stack = new Stack<Widget> ();
+			stack.Push (widget);
+			while (stack.Count > 0) {
+				var cur = stack.Pop ();
+				if (cur.HasFocus) {
+					return true;
+				}
+				var c = cur as Gtk.Container;
+				if (c!= null) {
+					foreach (var child in c.Children) {
+						stack.Push (child);
+					}
+				}
+			}
+			return false;
+		}
 		
 		public class Border : Gtk.DrawingArea
 		{
@@ -176,7 +200,7 @@ namespace MonoDevelop.SourceEditor
 		class DecoratedScrolledWindow : HBox
 		{
 			SourceEditorWidget parent;
-			SmartScrolledWindow scrolledWindow;
+			ScrolledWindow scrolledWindow;
 			
 			QuickTaskStrip strip;
 			
@@ -191,59 +215,53 @@ namespace MonoDevelop.SourceEditor
 					return scrolledWindow.Vadjustment;
 				}
 			}
-			
+
 			public DecoratedScrolledWindow (SourceEditorWidget parent)
 			{
 				this.parent = parent;
 				this.strip = new QuickTaskStrip ();
-				/*
-				Border border = new Border ();
-				border.HeightRequest = 1;
-				PackStart (border, false, true, 0);
-								 
-				HBox box = new HBox ();
-				
-				border = new Border ();
-				border.WidthRequest = 1;
-				box.PackStart (border, false, true, 0);
-				
-				scrolledWindow = new ScrolledWindow ();
-				scrolledWindow.BorderWidth = 0;
-				scrolledWindow.ShadowType = ShadowType.None;
-				scrolledWindow.ButtonPressEvent += PrepareEvent;
-				box.PackStart (scrolledWindow, true, true, 0);
-				
-				
-				border = new Border ();
-				border.WidthRequest = 1;
-				box.PackStart (border, false, true, 0);
-				
-				PackStart (box, true, true, 0);
-				
-				border = new Border ();
-				border.HeightRequest = 1;
-				PackStart (border, false, true, 0);*/
-				
-				scrolledWindow = new SmartScrolledWindow ();
-//				scrolledWindow.BorderWidth = 0;
-//				scrolledWindow.ShadowType = ShadowType.In;
+
+				scrolledWindow = new CompactScrolledWindow ();
 				scrolledWindow.ButtonPressEvent += PrepareEvent;
 				PackStart (scrolledWindow, true, true, 0);
-				if (parent.quickTaskProvider.Count > 0) {
-					strip.VAdjustment = scrolledWindow.Vadjustment;
-					scrolledWindow.ReplaceVScrollBar (strip);
+				strip.VAdjustment = scrolledWindow.Vadjustment;
+				PackEnd (strip, false, true, 0);
+
+				parent.quickTaskProvider.ForEach (AddQuickTaskProvider);
+
+				QuickTaskStrip.EnableFancyFeatures.Changed += FancyFeaturesChanged;
+				FancyFeaturesChanged (null, null);
+			}
+
+			void FancyFeaturesChanged (object sender, EventArgs e)
+			{
+				if (QuickTaskStrip.EnableFancyFeatures) {
+					GtkWorkarounds.SetOverlayScrollbarPolicy (scrolledWindow, PolicyType.Automatic, PolicyType.Never);
+					scrolledWindow.VScrollbar.SizeRequested += SuppressSize;
+					scrolledWindow.VScrollbar.ExposeEvent += SuppressExpose;
 				} else {
-					strip.Visible = false;
+					GtkWorkarounds.SetOverlayScrollbarPolicy (scrolledWindow, PolicyType.Automatic, PolicyType.Automatic);
+					scrolledWindow.VScrollbar.SizeRequested -= SuppressSize;
+					scrolledWindow.VScrollbar.ExposeEvent -= SuppressExpose;
 				}
-				parent.quickTaskProvider.ForEach (p => AddQuickTaskProvider (p));
+				QueueResize ();
+			}
+
+			[GLib.ConnectBefore]
+			static void SuppressExpose (object o, ExposeEventArgs args)
+			{
+				args.RetVal = true;
+			}
+
+			[GLib.ConnectBefore]
+			static void SuppressSize (object o, SizeRequestedArgs args)
+			{
+				args.Requisition = Requisition.Zero;
+				args.RetVal = true;
 			}
 			
 			public void AddQuickTaskProvider (IQuickTaskProvider p)
 			{
-				if (!strip.Visible) {
-					strip.VAdjustment = scrolledWindow.Vadjustment;
-					scrolledWindow.ReplaceVScrollBar (strip);
-				}
 				p.TasksUpdated += HandleTasksUpdated; 
 			}
 
@@ -261,10 +279,6 @@ namespace MonoDevelop.SourceEditor
 
 			public void AddUsageProvider (IUsageProvider p)
 			{
-				if (!strip.Visible) {
-					strip.VAdjustment = scrolledWindow.Vadjustment;
-					scrolledWindow.ReplaceVScrollBar (strip);
-				}
 				p.UsagesUpdated += (sender, e) => strip.Update (p);
 			}
 			
@@ -272,7 +286,8 @@ namespace MonoDevelop.SourceEditor
 			{
 				if (scrolledWindow.Child != null)
 					RemoveEvents ();
-				
+
+				QuickTaskStrip.EnableFancyFeatures.Changed -= FancyFeaturesChanged;
 				scrolledWindow.ButtonPressEvent -= PrepareEvent;
 				base.OnDestroyed ();
 			}
@@ -282,38 +297,38 @@ namespace MonoDevelop.SourceEditor
 				args.RetVal = true;
 			}
 		
-			public void SetTextEditor (TextEditorContainer container)
+			public void SetTextEditor (TextEditor container)
 			{
 				scrolledWindow.Child = container;
-				this.strip.TextEditor = container.TextEditorWidget;
+				this.strip.TextEditor = container;
 //				container.TextEditorWidget.EditorOptionsChanged += OptionsChanged;
-				container.TextEditorWidget.Caret.ModeChanged += parent.UpdateLineColOnEventHandler;
-				container.TextEditorWidget.Caret.PositionChanged += parent.CaretPositionChanged;
-				container.TextEditorWidget.SelectionChanged += parent.UpdateLineColOnEventHandler;
+				container.Caret.ModeChanged += parent.UpdateLineColOnEventHandler;
+				container.Caret.PositionChanged += parent.CaretPositionChanged;
+				container.SelectionChanged += parent.UpdateLineColOnEventHandler;
 			}
 			
 			void OptionsChanged (object sender, EventArgs e)
 			{
 				TextEditor editor = (TextEditor)sender;
-				scrolledWindow.ModifyBg (StateType.Normal, editor.ColorStyle.Default.BackgroundColor);
+				scrolledWindow.ModifyBg (StateType.Normal, (Mono.TextEditor.HslColor)editor.ColorStyle.PlainText.Background);
 			}
 			
 			void RemoveEvents ()
 			{
-				TextEditorContainer container = scrolledWindow.Child as TextEditorContainer;
+				var container = scrolledWindow.Child as TextEditor;
 				if (container == null) {
 					LoggingService.LogError ("can't remove events from text editor container.");
 					return;
 				}
 //				container.TextEditorWidget.EditorOptionsChanged -= OptionsChanged;
-				container.TextEditorWidget.Caret.ModeChanged -= parent.UpdateLineColOnEventHandler;
-				container.TextEditorWidget.Caret.PositionChanged -= parent.CaretPositionChanged;
-				container.TextEditorWidget.SelectionChanged -= parent.UpdateLineColOnEventHandler;
+				container.Caret.ModeChanged -= parent.UpdateLineColOnEventHandler;
+				container.Caret.PositionChanged -= parent.CaretPositionChanged;
+				container.SelectionChanged -= parent.UpdateLineColOnEventHandler;
 			}
 			
-			public TextEditorContainer RemoveTextEditor ()
+			public TextEditor RemoveTextEditor ()
 			{
-				TextEditorContainer child = scrolledWindow.Child as TextEditorContainer;
+				var child = scrolledWindow.Child as TextEditor;
 				if (child == null)
 					return null;
 				RemoveEvents ();
@@ -323,20 +338,21 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 		
-		TextEditorContainer textEditorContainer;
 		public SourceEditorWidget (SourceEditorView view)
 		{
 			this.view = view;
 			vbox.SetSizeRequest (32, 32);
 			this.lastActiveEditor = this.textEditor = new MonoDevelop.SourceEditor.ExtensibleTextEditor (view);
-			this.textEditor.FocusInEvent += (o, s) => lastActiveEditor = (ExtensibleTextEditor)o;
-			this.textEditor.FocusOutEvent += delegate {
-				if (this.splittedTextEditor == null || !splittedTextEditor.HasFocus)
+			this.textEditor.TextArea.FocusInEvent += (o, s) => {
+				lastActiveEditor = (ExtensibleTextEditor)((TextArea)o).GetTextEditorData ().Parent;
+				view.FireCompletionContextChanged ();
+			};
+			this.textEditor.TextArea.FocusOutEvent += delegate {
+				if (this.splittedTextEditor == null || !splittedTextEditor.TextArea.HasFocus)
 					OnLostFocus ();
 			};
 			mainsw = new DecoratedScrolledWindow (this);
-			this.textEditorContainer = new TextEditorContainer (textEditor);
-			mainsw.SetTextEditor (textEditorContainer);
+			mainsw.SetTextEditor (textEditor);
 			
 			vbox.PackStart (mainsw, true, true, 0);
 			
@@ -355,12 +371,17 @@ namespace MonoDevelop.SourceEditor
 				RemoveErrorUndelinesResetTimerId ();
 				StopParseInfoThread ();
 				KillWidgets ();
-				
+
+				foreach (var provider in quickTaskProvider.ToArray ()) {
+					RemoveQuickTaskProvider (provider);
+				}
+
 				this.lastActiveEditor = null;
 				this.splittedTextEditor = null;
 				view = null;
-				
-				IdeApp.Workbench.StatusBar.ClearCaretState ();
+				parsedDocument = null;
+
+//				IdeApp.Workbench.StatusBar.ClearCaretState ();
 				if (parseInformationUpdaterWorkerThread != null) {
 					parseInformationUpdaterWorkerThread.Dispose ();
 					parseInformationUpdaterWorkerThread = null;
@@ -375,8 +396,6 @@ namespace MonoDevelop.SourceEditor
 
 		void OnLostFocus ()
 		{
-			//clears search status messages
-			IdeApp.Workbench.StatusBar.ShowReady ();
 		}
 
 		void UpdateLineColOnEventHandler (object sender, EventArgs e)
@@ -387,8 +406,7 @@ namespace MonoDevelop.SourceEditor
 		void ResetFocusChain ()
 		{
 			List<Widget> focusChain = new List<Widget> ();
-			
-			focusChain.Add (this.textEditor);
+			focusChain.Add (this.textEditor.TextArea);
 			if (this.searchAndReplaceWidget != null) {
 				focusChain.Add (this.searchAndReplaceWidget);
 			}
@@ -431,7 +449,7 @@ namespace MonoDevelop.SourceEditor
 			if (!options.ShowFoldMargin)
 				return;
 			// don't update parsed documents that contain errors - the foldings from there may be invalid.
-			if (doc.HasFoldSegments && parsedDocument.Errors.Any (err => err.ErrorType == ErrorType.Error))
+			if (parsedDocument.HasErrors)
 				return;
 			try {
 				List<FoldSegment > foldSegments = new List<FoldSegment> ();
@@ -645,13 +663,7 @@ namespace MonoDevelop.SourceEditor
 		
 		public bool EditorHasFocus {
 			get {
-				Gtk.Container c = vbox;
-				while (c != null) {
-					if (c.FocusChild == textEditor)
-						return true;
-					c = c.FocusChild as Gtk.Container;
-				}
-				return false;
+				return TextEditor.TextArea.HasFocus;
 			}
 		}
 
@@ -696,9 +708,7 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 		DecoratedScrolledWindow secondsw;
-		TextEditorContainer splittedTextEditorContainer;
-		
-		
+
 		public void Split (bool vSplit)
 		{
 			double vadjustment = this.mainsw.Vadjustment.Value;
@@ -720,20 +730,25 @@ namespace MonoDevelop.SourceEditor
 				}
 			};
 			secondsw = new DecoratedScrolledWindow (this);
-			this.splittedTextEditor = new MonoDevelop.SourceEditor.ExtensibleTextEditor (view, this.textEditor.Options, textEditor.Document);
-			this.splittedTextEditor.FocusInEvent += (o, s) => lastActiveEditor = (ExtensibleTextEditor)o;
-			this.splittedTextEditor.FocusOutEvent += delegate {
-				 if (!textEditor.HasFocus)
+			splittedTextEditor = new MonoDevelop.SourceEditor.ExtensibleTextEditor (view, textEditor.Options, textEditor.Document);
+			splittedTextEditor.TextArea.FocusInEvent += (o, s) => {
+				lastActiveEditor = (ExtensibleTextEditor)((TextArea)o).GetTextEditorData ().Parent;
+				view.FireCompletionContextChanged ();
+			};
+			splittedTextEditor.TextArea.FocusOutEvent += delegate {
+				 if (!textEditor.TextArea.HasFocus)
 					OnLostFocus ();
 			};
-			this.splittedTextEditor.Extension = textEditor.Extension;
-			
-			this.splittedTextEditorContainer = new TextEditorContainer (this.splittedTextEditor);
-			secondsw.SetTextEditor (this.splittedTextEditorContainer);
+			splittedTextEditor.Extension = textEditor.Extension;
+			if (textEditor.GetTextEditorData ().HasIndentationTracker)
+				splittedTextEditor.GetTextEditorData ().IndentationTracker = textEditor.GetTextEditorData ().IndentationTracker;
+			splittedTextEditor.Document.BracketMatcher = textEditor.Document.BracketMatcher;
+
+			secondsw.SetTextEditor (splittedTextEditor);
 			splitContainer.Add2 (secondsw);
 			
 			vbox.PackStart (splitContainer, true, true, 0);
-			this.splitContainer.Position = (vSplit ? vbox.Allocation.Height : vbox.Allocation.Width) / 2 - 1;
+			splitContainer.Position = (vSplit ? vbox.Allocation.Height : vbox.Allocation.Width) / 2 - 1;
 			
 			vbox.ShowAll ();
 			secondsw.Vadjustment.Value = mainsw.Vadjustment.Value = vadjustment; 
@@ -744,16 +759,16 @@ namespace MonoDevelop.SourceEditor
 		{
 			// destroy old scrolled window to work around Bug 526721 - When splitting window vertically, 
 			// the slider under left split is not shown unitl window is resized
-			double vadjustment = this.mainsw.Vadjustment.Value;
-			double hadjustment = this.mainsw.Hadjustment.Value;
+			double vadjustment = mainsw.Vadjustment.Value;
+			double hadjustment = mainsw.Hadjustment.Value;
 			
-			var removedTextEditor = this.mainsw.RemoveTextEditor ();
-			this.mainsw.Destroy ();
+			var removedTextEditor = mainsw.RemoveTextEditor ();
+			mainsw.Destroy ();
 			
-			this.mainsw = new DecoratedScrolledWindow (this);
-			this.mainsw.SetTextEditor (removedTextEditor);
-			this.mainsw.Vadjustment.Value = vadjustment; 
-			this.mainsw.Hadjustment.Value = hadjustment;
+			mainsw = new DecoratedScrolledWindow (this);
+			mainsw.SetTextEditor (removedTextEditor);
+			mainsw.Vadjustment.Value = vadjustment; 
+			mainsw.Hadjustment.Value = hadjustment;
 			lastActiveEditor = textEditor;
 		}
 
@@ -787,9 +802,9 @@ namespace MonoDevelop.SourceEditor
 			if (messageBar == null) {
 				messageBar = new MonoDevelop.Components.InfoBar (MessageType.Warning);
 				messageBar.SetMessageLabel (GettextCatalog.GetString (
-					"<b>The file \"{0}\" has been changed outside of MonoDevelop.</b>\n" +
+					"<b>The file \"{0}\" has been changed outside of {1}.</b>\n" +
 					"Do you want to keep your changes, or reload the file from disk?",
-					EllipsizeMiddle (Document.FileName, 50)));
+					EllipsizeMiddle (Document.FileName, 50), BrandingService.ApplicationName));
 				
 				var b1 = new Button (GettextCatalog.GetString ("_Reload from disk"));
 				b1.Image = ImageService.GetImage (Gtk.Stock.Refresh, IconSize.Button);
@@ -839,17 +854,25 @@ namespace MonoDevelop.SourceEditor
 		#region Eol marker check
 		internal bool UseIncorrectMarkers { get; set; }
 		internal bool HasIncorrectEolMarker {
+			get {
+				string eol = DetectedEolMarker;
+				if (eol == null)
+					return false;
+				return eol != textEditor.Options.DefaultEolMarker;
+			}
+		}
+		string DetectedEolMarker {
 			get {
 				if (textEditor.IsDisposed) {
 					LoggingService.LogWarning ("SourceEditorWidget.cs: HasIncorrectEolMarker was called on disposed source editor widget." + Environment.NewLine + Environment.StackTrace);
-					return false;
+					return null;
 				}
 				var firstLine = Document.GetLine (1);
 				if (firstLine != null && firstLine.DelimiterLength > 0) {
 					string firstDelimiter = Document.GetTextAt (firstLine.Length, firstLine.DelimiterLength);
-					return firstDelimiter != textEditor.Options.DefaultEolMarker;
+					return firstDelimiter;
 				}
-				return false;
+				return null;
 			}
 		}
 
@@ -891,8 +914,10 @@ namespace MonoDevelop.SourceEditor
 			
 			if (messageBar == null) {
 				messageBar = new MonoDevelop.Components.InfoBar (MessageType.Warning);
+				string detectedEol = DetectedEolMarker.Replace ("\n", "NL").Replace ("\r", "CR");
+				string defaultEol = textEditor.Options.DefaultEolMarker.Replace ("\n", "NL").Replace ("\r", "CR");
 				messageBar.SetMessageLabel (GettextCatalog.GetString (
-					"<b>The file \"{0}\" has line endings which differ from the policy settings.</b>\n" +
+					"<b>The file \"{0}\" has line endings (" + detectedEol + ") which differ from the policy settings(" + defaultEol + ").</b>\n" +
 					"Do you want to convert the line endings?",
 					EllipsizeMiddle (Document.FileName, 50)));
 				
@@ -942,7 +967,7 @@ namespace MonoDevelop.SourceEditor
 			view.WorkbenchWindow.ShowNotification = true;
 			
 			// Ensure that one file with incorret EOL markers is shown.
-			var currentView = IdeApp.Workbench.ActiveDocument.PrimaryView as SourceEditorView;
+			var currentView = IdeApp.Workbench.ActiveDocument.PrimaryView.GetContent<SourceEditorView> ();
 			if (currentView == null || !currentView.IsDirty || !currentView.SourceEditorWidget.HasIncorrectEolMarker || currentView.SourceEditorWidget.UseIncorrectMarkers)
 				view.WorkbenchWindow.SelectWindow ();
 		}
@@ -953,11 +978,11 @@ namespace MonoDevelop.SourceEditor
 			TextEditor.Visible = false;
 			if (messageBar == null) {
 				messageBar = new MonoDevelop.Components.InfoBar (MessageType.Warning);
-				messageBar.SetMessageLabel (GettextCatalog.GetString (
+				messageBar.SetMessageLabel (BrandingService.BrandApplicationName (GettextCatalog.GetString (
 						"<b>An autosave file has been found for this file.</b>\n" +
 						"This could mean that another instance of MonoDevelop is editing this " +
 						"file, or that MonoDevelop crashed with unsaved changes.\n\n" +
-					    "Do you want to use the original file, or load from the autosave file?"));
+					    "Do you want to use the original file, or load from the autosave file?")));
 				
 				Button b1 = new Button (GettextCatalog.GetString("_Use original file"));
 				b1.Image = ImageService.GetImage (Gtk.Stock.Refresh, IconSize.Button);
@@ -1022,6 +1047,9 @@ namespace MonoDevelop.SourceEditor
 		public void Reload ()
 		{
 			try {
+				if (!System.IO.File.Exists (view.ContentName))
+					return;
+
 				view.StoreSettings ();
 				reloadSettings = true;
 				view.Load (view.ContentName);
@@ -1060,14 +1088,14 @@ namespace MonoDevelop.SourceEditor
 		
 		internal void UpdateLineCol ()
 		{
-			int offset = TextEditor.Caret.Offset;
-			if (offset < 0 || offset > TextEditor.Document.TextLength)
-				return;
-			DocumentLocation location = TextEditor.LogicalToVisualLocation (TextEditor.Caret.Location);
-			IdeApp.Workbench.StatusBar.ShowCaretState (TextEditor.Caret.Line,
-			                                           location.Column,
-			                                           TextEditor.IsSomethingSelected ? TextEditor.SelectionRange.Length : 0,
-			                                           TextEditor.Caret.IsInInsertMode);
+//			int offset = TextEditor.Caret.Offset;
+//			if (offset < 0 || offset > TextEditor.Document.TextLength)
+//				return;
+//			DocumentLocation location = TextEditor.LogicalToVisualLocation (TextEditor.Caret.Location);
+//			IdeApp.Workbench.StatusBar.ShowCaretState (TextEditor.Caret.Line,
+//			                                           location.Column,
+//			                                           TextEditor.IsSomethingSelected ? TextEditor.SelectionRange.Length : 0,
+//			                                           TextEditor.Caret.IsInInsertMode);
 		}
 		
 		#endregion
@@ -1174,7 +1202,7 @@ namespace MonoDevelop.SourceEditor
 				searchAndReplaceWidgetFrame.Child = searchAndReplaceWidget = new SearchAndReplaceWidget (TextEditor, searchAndReplaceWidgetFrame);
 				searchAndReplaceWidget.Destroyed += (sender, e) => RemoveSearchWidget ();
 				searchAndReplaceWidgetFrame.ShowAll ();
-				this.TextEditorContainer.AddAnimatedWidget (searchAndReplaceWidgetFrame, 300, Easing.ExponentialInOut, Blocking.Downstage, TextEditor.Allocation.Width - 400, -searchAndReplaceWidget.Allocation.Height);
+				this.TextEditor.AddAnimatedWidget (searchAndReplaceWidgetFrame, 300, Mono.TextEditor.Theatrics.Easing.ExponentialInOut, Blocking.Downstage, TextEditor.Allocation.Width - 400, -searchAndReplaceWidget.Allocation.Height);
 //				this.PackEnd (searchAndReplaceWidget);
 //				this.SetChildPacking (searchAndReplaceWidget, false, false, CHILD_PADDING, PackType.End);
 				//		searchAndReplaceWidget.ShowAll ();
@@ -1214,7 +1242,7 @@ namespace MonoDevelop.SourceEditor
 				gotoLineNumberWidgetFrame.Child = gotoLineNumberWidget = new GotoLineNumberWidget (textEditor, gotoLineNumberWidgetFrame);
 				gotoLineNumberWidget.Destroyed += (sender, e) => RemoveSearchWidget ();
 				gotoLineNumberWidgetFrame.ShowAll ();
-				TextEditorContainer.AddAnimatedWidget (gotoLineNumberWidgetFrame, 300, Easing.ExponentialInOut, Mono.TextEditor.Theatrics.Blocking.Downstage, this.TextEditor.Allocation.Width - 400, -gotoLineNumberWidget.Allocation.Height);
+				TextEditor.AddAnimatedWidget (gotoLineNumberWidgetFrame, 300, Mono.TextEditor.Theatrics.Easing.ExponentialInOut, Mono.TextEditor.Theatrics.Blocking.Downstage, this.TextEditor.Allocation.Width - 400, -gotoLineNumberWidget.Allocation.Height);
 				
 				ResetFocusChain ();
 			}
@@ -1250,10 +1278,21 @@ namespace MonoDevelop.SourceEditor
 		
 		void SetSearchPatternToSelection ()
 		{
+			if (!TextEditor.IsSomethingSelected) {
+				int start = textEditor.Caret.Offset;
+				int end = start;
+				while (start - 1 >= 0 && DynamicAbbrevHandler.IsIdentifierPart (textEditor.GetCharAt (start - 1)))
+					start--;
+
+				while (end < textEditor.Length && DynamicAbbrevHandler.IsIdentifierPart (textEditor.GetCharAt (end)))
+					end++;
+				textEditor.Caret.Offset = end;
+				TextEditor.SetSelection (start, end);
+			}
+
 			if (TextEditor.IsSomethingSelected) {
 				var pattern = FormatPatternToSelectionOption (TextEditor.SelectedText);
-					
-				TextEditor.SearchPattern = pattern;
+				SearchAndReplaceOptions.SearchPattern = pattern;
 				SearchAndReplaceWidget.UpdateSearchHistory (TextEditor.SearchPattern);
 			}
 			if (searchAndReplaceWidget != null)
@@ -1296,6 +1335,18 @@ namespace MonoDevelop.SourceEditor
 		{
 			DomRegion region;
 			var res = TextEditor.GetLanguageItem (TextEditor.Caret.Offset, out region);
+			if (res is UnknownIdentifierResolveResult) {
+				var uir = (UnknownIdentifierResolveResult)res;
+				IdeApp.HelpOperations.SearchHelpFor (uir.Identifier);
+				return;
+			}
+
+			if (res is UnknownMemberResolveResult) {
+				var uir = (UnknownMemberResolveResult)res;
+				IdeApp.HelpOperations.SearchHelpFor (uir.MemberName);
+				return;
+			}
+
 			string url = HelpService.GetMonoDocHelpUrl (res);
 			if (url != null)
 				IdeApp.HelpOperations.ShowHelp (url);
@@ -1305,7 +1356,7 @@ namespace MonoDevelop.SourceEditor
 		{
 			DomRegion region;
 			var res = TextEditor.GetLanguageItem (TextEditor.Caret.Offset, out region);
-			if (res == null || !IdeApp.HelpOperations.CanShowHelp (res))
+			if (res == null || !IdeApp.HelpOperations.CanShowHelp (res) && !(res is UnknownIdentifierResolveResult || res is UnknownMemberResolveResult))
 				cinfo.Bypass = true;
 		}
 		
@@ -1360,7 +1411,7 @@ namespace MonoDevelop.SourceEditor
 				}
 				string startLineText = Document.GetTextAt (startLine.Offset, startLine.Length);
 				string endLineText = Document.GetTextAt (endLine.Offset, endLine.Length);
-				if (startLineText.StartsWith (blockStart) && endLineText.EndsWith (blockEnd)) {
+				if (startLineText.StartsWith (blockStart) && endLineText.EndsWith (blockEnd, StringComparison.Ordinal)) {
 					textEditor.Remove (endLine.Offset + endLine.Length - blockEnd.Length, blockEnd.Length);
 					textEditor.Remove (startLine.Offset, blockStart.Length);
 					if (TextEditor.IsSomethingSelected) {
@@ -1395,7 +1446,7 @@ namespace MonoDevelop.SourceEditor
 					continue;
 				string text = Document.GetTextAt (line);
 				string trimmedText = text.TrimStart ();
-				if (!trimmedText.StartsWith (commentTag)) {
+				if (!trimmedText.StartsWith (commentTag, StringComparison.Ordinal)) {
 					comment = true;
 					break;
 				}
@@ -1569,7 +1620,6 @@ namespace MonoDevelop.SourceEditor
 			// may be null if no line is assigned to the error.
 			Wave = true;
 			
-			ColorName = info.ErrorType == ErrorType.Warning ? Mono.TextEditor.Highlighting.ColorScheme.WarningUnderlineString : Mono.TextEditor.Highlighting.ColorScheme.ErrorUnderlineString;
 			StartCol = Info.Region.BeginColumn + 1;
 			if (Info.Region.EndColumn > StartCol) {
 				EndCol = Info.Region.EndColumn;
@@ -1589,5 +1639,12 @@ namespace MonoDevelop.SourceEditor
 				EndCol = Info.Region.BeginColumn + o - start + 1;
 			}
 		}
+
+		public override void Draw (TextEditor editor, Cairo.Context cr, double y, LineMetrics metrics)
+		{
+			Color = Info.ErrorType == ErrorType.Warning ? editor.ColorStyle.UnderlineWarning.Color : editor.ColorStyle.UnderlineError.Color;
+
+			base.Draw (editor, cr, y, metrics);
+		}
 	}
-}
\ No newline at end of file
+}
