@@ -1,5 +1,6 @@
 using System;
 using System.IO;
+using System.Linq;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
@@ -8,12 +9,18 @@ using MonoDevelop.Ide.Gui.Components;
 using MonoDevelop.Ide;
 using System.Collections.Generic;
 using MonoDevelop.Core.Assemblies;
+using System.Threading.Tasks;
+using System.ServiceModel;
 
 namespace MonoDevelop.WebReferences.Commands
 {
 	/// <summary>Defines the properties and methods for the WebReferenceCommandHandler class.</summary>
 	public class WebReferenceCommandHandler : NodeCommandHandler
 	{
+		StatusBarContext UpdateReferenceContext {
+			get; set;
+		}
+		
 		/// <summary>Execute the command for adding a new web reference to a project.</summary>
 		[CommandHandler (MonoDevelop.WebReferences.WebReferenceCommands.Add)]
 		public void NewWebReference()
@@ -39,47 +46,81 @@ namespace MonoDevelop.WebReferences.Commands
 			dialog.NamespacePrefix = project.DefaultNamespace;
 			
 			try {
-				if (MessageService.RunCustomDialog (dialog) == (int)Gtk.ResponseType.Ok) {
-					dialog.SelectedService.GenerateFiles (project, dialog.Namespace, dialog.ReferenceName);
-					IdeApp.ProjectOperations.Save(project);
-				}
-			}
-			catch(Exception exception) {
+				if (MessageService.RunCustomDialog (dialog) != (int)Gtk.ResponseType.Ok)
+					return;
+
+				dialog.SelectedService.GenerateFiles (project, dialog.Namespace, dialog.ReferenceName);
+				IdeApp.ProjectOperations.Save(project);
+			} catch (Exception exception) {
 				MessageService.ShowException (exception);
 			} finally {
 				dialog.Destroy ();
 			}
 		}
+
+		[CommandUpdateHandler (MonoDevelop.WebReferences.WebReferenceCommands.Update)]
+		[CommandUpdateHandler (MonoDevelop.WebReferences.WebReferenceCommands.UpdateAll)]
+		void CanUpdateWebReferences (CommandInfo ci)
+		{
+			// This does not appear to work.
+			ci.Enabled = UpdateReferenceContext == null;
+		}
 		
 		/// <summary>Execute the command for updating a web reference in a project.</summary>
 		[CommandHandler (MonoDevelop.WebReferences.WebReferenceCommands.Update)]
 		public void Update()
 		{
-			using (StatusBarContext sbc = IdeApp.Workbench.StatusBar.CreateContext ()) {
-				sbc.BeginProgress (GettextCatalog.GetString ("Updating web reference"));
-				sbc.AutoPulse = true;
-				WebReferenceItem item = (WebReferenceItem) CurrentNode.DataItem;
-				DispatchService.BackgroundDispatchAndWait (item.Update);
-				IdeApp.ProjectOperations.Save (item.Project);
-				IdeApp.Workbench.StatusBar.ShowMessage("Updated Web Reference " + item.Name);
-			}
+			UpdateReferences (new [] { (WebReferenceItem) CurrentNode.DataItem });
 		}
-		
+
 		/// <summary>Execute the command for updating all web reference in a project.</summary>
 		[CommandHandler (MonoDevelop.WebReferences.WebReferenceCommands.UpdateAll)]
 		public void UpdateAll()
 		{
-			using (StatusBarContext sbc = IdeApp.Workbench.StatusBar.CreateContext ()) {
-				sbc.BeginProgress (GettextCatalog.GetString ("Updating web references"));
-				sbc.AutoPulse = true;
-				DotNetProject project = ((WebReferenceFolder) CurrentNode.DataItem).Project;
-				List<WebReferenceItem> items = new List<WebReferenceItem> (WebReferencesService.GetWebReferenceItems (project));
-				DispatchService.BackgroundDispatchAndWait (delegate {
-					foreach (var item in items)
-						item.Update();
+			DotNetProject project = ((WebReferenceFolder) CurrentNode.DataItem).Project;
+			UpdateReferences (WebReferencesService.GetWebReferenceItems (project).ToArray ());
+		}
+		
+		void UpdateReferences (IList<WebReferenceItem> items)
+		{
+			try {
+				UpdateReferenceContext = IdeApp.Workbench.StatusBar.CreateContext ();
+				UpdateReferenceContext.BeginProgress (GettextCatalog.GetPluralString ("Updating web reference", "Updating web references", items.Count));
+				
+				DispatchService.ThreadDispatch (() => {
+					for (int i = 0; i < items.Count; i ++) {
+						DispatchService.GuiDispatch (() => UpdateReferenceContext.SetProgressFraction (Math.Max (0.1, (double)i / items.Count)));
+						try {
+							items [i].Update();
+						} catch (Exception ex) {
+							DispatchService.GuiSyncDispatch (() => {
+								MessageService.ShowException (ex, GettextCatalog.GetString ("Failed to update Web Reference '{0}'", items [i].Name));
+								DisposeUpdateContext ();
+							});
+							return;
+						}
+					}
+					
+					DispatchService.GuiDispatch (() => {
+						// Make sure that we save all relevant projects, there should only be 1 though
+						foreach (var project in items.Select (i =>i.Project).Distinct ())
+							IdeApp.ProjectOperations.Save (project);
+						
+						IdeApp.Workbench.StatusBar.ShowMessage(GettextCatalog.GetPluralString ("Updated Web Reference {0}", "Updated Web References", items.Count, items[0].Name));
+						DisposeUpdateContext ();
+					});
 				});
-				IdeApp.ProjectOperations.Save (project);
-				IdeApp.Workbench.StatusBar.ShowMessage("Updated all Web References");
+			} catch {
+				DisposeUpdateContext ();
+				throw;
+			}
+		}
+	
+		void DisposeUpdateContext ()
+		{
+			if (UpdateReferenceContext != null) {
+				UpdateReferenceContext.Dispose ();
+				UpdateReferenceContext = null;
 			}
 		}
 		
@@ -107,5 +148,50 @@ namespace MonoDevelop.WebReferences.Commands
 			IdeApp.ProjectOperations.Save(project);
 			IdeApp.Workbench.StatusBar.ShowMessage("Deleted all Web References");
 		}
+
+		[CommandUpdateHandler (MonoDevelop.WebReferences.WebReferenceCommands.Configure)]
+		void CanConfigureWebReferences (CommandInfo ci)
+		{
+			var item = CurrentNode.DataItem as WebReferenceItem;
+			ci.Enabled = item != null && WCFConfigWidget.IsSupported (item);
+		}
+
+		/// <summary>Execute the command for configuring a web reference in a project.</summary>
+		[CommandHandler (MonoDevelop.WebReferences.WebReferenceCommands.Configure)]
+		public void Configure ()
+		{
+			var item = (WebReferenceItem) CurrentNode.DataItem;
+
+			if (!WCFConfigWidget.IsSupported (item))
+				return;
+
+			WCF.ReferenceGroup refgroup;
+			WCF.ClientOptions options;
+
+			try {
+				refgroup = WCF.ReferenceGroup.Read (item.MapFile.FilePath);
+				if (refgroup == null || refgroup.ClientOptions == null)
+					return;
+				options = refgroup.ClientOptions;
+			} catch {
+				return;
+			}
+
+			var dialog = new WebReferenceDialog (item, options);
+
+			try {
+				if (MessageService.RunCustomDialog (dialog) != (int)Gtk.ResponseType.Ok)
+					return;
+				if (!dialog.Modified)
+					return;
+				
+				refgroup.Save (item.MapFile.FilePath);
+				UpdateReferences (new [] { item });
+			} catch (Exception exception) {
+				MessageService.ShowException (exception);
+			} finally {
+				dialog.Destroy ();
+			}
+		}
 	}	
 }
