@@ -42,6 +42,7 @@ using MonoDevelop.Components.Docking;
 using MonoDevelop.Ide;
 using MonoDevelop.Projects;
 using Mono.TextEditor;
+using System.Linq;
 
 namespace MonoDevelop.NUnit
 {
@@ -420,6 +421,33 @@ namespace MonoDevelop.NUnit
 			}
 		}
 		
+		[CommandHandler (TestCommands.DebugTest)]
+		protected void OnDebugTest (object data)
+		{
+			var debugModeSet = Runtime.ProcessService.GetDebugExecutionMode ();
+			var mode = debugModeSet.ExecutionModes.First (m => m.Id == (string)data);
+			RunSelectedTest (mode.ExecutionHandler);
+		}
+
+		[CommandUpdateHandler (TestCommands.DebugTest)]
+		protected void OnUpdateDebugTest (CommandArrayInfo info)
+		{
+			var debugModeSet = Runtime.ProcessService.GetDebugExecutionMode ();
+			if (debugModeSet == null)
+				return;
+
+			UnitTest test = GetSelectedTest ();
+			if (test == null)
+				return;
+
+			foreach (var mode in debugModeSet.ExecutionModes) {
+				if (test.CanRun (mode.ExecutionHandler))
+					info.Add (GettextCatalog.GetString ("Debug Test ({0})", mode.Name), mode.Id);
+			}
+			if (info.Count == 1)
+				info [0].Text = GettextCatalog.GetString ("Debug Test");
+		}
+
 		public TestPad ()
 		{
 			base.TreeView.CurrentItemActivated += delegate {
@@ -453,7 +481,7 @@ namespace MonoDevelop.NUnit
 			UnitTest test = nav.DataItem as UnitTest;
 			if (test == null)
 				return;
-			TestSession.ResetResult (test.RootTest);
+			NUnitService.ResetResult (test.RootTest);
 			
 			this.buttonRun.Sensitive = false;
 			this.buttonRunAll.Sensitive = false;
@@ -716,7 +744,7 @@ namespace MonoDevelop.NUnit
 		void OnChartButtonPress (object o, Gtk.ButtonPressEventArgs args)
 		{
 			if (args.Event.TriggersContextMenu ()) {
-				IdeApp.CommandService.ShowContextMenu ("/MonoDevelop/NUnit/ContextMenu/TestChart");
+				IdeApp.CommandService.ShowContextMenu (Control, args.Event, "/MonoDevelop/NUnit/ContextMenu/TestChart");
 				args.RetVal = true;
 			}
 		}
