@@ -56,11 +56,7 @@ namespace MonoDevelop.NUnit
 			IdeApp.Workspace.WorkspaceItemClosed += OnWorkspaceChanged;
 			IdeApp.Workspace.ItemAddedToSolution += OnWorkspaceChanged;
 			IdeApp.Workspace.ItemRemovedFromSolution += OnWorkspaceChanged;
-			
-			ProjectService ps = MonoDevelop.Projects.Services.ProjectService;
-			ps.DataContext.IncludeType (typeof(UnitTestOptionsSet));
-			ps.DataContext.RegisterProperty (typeof(SolutionItemConfiguration), "UnitTestInformation", typeof(UnitTestOptionsSet));
-			
+
 			Mono.Addins.AddinManager.AddExtensionNodeHandler ("/MonoDevelop/NUnit/TestProviders", OnExtensionChange);
 		}
 		
@@ -191,6 +187,17 @@ namespace MonoDevelop.NUnit
 			}
 			return null;
 		}
+
+		public UnitTest SearchTestById (string id)
+		{
+			foreach (UnitTest t in RootTests) {
+				UnitTest r = SearchTestById (t, id);
+				if (r != null)
+					return r;
+			}
+			return null;
+		}
+
 		
 		UnitTest SearchTest (UnitTest test, string fullName)
 		{
@@ -198,7 +205,7 @@ namespace MonoDevelop.NUnit
 				return null;
 			if (test.FullName == fullName)
 				return test;
-			
+
 			UnitTestGroup group = test as UnitTestGroup;
 			if (group != null)  {
 				foreach (UnitTest t in group.Tests) {
@@ -209,7 +216,25 @@ namespace MonoDevelop.NUnit
 			}
 			return null;
 		}
-		
+
+		UnitTest SearchTestById (UnitTest test, string id)
+		{
+			if (test == null)
+				return null;
+			if (test.TestId == id)
+				return test;
+
+			UnitTestGroup group = test as UnitTestGroup;
+			if (group != null)  {
+				foreach (UnitTest t in group.Tests) {
+					UnitTest result = SearchTestById (t, id);
+					if (result != null)
+						return result;
+				}
+			}
+			return null;
+		}
+
 		public UnitTest FindRootTest (IWorkspaceObject item)
 		{
 			return FindRootTest (RootTests, item);
@@ -256,8 +281,12 @@ namespace MonoDevelop.NUnit
 		public UnitTest BuildTest (IWorkspaceObject entry)
 		{
 			foreach (ITestProvider p in providers) {
-				UnitTest t = p.CreateUnitTest (entry);
-				if (t != null) return t;
+				try {
+					UnitTest t = p.CreateUnitTest (entry);
+					if (t != null)
+						return t;
+				} catch {
+				}
 			}
 			return null;
 		}
@@ -278,11 +307,24 @@ namespace MonoDevelop.NUnit
 			if (TestSuiteChanged != null)
 				TestSuiteChanged (this, EventArgs.Empty);
 		}
-		
+
+		public static void ResetResult (UnitTest test)
+		{
+			if (test == null)
+				return;
+			test.ResetLastResult ();
+			UnitTestGroup group = test as UnitTestGroup;
+			if (group == null) 
+				return;
+			foreach (UnitTest t in new List<UnitTest> (group.Tests))
+				ResetResult (t);
+		}
+
 		public event EventHandler TestSuiteChanged;
 	}
 	
-	
+
+
 	class TestSession: IAsyncOperation, ITestProgressMonitor
 	{
 		UnitTest test;
@@ -310,7 +352,7 @@ namespace MonoDevelop.NUnit
 		void RunTests ()
 		{
 			try {
-				ResetResult (test);
+				NUnitService.ResetResult (test);
 				monitor.InitializeTestRun (test);
 				TestContext ctx = new TestContext (monitor, context, DateTime.Now);
 				test.Run (ctx);
@@ -332,18 +374,6 @@ namespace MonoDevelop.NUnit
 				Completed (this);
 		}
 		
-		public static void ResetResult (UnitTest test)
-		{
-			if (test == null)
-				return;
-			test.ResetLastResult ();
-			UnitTestGroup group = test as UnitTestGroup;
-			if (group == null) 
-				return;
-			foreach (UnitTest t in new List<UnitTest> (group.Tests))
-				ResetResult (t);
-		}
-		
 		void ITestProgressMonitor.BeginTest (UnitTest test)
 		{
 			monitor.BeginTest (test);
@@ -359,6 +389,11 @@ namespace MonoDevelop.NUnit
 			monitor.ReportRuntimeError (message, exception);
 		}
 		
+		void ITestProgressMonitor.WriteGlobalLog (string message)
+		{
+			monitor.WriteGlobalLog (message);
+		}
+
 		bool ITestProgressMonitor.IsCancelRequested {
 			get { return monitor.IsCancelRequested; }
 		}
