@@ -76,6 +76,14 @@ namespace MonoDevelop.NUnit
 			return res;
 		}
 		
+		public static UnitTestResult CreateInconclusive (string message)
+		{
+			UnitTestResult res = new UnitTestResult ();
+			res.status = ResultStatus.Inconclusive;
+			res.Message = message;
+			return res;
+		}
+		
 		public static UnitTestResult CreateSuccess ()
 		{
 			UnitTestResult res = new UnitTestResult ();
@@ -90,6 +98,7 @@ namespace MonoDevelop.NUnit
 		
 		public ResultStatus Status {
 			get { return status; }
+			set { status = value; }
 		}
 		
 		public bool IsFailure {
@@ -100,6 +109,10 @@ namespace MonoDevelop.NUnit
 			get { return ErrorsAndFailures == 0 && Passed > 0; }
 		}
 
+		public bool IsInconclusive {
+			get { return Passed == 0 && ErrorsAndFailures == 0 && Inconclusive > 0; }
+		}
+
 		public bool IsNotRun {
 			get {
 				return Passed == 0 && ErrorsAndFailures == 0 && TestsNotRun > 0;
