@@ -1,6 +1,7 @@
 
 using System;
 using MonoDevelop.Core;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.VersionControl.Subversion.Gui
 {
@@ -34,9 +35,8 @@ namespace MonoDevelop.VersionControl.Subversion.Gui
 			
 			EventHandler del = delegate {
 					ClientCertificateDialog dlg = new ClientCertificateDialog (realm, may_save);
-					dlg.TransientFor = MonoDevelop.Ide.Gui.IdeApp.Workbench.RootWindow;
 					try {
-						res = (dlg.Run () == (int) Gtk.ResponseType.Ok);
+						res = (MessageService.RunCustomDialog (dlg) == (int) Gtk.ResponseType.Ok);
 						if (res) {
 							local_save = dlg.Save;
 							local_cert_file = dlg.File;
