@@ -13,8 +13,6 @@ namespace MonoDevelop.VersionControl.Subversion
 {
 	public class SubversionRepository: UrlBasedRepository
 	{
-		FilePath rootPath;
-		
 		public SubversionRepository ()
 		{
 			Url = "svn://";
@@ -23,13 +21,9 @@ namespace MonoDevelop.VersionControl.Subversion
 		public SubversionRepository (SubversionVersionControl vcs, string url, FilePath rootPath): base (vcs)
 		{
 			Url = url;
-			this.rootPath = !rootPath.IsNullOrEmpty ? rootPath.CanonicalPath : null;
+			RootPath = !rootPath.IsNullOrEmpty ? rootPath.CanonicalPath : null;
 		}
-		
-		public FilePath RootPath {
-			get { return rootPath; }
-		}
-		
+
 		public override string[] SupportedProtocols {
 			get {
 				return new string[] {"svn", "svn+ssh", "http", "https", "file"};
@@ -47,7 +41,7 @@ namespace MonoDevelop.VersionControl.Subversion
 				
 				foreach (DirectoryEntry ent in Svn.ListUrl (Url, false)) {
 					if (ent.IsDirectory) {
-						SubversionRepository rep = new SubversionRepository (Svn, Url + "/" + ent.Name, null);
+						SubversionRepository rep = new SubversionRepository (VersionControlSystem, Url + "/" + ent.Name, null);
 						rep.Name = ent.Name;
 						list.Add (rep);
 					}
@@ -55,35 +49,36 @@ namespace MonoDevelop.VersionControl.Subversion
 				return list;
 			}
 		}
-		
-		SubversionVersionControl Svn {
-			get { return (SubversionVersionControl) VersionControlSystem; }
+
+		new SubversionVersionControl VersionControlSystem {
+			get { return (SubversionVersionControl)base.VersionControlSystem; }
+		}
+
+		SubversionBackend backend;
+		protected internal SubversionBackend Svn {
+			get {
+				if (backend == null)
+					backend = VersionControlSystem.CreateBackend ();
+				return backend;
+			}
 		}
 
 		bool IsVersioned (FilePath sourcefile)
 		{
-			return Svn.IsVersioned (sourcefile);
+			return GetVersionInfo (sourcefile, VersionInfoQueryFlags.IgnoreCache).IsVersioned;
 		}
 		
 		public override string GetBaseText (FilePath sourcefile)
 		{
-			// The base file will not exist if the file has just been
-			// added to svn and not committed
-			var baseFile = Svn.GetPathToBaseText (sourcefile);
-			return File.Exists (baseFile) ? File.ReadAllText (baseFile) : "";
+			return Svn.GetTextBase (sourcefile);
 		}
 
-		public string GetPathToBaseText (FilePath sourcefile)
+		protected override string OnGetTextAtRevision (FilePath repositoryPath, Revision revision)
 		{
-			return Svn.GetPathToBaseText (sourcefile);
+			return Svn.GetTextAtRevision (repositoryPath, revision, RootPath);
 		}
 
-		public override string GetTextAtRevision (FilePath repositoryPath, Revision revision)
-		{
-			return Svn.GetTextAtRevision (repositoryPath, revision);
-		}
-
-		public override Revision[] GetHistory (FilePath sourcefile, Revision since)
+		protected override Revision[] OnGetHistory (FilePath sourcefile, Revision since)
 		{
 			return Svn.GetHistory (this, sourcefile, since);
 		}
@@ -130,17 +125,17 @@ namespace MonoDevelop.VersionControl.Subversion
 			return true;
 		}
 
-		public override void Lock (IProgressMonitor monitor, params FilePath[] localPaths)
+		protected override void OnLock (IProgressMonitor monitor, params FilePath[] localPaths)
 		{
 			Svn.Lock (monitor, "", false, localPaths);
 		}
 
-		public override void Unlock (IProgressMonitor monitor, params FilePath[] localPaths)
+		protected override void OnUnlock (IProgressMonitor monitor, params FilePath[] localPaths)
 		{
 			Svn.Unlock (monitor, false, localPaths);
 		}
 
-		public override Repository Publish (string serverPath, FilePath localPath, FilePath[] files, string message, IProgressMonitor monitor)
+		protected override Repository OnPublish (string serverPath, FilePath localPath, FilePath[] files, string message, IProgressMonitor monitor)
 		{
 			string url = Url;
 			if (!serverPath.StartsWith ("/") && !url.EndsWith ("/"))
@@ -148,10 +143,11 @@ namespace MonoDevelop.VersionControl.Subversion
 			url += serverPath;
 			
 			string[] paths = new string[] {url};
-			
+
 			CreateDirectory (paths, message, monitor);
 			Svn.Checkout (this.Url + "/" + serverPath, localPath, null, true, monitor);
 
+			RootPath = localPath;
 			Set<FilePath> dirs = new Set<FilePath> ();
 			PublishDir (dirs, localPath, false, monitor);
 
@@ -162,33 +158,26 @@ namespace MonoDevelop.VersionControl.Subversion
 
 			Svn.Commit (new FilePath[] { localPath }, message, monitor);
 			
-			return new SubversionRepository (Svn, paths[0], localPath);
+			return new SubversionRepository (VersionControlSystem, paths[0], localPath);
 		}
 
 		void PublishDir (Set<FilePath> dirs, FilePath dir, bool rec, IProgressMonitor monitor)
 		{
-			string ndir = (string) dir;
-			while (ndir[ndir.Length - 1] == Path.DirectorySeparatorChar)
-				ndir = ndir.Substring (0, ndir.Length - 1);
-
-			dir = ndir;
-			if (dirs.Contains (dir))
-				return;
-
-			dirs.Add (dir);
-			if (rec) {
-				PublishDir (dirs, dir.ParentDirectory, true, monitor);
-				Add (dir, false, monitor);
+			if (dirs.Add (dir.CanonicalPath)) {
+				if (rec) {
+					PublishDir (dirs, dir.ParentDirectory, true, monitor);
+					Add (dir, false, monitor);
+				}
 			}
 		}
 
-		public override void Update (FilePath[] paths, bool recurse, IProgressMonitor monitor)
+		protected override void OnUpdate (FilePath[] paths, bool recurse, IProgressMonitor monitor)
 		{
 			foreach (string path in paths)
 				Svn.Update (path, recurse, monitor);
 		}
 		
-		public override void Commit (ChangeSet changeSet, IProgressMonitor monitor)
+		protected override void OnCommit (ChangeSet changeSet, IProgressMonitor monitor)
 		{
 			List<FilePath> list = new List<FilePath> ();
 			foreach (ChangeSetItem it in changeSet.Items)
@@ -201,7 +190,7 @@ namespace MonoDevelop.VersionControl.Subversion
 			Svn.Mkdir (paths, message, monitor);
 		}
 
-		public override void Checkout (FilePath path, Revision rev, bool recurse, IProgressMonitor monitor)
+		protected override void OnCheckout (FilePath path, Revision rev, bool recurse, IProgressMonitor monitor)
 		{
 			Svn.Checkout (this.Url, path, rev, recurse, monitor);
 		}
@@ -215,7 +204,7 @@ namespace MonoDevelop.VersionControl.Subversion
 			VersionControlService.NotifyFileStatusChanged (args);
 		}
 
-		public override void Revert (FilePath[] localPaths, bool recurse, IProgressMonitor monitor)
+		protected override void OnRevert (FilePath[] localPaths, bool recurse, IProgressMonitor monitor)
 		{
 			// If we have an array of paths such as: new [] { "/Foo/Directory", "/Foo/Directory/File1", "/Foo/Directory/File2" }
 			// svn will successfully revert the first entry (the directory) and then throw an error when trying to revert the
@@ -226,23 +215,23 @@ namespace MonoDevelop.VersionControl.Subversion
 			Svn.Revert (localPaths, recurse, monitor);
 		}
 
-		public override void RevertRevision (FilePath localPath, Revision revision, IProgressMonitor monitor)
+		protected override void OnRevertRevision (FilePath localPath, Revision revision, IProgressMonitor monitor)
 		{
 			Svn.RevertRevision (localPath, revision, monitor);
 		}
 
-		public override void RevertToRevision (FilePath localPath, Revision revision, IProgressMonitor monitor)
+		protected override void OnRevertToRevision (FilePath localPath, Revision revision, IProgressMonitor monitor)
 		{
 			Svn.RevertToRevision (localPath, revision, monitor);
 		}
 
-		public override void Add (FilePath[] paths, bool recurse, IProgressMonitor monitor)
+		protected override void OnAdd (FilePath[] paths, bool recurse, IProgressMonitor monitor)
 		{
 			foreach (FilePath path in paths) {
 				if (IsVersioned (path) && File.Exists (path) && !Directory.Exists (path)) {
-					if (rootPath == null)
+					if (RootPath.IsNull)
 						throw new UserException (GettextCatalog.GetString ("Project publishing failed. There is a stale .svn folder in the path '{0}'", path.ParentDirectory));
-					VersionInfo srcInfo = GetVersionInfo (path, false);
+					VersionInfo srcInfo = GetVersionInfo (path, VersionInfoQueryFlags.IgnoreCache);
 					if (srcInfo.HasLocalChange (VersionStatus.ScheduledDelete)) {
 						// It is a file that was deleted. It can be restored now since it's going
 						// to be added again.
@@ -256,25 +245,27 @@ namespace MonoDevelop.VersionControl.Subversion
 						// Copy the file over the old one and clean up
 						File.Copy (tmp, path, true);
 						File.Delete (tmp);
+						continue;
 					}
 				}
 				else {
+					VersionInfo srcInfo = GetVersionInfo (path, VersionInfoQueryFlags.IgnoreCache);
 					if (!IsVersioned (path.ParentDirectory)) {
 						// The file/folder belongs to an unversioned folder. We can add it by versioning the parent
 						// folders up to the root of the repository
 						
-						if (!path.IsChildPathOf (rootPath))
+						if (!path.IsChildPathOf (RootPath))
 							throw new InvalidOperationException ("File outside the repository directory");
 
 						List<FilePath> dirChain = new List<FilePath> ();
 						FilePath parentDir = path.CanonicalPath;
 						do {
 							parentDir = parentDir.ParentDirectory;
-							if (Directory.Exists (SubversionVersionControl.GetDirectoryDotSvn (parentDir)))
+							if (IsVersioned (parentDir))
 								break;
 							dirChain.Add (parentDir);
 						}
-						while (parentDir != rootPath);
+						while (parentDir != RootPath);
 
 						// Found all parent unversioned dirs. Versin them now.
 						dirChain.Reverse ();
@@ -285,8 +276,8 @@ namespace MonoDevelop.VersionControl.Subversion
 						}
 						VersionControlService.NotifyFileStatusChanged (args);
 					}
-					Svn.Add (path, recurse, monitor);
 				}
+				Svn.Add (path, recurse, monitor);
 			}
 		}
 		
@@ -308,7 +299,7 @@ namespace MonoDevelop.VersionControl.Subversion
 			return (srcRepository is SubversionRepository) && ((SubversionRepository)srcRepository).Root == Root;
 		}
 
-		public override void MoveFile (FilePath srcPath, FilePath destPath, bool force, IProgressMonitor monitor)
+		protected override void OnMoveFile (FilePath srcPath, FilePath destPath, bool force, IProgressMonitor monitor)
 		{
 			bool destIsVersioned = false;
 			
@@ -323,13 +314,13 @@ namespace MonoDevelop.VersionControl.Subversion
 				destIsVersioned = true;
 			}
 			
-			VersionInfo srcInfo = GetVersionInfo (srcPath, false);
+			VersionInfo srcInfo = GetVersionInfo (srcPath, VersionInfoQueryFlags.IgnoreCache);
 			if (srcInfo != null && srcInfo.HasLocalChange (VersionStatus.ScheduledAdd)) {
 				// If the file is scheduled to add, cancel it, move the file, and schedule to add again
 				Revert (srcPath, false, monitor);
 				if (!destIsVersioned)
 					MakeDirVersioned (Path.GetDirectoryName (destPath), monitor);
-				base.MoveFile (srcPath, destPath, force, monitor);
+				base.OnMoveFile (srcPath, destPath, force, monitor);
 				if (!destIsVersioned)
 					Add (destPath, false, monitor);
 			} else {
@@ -337,15 +328,15 @@ namespace MonoDevelop.VersionControl.Subversion
 					MakeDirVersioned (Path.GetDirectoryName (destPath), monitor);
 					Svn.Move (srcPath, destPath, force, monitor);
 				} else
-					base.MoveFile (srcPath, destPath, force, monitor);
+					base.OnMoveFile (srcPath, destPath, force, monitor);
 			}
 		}
 
-		public override void MoveDirectory (FilePath srcPath, FilePath destPath, bool force, IProgressMonitor monitor)
+		protected override void OnMoveDirectory (FilePath srcPath, FilePath destPath, bool force, IProgressMonitor monitor)
 		{
 			if (IsVersioned (destPath))
 			{
-				VersionInfo vinfo = GetVersionInfo (destPath, false);
+				VersionInfo vinfo = GetVersionInfo (destPath, VersionInfoQueryFlags.IgnoreCache);
 				if (!vinfo.HasLocalChange (VersionStatus.ScheduledDelete) && Directory.Exists (destPath))
 					throw new InvalidOperationException ("Cannot move directory. Destination directory already exist.");
 					
@@ -391,12 +382,12 @@ namespace MonoDevelop.VersionControl.Subversion
 					string df = Path.GetDirectoryName (dst);
 					copiedFolders [df] = df;
 				}
-				
+
 				// Delete all old files which have not been replaced
 				ArrayList foldersToDelete = new ArrayList ();
 				foreach (string oldFile in oldFiles) {
 					if (!copiedFiles.Contains (oldFile)) {
-						DeleteFile (oldFile, true, monitor);
+						DeleteFile (oldFile, true, monitor, false);
 						string fd = Path.GetDirectoryName (oldFile);
 						if (!copiedFolders.Contains (fd) && !foldersToDelete.Contains (fd))
 							foldersToDelete.Add (fd);
@@ -409,25 +400,25 @@ namespace MonoDevelop.VersionControl.Subversion
 				}
 				
 				// Delete the source directory
-				DeleteDirectory (srcPath, true, monitor);
+				DeleteDirectory (srcPath, true, monitor, false);
 			}
 			else {
 				if (Directory.Exists (destPath))
 					throw new InvalidOperationException ("Cannot move directory. Destination directory already exist.");
 				
-				VersionInfo srcInfo = GetVersionInfo (srcPath, false);
+				VersionInfo srcInfo = GetVersionInfo (srcPath, VersionInfoQueryFlags.IgnoreCache);
 				if (srcInfo != null && srcInfo.HasLocalChange (VersionStatus.ScheduledAdd)) {
 					// If the directory is scheduled to add, cancel it, move the directory, and schedule to add it again
 					MakeDirVersioned (Path.GetDirectoryName (destPath), monitor);
 					Revert (srcPath, true, monitor);
-					base.MoveDirectory (srcPath, destPath, force, monitor);
+					base.OnMoveDirectory (srcPath, destPath, force, monitor);
 					Add (destPath, true, monitor);
 				} else {
 					if (IsVersioned (srcPath)) {
 						MakeDirVersioned (Path.GetDirectoryName (destPath), monitor);
 						Svn.Move (srcPath, destPath, force, monitor);
 					} else
-						base.MoveDirectory (srcPath, destPath, force, monitor);
+						base.OnMoveDirectory (srcPath, destPath, force, monitor);
 				}
 			}
 		}
@@ -459,30 +450,68 @@ namespace MonoDevelop.VersionControl.Subversion
 				collection.Add(f);
 		}
 
+		protected override void OnDeleteFiles (FilePath[] paths, bool force, IProgressMonitor monitor)
+		{
+		}
 
-		public override void DeleteFiles (FilePath[] paths, bool force, IProgressMonitor monitor)
+		protected override void OnDeleteFiles (FilePath[] paths, bool force, IProgressMonitor monitor, bool keepLocal)
 		{
 			foreach (string path in paths) {
-				if (IsVersioned (path))
-					Svn.Delete (path, force, monitor);
-				else {
-					VersionInfo srcInfo = GetVersionInfo (path, false);
-					if (srcInfo != null && srcInfo.HasLocalChange (VersionStatus.ScheduledAdd)) {
-						// Revert the add command
-						Revert (path, false, monitor);
+				if (IsVersioned (path)) {
+					string newPath = String.Empty;
+					if (keepLocal) {
+						newPath = Path.GetTempFileName ();
+						File.Copy (path, newPath, true);
 					}
-					File.Delete (path);
+					try {
+						Svn.Delete (path, force, monitor);
+					} finally {
+						if (keepLocal)
+							File.Move (newPath, path);
+					}
+				} else {
+					if (keepLocal) {
+						VersionInfo srcInfo = GetVersionInfo (path, VersionInfoQueryFlags.IgnoreCache);
+						if (srcInfo != null && srcInfo.HasLocalChange (VersionStatus.ScheduledAdd)) {
+							// Revert the add command
+							Revert (path, false, monitor);
+						}
+					} else
+						File.Delete (path);
 				}
 			}
 		}
 
-		public override void DeleteDirectories (FilePath[] paths, bool force, IProgressMonitor monitor)
+		protected override void OnDeleteDirectories (FilePath[] paths, bool force, IProgressMonitor monitor)
+		{
+		}
+
+		protected override void OnDeleteDirectories (FilePath[] paths, bool force, IProgressMonitor monitor, bool keepLocal)
 		{
 			foreach (string path in paths) {
-				if (IsVersioned (path))
-					Svn.Delete (path, force, monitor);
-				else
-					Directory.Delete (path, true);
+				if (IsVersioned (path)) {
+					string newPath = String.Empty;
+					if (keepLocal) {
+						newPath = FileService.CreateTempDirectory ();
+						FileService.CopyDirectory (path, newPath);
+					}
+					try {
+						Svn.Delete (path, force, monitor);
+					} finally {
+						if (keepLocal)
+							FileService.MoveDirectory (newPath, path);
+					}
+				} else {
+					if (keepLocal) {
+						foreach (var info in GetDirectoryVersionInfo (path, false, true)) {
+							if (info != null && info.HasLocalChange (VersionStatus.ScheduledAdd)) {
+								// Revert the add command
+								Revert (path, false, monitor);
+							}
+						}
+					} else
+						Directory.Delete (path, true);
+				}
 			}
 		}
 		
@@ -521,14 +550,14 @@ namespace MonoDevelop.VersionControl.Subversion
 		{
 			List<Annotation> annotations = new List<Annotation> (Svn.GetAnnotations (this, localPath, SvnRevision.First, SvnRevision.Base));
 			Annotation nextRev = new Annotation (GettextCatalog.GetString ("working copy"), "", DateTime.MinValue);
-			var baseDocument = new Mono.TextEditor.TextDocument (File.ReadAllText (GetPathToBaseText (localPath)));
+			var baseDocument = new Mono.TextEditor.TextDocument (GetBaseText (localPath));
 			var workingDocument = new Mono.TextEditor.TextDocument (File.ReadAllText (localPath));
 			
 			// "SubversionException: blame of the WORKING revision is not supported"
 			foreach (var hunk in baseDocument.Diff (workingDocument)) {
-				annotations.RemoveRange (hunk.InsertStart, hunk.Inserted);
+				annotations.RemoveRange (hunk.RemoveStart, hunk.Removed);
 				for (int i = 0; i < hunk.Inserted; ++i) {
-					annotations.Insert (hunk.InsertStart, nextRev);
+					annotations.Insert (hunk.InsertStart - 1, nextRev);
 				}
 			}
 			
@@ -557,5 +586,15 @@ namespace MonoDevelop.VersionControl.Subversion
 			
 			return patch.ToString ();
 		}
+
+		protected override void OnIgnore (FilePath[] paths)
+		{
+			Svn.Ignore (paths);
+		}
+
+		protected override void OnUnignore (FilePath[] paths)
+		{
+			Svn.Unignore (paths);
+		}
 	}
 }
