@@ -85,7 +85,7 @@ namespace MonoDevelop.VersionControl.Views
 		
 		public Ide.Gui.Document Document {
 			get {
-				return info.Document.WorkbenchWindow.Document;
+				return info.Document.ParentDocument;
 			}
 		}
 		public VersionControlItem VersionControlItem {
@@ -101,15 +101,27 @@ namespace MonoDevelop.VersionControl.Views
 			this.info = info;
 			var sourceEditor = info.Document.GetContent<MonoDevelop.SourceEditor.SourceEditorView> ();
 			
-			vAdjustment = new Adjustment (0, 0, 0, 0, 0, 0);
+			vAdjustment = new Adjustment (
+				sourceEditor.TextEditor.VAdjustment.Value, 
+				sourceEditor.TextEditor.VAdjustment.Lower, 
+				sourceEditor.TextEditor.VAdjustment.Upper, 
+				sourceEditor.TextEditor.VAdjustment.StepIncrement, 
+				sourceEditor.TextEditor.VAdjustment.PageIncrement, 
+				sourceEditor.TextEditor.VAdjustment.PageSize);
 			vAdjustment.Changed += HandleAdjustmentChanged;
 			
 			vScrollBar = new VScrollbar (vAdjustment);
 			AddChild (vScrollBar);
 			
-			hAdjustment = new Adjustment (0, 0, 0, 0, 0, 0);
+			hAdjustment = new Adjustment (
+				sourceEditor.TextEditor.HAdjustment.Value, 
+				sourceEditor.TextEditor.HAdjustment.Lower, 
+				sourceEditor.TextEditor.HAdjustment.Upper, 
+				sourceEditor.TextEditor.HAdjustment.StepIncrement, 
+				sourceEditor.TextEditor.HAdjustment.PageIncrement, 
+				sourceEditor.TextEditor.HAdjustment.PageSize);
 			hAdjustment.Changed += HandleAdjustmentChanged;
-			
+
 			hScrollBar = new HScrollbar (hAdjustment);
 			AddChild (hScrollBar);
 			
@@ -177,7 +189,7 @@ namespace MonoDevelop.VersionControl.Views
 				children.ForEach (child => callback (child.Child));
 		}
 		
-		public void AddChild (Gtk.Widget child)
+		void AddChild (Gtk.Widget child)
 		{
 			child.Parent = this;
 			children.Add (new ContainerChild (this, child));
@@ -205,8 +217,14 @@ namespace MonoDevelop.VersionControl.Views
 		protected override void OnDestroyed ()
 		{
 			base.OnDestroyed ();
-			children.ForEach (child => child.Child.Destroy ());
-			children.Clear ();
+			hScrollBar.Destroy ();
+			hAdjustment.Destroy ();
+
+			vScrollBar.Destroy ();
+			vAdjustment.Destroy ();
+
+			editor.Destroy ();
+			overview.Destroy ();
 		}
 		 
 		protected override void OnSizeAllocated (Rectangle allocation)
@@ -358,6 +376,9 @@ namespace MonoDevelop.VersionControl.Views
 			Pango.Layout layout;
 
 			double dragPosition = -1;
+
+			TextDocument document;
+
 			public BlameRenderer (BlameWidget widget)
 			{
 				this.widget = widget;
@@ -365,8 +386,9 @@ namespace MonoDevelop.VersionControl.Views
 				annotations = new List<Annotation> ();
 				UpdateAnnotations (null, null);
 	//			widget.Document.Saved += UpdateAnnotations;
-				widget.Editor.Document.TextReplacing += EditorDocumentTextReplacing;
-				widget.Editor.Document.LineChanged += EditorDocumentLineChanged;
+				document = widget.Editor.Document;
+				document.TextReplacing += EditorDocumentTextReplacing;
+				document.LineChanged += EditorDocumentLineChanged;
 				widget.vScrollBar.ValueChanged += delegate {
 					QueueDraw ();
 				};
@@ -388,8 +410,11 @@ namespace MonoDevelop.VersionControl.Views
 			{
 				base.OnDestroyed ();
 //				widget.Document.Saved -= UpdateAnnotations;
-				widget.Editor.Document.TextReplacing -= EditorDocumentTextReplacing;
-				widget.Editor.Document.LineChanged -= EditorDocumentLineChanged;
+				if (document != null) { 
+					document.TextReplacing -= EditorDocumentTextReplacing;
+					document.LineChanged -= EditorDocumentLineChanged;
+					document = null;
+				}
 				if (layout != null) {
 					layout.Dispose ();
 					layout = null;
@@ -472,10 +497,10 @@ namespace MonoDevelop.VersionControl.Views
 				if (highlightAnnotation == null)
 					return;
 				int i = 1;
-				foreach (var content in widget.info.Document.WorkbenchWindow.SubViewContents) {
-					DiffView diffView = content as DiffView;
+				foreach (var view in widget.info.Document.ParentDocument.Views) {
+					DiffView diffView = view.GetContent<DiffView> ();
 					if (diffView != null) {
-						widget.info.Document.WorkbenchWindow.SwitchView (i);
+						view.Select ();
 						var rev = widget.info.History.FirstOrDefault (h => h.ToString () == highlightAnnotation.Revision);
 						if (rev == null)
 							return;
@@ -493,10 +518,10 @@ namespace MonoDevelop.VersionControl.Views
 				if (highlightAnnotation == null)
 					return;
 				int i = 1;
-				foreach (var content in widget.info.Document.WorkbenchWindow.SubViewContents) {
-					LogView logView = content as LogView;
+				foreach (var view in widget.info.Document.ParentDocument.Views) {
+					LogView logView = view.GetContent<LogView> ();
 					if (logView != null) {
-						widget.info.Document.WorkbenchWindow.SwitchView (i);
+						view.Select ();
 						var rev = widget.info.History.FirstOrDefault (h => h.ToString () == highlightAnnotation.Revision);
 						if (rev == null)
 							return;
@@ -524,7 +549,7 @@ namespace MonoDevelop.VersionControl.Views
 			{
 				StatusBarContext ctx = IdeApp.Workbench.StatusBar.CreateContext ();
 				ctx.AutoPulse = true;
-				ctx.ShowMessage (ImageService.GetImage ("md-version-control", IconSize.Menu), GettextCatalog.GetString ("Retrieving history"));
+				ctx.ShowMessage ("md-version-control", GettextCatalog.GetString ("Retrieving history"));
 				
 				ThreadPool.QueueUserWorkItem (delegate {
 					try {
@@ -575,14 +600,14 @@ namespace MonoDevelop.VersionControl.Views
 					
 					if (lineCount > 0)
 						annotations.RemoveRange (startLine - 1, lineCount);
-					if (!string.IsNullOrEmpty (e.InsertedText)) {
+					if (!string.IsNullOrEmpty (e.InsertedText.Text)) {
 						for (int i=0; i<lineCount; ++i)
 							annotations.Insert (startLine - 1, locallyModified);
 					}
 					return;
 				} else if (0 == e.RemovalLength) {
 					// insert
-					tokens = e.InsertedText.Split (new string[]{Environment.NewLine}, StringSplitOptions.None);
+					tokens = e.InsertedText.Text.Split (new string[]{Environment.NewLine}, StringSplitOptions.None);
 						lineCount = tokens.Length - 1;
 						for (int i=0; i<lineCount; ++i) {
 							annotations.Insert (Math.Min (startLine, annotations.Count), locallyModified);
