@@ -50,17 +50,10 @@ namespace MonoDevelop.VersionControl.Views
 			}
 		}
 		
-		public Toolbar CommandBar {
-			get {
-				return commandBar;
-			}
-		}
-		
-		
 		ListStore logstore = new ListStore (typeof (Revision));
 		FileTreeView treeviewFiles;
 		TreeStore changedpathstore;
-		Gtk.ToolButton revertButton, revertToButton;
+		Gtk.Button revertButton, revertToButton, refreshButton;
 		SearchEntry searchEntry;
 		string currentFilter;
 		
@@ -122,22 +115,27 @@ namespace MonoDevelop.VersionControl.Views
 			this.info = info;
 			if (info.Document != null)
 				this.preselectFile = info.Item.Path;
+
+			var separator = new HeaderBox ();
+			separator.SetMargins (1, 0, 0, 0);
+			separator.HeightRequest = 4;
+			separator.ShowAll ();
 			
-			revertButton = new Gtk.ToolButton (new Gtk.Image ("vc-revert-command", Gtk.IconSize.Menu), GettextCatalog.GetString ("Revert changes from this revision"));
-			revertButton.IsImportant = true;
+			hpaned1 = hpaned1.ReplaceWithWidget (new HPanedThin (), true);
+			vpaned1 = vpaned1.ReplaceWithWidget (new VPanedThin () { HandleWidget = separator }, true);
+
+			revertButton = new DocumentToolButton ("vc-revert-command", GettextCatalog.GetString ("Revert changes from this revision"));
 //			revertButton.Sensitive = false;
 			revertButton.Clicked += new EventHandler (RevertRevisionClicked);
-			CommandBar.Insert (revertButton, -1);
-			
-			revertToButton = new Gtk.ToolButton (new Gtk.Image ("vc-revert-command", Gtk.IconSize.Menu), GettextCatalog.GetString ("Revert to this revision"));
-			revertToButton.IsImportant = true;
+
+			revertToButton = new DocumentToolButton ("vc-revert-command", GettextCatalog.GetString ("Revert to this revision"));
 //			revertToButton.Sensitive = false;
 			revertToButton.Clicked += new EventHandler (RevertToRevisionClicked);
-			CommandBar.Insert (revertToButton, -1);
-			
-			Gtk.ToolItem item = new Gtk.ToolItem ();
-			Gtk.HBox a = new Gtk.HBox ();
-			item.Add (a);
+
+			refreshButton = new DocumentToolButton (Gtk.Stock.Refresh, GettextCatalog.GetString ("Refresh"));
+//			refreshButton.Sensitive = false;
+			refreshButton.Clicked += new EventHandler (RefreshClicked);
+
 			searchEntry = new SearchEntry ();
 			searchEntry.WidthRequest = 200;
 			searchEntry.ForceFilterButtonVisible = true;
@@ -145,12 +143,7 @@ namespace MonoDevelop.VersionControl.Views
 			searchEntry.Changed += HandleSearchEntryFilterChanged;
 			searchEntry.Ready = true;
 			searchEntry.Show ();
-			a.PackEnd (searchEntry, false, false, 0);
-			CommandBar.Insert (item, -1);
-			((Gtk.Toolbar.ToolbarChild)CommandBar[item]).Expand = true;
-			
-			CommandBar.ShowAll ();
-			
+
 			messageRenderer.Ellipsize = Pango.EllipsizeMode.End;
 			TreeViewColumn colRevMessage = new TreeViewColumn ();
 			colRevMessage.Title = GettextCatalog.GetString ("Message");
@@ -194,11 +187,11 @@ namespace MonoDevelop.VersionControl.Views
 			scrolledwindowFiles.Child = treeviewFiles;
 			scrolledwindowFiles.ShowAll ();
 			
-			changedpathstore = new TreeStore (typeof(Gdk.Pixbuf), typeof (string), // icon/file name
-				typeof(Gdk.Pixbuf), typeof (string), // icon/operation
+			changedpathstore = new TreeStore (typeof (Gdk.Pixbuf), typeof (string), // icon/file name
+				typeof (Gdk.Pixbuf), typeof (string), // icon/operation
 				typeof (string), // path
 				typeof (string), // revision path (invisible)
-				typeof (string[]) // diff
+				typeof (string []) // diff
 				);
 			
 			TreeViewColumn colChangedFile = new TreeViewColumn ();
@@ -235,6 +228,42 @@ namespace MonoDevelop.VersionControl.Views
 			labelAuthor.Text = "";
 			labelDate.Text = "";
 			labelRevision.Text = "";
+
+			vbox2.Remove (scrolledwindow1);
+			HeaderBox tb = new HeaderBox ();
+			tb.Show ();
+			tb.SetMargins (1, 0, 0, 0);
+			tb.ShowTopShadow = true;
+			tb.ShadowSize = 4;
+			tb.SetPadding (8, 8, 8, 8);
+			tb.UseChildBackgroundColor = true;
+			tb.Add (scrolledwindow1);
+			vbox2.PackStart (tb, true, true, 0);
+		}
+
+		protected override void OnRealized ()
+		{
+			base.OnRealized ();
+			var c = new HslColor (Style.Base (StateType.Normal));
+			c.L *= 0.8;
+			commitBox.ModifyBg (StateType.Normal, c);
+
+			var tcol = new Gdk.Color (255, 251, 242);
+			textviewDetails.ModifyBase (StateType.Normal, tcol);
+			scrolledwindow1.ModifyBase (StateType.Normal, tcol);
+		}
+
+		internal void SetToolbar (DocumentToolbar toolbar)
+		{
+			toolbar.Add (revertButton);
+			toolbar.Add (revertToButton);
+			toolbar.Add (refreshButton);
+
+			Gtk.HBox a = new Gtk.HBox ();
+			a.PackEnd (searchEntry, false, false, 0);
+			toolbar.Add (a, true);
+
+			toolbar.ShowAll ();
 		}
 		
 		bool filtering;
@@ -257,20 +286,28 @@ namespace MonoDevelop.VersionControl.Views
 			scrolledLog.Hide ();
 		}
 		
-		void RevertToRevisionClicked (object src, EventArgs args) {
+		void RevertToRevisionClicked (object src, EventArgs args)
+		{
 			Revision d = SelectedRevision;
 			if (RevertRevisionsCommands.RevertToRevision (info.Repository, info.Item.Path, d, false))
 				VersionControlService.SetCommitComment (info.Item.Path, 
 				  string.Format ("(Revert to revision {0})", d.ToString ()), true);
 		}
 		
-		void RevertRevisionClicked (object src, EventArgs args) {
+		void RevertRevisionClicked (object src, EventArgs args)
+		{
 			Revision d = SelectedRevision;
 			if (RevertRevisionsCommands.RevertRevision (info.Repository, info.Item.Path, d, false))
 				VersionControlService.SetCommitComment (info.Item.Path, 
 				  string.Format ("(Revert revision {0})", d.ToString ()), true);
 		}
 
+		void RefreshClicked (object src, EventArgs args)
+		{
+			ShowLoading ();
+			info.Start (true);
+		}
+
 		void HandleTreeviewFilesDiffLineActivated (object sender, EventArgs e)
 		{
 			TreePath[] paths = treeviewFiles.Selection.GetSelectedRows ();
@@ -301,23 +338,25 @@ namespace MonoDevelop.VersionControl.Views
 				i++;
 			}
 		}
-		
+
+		const int colOperation = 4;
 		const int colPath = 5;
 		const int colDiff = 6;
 		
 		void HandleTreeviewFilesTestExpandRow (object o, TestExpandRowArgs args)
 		{
-			string[] diff = changedpathstore.GetValue (args.Iter, colDiff) as string[];
-			if (diff != null) {
-				return;
-			}
 			TreeIter iter;
 			if (changedpathstore.IterChildren (out iter, args.Iter)) {
-				string path = (string)changedpathstore .GetValue (args.Iter, colPath);
+				string[] diff = changedpathstore.GetValue (iter, colDiff) as string[];
+				if (diff != null)
+					return;
+
+				string path = (string)changedpathstore.GetValue (args.Iter, colPath);
+
 				changedpathstore.SetValue (iter, colDiff, new string[] { GettextCatalog.GetString ("Loading data...") });
 				var rev = SelectedRevision;
 				ThreadPool.QueueUserWorkItem (delegate {
-					string text;
+					string text = "";
 					try {
 						text = info.Repository.GetTextAtRevision (path, rev);
 					} catch (Exception e) {
@@ -338,25 +377,42 @@ namespace MonoDevelop.VersionControl.Views
 						return;
 					}
 					string[] lines;
-					var changedDocument = new Mono.TextEditor.Document (text);
-					if (prevRev == null) {
-						lines = new string[changedDocument.LineCount];
-						for (int i = 0; i < changedDocument.LineCount; i++) {
-							lines[i] = "+ " + changedDocument.GetLineText (i + 1).TrimEnd ('\r','\n');
-						}
-						
+					// Indicator that the file was binary
+					if (text == null) {
+						lines = new [] { " Binary files differ" };
 					} else {
-						string prevRevisionText = "";
-						try {
-							prevRevisionText = info.Repository.GetTextAtRevision (path, prevRev);
-						} catch (Exception e) {
-							// The file did not exist at this point in time, so just treat it as empty
+						var changedDocument = new Mono.TextEditor.TextDocument (text);
+						if (prevRev == null) {
+							lines = new string [changedDocument.LineCount];
+							for (int i = 0; i < changedDocument.LineCount; i++) {
+								lines[i] = "+ " + changedDocument.GetLineText (i + 1).TrimEnd ('\r','\n');
+							}
+						} else {
+							string prevRevisionText = "";
+							try {
+								prevRevisionText = info.Repository.GetTextAtRevision (path, prevRev);
+							} catch (Exception e) {
+								Application.Invoke (delegate {
+									LoggingService.LogError ("Error while getting revision text", e);
+									MessageService.ShowError ("Error while getting revision text.", "The file may not be part of the working copy.");
+								});
+								return;
+							}
+
+							if (String.IsNullOrEmpty (text)) {
+								if (!String.IsNullOrEmpty (prevRevisionText)) {
+									lines = new string [changedDocument.LineCount];
+									for (int i = 0; i < changedDocument.LineCount; i++) {
+										lines [i] = "- " + changedDocument.GetLineText (i + 1).TrimEnd ('\r','\n');
+									}
+								}
+							}
+
+							var originalDocument = new Mono.TextEditor.TextDocument (prevRevisionText);
+							originalDocument.FileName = "Revision " + prevRev;
+							changedDocument.FileName = "Revision " + rev;
+							lines = Mono.TextEditor.Utils.Diff.GetDiffString (originalDocument, changedDocument).Split ('\n');
 						}
-						
-						var originalDocument = new Mono.TextEditor.Document (prevRevisionText);
-						originalDocument.FileName = "Revision " + prevRev.ToString ();
-						changedDocument.FileName = "Revision " + rev.ToString ();
-						lines = Mono.TextEditor.Utils.Diff.GetDiffString (originalDocument, changedDocument).Split ('\n');
 					}
 					Application.Invoke (delegate {
 						changedpathstore.SetValue (iter, colDiff, lines);
@@ -376,7 +432,7 @@ namespace MonoDevelop.VersionControl.Views
 			TreeIter iter;
 			if (!treeviewFiles.Selection.GetSelected (out iter))
 				return;
-			string path = (string)changedpathstore.GetValue (iter, 5);
+			string path = (string)changedpathstore.GetValue (iter, colPath);
 			ThreadPool.QueueUserWorkItem (delegate {
 				string text = info.Repository.GetTextAtRevision (path, rev);
 				string prevRevision = text; // info.Repository.GetTextAtRevision (path, rev.GetPrevious ());
@@ -473,6 +529,7 @@ namespace MonoDevelop.VersionControl.Views
 			if (img.LoadOperation.IsCompleted)
 				renderer.Pixbuf = img.Pixbuf;
 			else {
+				renderer.Pixbuf = null;
 				img.LoadOperation.Completed += delegate {
 					Gtk.Application.Invoke (delegate {
 						if (logstore.IterIsValid (iter))
@@ -497,7 +554,7 @@ namespace MonoDevelop.VersionControl.Views
 			CellRendererDiff rc = (CellRendererDiff)cell;
 			string[] lines = (string[])changedpathstore.GetValue (iter, colDiff);
 			if (lines == null)
-				lines = new string[] { (string)changedpathstore.GetValue (iter, 4) };
+				lines = new string[] { (string)changedpathstore.GetValue (iter, colOperation) };
 			rc.InitCell (treeviewFiles, changedpathstore.IterDepth (iter) != 0, lines, changedpathstore.GetPath (iter));
 		}
 		
@@ -588,8 +645,10 @@ namespace MonoDevelop.VersionControl.Views
 			labelRevision.Text = GettextCatalog.GetString ("revision: {0}", rev);
 			textviewDetails.Buffer.Text = d.Message;
 			
-			if (select)
+			if (select) {
 				treeviewFiles.Selection.SelectIter (selectIter);
+				treeviewFiles.ExpandRow (treeviewFiles.Model.GetPath (selectIter), true);
+			}
 		}
 		
 		void UpdateHistory ()
@@ -654,5 +713,17 @@ namespace MonoDevelop.VersionControl.Views
 				currentRevisionShortened = false;
 			}
 		}
+
+		internal string DiffText {
+			get {
+				TreeIter iter;
+				if (treeviewFiles.Selection.GetSelected (out iter)) {
+					string [] items = changedpathstore.GetValue (iter, colDiff) as string [];
+					if (items != null)
+						return String.Join (Environment.NewLine, items);
+				}
+				return null;
+			}
+		}
 	}
 }
