@@ -35,22 +35,18 @@ using MonoDevelop.Ide;
 using MonoDevelop.Components;
 using System.ComponentModel;
 using MonoDevelop.Core;
+using MonoDevelop.Ide.Gui;
 
 namespace MonoDevelop.VersionControl.Views
 {
-	[ToolboxItem (true)]
 	public class MergeWidget : EditorCompareWidgetBase
 	{
 		protected override TextEditor MainEditor {
 			get {
-				return editors[1];
+				return editors != null && editors.Length >= 2 ? editors[1] : null;
 			}
 		}
 
-		protected MergeWidget (IntPtr ptr) : base (ptr)
-		{
-		}
-
 		public MergeWidget ()
 		{
 			MainEditor.Document.TextReplaced += UpdateConflictsOnTextReplace;
@@ -86,7 +82,7 @@ namespace MonoDevelop.VersionControl.Views
 		public void Load (string fileName)
 		{
 			MainEditor.Document.MimeType = DesktopService.GetMimeTypeForUri (fileName);
-			MainEditor.Document.Text = System.IO.File.ReadAllText (fileName);
+			MainEditor.Document.Text = Mono.TextEditor.Utils.TextFileUtility.ReadAllText (fileName);
 
 			this.CreateDiff ();
 			Show ();
@@ -95,7 +91,8 @@ namespace MonoDevelop.VersionControl.Views
 		protected override void OnDestroyed ()
 		{
 			base.OnDestroyed ();
-			MainEditor.Document.TextReplaced -= UpdateConflictsOnTextReplace;
+			if (MainEditor != null && MainEditor.Document != null)
+				MainEditor.Document.TextReplaced -= UpdateConflictsOnTextReplace;
 		}
 		
 		public string GetResultText ()
@@ -105,7 +102,12 @@ namespace MonoDevelop.VersionControl.Views
 
 		protected override void CreateComponents ()
 		{
-			this.editors = new [] { new TextEditor (), new TextEditor (), new TextEditor () };
+			this.editors = new [] {
+				new TextEditor (new Mono.TextEditor.TextDocument (), new CommonTextEditorOptions ()),
+				new TextEditor (new Mono.TextEditor.TextDocument (), new CommonTextEditorOptions ()),
+				new TextEditor (new Mono.TextEditor.TextDocument (), new CommonTextEditorOptions ()),
+			};
+			
 			this.editors[0].Document.ReadOnly = true;
 			this.editors[2].Document.ReadOnly = true;
 			
@@ -117,36 +119,38 @@ namespace MonoDevelop.VersionControl.Views
 		}
 
 		// todo: move to version control backend
-		IEnumerable<Conflict> Conflicts (Mono.TextEditor.Document doc)
+		IEnumerable<Conflict> Conflicts (Mono.TextEditor.TextDocument doc)
 		{
-			foreach (int mergeStart in doc.SearchForward ("<<<<<<<", 0)) {
-				LineSegment start = doc.GetLineByOffset (mergeStart);
+			int mergeStart = 0;
+			while ((mergeStart = doc.IndexOf ("<<<<<<<", mergeStart, doc.TextLength - mergeStart, StringComparison.Ordinal)) >= 0) {
+				DocumentLine start = doc.GetLineByOffset (mergeStart);
 				if (start.Offset != mergeStart)
 					continue;
-				int dividerOffset = doc.SearchForward ("=======", mergeStart).First ();
-				LineSegment divider = doc.GetLineByOffset (dividerOffset);
+				int dividerOffset = doc.IndexOf ("=======", mergeStart, doc.TextLength - mergeStart, StringComparison.Ordinal);
+				DocumentLine divider = doc.GetLineByOffset (dividerOffset);
 
-				int endOffset      = doc.SearchForward (">>>>>>>", dividerOffset).First ();
-				LineSegment end = doc.GetLineByOffset (endOffset);
+				int endOffset = doc.IndexOf (">>>>>>>", dividerOffset, doc.TextLength - dividerOffset, StringComparison.Ordinal);
+				DocumentLine end = doc.GetLineByOffset (endOffset);
+				mergeStart = dividerOffset + 1;
 
-				yield return new Conflict (new Mono.TextEditor.Segment (start.EndOffset, divider.Offset - start.EndOffset),
-					new Mono.TextEditor.Segment (divider.EndOffset, end.Offset - divider.EndOffset),
-					new Mono.TextEditor.Segment (start),
-					new Mono.TextEditor.Segment (divider),
-					new Mono.TextEditor.Segment (end));
+				yield return new Conflict (new TextSegment (start.EndOffsetIncludingDelimiter, divider.Offset - start.EndOffsetIncludingDelimiter),
+					new TextSegment (divider.EndOffsetIncludingDelimiter, end.Offset - divider.EndOffsetIncludingDelimiter),
+					start,
+					divider,
+					end);
 			}
 		}
 
 		class Conflict
 		{
-			public readonly Mono.TextEditor.Segment MySegment;
-			public readonly Mono.TextEditor.Segment TheirSegment;
+			public TextSegment MySegment;
+			public TextSegment TheirSegment;
 
-			public readonly Mono.TextEditor.Segment StartSegment;
-			public readonly Mono.TextEditor.Segment DividerSegment;
-			public readonly Mono.TextEditor.Segment EndSegment;
+			public TextSegment StartSegment;
+			public TextSegment DividerSegment;
+			public TextSegment EndSegment;
 
-			public Conflict (Mono.TextEditor.Segment mySegment, Mono.TextEditor.Segment theirSegment, Mono.TextEditor.Segment startSegment, Mono.TextEditor.Segment dividerSegment, Mono.TextEditor.Segment endSegment)
+			public Conflict (TextSegment mySegment, TextSegment theirSegment, TextSegment startSegment, TextSegment dividerSegment, TextSegment endSegment)
 			{
 				this.MySegment = mySegment;
 				this.TheirSegment = theirSegment;
@@ -165,7 +169,7 @@ namespace MonoDevelop.VersionControl.Views
 			LeftDiff  = new List<Mono.TextEditor.Utils.Hunk> (editors[0].Document.Diff (MainEditor.Document));
 			RightDiff = new List<Mono.TextEditor.Utils.Hunk> (editors[2].Document.Diff (MainEditor.Document));
 
-			LineSegment line;
+			DocumentLine line;
 			LeftDiff.RemoveAll (item => null != (line = MainEditor.Document.GetLine (item.InsertStart)) &&
 				currentConflicts.Any (c => c.StartSegment.Offset <= line.Offset && line.Offset < c.EndSegment.EndOffset));
 			RightDiff.RemoveAll (item => null != (line = MainEditor.Document.GetLine (item.InsertStart)) &&
@@ -201,14 +205,14 @@ namespace MonoDevelop.VersionControl.Views
 				Conflict conflict = currentConflicts[i];
 
 				string above = MainEditor.Document.GetTextBetween (curOffset, conflict.StartSegment.Offset);
-				editors[0].Insert (editors[0].Document.Length, above);
+				editors[0].Insert (editors[0].Document.TextLength, above);
 				int leftA = editors[0].Document.LineCount;
-				editors[0].Insert (editors[0].Document.Length, MainEditor.Document.GetTextAt (conflict.MySegment));
+				editors[0].Insert (editors[0].Document.TextLength, MainEditor.Document.GetTextAt (conflict.MySegment));
 				int leftB = editors[0].Document.LineCount;
 
-				editors[2].Insert (editors[2].Document.Length, above);
+				editors[2].Insert (editors[2].Document.TextLength, above);
 				int rightA = editors[2].Document.LineCount;
-				editors[2].Insert (editors[2].Document.Length, MainEditor.Document.GetTextAt (conflict.TheirSegment));
+				editors[2].Insert (editors[2].Document.TextLength, MainEditor.Document.GetTextAt (conflict.TheirSegment));
 				int rightB = editors[2].Document.LineCount;
 
 				int middleA = MainEditor.Document.OffsetToLineNumber (conflict.StartSegment.Offset);
@@ -221,28 +225,23 @@ namespace MonoDevelop.VersionControl.Views
 			if (currentConflicts.Count > 0)
 				endOffset = currentConflicts.Last ().EndSegment.EndOffset;
 
-			string lastPart = MainEditor.Document.GetTextBetween (endOffset, MainEditor.Document.Length);
-			editors[0].Insert (editors[0].Document.Length, lastPart);
-			editors[2].Insert (editors[2].Document.Length, lastPart);
+			string lastPart = MainEditor.Document.GetTextBetween (endOffset, MainEditor.Document.TextLength);
+			editors[0].Insert (editors[0].Document.TextLength, lastPart);
+			editors[2].Insert (editors[2].Document.TextLength, lastPart);
 
 			UpdateDiff ();
 		}
 
-		IEnumerable<ISegment> GetAllConflictingSegments ()
-		{
-			foreach (var conflict in currentConflicts) {
-				yield return conflict.StartSegment;
-				yield return conflict.DividerSegment;
-				yield return conflict.EndSegment;
-				yield return conflict.MySegment;
-				yield return conflict.TheirSegment;
-			}
-		}
-
-		void UpdateConflictsOnTextReplace (object sender, ReplaceEventArgs e)
+		void UpdateConflictsOnTextReplace (object sender, DocumentChangeEventArgs e)
 		{
 			this.UpdateDiff ();
-			Document.UpdateSegments (GetAllConflictingSegments (), e);
+			foreach (var conflict in currentConflicts) {
+				conflict.StartSegment = conflict.StartSegment.AdjustSegment (e);
+				conflict.DividerSegment = conflict.DividerSegment.AdjustSegment (e);
+				conflict.EndSegment = conflict.EndSegment.AdjustSegment (e);
+				conflict.MySegment = conflict.MySegment.AdjustSegment (e);
+				conflict.TheirSegment = conflict.TheirSegment.AdjustSegment (e);
+			}
 		}
 	}
 }
\ No newline at end of file
