@@ -26,29 +26,32 @@ namespace MonoDevelop.VersionControl
 		Annotate,
 		ShowAnnotations,
 		HideAnnotations,
-		CreatePatch
+		CreatePatch,
+		Ignore,
+		Unignore
 	}
 	
-	class SolutionVersionControlCommandHalder: CommandHandler
+	class SolutionVersionControlCommandHandler: CommandHandler
 	{
 		VersionControlItemList GetItems ()
 		{
 			VersionControlItemList list = new VersionControlItemList ();
 			
-			IWorkspaceObject wob;
+			WorkspaceItem wob;
+			SolutionItem sol;
 			Repository repo = null;
 			wob = IdeApp.ProjectOperations.CurrentSelectedWorkspaceItem;
 			if (wob != null)
 				repo = VersionControlService.GetRepository (wob);
 			if (repo == null) {
-				wob = IdeApp.ProjectOperations.CurrentSelectedSolutionItem;
-				if (wob != null)
-					repo = VersionControlService.GetRepository (wob);
+				sol = IdeApp.ProjectOperations.CurrentSelectedSolutionItem;
+				if (sol != null)
+					repo = VersionControlService.GetRepository (sol);
 			}
 			if (repo == null || repo.VersionControlSystem == null || !repo.VersionControlSystem.IsInstalled)
 				return list;
-			
-			list.Add (new VersionControlItem (repo, wob, wob.BaseDirectory, true, null));
+
+			list.Add (new VersionControlItem (repo, wob, wob.FileName, true, null));
 			return list;
 		}
 		
@@ -70,12 +73,13 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class FileVersionControlCommandHalder: CommandHandler
+	class FileVersionControlCommandHandler: CommandHandler
 	{
 		protected VersionControlItemList GetItems ()
 		{
 			VersionControlItemList list = new VersionControlItemList ();
 			VersionControlItem it = GetItem ();
+
 			if (it != null)
 				list.Add (it);
 			return list;
@@ -116,7 +120,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}	
 
-	class UpdateCommandHandler: SolutionVersionControlCommandHalder
+	class UpdateCommandHandler: SolutionVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
@@ -124,23 +128,23 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class StatusCommandHandler: SolutionVersionControlCommandHalder
+	class StatusCommandHandler: SolutionVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
 			return StatusView.Show (items, test);
 		}
 	}
-	
-	class CommitCommandHandler: SolutionVersionControlCommandHalder
+
+	class CommitCommandHandler: FileVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
 			return CommitCommand.Commit (items, test);
 		}
 	}
-	
-	class AddCommandHandler: FileVersionControlCommandHalder
+
+	class AddCommandHandler: FileVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
@@ -154,7 +158,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class RemoveCommandHandler: FileVersionControlCommandHalder
+	class RemoveCommandHandler: FileVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
@@ -168,7 +172,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class RevertCommandHandler: FileVersionControlCommandHalder
+	class RevertCommandHandler: FileVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
@@ -182,7 +186,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class LockCommandHandler: FileVersionControlCommandHalder
+	class LockCommandHandler: FileVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
@@ -196,7 +200,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class UnlockCommandHandler: FileVersionControlCommandHalder
+	class UnlockCommandHandler: FileVersionControlCommandHandler
 	{
 		protected override bool RunCommand (VersionControlItemList items, bool test)
 		{
@@ -210,7 +214,35 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 
-	class CurrentFileDiffHandler : FileVersionControlCommandHalder
+	class IgnoreCommandHandler : FileVersionControlCommandHandler
+	{
+		protected override bool RunCommand (VersionControlItemList items, bool test)
+		{
+			return IgnoreCommand.Ignore (items, test);
+		}
+
+		protected override void Update (CommandInfo info)
+		{
+			base.Update (info);
+			info.Text = GettextCatalog.GetString ("Add to ignore list");
+		}
+	}
+
+	class UnignoreCommandHandler : FileVersionControlCommandHandler
+	{
+		protected override bool RunCommand (VersionControlItemList items, bool test)
+		{
+			return UnignoreCommand.Unignore (items, test);
+		}
+
+		protected override void Update (CommandInfo info)
+		{
+			base.Update (info);
+			info.Text = GettextCatalog.GetString ("Remove from ignore list");
+		}
+	}
+
+	class CurrentFileDiffHandler : FileVersionControlCommandHandler
 	{
 		protected override void Run ()
 		{
@@ -219,7 +251,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class CurrentFileBlameHandler : FileVersionControlCommandHalder
+	class CurrentFileBlameHandler : FileVersionControlCommandHandler
 	{
 		protected override void Run ()
 		{
@@ -228,7 +260,7 @@ namespace MonoDevelop.VersionControl
 		}
 	}
 	
-	class CurrentFileLogHandler : FileVersionControlCommandHalder
+	class CurrentFileLogHandler : FileVersionControlCommandHandler
 	{
 		protected override void Run ()
 		{
