@@ -1,17 +1,9 @@
 using System;
-using System.Collections;
-using System.IO;
+using System.Linq;
 
-using Gtk;
-
-using MonoDevelop.Projects;
-using MonoDevelop.Ide.Gui.Pads;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.Core;
-using MonoDevelop.Components.Commands;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Components;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.VersionControl
 {
@@ -31,12 +23,8 @@ namespace MonoDevelop.VersionControl
 		private static bool RevertInternal (VersionControlItemList items, bool test)
 		{
 			try {
-				if (test) {
-					foreach (VersionControlItem item in items)
-						if (!item.Repository.CanRevert (item.Path))
-							return false;
-					return true;
-				}
+				if (test)
+					return items.All (i => i.VersionInfo.CanRevert);
 
 				if (MessageService.AskQuestion (GettextCatalog.GetString ("Are you sure you want to revert the changes done in the selected files?"), 
 				                                GettextCatalog.GetString ("All changes made to the selected files will be permanently lost."),
@@ -69,19 +57,20 @@ namespace MonoDevelop.VersionControl
 			protected override void Run ()
 			{
 				foreach (VersionControlItemList list in items.SplitByRepository ())
-					list[0].Repository.Revert (list.Paths, true, GetProgressMonitor ());
+					list[0].Repository.Revert (list.Paths, true, Monitor);
 				
+				Monitor.ReportSuccess (GettextCatalog.GetString ("Revert operation completed."));
 				Gtk.Application.Invoke (delegate {
 					foreach (VersionControlItem item in items) {
 						if (!item.IsDirectory) {
+							FileService.NotifyFileChanged (item.Path);
 							// Reload reverted files
 							Document doc = IdeApp.Workbench.GetDocument (item.Path);
-							if (doc != null)
+							if (doc != null && System.IO.File.Exists (item.Path))
 								doc.Reload ();
-							FileService.NotifyFileChanged (item.Path);
 						}
-						VersionControlService.NotifyFileStatusChanged (item.Repository, item.Path, item.IsDirectory);
 					}
+					VersionControlService.NotifyFileStatusChanged (items);
 				});
 			}
 		}
