@@ -11,6 +11,8 @@ namespace MonoDevelop.VersionControl
 	{
 		IProgressMonitor tracker;
 		ThreadNotify threadnotify;
+
+		protected VersionControlOperationType OperationType { get; set; }
 		
 		protected abstract string GetDescription();
 		
@@ -25,6 +27,7 @@ namespace MonoDevelop.VersionControl
 
 		protected Task()
 		{
+			OperationType = VersionControlOperationType.Other;
 			threadnotify = new ThreadNotify(new ReadyEvent(Wakeup));
 		}
 		
@@ -34,7 +37,7 @@ namespace MonoDevelop.VersionControl
 		
 		protected virtual IProgressMonitor CreateProgressMonitor ()
 		{
-			return VersionControlService.GetProgressMonitor (GetDescription ());
+			return VersionControlService.GetProgressMonitor (GetDescription (), OperationType);
 		}
 		
 		public void Start() {
@@ -51,13 +54,6 @@ namespace MonoDevelop.VersionControl
 				tracker.ReportError("The operation could not be completed because a shared library is missing: " + e.Message, null);
 			} catch (Exception e) {
 				string msg = GettextCatalog.GetString ("Version control operation failed: ");
-				msg += e.Message;
-				if (e.InnerException != null && e.InnerException.Message != e.Message) {
-					msg = msg.Trim ();
-					if (!msg.EndsWith ("."))
-						msg += ".";
-					msg += " " + e.InnerException.Message;
-				}
 				tracker.ReportError (msg, e);
 			} finally {			
 				threadnotify.WakeupMain();
