@@ -16,15 +16,8 @@ namespace MonoDevelop.VersionControl
 {
 	class VersionControlNodeExtension : NodeBuilderExtension
 	{
-		Dictionary<FilePath,DirData> filePaths = new Dictionary<FilePath, DirData> ();
+		Dictionary<FilePath,object> pathToObject = new Dictionary<FilePath, object> ();
 		
-		class DirData
-		{
-			public object Object;
-			public Dictionary<FilePath,VersionInfo> FileData;
-			public DateTime Timestamp;
-		}
-	
 		public override bool CanBuildNode (Type dataType)
 		{
 			//Console.Error.WriteLine(dataType);
@@ -38,7 +31,17 @@ namespace MonoDevelop.VersionControl
 		{
 			VersionControlService.FileStatusChanged += Monitor;
 		}
-		
+
+		protected override void Initialize ()
+		{
+			base.Initialize ();
+		}
+
+		public override void Dispose ()
+		{
+			base.Dispose ();
+		}
+
 		public override void BuildNode (ITreeBuilder builder, object dataObject, ref string label, ref Gdk.Pixbuf icon, ref Gdk.Pixbuf closedIcon)
 		{
 			if (!builder.Options["ShowVersionControlOverlays"])
@@ -48,18 +51,20 @@ namespace MonoDevelop.VersionControl
 			
 			if (dataObject is IWorkspaceObject) {
 				IWorkspaceObject ce = (IWorkspaceObject) dataObject;
-				ClearDirCache (ce.BaseDirectory);
 				Repository rep = VersionControlService.GetRepository (ce);
-				if (rep != null)
+				if (rep != null) {
 					AddFolderOverlay (rep, ce.BaseDirectory, ref icon, ref closedIcon, false);
+					rep.GetDirectoryVersionInfo (ce.BaseDirectory, false, false);
+				}
 				return;
 			} else if (dataObject is ProjectFolder) {
 				ProjectFolder ce = (ProjectFolder) dataObject;
 				if (ce.ParentWorkspaceObject != null) {
-					ClearDirCache (ce.Path);
 					Repository rep = VersionControlService.GetRepository (ce.ParentWorkspaceObject);
-					if (rep != null)
+					if (rep != null) {
 						AddFolderOverlay (rep, ce.Path, ref icon, ref closedIcon, true);
+						rep.GetDirectoryVersionInfo (ce.Path, false, false);
+					}
 				}
 				return;
 			}
@@ -84,19 +89,35 @@ namespace MonoDevelop.VersionControl
 			if (repo == null)
 				return;
 			
-			VersionInfo vi = GetVersionInfo (repo, file);
-			if (dataObject is ProjectFile)
-				((ProjectFile)dataObject).ExtendedProperties [typeof(VersionInfo)] = vi;
-			
+			VersionInfo vi = repo.GetVersionInfo (file);
+
 			Gdk.Pixbuf overlay = VersionControlService.LoadOverlayIconForStatus (vi.Status);
 			if (overlay != null)
 				AddOverlay (ref icon, overlay);
 		}
-		
+
+/*		public override void PrepareChildNodes (object dataObject)
+		{
+			if (dataObject is IWorkspaceObject) {
+				IWorkspaceObject ce = (IWorkspaceObject) dataObject;
+				Repository rep = VersionControlService.GetRepository (ce);
+				if (rep != null)
+					rep.GetDirectoryVersionInfo (ce.BaseDirectory, false, false);
+			} else if (dataObject is ProjectFolder) {
+				ProjectFolder ce = (ProjectFolder) dataObject;
+				if (ce.ParentWorkspaceObject != null) {
+					Repository rep = VersionControlService.GetRepository (ce.ParentWorkspaceObject);
+					if (rep != null)
+						rep.GetDirectoryVersionInfo (ce.Path, false, false);
+				}
+			}
+			base.PrepareChildNodes (dataObject);
+		}
+*/		
 		void AddFolderOverlay (Repository rep, string folder, ref Gdk.Pixbuf icon, ref Gdk.Pixbuf closedIcon, bool skipVersionedOverlay)
 		{
 			Gdk.Pixbuf overlay = null;
-			VersionInfo vinfo = rep.GetVersionInfo (folder, false);
+			VersionInfo vinfo = rep.GetVersionInfo (folder);
 			if (vinfo == null || !vinfo.IsVersioned) {
 				overlay = VersionControlService.LoadOverlayIconForStatus (VersionStatus.Unversioned);
 			} else if (vinfo.IsVersioned && !vinfo.HasLocalChanges) {
@@ -137,93 +158,76 @@ namespace MonoDevelop.VersionControl
 			icon = res;
 		}
 		
-		void ClearDirCache (FilePath path)
-		{
-			path = path.CanonicalPath;
-			DirData data;
-			if (filePaths.TryGetValue (path, out data))
-				data.FileData = null;
-		}
-		
-		VersionInfo GetVersionInfo (Repository vc, FilePath filepath)
-		{
-			FilePath dir = filepath;
-			dir = dir.ParentDirectory.CanonicalPath;
-
-			DirData data;
-			if (filePaths.TryGetValue (dir, out data)) {
-				if (data.FileData == null) {
-					data.FileData = new Dictionary<FilePath, VersionInfo> ();
-					foreach (VersionInfo vin in vc.GetDirectoryVersionInfo (dir, false, false))
-						data.FileData [vin.LocalPath.CanonicalPath] = vin;
-					data.Timestamp = DateTime.Now;
-				}
-				VersionInfo vi;
-				if (data.FileData.TryGetValue (filepath.CanonicalPath, out vi))
-					return vi;
-			}
-			
-			VersionInfo node = vc.GetVersionInfo (filepath, false);
-			if (node != null) {
-				if (data != null)
-					data.FileData [filepath] = node;
-				return node;
-			}
-			return VersionInfo.CreateUnversioned (filepath, false);
-		}
-		
 		void Monitor (object sender, FileUpdateEventArgs args)
 		{
-			foreach (var dir in args.GroupByDirectory ()) {
-				if (dir.Count () > 3 || dir.Any (f => f.IsDirectory)) {
-					FilePath path = dir.Key.CanonicalPath;
-					DirData dd;
-					if (filePaths.TryGetValue (path, out dd)) {
-						dd.FileData = null; // Clear the status cache
-						ITreeBuilder builder = Context.GetTreeBuilder (dd.Object);
-						if (builder != null)
-							builder.UpdateAll();
-					}
-					continue;
+			foreach (FileUpdateEventInfo uinfo in args) {
+				foreach (var ob in GetObjectsForPath (uinfo.FilePath)) {
+					ITreeBuilder builder = Context.GetTreeBuilder (ob);
+					if (builder != null)
+						builder.Update();
 				}
-				else {
-					// All files, clear the cached version info for each file, if exists
-					foreach (FileUpdateEventInfo uinfo in dir) {
-						FilePath path = uinfo.FilePath.ParentDirectory;
-						DirData dd;
-						if (filePaths.TryGetValue (path.CanonicalPath, out dd) && dd.FileData != null) {
-							dd.FileData.Remove (uinfo.FilePath.CanonicalPath);
-						}
-						if (filePaths.TryGetValue (uinfo.FilePath.CanonicalPath, out dd)) {
-							ITreeBuilder builder = Context.GetTreeBuilder (dd.Object);
-							if (builder != null)
-								builder.UpdateAll();
-						}
-					}
+			}
+		}
+
+		void RegisterObjectPath (FilePath path, object ob)
+		{
+			path = path.CanonicalPath;
+			object currentObj;
+			if (pathToObject.TryGetValue (path, out currentObj)) {
+				if (currentObj is List<object>) {
+					var list = (List<object>) currentObj;
+					list.Add (ob);
+				} else {
+					var list = new List<object> (2);
+					list.Add (currentObj);
+					list.Add (ob);
+					pathToObject [path] = list;
 				}
+			} else
+				pathToObject [path] = ob;
+		}
+
+		void UnregisterObjectPath (FilePath path, object ob)
+		{
+			path = path.CanonicalPath;
+			object currentObj;
+			if (pathToObject.TryGetValue (path, out currentObj)) {
+				if (currentObj is List<object>) {
+					var list = (List<object>) currentObj;
+					if (list.Remove (ob)) {
+						if (list.Count == 1)
+							pathToObject [path] = list[0];
+					}
+				} else if (currentObj == ob)
+					pathToObject.Remove (path);
+			}
+		}
+
+		IEnumerable<object> GetObjectsForPath (FilePath path)
+		{
+			path = path.CanonicalPath;
+			object currentObj;
+			if (pathToObject.TryGetValue (path, out currentObj)) {
+				if (currentObj is List<object>) {
+					foreach (var ob in (List<object>) currentObj)
+						yield return ob;
+				} else
+					yield return currentObj;
 			}
 		}
 		
 		public override void OnNodeAdded (object dataObject)
 		{
 			FilePath path = GetPath (dataObject);
-			if (path != FilePath.Null) {
-				DirData dd = new DirData ();
-				dd.Object = dataObject;
-				filePaths [path.CanonicalPath] = dd;
-			}
+			if (path != FilePath.Null)
+				RegisterObjectPath (path, dataObject);
 		}
 		
 		public override void OnNodeRemoved (object dataObject)
 		{
 			FilePath path = GetPath (dataObject);
-			if (path != FilePath.Null) {
-				path = path.CanonicalPath;
-				DirData data;
-				if (filePaths.TryGetValue (path, out data) && data.Object == dataObject) {
-					filePaths.Remove (path);
-				}
-			}
+			if (path != FilePath.Null)
+				UnregisterObjectPath (path, dataObject);
 		}
 		
 		internal static string GetPath (object dataObject)
@@ -292,7 +296,8 @@ namespace MonoDevelop.VersionControl
 		protected void UpdateStatus(CommandInfo item) {
 			TestCommand(Commands.Status, item);
 		}
-		
+
+		[AllowMultiSelection]
 		[CommandHandler (Commands.Commit)]
 		protected void OnCommit() {
 			RunCommand (Commands.Commit, false);
@@ -339,12 +344,12 @@ namespace MonoDevelop.VersionControl
 		[AllowMultiSelection]
 		[CommandHandler (Commands.Revert)]
 		protected void OnRevert() {
-			RunCommand(Commands.Revert, false);
+			RunCommand(Commands.Revert, false, false);
 		}
 		
 		[CommandUpdateHandler (Commands.Revert)]
 		protected void UpdateRevert(CommandInfo item) {
-			TestCommand(Commands.Revert, item);
+			TestCommand(Commands.Revert, item, false);
 		}
 		
 		[AllowMultiSelection]
@@ -390,9 +395,36 @@ namespace MonoDevelop.VersionControl
 		protected void UpdateCreatePatch(CommandInfo item) {
 			TestCommand(Commands.CreatePatch, item);
 		}
-			
-		private void TestCommand(Commands cmd, CommandInfo item) {
-			TestResult res = RunCommand(cmd, true);
+
+		[AllowMultiSelection]
+		[CommandHandler (Commands.Ignore)]
+		protected void OnIgnore ()
+		{
+			RunCommand(Commands.Ignore, false);
+		}
+
+		[CommandUpdateHandler (Commands.Ignore)]
+		protected void UpdateIgnore (CommandInfo item)
+		{
+			TestCommand(Commands.Ignore, item);
+		}
+
+		[AllowMultiSelection]
+		[CommandHandler (Commands.Unignore)]
+		protected void OnUnignore ()
+		{
+			RunCommand(Commands.Unignore, false);
+		}
+
+		[CommandUpdateHandler (Commands.Unignore)]
+		protected void UpdateUnignore (CommandInfo item)
+		{
+			TestCommand(Commands.Unignore, item);
+		}
+
+		private void TestCommand(Commands cmd, CommandInfo item, bool projRecurse = true)
+		{
+			TestResult res = RunCommand(cmd, true, projRecurse);
 			if (res == TestResult.NoVersionControl && cmd == Commands.Log) {
 				// Use the update command to show the "not available" message
 				item.Icon = null;
@@ -402,9 +434,9 @@ namespace MonoDevelop.VersionControl
 				item.Visible = res == TestResult.Enable;
 		}
 		
-		private TestResult RunCommand (Commands cmd, bool test)
+		private TestResult RunCommand (Commands cmd, bool test, bool projRecurse = true)
 		{
-			VersionControlItemList items = GetItems ();
+			VersionControlItemList items = GetItems (projRecurse);
 
 			foreach (VersionControlItem it in items) {
 				if (it.Repository == null) {
@@ -460,6 +492,12 @@ namespace MonoDevelop.VersionControl
 				case Commands.CreatePatch:
 					res = CreatePatchCommand.CreatePatch (items, test);
 					break;
+				case Commands.Ignore:
+					res = IgnoreCommand.Ignore (items, test);
+					break;
+				case Commands.Unignore:
+					res = UnignoreCommand.Unignore (items, test);
+					break;
 				}
 			}
 			catch (Exception ex) {
@@ -472,6 +510,15 @@ namespace MonoDevelop.VersionControl
 			
 			return res ? TestResult.Enable : TestResult.Disable;
 		}
+
+		public override void RefreshItem ()
+		{
+			foreach (VersionControlItem it in GetItems ()) {
+				if (it.Repository != null)
+					it.Repository.ClearCachedVersionInfo (it.Path);
+			}
+			base.RefreshItem ();
+		}
 	}
 
 	class OpenCommandHandler : VersionControlCommandHandler 
