@@ -28,7 +28,6 @@
 
 using MonoDevelop.Core;
 using MonoDevelop.Core.ProgressMonitoring;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Projects;
 
@@ -37,8 +36,8 @@ using System.Collections.Generic;
 using System.IO;
 using System.Text.RegularExpressions;
 using System.Xml;
-using System.Threading;
 using MonoDevelop.Projects.Extensions;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.Prj2Make
 {
@@ -209,26 +208,14 @@ namespace MonoDevelop.Prj2Make
 		// Utility function to determine the sln file version
 		string GetSlnFileVersion(string strInSlnFile)
 		{
-			string strVersion = null;
-			string strInput = null;
-			Match match;
-			StreamReader reader = new StreamReader(strInSlnFile);
-			strInput = reader.ReadLine();
-
-			match = SlnMaker.SlnVersionRegex.Match(strInput);
-			if (!match.Success) {
-				match = SlnMaker.SlnVersionRegex.Match (reader.ReadLine ());
+			using (var stream = File.OpenText (strInSlnFile)) {
+				// ReadLine returns null at EOF which makes the regex throw an ArgNull exception
+				var match = SlnMaker.SlnVersionRegex.Match (stream.ReadLine () ?? "");
+				if (!match.Success)
+					match = SlnMaker.SlnVersionRegex.Match (stream.ReadLine () ?? "");
+				
+				return match.Success ? match.Groups[1].Value : null;
 			}
-
-			if (match.Success)
-			{
-				strVersion = match.Groups[1].Value;
-			}
-			
-			// Close the stream
-			reader.Close();
-
-			return strVersion;
 		}
 		
 		public bool SupportsMixedFormats {
@@ -239,6 +226,11 @@ namespace MonoDevelop.Prj2Make
 		{
 			yield break;
 		}
+		
+		public bool SupportsFramework (MonoDevelop.Core.Assemblies.TargetFramework framework)
+		{
+			return framework.Id == MonoDevelop.Core.Assemblies.TargetFrameworkMoniker.NET_1_1;
+		}
 	}
 
 	enum TargetConvert
@@ -252,15 +244,12 @@ namespace MonoDevelop.Prj2Make
 	{
 		TargetConvert QueryConversion (string text)
 		{
-			AlertButton monodevelop = new AlertButton (GettextCatalog.GetString ("Convert to MonoDevelop"));
 			AlertButton vs2005      = new AlertButton (GettextCatalog.GetString ("Convert to MSBuild"));
 
 			AlertButton choice = MessageService.AskQuestion (text,
-			                                                 GettextCatalog.GetString ("Converting to Visual Studio 2005 format will overwrite existing files."),
-			                                                 AlertButton.Cancel, vs2005, monodevelop);
-			if (choice == monodevelop)
-				return TargetConvert.MonoDevelop;
-			else if (choice == vs2005)
+			                                                 GettextCatalog.GetString ("Converting to MSBuild format will overwrite existing files."),
+			                                                 AlertButton.Cancel, vs2005);
+			if (choice == vs2005)
 				return TargetConvert.VisualStudio;
 			else
 				return TargetConvert.None;
@@ -268,19 +257,19 @@ namespace MonoDevelop.Prj2Make
 		
 		public TargetConvert QueryProjectConversion (string file)
 		{
-			string text = GettextCatalog.GetString ("The project file {0} is a Visual Studio 2003 project. It must be converted to either a MonoDevelop or a Visual Studio 2005 project.", file);
+			string text = GettextCatalog.GetString ("The project file {0} is a Visual Studio 2003 project. It must be converted to a MSBuild project.", file);
 			return QueryConversion (text);
 		}
 		
 		public TargetConvert QuerySolutionConversion (string file)
 		{
-			string text = GettextCatalog.GetString ("The solution file {0} is a Visual Studio 2003 solution. It must be converted to either a MonoDevelop or a Visual Studio 2005 project.", file);
+			string text = GettextCatalog.GetString ("The solution file {0} is a Visual Studio 2003 solution. It must be converted to a MSBuild project.", file);
 			return QueryConversion (text);
 		}
 
 		public IProgressMonitor CreateProgressMonitor ()
 		{
-			return new MonoDevelop.Core.Gui.ProgressMonitoring.MessageDialogProgressMonitor (true, false, true, false);
+			return new MonoDevelop.Ide.ProgressMonitoring.MessageDialogProgressMonitor (true, false, true, false);
 		}
 	}
 }
