@@ -44,7 +44,7 @@ namespace Mono.TextEditor.PopupWindow
 	
 	public class ListWindow<T> : Gtk.Window
 	{
-		VScrollbar scrollbar;
+		ScrolledWindow scrollbar;
 		ListWidget<T> list;
 		IListDataProvider<T> provider;
 		Widget footer;
@@ -60,27 +60,30 @@ namespace Mono.TextEditor.PopupWindow
 			HBox box = new HBox ();
 			list = new ListWidget<T> (this);
 			list.SelectionChanged += new EventHandler (OnSelectionChanged);
-			list.ScrollEvent += new ScrollEventHandler (OnScrolled);
-			box.PackStart (list, true, true, 0);
-			this.BorderWidth = 1;
+			this.BorderWidth = 0;
 			
-			scrollbar = new VScrollbar (null);
-			scrollbar.ValueChanged += new EventHandler (OnScrollChanged); 
-			box.PackStart (scrollbar, false, false, 0);
+			scrollbar = new Gtk.ScrolledWindow ();
+			scrollbar.Child = list;
+			box.PackStart (scrollbar, true, true, 0);
 			list.ButtonPressEvent += delegate (object o, ButtonPressEventArgs args) {
 				if (args.Event.Button == 1 && args.Event.Type == Gdk.EventType.TwoButtonPress)
-					DoubleClick ();
+					OnDoubleClicked (EventArgs.Empty);
 			};
 			vbox.PackStart (box, true, true, 0);
 			Add (vbox);
 			
 			this.TypeHint = WindowTypeHint.Menu;
 		}
-		
-		protected virtual void DoubleClick ()
+
+		public event EventHandler DoubleClicked;
+
+		protected virtual void OnDoubleClicked (EventArgs e)
 		{
+			EventHandler handler = this.DoubleClicked;
+			if (handler != null)
+				handler (this, e);
 		}
-		
+
 		public new void Show ()
 		{
 			this.ShowAll ();
@@ -126,15 +129,6 @@ namespace Mono.TextEditor.PopupWindow
 		
 		void ResetSizes ()
 		{
-			scrollbar.Adjustment.Lower = 0;
-			scrollbar.Adjustment.Upper = System.Math.Max(0, provider.Count - list.VisibleRows);
-			scrollbar.Adjustment.PageIncrement = list.VisibleRows - 1;
-			scrollbar.Adjustment.StepIncrement = 1;
-			
-			if (list.VisibleRows >= provider.Count) {
-				this.scrollbar.Hide();
-			}
-
 			this.Resize(this.list.WidthRequest, this.list.HeightRequest);
 		}
 		
@@ -332,7 +326,7 @@ namespace Mono.TextEditor.PopupWindow
 			if (hasMismatches)
 				list.SelectionDisabled = true;
 		}
-		
+		/*
 		void OnScrollChanged (object o, EventArgs args)
 		{
 			list.Page = (int) scrollbar.Value;
@@ -364,10 +358,9 @@ namespace Mono.TextEditor.PopupWindow
 			adj.AddValueClamped (discreteItemDelta);
 			args.RetVal = true;
 		}
-		
+		*/
 		void OnSelectionChanged (object o, EventArgs args)
 		{
-			scrollbar.Value = list.Page;
 			OnSelectionChanged ();
 		}
 		
