@@ -61,7 +61,7 @@ namespace Mono.TextEditor.Utils
 			// Encoding verifiers
 			var verifierList = new List<Verifier> () {
 				new Utf8Verifier (),
-				new CodePage1252Verifier (),
+				new WindowsCodePageVerifier (),
 				new UnicodeVerifier (),
 				new BigEndianUnicodeVerifier (),
 				new CodePage858Verifier ()
@@ -78,6 +78,17 @@ namespace Mono.TextEditor.Utils
 		}
 
 		#region stream reader methods
+		public static StreamReader OpenStream (string fileName)
+		{
+			bool hadBom;
+			return OpenStream (File.ReadAllBytes (fileName), out hadBom);
+		}
+
+		public static StreamReader OpenStream (string fileName, out bool hadBom)
+		{
+			return OpenStream (File.ReadAllBytes (fileName), out hadBom);
+		}
+
 		public static StreamReader OpenStream (byte[] bytes)
 		{
 			bool hadBom;
@@ -134,7 +145,7 @@ namespace Mono.TextEditor.Utils
 			}
 		}
 
-		public static string GetText (byte[] bytes, out bool hadBom, out Encoding encoding)
+		public static string GetText (byte[] bytes, out Encoding encoding, out bool hadBom)
 		{
 			if (bytes == null)
 				throw new ArgumentNullException ("bytes");
@@ -151,7 +162,7 @@ namespace Mono.TextEditor.Utils
 			}
 		}
 
-		public static string GetText (Stream inputStream, out bool hadBom, out Encoding encoding)
+		public static string GetText (Stream inputStream, out Encoding encoding, out bool hadBom)
 		{
 			if (inputStream == null)
 				throw new ArgumentNullException ("inputStream");
@@ -161,6 +172,23 @@ namespace Mono.TextEditor.Utils
 			}
 		}
 
+		public static string GetText (string fileName)
+		{
+			using (var stream = OpenStream (fileName)) {
+				return stream.ReadToEnd ();
+			}
+		}
+
+		public static string GetText (string fileName, out Encoding encoding, out bool hadBom)
+		{
+			if (fileName == null)
+				throw new ArgumentNullException ("fileName");
+			using (var stream = OpenStream (fileName, out hadBom)) {
+				encoding = stream.CurrentEncoding;
+				return stream.ReadToEnd ();
+			}
+		}
+
 		#endregion
 
 		#region file methods
@@ -187,6 +215,26 @@ namespace Mono.TextEditor.Utils
 			SystemRename (tmpPath, fileName);
 		}
 
+		/// <summary>
+		/// Returns a byte array containing the text encoded by a specified encoding & bom.
+		/// </summary>
+		/// <param name="text">The text to encode.</param>
+		/// <param name="encoding">The encoding.</param>
+		/// <param name="hadBom">If set to <c>true</c> a bom will be prepended.</param>
+		public static byte[] GetBuffer (string text, Encoding encoding, bool hadBom)
+		{
+			using (var stream = new MemoryStream ()) {
+				if (hadBom) {
+					var bom = encoding.GetPreamble ();
+					if (bom != null && bom.Length > 0)
+						stream.Write (bom, 0, bom.Length);
+				}
+				byte[] bytes = encoding.GetBytes (text);
+				stream.Write (bytes, 0, bytes.Length);
+				return stream.GetBuffer ();
+			}
+		}
+
 		// Code taken from FileService.cs
 		static void SystemRename (string sourceFile, string destFile)
 		{
@@ -234,7 +282,7 @@ namespace Mono.TextEditor.Utils
 			if (fileName == null)
 				throw new ArgumentNullException ("fileName");
 			using (var stream = new FileStream (fileName, FileMode.Open, FileAccess.Read, FileShare.Read)) {
-				return GetText (stream, out hadBom, out encoding);
+				return GetText (stream, out encoding, out hadBom);
 			}
 		}
 		public static string ReadAllText (string fileName, Encoding encoding, out bool hadBom)
@@ -602,23 +650,40 @@ namespace Mono.TextEditor.Utils
 		/// <summary>
 		/// Code page 1252 was the long time default on windows. This encoding is a superset of ISO 8859-1.
 		/// </summary>
-		class CodePage1252Verifier : Verifier
+		class WindowsCodePageVerifier : Verifier
 		{
 			const byte Valid = 1;
 			const byte LAST = 2;
 			static byte[][] table;
-			static Encoding EncodingCp1252;
+			static Encoding EncodingWindows;
 
 			public override byte InitalState { get { return Valid; } }
 
-			public override Encoding Encoding { get { return EncodingCp1252; } }
+			public override Encoding Encoding { get { return EncodingWindows; } }
 
 			public override byte[][] StateTable { get { return table; } }
 
+
+			const int westernEncodingCodePage = 1252;
+			/// <summary>
+			/// Try to guess the windows code page using the default encoding, on non windows system default
+			/// to 1252 (western encoding).
+			/// </summary>
+			int WindowsCodePage {
+				get {
+					if (Platform.IsWindows) {
+						int cp = Encoding.Default.CodePage;
+						if (cp >= 1250 && cp < 1260)
+							return cp;
+					}
+					return westernEncodingCodePage;
+				}
+			}
+
 			public override bool IsSupported {
 				get {
 					try {
-						return Encoding.GetEncoding (1252) != null;
+						return Encoding.GetEncoding (WindowsCodePage) != null;
 					} catch (Exception) {
 						return false;
 					}
@@ -627,7 +692,7 @@ namespace Mono.TextEditor.Utils
 
 			protected override void Init ()
 			{
-				EncodingCp1252 = Encoding.GetEncoding (1252);
+				EncodingWindows = Encoding.GetEncoding (WindowsCodePage);
 				table = new byte[LAST][];
 				table [0] = errorTable;
 				for (int i = 1; i < LAST; i++)
