@@ -34,82 +34,30 @@ using System.Linq;
 
 namespace Mono.TextEditor.Vi
 {
-	public class NewViEditMode : EditMode
-	{
-		protected ViEditor ViEditor { get ; private set ;}
-		
-		public NewViEditMode ()
-		{
-			ViEditor = new ViEditor (this);
-		}
-		
-		protected override void HandleKeypress (Gdk.Key key, uint unicodeKey, Gdk.ModifierType modifier)
-		{
-			ViEditor.ProcessKey (modifier, key, (char)unicodeKey);
-		}
-		
-		public new TextEditor Editor { get { return base.Editor; } }
-		public new TextEditorData Data { get { return base.Data; } }
-		
-		public override bool WantsToPreemptIM {
-			get {
-				switch (ViEditor.Mode) {
-				case ViEditorMode.Insert:
-				case ViEditorMode.Replace:
-					return false;
-				case ViEditorMode.Normal:
-				case ViEditorMode.Visual:
-				case ViEditorMode.VisualLine:
-				default:
-					return true;
-				}
-			}
-		}
-		
-		protected override void OnAddedToEditor (TextEditorData data)
-		{
-			ViEditor.SetMode (ViEditorMode.Normal);
-			SetCaretMode (CaretMode.Block, data);
-			ViActions.RetreatFromLineEnd (data);
-		}
-		
-		protected override void OnRemovedFromEditor (TextEditorData data)
-		{
-			SetCaretMode (CaretMode.Insert, data);
-		}
-		
-		protected override void CaretPositionChanged ()
-		{
-			ViEditor.OnCaretPositionChanged ();
-		}
-		
-		public void SetCaretMode (CaretMode mode)
-		{
-			SetCaretMode (mode, Data);
-		}
-		
-		static void SetCaretMode (CaretMode mode, TextEditorData data)
-		{
-			if (data.Caret.Mode == mode)
-				return;
-			data.Caret.Mode = mode;
-			data.Document.RequestUpdate (new SinglePositionUpdate (data.Caret.Line, data.Caret.Column));
-			data.Document.CommitDocumentUpdate ();
-		}
-	}
-	
 	public class ViEditMode : EditMode
 	{
 		bool searchBackward;
 		static string lastPattern;
 		static string lastReplacement;
-		State state;
+		State curState;
+		State CurState {
+			get {
+				return curState;
+			}
+			set {
+				curState = value;
+				if (statusArea != null) {
+					statusArea.ShowCaret = curState == State.Command;
+				}
+			}
+		}
+
+		Motion motion;
 		const string substMatch = @"^:s(?<sep>.)(?<pattern>.+?)\k<sep>(?<replacement>.*?)(\k<sep>(?<trailer>i?))?$";
 		StringBuilder commandBuffer = new StringBuilder ();
 		Dictionary<char,ViMark> marks = new Dictionary<char, ViMark>();
 		Dictionary<char,ViMacro> macros = new Dictionary<char, ViMacro>();
 		char macros_lastplayed = '@'; // start with the illegal macro character
-		string statusText = "";
 		
 		/// <summary>
 		/// The macro currently being implemented. Will be set to null and checked as a flag when required.
@@ -119,17 +67,17 @@ namespace Mono.TextEditor.Vi
 		public virtual string Status {
 		
 			get {
-				return statusText;
+				return statusArea.Message;
 			}
 			
 			protected set {
-				if (currentMacro == null) {
-					statusText = value;
-				} else {
-					statusText = value + " recording";
+				if (currentMacro != null) {
+					value = value + " recording";
+				}
+				if (viTextEditor != null) {
+					statusArea.Message = value;
 				}
 			}
-	
 		}
 		
 		protected virtual string RunExCommand (string command)
@@ -214,38 +162,41 @@ namespace Mono.TextEditor.Vi
 		
 		public override bool WantsToPreemptIM {
 			get {
-				return state != State.Insert && state != State.Replace;
+				return CurState != State.Insert && CurState != State.Replace;
 			}
 		}
 		
 		protected override void SelectionChanged ()
 		{
 			if (Data.IsSomethingSelected) {
-				state = ViEditMode.State.Visual;
+				CurState = ViEditMode.State.Visual;
 				Status = "-- VISUAL --";
-			} else if (state == State.Visual && !Data.IsSomethingSelected) {
+			} else if (CurState == State.Visual && !Data.IsSomethingSelected) {
 				Reset ("");
 			}
 		}
 		
 		protected override void CaretPositionChanged ()
 		{
-			if (state == State.Replace || state == State.Insert || state == State.Visual)
+			if (CurState == State.Replace || CurState == State.Insert || CurState == State.Visual)
 				return;
-			else if (state == ViEditMode.State.Normal || state == ViEditMode.State.Unknown)
+			else if (CurState == ViEditMode.State.Normal || CurState == ViEditMode.State.Unknown)
 				ViActions.RetreatFromLineEnd (Data);
 			else
 				Reset ("");
 		}
-		
+
+		ViStatusArea statusArea;
+		TextEditor viTextEditor;
+
 		void CheckVisualMode ()
 		{
-			if (state == ViEditMode.State.Visual || state == ViEditMode.State.Visual) {
+			if (CurState == ViEditMode.State.Visual || CurState == ViEditMode.State.Visual) {
 				if (!Data.IsSomethingSelected)
-					state = ViEditMode.State.Normal;
+					CurState = ViEditMode.State.Normal;
 			} else {
 				if (Data.IsSomethingSelected) {
-					state = ViEditMode.State.Visual;
+					CurState = ViEditMode.State.Visual;
 					Status = "-- VISUAL --";
 				}
 			}
@@ -274,16 +225,32 @@ namespace Mono.TextEditor.Vi
 		{
 			data.Caret.Mode = CaretMode.Block;
 			ViActions.RetreatFromLineEnd (data);
+
+			viTextEditor = data.Parent;
+			if (viTextEditor != null) {
+				statusArea = new ViStatusArea (viTextEditor);
+			}
 		}
 		
 		protected override void OnRemovedFromEditor (TextEditorData data)
 		{
 			data.Caret.Mode = CaretMode.Insert;
+
+			if (viTextEditor != null) {
+				statusArea.RemoveFromParentAndDestroy ();
+				statusArea = null;
+				viTextEditor = null;
+			}
+		}
+
+		public override void AllocateTextArea (TextEditor textEditor, TextArea textArea, Gdk.Rectangle allocation)
+		{
+			statusArea.AllocateArea (textArea, allocation);
 		}
 		
 		void Reset (string status)
 		{
-			state = State.Normal;
+			CurState = State.Normal;
 			ResetEditorState (Data);
 			
 			commandBuffer.Length = 0;
@@ -330,7 +297,7 @@ namespace Mono.TextEditor.Vi
 			Action<TextEditorData> action = null;
 			bool lineAction = false;
 			
-			switch (state) {
+			switch (CurState) {
 			case State.Unknown:
 				Reset (string.Empty);
 				goto case State.Normal;
@@ -342,7 +309,7 @@ namespace Mono.TextEditor.Vi
 					case '?':
 					case '/':
 					case ':':
-						state = State.Command;
+						CurState = State.Command;
 						commandBuffer.Append ((char)unicodeKey);
 						Status = commandBuffer.ToString ();
 						return;
@@ -362,39 +329,39 @@ namespace Mono.TextEditor.Vi
 					case 'i':
 						Caret.Mode = CaretMode.Insert;
 						Status = "-- INSERT --";
-						state = State.Insert;
+						CurState = State.Insert;
 						return;
 						
 					case 'R':
 						Caret.Mode = CaretMode.Underscore;
 						Status = "-- REPLACE --";
-						state = State.Replace;
+						CurState = State.Replace;
 						return;
 
 					case 'V':
 						Status = "-- VISUAL LINE --";
 						Data.SetSelectLines (Caret.Line, Caret.Line);
-						state = State.VisualLine;
+						CurState = State.VisualLine;
 						return;
 						
 					case 'v':
 						Status = "-- VISUAL --";
-						state = State.Visual;
+						CurState = State.Visual;
 						RunAction (ViActions.VisualSelectionFromMoveAction (ViActions.Right));
 						return;
 						
 					case 'd':
 						Status = "d";
-						state = State.Delete;
+						CurState = State.Delete;
 						return;
 						
 					case 'y':
 						Status = "y";
-						state = State.Yank;
+						CurState = State.Yank;
 						return;
 
 					case 'Y':
-						state = State.Yank;
+						CurState = State.Yank;
 						HandleKeypress (Gdk.Key.y, (int)'y', Gdk.ModifierType.None);
 						return;
 						
@@ -409,13 +376,13 @@ namespace Mono.TextEditor.Vi
 					case 'r':
 						Caret.Mode = CaretMode.Underscore;
 						Status = "-- REPLACE --";
-						state = State.WriteChar;
+						CurState = State.WriteChar;
 						return;
 						
 					case 'c':
 						Caret.Mode = CaretMode.Insert;
 						Status = "c";
-						state = State.Change;
+						CurState = State.Change;
 						return;
 						
 					case 'x':
@@ -449,12 +416,12 @@ namespace Mono.TextEditor.Vi
 						
 					case '>':
 						Status = ">";
-						state = State.Indent;
+						CurState = State.Indent;
 						return;
 						
 					case '<':
 						Status = "<";
-						state = State.Unindent;
+						CurState = State.Unindent;
 						return;
 					case 'n':
 						Search ();
@@ -484,7 +451,7 @@ namespace Mono.TextEditor.Vi
 						
 					case 'g':
 						Status = "g";
-						state = State.G;
+						CurState = State.G;
 						return;
 						
 					case 'H':
@@ -512,28 +479,28 @@ namespace Mono.TextEditor.Vi
 						
 					case 'z':
 						Status = "z";
-						state = State.Fold;
+						CurState = State.Fold;
 						return;
 						
 					case 'm':
 						Status = "m";
-						state = State.Mark;
+						CurState = State.Mark;
 						return;
 						
 					case '`':
 						Status = "`";
-						state = State.GoToMark;
+						CurState = State.GoToMark;
 						return;
 						
 					case '@':
 						Status = "@";
-						state = State.PlayMacro;
+						CurState = State.PlayMacro;
 						return;
 	
 					case 'q':
 						if (currentMacro == null) {
 							Status = "q";
-							state = State.NameMacro;
+							CurState = State.NameMacro;
 							return;
 						} 
 						currentMacro = null;
@@ -560,7 +527,12 @@ namespace Mono.TextEditor.Vi
 				return;
 				
 			case State.Delete:
-				if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
+				if (IsInnerOrOuterMotionKey (unicodeKey, ref motion)) return;
+
+				if (motion != Motion.None) {
+					action = ViActionMaps.GetEditObjectCharAction((char) unicodeKey, motion);
+				}
+				else if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
 				     && unicodeKey == 'd'))
 				{
 					action = SelectionActions.LineActionFromMoveAction (CaretMoveActions.LineEnd);
@@ -586,9 +558,13 @@ namespace Mono.TextEditor.Vi
 				return;
 
 			case State.Yank:
+				if (IsInnerOrOuterMotionKey (unicodeKey, ref motion)) return;
 				int offset = Caret.Offset;
-				
-				if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
+
+				if (motion != Motion.None) {
+					action = ViActionMaps.GetEditObjectCharAction((char) unicodeKey, motion);
+				}
+				else if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0
 				     && unicodeKey == 'y'))
 				{
 					action = SelectionActions.LineActionFromMoveAction (CaretMoveActions.LineEnd);
@@ -615,8 +591,13 @@ namespace Mono.TextEditor.Vi
 				return;
 				
 			case State.Change:
+				if (IsInnerOrOuterMotionKey (unicodeKey, ref motion)) return;
+
+				if (motion != Motion.None) {
+					action = ViActionMaps.GetEditObjectCharAction((char) unicodeKey, motion);
+				}
 				//copied from delete action
-				if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
+				else if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
 				     && unicodeKey == 'c'))
 				{
 					action = SelectionActions.LineActionFromMoveAction (CaretMoveActions.LineEnd);
@@ -635,7 +616,7 @@ namespace Mono.TextEditor.Vi
 					else
 						RunActions (action, ClipboardActions.Cut);
 					Status = "-- INSERT --";
-					state = State.Insert;
+					CurState = State.Insert;
 					Caret.Mode = CaretMode.Insert;
 				} else {
 					Reset ("Unrecognised motion");
@@ -681,6 +662,16 @@ namespace Mono.TextEditor.Vi
 				return;
 
 			case State.Visual:
+				if (IsInnerOrOuterMotionKey (unicodeKey, ref motion)) return;
+
+				if (motion != Motion.None) {
+					action = ViActionMaps.GetEditObjectCharAction((char) unicodeKey, motion);
+					if (action != null) {
+						RunAction (action);
+						return;
+					}
+				}
+
 				if (key == Gdk.Key.Delete)
 					unicodeKey = 'x';
 				switch ((char)unicodeKey) {
@@ -712,7 +703,7 @@ namespace Mono.TextEditor.Vi
 				case Gdk.Key.KP_Enter:
 					Status = RunExCommand (commandBuffer.ToString ());
 					commandBuffer.Length = 0;
-					state = State.Normal;
+					CurState = State.Normal;
 					break;
 				case Gdk.Key.BackSpace:
 				case Gdk.Key.Delete:
@@ -908,6 +899,19 @@ namespace Mono.TextEditor.Vi
 			}
 		}
 
+		static bool IsInnerOrOuterMotionKey (uint unicodeKey, ref Motion motion)
+		{
+			if (unicodeKey == 'i') {
+				motion = Motion.Inner;
+				return true;
+			} 
+			if (unicodeKey == 'a') {
+				motion = Motion.Outer;
+				return true;
+			}
+			return false;
+		}
+
 		/// <summary>
 		/// Runs an in-place replacement on the selection or the current line
 		/// using the "pattern", "replacement", and "trailer" groups of match.
@@ -971,7 +975,7 @@ namespace Mono.TextEditor.Vi
 				case 'c':
 					RunAction (ClipboardActions.Cut);
 					Caret.Mode = CaretMode.Insert;
-					state = State.Insert;
+					CurState = State.Insert;
 					Status = "-- INSERT --";
 					return;
 				case 'S':
@@ -991,7 +995,7 @@ namespace Mono.TextEditor.Vi
 				case ':':
 					commandBuffer.Append (":");
 					Status = commandBuffer.ToString ();
-					state = State.Command;
+					CurState = State.Command;
 					break;
 				case 'J':
 					RunAction (ViActions.Join);
@@ -1136,4 +1140,10 @@ namespace Mono.TextEditor.Vi
 			PlayMacro
 		}
 	}
+
+	public enum Motion {
+		None = 0,
+		Inner,
+		Outer
+	}
 }
