@@ -39,19 +39,19 @@ namespace Mono.TextEditor.Vi
 		/// <summary>
 		/// Move to next non-whitespace change in character class.
 		/// </summary>
-		public override int FindNextSubwordOffset (Document doc, int offset)
+		public override int FindNextSubwordOffset (TextDocument doc, int offset)
 		{
 			int myoffset = offset;
-			if (0 > myoffset || doc.Length-1 <= myoffset){ return myoffset; }
+			if (0 > myoffset || doc.TextLength-1 <= myoffset){ return myoffset; }
 			
 			char c = doc.GetCharAt (myoffset);
 			CharacterClass initialClass = GetCharacterClass (c);
 			
-			while (GetCharacterClass (c) == initialClass && 0 <= myoffset && doc.Length-1 > myoffset) {
+			while (GetCharacterClass (c) == initialClass && 0 <= myoffset && doc.TextLength-1 > myoffset) {
 				c = doc.GetCharAt (++myoffset);
 			}
 			for (c = doc.GetCharAt (myoffset);
-			     char.IsWhiteSpace (c) && 0 <= myoffset && doc.Length-1 > myoffset;
+			     char.IsWhiteSpace (c) && 0 <= myoffset && doc.TextLength-1 > myoffset;
 			     c = doc.GetCharAt (++myoffset));
 			     
 			return (myoffset == offset)? myoffset+1: myoffset;
@@ -60,16 +60,16 @@ namespace Mono.TextEditor.Vi
 		/// <summary>
 		/// Move past next whitespace group.
 		/// </summary>
-		public override int FindNextWordOffset (Document doc, int offset)
+		public override int FindNextWordOffset (TextDocument doc, int offset)
 		{
 			int myoffset = offset;
-			if (0 > myoffset || doc.Length-1 <= myoffset){ return myoffset; }
+			if (0 > myoffset || doc.TextLength-1 <= myoffset){ return myoffset; }
 			
 			for (char c = doc.GetCharAt (myoffset);
-			     !char.IsWhiteSpace (c) && 0 <= myoffset && doc.Length-1 > myoffset;
+			     !char.IsWhiteSpace (c) && 0 <= myoffset && doc.TextLength-1 > myoffset;
 			     c = doc.GetCharAt (++myoffset));
 			for (char c = doc.GetCharAt (myoffset);
-			     char.IsWhiteSpace (c) && 0 <= myoffset && doc.Length-1 > myoffset;
+			     char.IsWhiteSpace (c) && 0 <= myoffset && doc.TextLength-1 > myoffset;
 			     c = doc.GetCharAt (++myoffset));
 			     
 			return (myoffset == offset)? myoffset+1: myoffset;
@@ -78,20 +78,20 @@ namespace Mono.TextEditor.Vi
 		/// <summary>
 		/// Move to previous non-whitespace change in character class.
 		/// </summary>
-		public override int FindPrevSubwordOffset (Document doc, int offset)
+		public override int FindPrevSubwordOffset (TextDocument doc, int offset)
 		{
 			int myoffset = offset-1;
 			char c;
-			if (0 > myoffset || doc.Length-1 <= myoffset){ return myoffset; }
+			if (0 > myoffset || doc.TextLength-1 <= myoffset){ return myoffset; }
 			
 			for (c = doc.GetCharAt (myoffset);
-			     char.IsWhiteSpace (c) && 0 <= myoffset && doc.Length-1 > myoffset;
+			     char.IsWhiteSpace (c) && 0 <= myoffset && doc.TextLength-1 > myoffset;
 			     c = doc.GetCharAt (--myoffset));
 			     
 			CharacterClass initialClass = GetCharacterClass (c);
 			
 			for (; GetCharacterClass (c) == initialClass && 
-			     0 <= myoffset && doc.Length-1 > myoffset;
+			     0 <= myoffset && doc.TextLength-1 > myoffset;
 			     c = doc.GetCharAt (--myoffset));
 			     
 			return (0 == myoffset)? myoffset: myoffset+1;
@@ -100,16 +100,16 @@ namespace Mono.TextEditor.Vi
 		/// <summary>
 		/// Move to end of previous whitespace group.
 		/// </summary>
-		public override int FindPrevWordOffset (Document doc, int offset)
+		public override int FindPrevWordOffset (TextDocument doc, int offset)
 		{
 			--offset;
-			if (0 > offset || doc.Length-1 <= offset){ return offset; }
+			if (0 > offset || doc.TextLength-1 <= offset){ return offset; }
 			
 			for (char c = doc.GetCharAt (offset);
-			     char.IsWhiteSpace (c) && 0 < offset && doc.Length > offset;
+			     char.IsWhiteSpace (c) && 0 < offset && doc.TextLength > offset;
 			     c = doc.GetCharAt (--offset));
 			for (char c = doc.GetCharAt (offset);
-			     !char.IsWhiteSpace (c) && 0 < offset && doc.Length > offset;
+			     !char.IsWhiteSpace (c) && 0 < offset && doc.TextLength > offset;
 			     c = doc.GetCharAt (--offset));
 			     
 			return (0 == offset)? offset: offset+1;
@@ -131,7 +131,7 @@ namespace Mono.TextEditor.Vi
 			}
 		}	
 		
-		enum CharacterClass
+		new enum CharacterClass
 		{
 			AlphaNumeric, // Should be roughly equivalent to [\w\d]
 			Whitespace,
