@@ -38,7 +38,7 @@ using System.Threading;
 
 namespace Mono.TextEditor
 {
-	public class TextDocument : AbstractAnnotatable, IBuffer, ICSharpCode.NRefactory.Editor.IDocument
+	public class TextDocument : AbstractAnnotatable, ICSharpCode.NRefactory.Editor.IDocument
 	{
 		readonly IBuffer buffer;
 		readonly ILineSplitter splitter;
@@ -58,17 +58,34 @@ namespace Mono.TextEditor
 			}
 			set {
 				if (mimeType != value) {
-					mimeType = value;
-					SyntaxMode = SyntaxModeService.GetSyntaxMode (this);
+					lock (this) {
+						mimeType = value;
+						SyntaxMode = SyntaxModeService.GetSyntaxMode (this, value);
+					}
 				}
 			}
 		}
 		
+		string fileName;
 		public string FileName {
-			get;
-			set;
+			get {
+				return fileName;
+			}
+			set {
+				fileName = value;
+				OnFileNameChanged (EventArgs.Empty);
+			}
 		}	
-		
+
+		public event EventHandler FileNameChanged;
+
+		protected virtual void OnFileNameChanged (EventArgs e)
+		{
+			EventHandler handler = this.FileNameChanged;
+			if (handler != null)
+				handler (this, e);
+		}
+
 		public bool HeightChanged {
 			get;
 			set;
@@ -116,6 +133,7 @@ namespace Mono.TextEditor
 			splitter.LineChanged += SplitterLineSegmentTreeLineChanged;
 			splitter.LineRemoved += HandleSplitterLineSegmentTreeLineRemoved;
 			foldSegmentTree.tree.NodeRemoved += HandleFoldSegmentTreetreeNodeRemoved; 
+			textSegmentMarkerTree.InstallListener (this);
 		}
 
 		void HandleFoldSegmentTreetreeNodeRemoved (object sender, RedBlackTree<FoldSegment>.RedBlackTreeNodeEventArgs e)
@@ -166,8 +184,10 @@ namespace Mono.TextEditor
 			}
 			set {
 				var args = new DocumentChangeEventArgs (0, Text, value);
+				textSegmentMarkerTree.Clear ();
 				OnTextReplacing (args);
 				buffer.Text = value;
+				extendingTextMarkers = new List<TextLineMarker> ();
 				splitter.Initalize (value);
 				ClearFoldSegments ();
 				OnTextReplaced (args);
@@ -178,9 +198,9 @@ namespace Mono.TextEditor
 			}
 		}
 
-		public void Insert (int offset, string text)
+		public void Insert (int offset, string text, ICSharpCode.NRefactory.Editor.AnchorMovementType anchorMovementType = AnchorMovementType.Default)
 		{
-			Replace (offset, 0, text);
+			Replace (offset, 0, text, anchorMovementType);
 		}
 		
 		public void Remove (int offset, int count)
@@ -194,6 +214,11 @@ namespace Mono.TextEditor
 		}
 
 		public void Replace (int offset, int count, string value)
+		{
+			Replace (offset, count, value, AnchorMovementType.Default);
+		}
+
+		public void Replace (int offset, int count, string value, ICSharpCode.NRefactory.Editor.AnchorMovementType anchorMovementType = AnchorMovementType.Default)
 		{
 			if (offset < 0)
 				throw new ArgumentOutOfRangeException ("offset", "must be > 0, was: " + offset);
@@ -205,9 +230,9 @@ namespace Mono.TextEditor
 			InterruptFoldWorker ();
 			
 			//int oldLineCount = LineCount;
-			var args = new DocumentChangeEventArgs (offset, count > 0 ? GetTextAt (offset, count) : "", value);
+			var args = new DocumentChangeEventArgs (offset, count > 0 ? GetTextAt (offset, count) : "", value, anchorMovementType);
 			OnTextReplacing (args);
-			value = args.InsertedText;
+			value = args.InsertedText.Text;
 			UndoOperation operation = null;
 			if (!isInUndo) {
 				operation = new UndoOperation (args);
@@ -302,6 +327,10 @@ namespace Mono.TextEditor
 		
 		public char GetCharAt (int offset)
 		{
+			if (offset < 0)
+				throw new ArgumentException ("offset < 0");
+			if (offset >= TextLength)
+				throw new ArgumentException ("offset >= TextLength");
 			return buffer.GetCharAt (offset);
 		}
 
@@ -454,7 +483,8 @@ namespace Mono.TextEditor
 			if (lineNr < DocumentLocation.MinLine)
 				return DocumentLocation.Empty;
 			DocumentLine line = GetLine (lineNr);
-			return new DocumentLocation (lineNr, System.Math.Min (line.LengthIncludingDelimiter, offset - line.Offset) + 1);
+			var col = System.Math.Max (1, System.Math.Min (line.LengthIncludingDelimiter, offset - line.Offset) + 1);
+			return new DocumentLocation (lineNr, col);
 		}
 
 		public string GetLineIndent (int lineNumber)
@@ -544,13 +574,13 @@ namespace Mono.TextEditor
 			
 			public virtual void Undo (TextDocument doc)
 			{
-				doc.Replace (args.Offset, args.InsertionLength, args.RemovedText);
+				doc.Replace (args.Offset, args.InsertionLength, args.RemovedText.Text);
 				OnUndoDone ();
 			}
 			
 			public virtual void Redo (TextDocument doc)
 			{
-				doc.Replace (args.Offset, args.RemovalLength, args.InsertedText);
+				doc.Replace (args.Offset, args.RemovalLength, args.InsertedText.Text);
 				OnRedoDone ();
 			}
 			
@@ -666,7 +696,33 @@ namespace Mono.TextEditor
 //				op.InformTextReplace (args);
 //			}
 		}
-		
+
+		internal int UndoBeginOffset {
+			get {
+				if (undoStack.Count == 0)
+					return -1;
+				var op = undoStack.Peek ();
+				while (op is AtomicUndoOperation)
+					op = ((AtomicUndoOperation)op).Operations.FirstOrDefault ();
+				if (op == null)
+					return -1;
+				return ((UndoOperation)op).Args.Offset;
+			}
+		}
+
+		internal int RedoBeginOffset {
+			get {
+				if (redoStack.Count == 0)
+					return -1;
+				var op = redoStack.Peek ();
+				while (op is AtomicUndoOperation)
+					op = ((AtomicUndoOperation)op).Operations.FirstOrDefault ();
+				if (op == null)
+					return -1;
+				return ((UndoOperation)op).Args.Offset;
+			}
+		}
+
 		public bool CanUndo {
 			get {
 				return this.undoStack.Count > 0 || currentAtomicOperation != null;
@@ -797,7 +853,17 @@ namespace Mono.TextEditor
 			this.RequestUpdate (new UpdateAll ());
 			this.CommitDocumentUpdate ();
 		}
-		
+
+		public void RollbackTo (ICSharpCode.NRefactory.Editor.ITextSourceVersion version)
+		{
+			var steps = Version.CompareAge (version);
+			if (steps < 0)
+				throw new InvalidOperationException ("Invalid version");
+			while (steps-- > 0) {
+				undoStack.Pop ().Undo (this);
+			}
+		}
+
 		internal protected virtual void OnUndone (UndoOperationEventArgs e)
 		{
 			EventHandler<UndoOperationEventArgs> handler = this.Undone;
@@ -982,7 +1048,6 @@ namespace Mono.TextEditor
 			if (newSegments == null) {
 				return;
 			}
-			Console.WriteLine ("startTask={0}, useApplicationInvoke={1}", startTask, useApplicationInvoke);
 			
 			InterruptFoldWorker ();
 			bool update;
@@ -1206,7 +1271,9 @@ namespace Mono.TextEditor
 		
 		public event EventHandler<FoldSegmentEventArgs> Folded;
 		#endregion
-		
+
+		#region Text line markers
+
 		public event EventHandler<TextMarkerEvent> MarkerAdded;
 		protected virtual void OnMarkerAdded (TextMarkerEvent e)
 		{
@@ -1224,28 +1291,28 @@ namespace Mono.TextEditor
 		}
 
 		
-		List<TextMarker> extendingTextMarkers = new List<TextMarker> ();
+		List<TextLineMarker> extendingTextMarkers = new List<TextLineMarker> ();
 		public IEnumerable<DocumentLine> LinesWithExtendingTextMarkers {
 			get {
 				return from marker in extendingTextMarkers where marker.LineSegment != null select marker.LineSegment;
 			}
 		}
 		
-		public void AddMarker (int lineNumber, TextMarker marker)
+		public void AddMarker (int lineNumber, TextLineMarker marker)
 		{
 			AddMarker (this.GetLine (lineNumber), marker);
 		}
 		
-		public void AddMarker (DocumentLine line, TextMarker marker)
+		public void AddMarker (DocumentLine line, TextLineMarker marker)
 		{
 			AddMarker (line, marker, true);
 		}
-		
-		public void AddMarker (DocumentLine line, TextMarker marker, bool commitUpdate)
+
+		public void AddMarker (DocumentLine line, TextLineMarker marker, bool commitUpdate)
 		{
 			if (line == null || marker == null)
 				return;
-			if (marker is IExtendingTextMarker) {
+			if (marker is IExtendingTextLineMarker) {
 				lock (extendingTextMarkers) {
 					HeightChanged = true;
 					extendingTextMarkers.Add (marker);
@@ -1258,26 +1325,26 @@ namespace Mono.TextEditor
 				this.CommitLineUpdate (line);
 		}
 		
-		static int CompareMarkers (TextMarker left, TextMarker right)
+		static int CompareMarkers (TextLineMarker left, TextLineMarker right)
 		{
 			if (left.LineSegment == null || right.LineSegment == null)
 				return 0;
 			return left.LineSegment.Offset.CompareTo (right.LineSegment.Offset);
 		}
 		
-		public void RemoveMarker (TextMarker marker)
+		public void RemoveMarker (TextLineMarker marker)
 		{
 			RemoveMarker (marker, true);
 		}
 		
-		public void RemoveMarker (TextMarker marker, bool updateLine)
+		public void RemoveMarker (TextLineMarker marker, bool updateLine)
 		{
 			if (marker == null)
 				return;
 			var line = marker.LineSegment;
 			if (line == null)
 				return;
-			if (marker is IExtendingTextMarker) {
+			if (marker is IExtendingTextLineMarker) {
 				lock (extendingTextMarkers) {
 					HeightChanged = true;
 					extendingTextMarkers.Remove (marker);
@@ -1307,10 +1374,10 @@ namespace Mono.TextEditor
 		{
 			if (line == null || type == null)
 				return;
-			if (typeof(IExtendingTextMarker).IsAssignableFrom (type)) {
+			if (typeof(IExtendingTextLineMarker).IsAssignableFrom (type)) {
 				lock (extendingTextMarkers) {
 					HeightChanged = true;
-					foreach (TextMarker marker in line.Markers.Where (marker => marker is IExtendingTextMarker)) {
+					foreach (TextLineMarker marker in line.Markers.Where (marker => marker is IExtendingTextLineMarker)) {
 						extendingTextMarkers.Remove (marker);
 					}
 				}
@@ -1319,16 +1386,59 @@ namespace Mono.TextEditor
 			if (updateLine)
 				this.CommitLineUpdate (line);
 		}
+
+		#endregion
+
+		#region Text segment markers
+
+		SegmentTree<TextSegmentMarker> textSegmentMarkerTree = new SegmentTree<TextSegmentMarker> (); 
+
+		public IEnumerable<TextSegmentMarker> GetTextSegmentMarkersAt (DocumentLine line)
+		{
+			return textSegmentMarkerTree.GetSegmentsOverlapping (line.Segment);
+		}
+
+		public IEnumerable<TextSegmentMarker> GetTextSegmentMarkersAt (TextSegment segment)
+		{
+			return textSegmentMarkerTree.GetSegmentsOverlapping (segment);
+		}
+
+		public IEnumerable<TextSegmentMarker> GetTextSegmentMarkersAt (int offset)
+		{
+			return textSegmentMarkerTree.GetSegmentsAt (offset);
+		}
 		
+
+		public void AddMarker (TextSegmentMarker marker)
+		{
+			CommitLineUpdate (GetLineByOffset (marker.Offset));
+			textSegmentMarkerTree.Add (marker);
+		}
+
+		/// <summary>
+		/// Removes a marker from the document.
+		/// </summary>
+		/// <returns><c>true</c>, if marker was removed, <c>false</c> otherwise.</returns>
+		/// <param name="marker">Marker.</param>
+		public bool RemoveMarker (TextSegmentMarker marker)
+		{
+			bool wasRemoved = textSegmentMarkerTree.Remove (marker);
+			if (wasRemoved)
+				CommitLineUpdate (GetLineByOffset (marker.Offset));
+			return wasRemoved;
+		}
+
+		#endregion
+
 		void HandleSplitterLineSegmentTreeLineRemoved (object sender, LineEventArgs e)
 		{
-			foreach (TextMarker marker in e.Line.Markers) {
-				if (marker is IExtendingTextMarker) {
+			foreach (TextLineMarker marker in e.Line.Markers) {
+				if (marker is IExtendingTextLineMarker) {
 					lock (extendingTextMarkers) {
 						HeightChanged = true;
 						extendingTextMarkers.Remove (marker);
 					}
-					UnRegisterVirtualTextMarker ((IExtendingTextMarker)marker);
+					UnRegisterVirtualTextMarker ((IExtendingTextLineMarker)marker);
 				}
 			}
 		}
@@ -1423,7 +1533,7 @@ namespace Mono.TextEditor
 		
 		public static bool IsWordSeparator (char ch)
 		{
-			return Char.IsWhiteSpace (ch) || (Char.IsPunctuation (ch) && ch != '_');
+			return !(char.IsLetterOrDigit (ch) || ch == '_');
 		}
 
 		public bool IsWholeWordAt (int offset, int length)
@@ -1537,15 +1647,15 @@ namespace Mono.TextEditor
 			}
 		}
 		
-		Dictionary<int, IExtendingTextMarker> virtualTextMarkers = new Dictionary<int, IExtendingTextMarker> ();
-		public void RegisterVirtualTextMarker (int lineNumber, IExtendingTextMarker marker)
+		Dictionary<int, IExtendingTextLineMarker> virtualTextMarkers = new Dictionary<int, IExtendingTextLineMarker> ();
+		public void RegisterVirtualTextMarker (int lineNumber, IExtendingTextLineMarker marker)
 		{
 			virtualTextMarkers[lineNumber] = marker;
 		}
 		
-		public IExtendingTextMarker GetExtendingTextMarker (int lineNumber)
+		public IExtendingTextLineMarker GetExtendingTextMarker (int lineNumber)
 		{
-			IExtendingTextMarker result;
+			IExtendingTextLineMarker result;
 			if (virtualTextMarkers.TryGetValue (lineNumber, out result))
 				return result;
 			return null;
@@ -1557,7 +1667,7 @@ namespace Mono.TextEditor
 		/// <param name='marker'>
 		/// marker.
 		/// </param>
-		public void UnRegisterVirtualTextMarker (IExtendingTextMarker marker)
+		public void UnRegisterVirtualTextMarker (IExtendingTextLineMarker marker)
 		{
 			var keys = new List<int> (from pair in virtualTextMarkers where pair.Value == marker select pair.Key);
 			keys.ForEach (key => { virtualTextMarkers.Remove (key); CommitLineUpdate (key); });
@@ -1692,14 +1802,29 @@ namespace Mono.TextEditor
 			return OffsetToLocation (offset);
 		}
 
+		void ICSharpCode.NRefactory.Editor.IDocument.Insert (int offset, ITextSource text)
+		{
+			Insert (offset, text.Text);
+		}
+
+		void ICSharpCode.NRefactory.Editor.IDocument.Replace (int offset, int count, ITextSource text)
+		{
+			Replace (offset, count, text.Text);
+		}
+
 		void ICSharpCode.NRefactory.Editor.IDocument.Insert (int offset, string text)
 		{
 			Insert (offset, text);
 		}
 
-		public void Insert (int offset, string text, ICSharpCode.NRefactory.Editor.AnchorMovementType defaultAnchorMovementType)
+		void ICSharpCode.NRefactory.Editor.IDocument.Insert (int offset, ITextSource text, AnchorMovementType anchorMovementType)
 		{
-			Insert (offset, text);
+			Insert (offset, text.Text, anchorMovementType);
+		}
+
+		void ICSharpCode.NRefactory.Editor.IDocument.Insert (int offset, string text, AnchorMovementType anchorMovementType)
+		{
+			Insert (offset, text, anchorMovementType);
 		}
 
 		void ICSharpCode.NRefactory.Editor.IDocument.StartUndoableAction ()
@@ -1726,6 +1851,16 @@ namespace Mono.TextEditor
 		#endregion
 
 		#region ITextSource implementation
+		void ICSharpCode.NRefactory.Editor.ITextSource.WriteTextTo (System.IO.TextWriter writer)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void ICSharpCode.NRefactory.Editor.ITextSource.WriteTextTo (System.IO.TextWriter writer, int offset, int length)
+		{
+			throw new NotImplementedException ();
+		}
+
 		ICSharpCode.NRefactory.Editor.ITextSource ICSharpCode.NRefactory.Editor.ITextSource.CreateSnapshot ()
 		{
 			throw new NotImplementedException ();
@@ -1789,6 +1924,9 @@ namespace Mono.TextEditor
 		{
 			return new SnapshotDocument (Text, Version);
 		}
+
+
+
 		#endregion
 	}
 	
