@@ -103,15 +103,26 @@ namespace Mono.TextEditor
 		public virtual bool WantsToPreemptIM {
 			get { return false; }
 		}
-		
+
+		bool IsSpecialKeyForSelection (uint unicodeKey)
+		{
+			string start, end;
+			return textEditorData.SelectionSurroundingProvider.GetSelectionSurroundings (textEditorData, unicodeKey, out start, out end);
+		}
+
 		protected void InsertCharacter (uint unicodeKey)
 		{
 			if (!textEditorData.CanEdit (Data.Caret.Line))
 				return;
-			
+
 			HideMouseCursor ();
 
 			using (var undo = Document.OpenUndoGroup ()) {
+				if (textEditorData.IsSomethingSelected && textEditorData.Options.EnableSelectionWrappingKeys && IsSpecialKeyForSelection (unicodeKey)) {
+					textEditorData.SelectionSurroundingProvider.HandleSpecialSelectionKey (textEditorData, unicodeKey);
+					return;
+				}
+
 				textEditorData.DeleteSelectedText (
 					textEditorData.IsSomethingSelected ? textEditorData.MainSelection.SelectionMode != SelectionMode.Block : true);
 				// Needs to be called after delete text, delete text handles virtual caret postitions itself,
@@ -143,11 +154,13 @@ namespace Mono.TextEditor
 								}
 								textEditorData.Insert (lineSegment.Offset + insertOffset, textToInsert);
 							}
+							var visualColumn = textEditorData.GetLine (Caret.Location.Line).GetVisualColumn (textEditorData, Caret.Column);
 
 							textEditorData.MainSelection = new Selection (
-								new DocumentLocation (selection.Anchor.Line, Caret.Column),
-								new DocumentLocation (selection.Lead.Line, Caret.Column),
-								Mono.TextEditor.SelectionMode.Block);
+								new DocumentLocation (selection.Anchor.Line, textEditorData.GetLine (selection.Anchor.Line).GetLogicalColumn (textEditorData, visualColumn)),
+								new DocumentLocation (selection.Lead.Line, textEditorData.GetLine (selection.Lead.Line).GetLogicalColumn (textEditorData, visualColumn)),
+								SelectionMode.Block
+								);
 							Document.CommitMultipleLineUpdate (textEditorData.MainSelection.MinLine, textEditorData.MainSelection.MaxLine);
 						} else {
 							textEditorData.Insert (Caret.Offset, text);
@@ -253,5 +266,13 @@ namespace Mono.TextEditor
 			if (editor != null)
 				editor.HideMouseCursor ();
 		}
+
+		#region TextAreaControl
+		public virtual void AllocateTextArea (TextEditor textEditor, TextArea textArea, Rectangle allocation)
+		{
+			if (textArea.Allocation != allocation)
+				textArea.SizeAllocate (allocation);
+		}
+		#endregion
 	}
 }
