@@ -25,6 +25,7 @@
 
 using System;
 using System.Text.RegularExpressions;
+using System.Collections.Generic;
 
 namespace Mono.TextEditor
 {
@@ -49,13 +50,15 @@ namespace Mono.TextEditor
 		SearchResult GetMatchAt (int offset);
 		SearchResult GetMatchAt (int offset, int length);
 		
-		SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, int fromOffset);
-		SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, int fromOffset);
+		SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset);
+		SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset);
 		
 		SearchResult SearchForward (int fromOffset);
 		SearchResult SearchBackward (int fromOffset);
 		
 		void Replace (SearchResult result, string pattern);
+
+		int ReplaceAll (string withPattern);
 		
 		ISearchEngine Clone ();
 	}
@@ -111,18 +114,44 @@ namespace Mono.TextEditor
 		public abstract SearchResult GetMatchAt (int offset);
 		public abstract SearchResult GetMatchAt (int offset, int length);
 		
-		public abstract SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, int fromOffset);
-		public abstract SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, int fromOffset);
+		public abstract SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset);
+		public abstract SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset);
 		
 		public SearchResult SearchForward (int fromOffset)
 		{
-			return SearchForward (null, fromOffset);
+			return SearchForward (null, new TextViewMargin.SearchWorkerArguments () { Text = textEditorData.Text }, fromOffset);
 		}
+
 		public SearchResult SearchBackward (int fromOffset)
 		{
-			return SearchBackward (null, fromOffset);
+			return SearchBackward (null, new TextViewMargin.SearchWorkerArguments () { Text = textEditorData.Text }, fromOffset);
 		}
 		public abstract void Replace (SearchResult result, string pattern);
+
+		public virtual int ReplaceAll (string withPattern)
+		{
+			int result = 0;
+			using (var undo = textEditorData.OpenUndoGroup ()) {
+				int offset = 0;
+				if (!SearchRequest.SearchRegion.IsInvalid)
+					offset = SearchRequest.SearchRegion.Offset;
+				SearchResult searchResult; 
+				var text = textEditorData.Text;
+				var args = new TextViewMargin.SearchWorkerArguments () { Text = text };
+				while (true) {
+					searchResult = SearchForward (null, args, offset);
+					if (searchResult == null || searchResult.SearchWrapped)
+						break;
+					Replace (searchResult, withPattern);
+					offset = searchResult.EndOffset;
+					result++;
+				}
+				if (result > 0)
+					textEditorData.ClearSelection ();
+			}
+			return result;
+		}
+
 		public virtual ISearchEngine Clone ()
 		{
 			ISearchEngine result = (ISearchEngine)MemberwiseClone ();
@@ -183,12 +212,12 @@ namespace Mono.TextEditor
 			return null;
 		}
 		
-		public override SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, int fromOffset)
+		public override SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset)
 		{
 			if (!string.IsNullOrEmpty (SearchRequest.SearchPattern)) {
 				// TODO: Optimize
-				for (int i = 0; i < this.textEditorData.Document.TextLength; i++) {
-					int offset = (fromOffset + i) % this.textEditorData.Document.TextLength;
+				for (int i = 0; i < args.Text.Length; i++) {
+					int offset = (fromOffset + i) % args.Text.Length;
 					if (worker != null && worker.CancellationPending)
 						return null; 
 					if (IsMatchAt (offset) && (searchRequest.SearchRegion.IsInvalid || searchRequest.SearchRegion.Contains (offset)))
@@ -198,12 +227,12 @@ namespace Mono.TextEditor
 			return null;
 		}
 		
-		public override SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, int fromOffset)
+		public override SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset)
 		{
 			if (!string.IsNullOrEmpty (SearchRequest.SearchPattern)) {
 				// TODO: Optimize
-				for (int i = 0; i < this.textEditorData.Document.TextLength; i++) {
-					int offset = (fromOffset + this.textEditorData.Document.TextLength * 2 - 1 - i) % this.textEditorData.Document.TextLength;
+				for (int i = 0; i < args.Text.Length; i++) {
+					int offset = (fromOffset + args.Text.Length * 2 - 1 - i) % args.Text.Length;
 					if (worker != null && worker.CancellationPending)
 						return null;
 					if (IsMatchAt (offset) && (searchRequest.SearchRegion.IsInvalid || searchRequest.SearchRegion.Contains (offset)))
@@ -212,11 +241,58 @@ namespace Mono.TextEditor
 			}
 			return null;
 		}
-		
+
 		public override void Replace (SearchResult result, string pattern)
 		{
 			textEditorData.Replace (result.Offset, result.Length, pattern);
 		}
+
+		public override int ReplaceAll (string withPattern)
+		{
+			var searchResults = new List<SearchResult> ();
+
+			int offset = 0;
+			if (!SearchRequest.SearchRegion.IsInvalid)
+				offset = SearchRequest.SearchRegion.Offset;
+			SearchResult searchResult; 
+			var text = textEditorData.Text;
+			var args = new TextViewMargin.SearchWorkerArguments () { Text = text };
+			while (true) {
+				searchResult = SearchForward (null, args, offset);
+				if (searchResult == null || searchResult.SearchWrapped)
+					break;
+				searchResults.Add (searchResult);
+				offset = searchResult.EndOffset;
+			}
+			if (searchResults.Count < 100) {
+				using (var undo = textEditorData.OpenUndoGroup ()) {
+					for (int i = searchResults.Count - 1; i >= 0; i--) {
+						Replace (searchResults [i], withPattern);
+					}
+					if (searchResults.Count > 0)
+						textEditorData.ClearSelection ();
+				}
+			} else {
+				char[] oldText = text.ToCharArray ();
+				char[] newText = new char[oldText.Length + searchResults.Count * (withPattern.Length - compiledPattern.Length)];
+				char[] pattern = withPattern.ToCharArray ();
+				int curOffset = 0, destOffset = 0;
+				foreach (var sr in searchResults) {
+					var length = sr.Offset - curOffset;
+					Array.Copy (oldText, curOffset, newText, destOffset, length);
+					destOffset += length;
+					Array.Copy (pattern, 0, newText, destOffset, pattern.Length);
+					destOffset += withPattern.Length;
+					curOffset = sr.EndOffset;
+				}
+				var l = textEditorData.Length - curOffset;
+				Array.Copy (oldText, curOffset, newText, destOffset, l);
+
+				textEditorData.Replace (0, textEditorData.Length, new string (newText));
+				textEditorData.ClearSelection ();
+			}
+			return searchResults.Count;
+		}
 	}
 	
 	public class RegexSearchEngine : AbstractSearchEngine
@@ -272,16 +348,16 @@ namespace Mono.TextEditor
 			return null;
 		}
 		
-		public override SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, int fromOffset)
+		public override SearchResult SearchForward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args, int fromOffset)
 		{
 			if (regex == null || String.IsNullOrEmpty (searchRequest.SearchPattern))
 				return null;
-			System.Text.RegularExpressions.Match match = regex.Match (this.textEditorData.Document.Text, fromOffset);
+			System.Text.RegularExpressions.Match match = regex.Match (args.Text, fromOffset);
 			if (match.Success) {
 				return new SearchResult (match.Index, 
 				                         match.Length, false);
 			}
-			match = regex.Match (this.textEditorData.Document.Text, 0, fromOffset);
+			match = regex.Match (args.Text, 0, fromOffset);
 			if (match.Success) {
 				return new SearchResult (match.Index, 
 				                         match.Length, true);
@@ -289,13 +365,13 @@ namespace Mono.TextEditor
 			return null;
 		}
 		
-		public override SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, int fromOffset)
+		public override SearchResult SearchBackward (System.ComponentModel.BackgroundWorker worker, TextViewMargin.SearchWorkerArguments args,  int fromOffset)
 		{
 			if (regex == null || String.IsNullOrEmpty (searchRequest.SearchPattern))
 				return null;
 			System.Text.RegularExpressions.Match found = null; 
 			System.Text.RegularExpressions.Match last = null; 
-			foreach (System.Text.RegularExpressions.Match match in regex.Matches (this.textEditorData.Document.Text)) {
+			foreach (System.Text.RegularExpressions.Match match in regex.Matches (args.Text)) {
 				if (match.Index < fromOffset) {
 					found = match;
 				}
