@@ -29,6 +29,21 @@ using Mono.TextEditor.Highlighting;
 
 namespace Mono.TextEditor
 {
+	public enum ShowWhitespaces {
+		Never,
+		Selection,
+		Always
+	}
+
+	[Flags]
+	public enum IncludeWhitespaces {
+		None        = 0,
+		Space       = 1,
+		Tab         = 2,
+		LineEndings = 4,
+		All         = Space | Tab | LineEndings
+	}
+	
 	public interface ITextEditorOptions : IDisposable
 	{
 		double Zoom { get; set; }
@@ -51,25 +66,33 @@ namespace Mono.TextEditor
 		bool ShowIconMargin { get; set; }
 		bool ShowLineNumberMargin { get; set; }
 		bool ShowFoldMargin { get; set; }
-		bool ShowInvalidLines { get; set; }
-		bool ShowTabs { get; set; }
-		bool ShowEolMarkers { get; set; }
 		bool HighlightCaretLine { get; set; }
-		bool ShowSpaces { get; set; }
 		int RulerColumn { get; set; }
 		bool ShowRuler { get; set; }
 		IndentStyle IndentStyle { get; set; }
 		bool OverrideDocumentEolMarker { get; set; }
 		bool EnableSyntaxHighlighting { get; set; }
 		bool EnableAnimations { get; }
+		bool EnableSelectionWrappingKeys { get; }
+
+		bool EnableQuickDiff { get; set; }
+
+		bool DrawIndentationMarkers { get; set; }
+
 		bool UseAntiAliasing { get; set; }
+		bool WrapLines { get; set; }
 		string FontName { get;  set; }
 		Pango.FontDescription Font { get;  }
+
+		string GutterFontName { get; set; }
+		Pango.FontDescription GutterFont { get; }
 		
 		string ColorScheme { get; set;  }
 		string DefaultEolMarker { get; set; }
-		
-		ColorScheme GetColorStyle (Gtk.Style widgetStyle);
+
+		ShowWhitespaces ShowWhitespaces { get; set; }
+		IncludeWhitespaces IncludeWhitespaces { get; set; }
+		ColorScheme GetColorStyle ();
 		
 		event EventHandler Changed;
 	}
