@@ -31,6 +31,7 @@ using System.IO;
 using System.Diagnostics;
 using Mono.TextEditor.Highlighting;
 using ICSharpCode.NRefactory.Editor;
+using Xwt.Drawing;
 
 namespace Mono.TextEditor
 {
@@ -41,9 +42,9 @@ namespace Mono.TextEditor
 	
 	public class TextEditorData : IDisposable
 	{
-		ITextEditorOptions options;
-		readonly TextDocument document; 
-		Caret      caret;
+		ITextEditorOptions    options;
+		TextDocument document; 
+		readonly Caret        caret;
 		
 		static Adjustment emptyAdjustment = new Adjustment (0, 0, 0, 0, 0, 0);
 		
@@ -88,13 +89,13 @@ namespace Mono.TextEditor
 		
 		public string FileName {
 			get {
-				return Document.FileName;
+				return Document != null ? Document.FileName : null;
 			}
 		}
 		
 		public string MimeType {
 			get {
-				return Document.MimeType;
+				return Document != null ? Document.MimeType : null;
 			}
 		}
 
@@ -103,7 +104,63 @@ namespace Mono.TextEditor
 			protected set;
 		}
 
+		ISelectionSurroundingProvider selectionSurroundingProvider = new DefaultSelectionSurroundingProvider ();
+		public ISelectionSurroundingProvider SelectionSurroundingProvider {
+			get {
+				return selectionSurroundingProvider;
+			}
+			set {
+				if (value == null)
+					throw new ArgumentNullException ("surrounding provider needs to be != null");
+				selectionSurroundingProvider = value;
+			}
+		}
+
+		bool? customTabsToSpaces;
+		public bool TabsToSpaces {
+			get {
+				return customTabsToSpaces.HasValue ? customTabsToSpaces.Value : options.TabsToSpaces;
+			}
+			set {
+				customTabsToSpaces = value;
+			}
+		}
+		#region Tooltip providers
+		internal List<TooltipProvider> tooltipProviders = new List<TooltipProvider> ();
+		public IEnumerable<TooltipProvider> TooltipProviders {
+			get { return tooltipProviders; }
+		}
 		
+		/// <summary>
+		/// If set the tooltips wont show up.
+		/// </summary>
+		public bool SuppressTooltips {
+			get;
+			set;
+		}
+
+		public void ClearTooltipProviders ()
+		{
+			foreach (var tp in tooltipProviders) {
+				var disposableProvider = tp as IDisposable;
+				if (disposableProvider == null)
+					continue;
+				disposableProvider.Dispose ();
+			}
+			tooltipProviders.Clear ();
+		}
+		
+		public void AddTooltipProvider (TooltipProvider provider)
+		{
+			tooltipProviders.Add (provider);
+		}
+		
+		public void RemoveTooltipProvider (TooltipProvider provider)
+		{
+			tooltipProviders.Remove (provider);
+		}
+		#endregion
+
 		public TextEditorData () : this (new TextDocument ())
 		{
 		}
@@ -142,9 +199,13 @@ namespace Mono.TextEditor
 
 		void HandleDocTextSet (object sender, EventArgs e)
 		{
-			caret.SetDocument (document);
+			if (vadjustment != null)
+				vadjustment.Value = vadjustment.Lower;
+			if (hadjustment != null)
+				hadjustment.Value = hadjustment.Lower;
 			HeightTree.Rebuild ();
 			ClearSelection ();
+			caret.SetDocument (document);
 		}
 
 		public double GetLineHeight (DocumentLine line)
@@ -175,7 +236,7 @@ namespace Mono.TextEditor
 
 		void HandleTextReplaced (object sender, DocumentChangeEventArgs e)
 		{
-			caret.UpdateCaretPosition ();
+			caret.UpdateCaretPosition (e);
 		}
 
 
@@ -216,7 +277,7 @@ namespace Mono.TextEditor
 		ColorScheme colorStyle;
 		public ColorScheme ColorStyle {
 			get {
-				return colorStyle ?? ColorScheme.Empty;
+				return colorStyle ?? Mono.TextEditor.Highlighting.SyntaxModeService.DefaultColorStyle;
 			}
 			set {
 				colorStyle = value;
@@ -234,17 +295,17 @@ namespace Mono.TextEditor
 			while (curOffset < offset + length && curOffset < Document.TextLength) {
 				DocumentLine line = Document.GetLineByOffset (curOffset);
 				int toOffset = System.Math.Min (line.Offset + line.Length, offset + length);
-				Stack<ChunkStyle> styleStack = new Stack<ChunkStyle> ();
-				foreach (var chunk in mode.GetChunks (ColorStyle, line, curOffset, toOffset - curOffset)) {
+				var styleStack = new Stack<ChunkStyle> ();
 
-					ChunkStyle chunkStyle = ColorStyle.GetChunkStyle (chunk);
-					bool setBold = chunkStyle.Bold && (styleStack.Count == 0 || !styleStack.Peek ().Bold) ||
-							!chunkStyle.Bold && (styleStack.Count == 0 || styleStack.Peek ().Bold);
-					bool setItalic = chunkStyle.Italic && (styleStack.Count == 0 || !styleStack.Peek ().Italic) ||
-							!chunkStyle.Italic && (styleStack.Count == 0 || styleStack.Peek ().Italic);
+				foreach (var chunk in mode.GetChunks (ColorStyle, line, curOffset, toOffset - curOffset)) {
+					var chunkStyle = ColorStyle.GetChunkStyle (chunk);
+					bool setBold = (styleStack.Count > 0 && styleStack.Peek ().FontWeight != chunkStyle.FontWeight) || 
+						chunkStyle.FontWeight != FontWeight.Normal;
+					bool setItalic = (styleStack.Count > 0 && styleStack.Peek ().FontStyle != chunkStyle.FontStyle) || 
+						chunkStyle.FontStyle != FontStyle.Normal;
 					bool setUnderline = chunkStyle.Underline && (styleStack.Count == 0 || !styleStack.Peek ().Underline) ||
 							!chunkStyle.Underline && (styleStack.Count == 0 || styleStack.Peek ().Underline);
-					bool setColor = styleStack.Count == 0 || TextViewMargin.GetPixel (styleStack.Peek ().Color) != TextViewMargin.GetPixel (chunkStyle.Color);
+					bool setColor = styleStack.Count == 0 || TextViewMargin.GetPixel (styleStack.Peek ().Foreground) != TextViewMargin.GetPixel (chunkStyle.Foreground);
 					if (setColor || setBold || setItalic || setUnderline) {
 						if (styleStack.Count > 0) {
 							result.Append ("</span>");
@@ -253,13 +314,13 @@ namespace Mono.TextEditor
 						result.Append ("<span");
 						if (useColors) {
 							result.Append (" foreground=\"");
-							result.Append (SyntaxMode.ColorToPangoMarkup (chunkStyle.Color));
+							result.Append (SyntaxMode.ColorToPangoMarkup (chunkStyle.Foreground));
 							result.Append ("\"");
 						}
-						if (chunkStyle.Bold)
-							result.Append (" weight=\"bold\"");
-						if (chunkStyle.Italic)
-							result.Append (" style=\"italic\"");
+						if (chunkStyle.FontWeight != Xwt.Drawing.FontWeight.Normal)
+							result.Append (" weight=\"" + chunkStyle.FontWeight + "\"");
+						if (chunkStyle.FontStyle != Xwt.Drawing.FontStyle.Normal)
+							result.Append (" style=\"" + chunkStyle.FontStyle + "\"");
 						if (chunkStyle.Underline)
 							result.Append (" underline=\"single\"");
 						result.Append (">");
@@ -335,7 +396,7 @@ namespace Mono.TextEditor
 			if (string.IsNullOrEmpty (str))
 				return "";
 			StringBuilder sb = new StringBuilder ();
-			bool convertTabs = Options.TabsToSpaces;
+			bool convertTabs = TabsToSpaces;
 			
 			for (int i = 0; i < str.Length; i++) {
 				char ch = str [i];
@@ -385,6 +446,11 @@ namespace Mono.TextEditor
 			if (String.IsNullOrEmpty (text))
 				return;
 			using (var undo = OpenUndoGroup ()) {
+				DeleteSelectedText (IsSomethingSelected ? MainSelection.SelectionMode != SelectionMode.Block : true);
+				// Needs to be called after delete text, delete text handles virtual caret postitions itself,
+				// but afterwards the virtual position may need to be restored.
+				EnsureCaretIsNotVirtual ();
+
 				if (IsSomethingSelected && MainSelection.SelectionMode == SelectionMode.Block) {
 					var visualInsertLocation = LogicalToVisualLocation (MainSelection.Anchor);
 					var selection = MainSelection;
@@ -404,10 +470,12 @@ namespace Mono.TextEditor
 						}
 						Insert (lineSegment.Offset + insertOffset, textToInsert);
 					}
+					var visualColumn = GetLine (Caret.Location.Line).GetVisualColumn (this, Caret.Column);
 					MainSelection = new Selection (
-								new DocumentLocation (selection.Anchor.Line, Caret.Column),
-								new DocumentLocation (selection.Lead.Line, Caret.Column),
-								Mono.TextEditor.SelectionMode.Block);
+						new DocumentLocation (selection.Anchor.Line, GetLine (selection.Anchor.Line).GetLogicalColumn (this, visualColumn)),
+						new DocumentLocation (selection.Lead.Line, GetLine (selection.Lead.Line).GetLogicalColumn (this, visualColumn)),
+						SelectionMode.Block
+					);
 					Caret.PreserveSelection = false;
 					Document.CommitMultipleLineUpdate (selection.MinLine, selection.MaxLine);
 				} else {
@@ -419,16 +487,20 @@ namespace Mono.TextEditor
 
 		void DetachDocument ()
 		{
+			if (document == null)
+				return;
 			document.BeginUndo -= OnBeginUndo;
 			document.EndUndo -= OnEndUndo;
 
 			document.Undone -= DocumentHandleUndone;
 			document.Redone -= DocumentHandleRedone;
 			document.LineChanged -= HandleDocLineChanged;
-			
+			document.TextReplaced -= HandleTextReplaced;
+
 			document.TextSet -= HandleDocTextSet;
 			document.Folded -= HandleTextEditorDataDocumentFolded;
 			document.FoldTreeUpdated -= HandleFoldTreeUpdated;
+			document = null;
 		}
 
 		public void Dispose ()
@@ -440,6 +512,8 @@ namespace Mono.TextEditor
 			options = options.Kill ();
 			HeightTree.Dispose ();
 			DetachDocument ();
+			ClearTooltipProviders ();
+			tooltipProviders = null;
 		}
 
 		/// <summary>
@@ -506,17 +580,7 @@ namespace Mono.TextEditor
 		{
 			return Options.WordFindStrategy.FindCurrentWordStart (Document, offset);
 		}
-		
-		public delegate void PasteCallback (int insertionOffset, string text, int insertedChars);
-		
-		public event PasteCallback Paste;
-		
-		public void PasteText (int insertionOffset, string text, int insertedChars)
-		{
-			if (Paste != null)
-				Paste (insertionOffset, text, insertedChars);
-		}
-		
+
 		#region undo/redo handling
 		DocumentLocation savedCaretPos;
 		Selection savedSelection;
@@ -525,7 +589,7 @@ namespace Mono.TextEditor
 		void OnBeginUndo (object sender, EventArgs args)
 		{
 			savedCaretPos  = Caret.Location;
-			savedSelection = Selection.Clone (MainSelection);
+			savedSelection = MainSelection;
 		}
 
 		void OnEndUndo (object sender, TextDocument.UndoOperationEventArgs e)
@@ -566,7 +630,7 @@ namespace Mono.TextEditor
 				undoSelection = selection;
 				
 				redoCaretPos  = editor.Caret.Location;
-				redoSelection = Mono.TextEditor.Selection.Clone (editor.MainSelection);
+				redoSelection = editor.MainSelection;
 			}
 			
 			public void UndoState ()
@@ -586,7 +650,7 @@ namespace Mono.TextEditor
 		#region Selection management
 		public bool IsSomethingSelected {
 			get {
-				return MainSelection != null && MainSelection.Anchor != MainSelection.Lead; 
+				return !MainSelection.IsEmpty && MainSelection.Anchor != MainSelection.Lead; 
 			}
 		}
 		
@@ -629,39 +693,31 @@ namespace Mono.TextEditor
 		
 		public SelectionMode SelectionMode {
 			get {
-				return MainSelection != null ? MainSelection.SelectionMode : SelectionMode.Normal;
+				return !MainSelection.IsEmpty ? MainSelection.SelectionMode : SelectionMode.Normal;
 			}
 			set {
-				if (MainSelection != null)
-					MainSelection.SelectionMode = value;
+				if (MainSelection.IsEmpty)
+					return;
+				MainSelection = MainSelection.WithSelectionMode (value);
 			}
 		}
 		
-		Selection mainSelection = null;
+		Selection mainSelection = Selection.Empty;
 		public Selection MainSelection {
 			get {
 				return mainSelection;
 			}
 			set {
-				if (mainSelection == null && value == null)
+				if (mainSelection.IsEmpty && value.IsEmpty)
 					return;
-				if (mainSelection == null && value != null || mainSelection != null && value == null || !mainSelection.Equals (value)) {
+				if (mainSelection.IsEmpty && !value.IsEmpty || !mainSelection.IsEmpty && value.IsEmpty || !mainSelection.Equals (value)) {
 					OnSelectionChanging (EventArgs.Empty);
-					if (mainSelection != null)
-						mainSelection.Changed -= HandleMainSelectionChanged;
 					mainSelection = value;
-					if (mainSelection != null) 
-						mainSelection.Changed += HandleMainSelectionChanged;
 					OnSelectionChanged (EventArgs.Empty);
 				}
 			}
 		}
 
-		void HandleMainSelectionChanged (object sender, EventArgs e)
-		{
-			OnSelectionChanged (EventArgs.Empty);
-		}
-		
 		public IEnumerable<Selection> Selections {
 			get {
 				yield return MainSelection;
@@ -681,18 +737,20 @@ namespace Mono.TextEditor
 //		}
 		public int SelectionAnchor {
 			get {
-				if (MainSelection == null)
+				if (MainSelection.IsEmpty)
 					return -1;
 				return MainSelection.GetAnchorOffset (this);
 			}
 			set {
 				DocumentLocation location = Document.OffsetToLocation (value);
-				if (mainSelection == null) {
+				if (mainSelection.IsEmpty) {
 					MainSelection = new Selection (location, location);
 				} else {
-					if (MainSelection.Lead == location)
-						MainSelection.Lead = MainSelection.Anchor;
-					MainSelection.Anchor = location;
+					if (MainSelection.Lead == location) {
+						MainSelection = MainSelection.WithLead (MainSelection.Anchor);
+					} else {
+						MainSelection = MainSelection.WithAnchor (location);
+					}
 				}
 			}
 		}
@@ -702,30 +760,29 @@ namespace Mono.TextEditor
 		/// </summary>
 		public TextSegment SelectionRange {
 			get {
-				return MainSelection != null ? MainSelection.GetSelectionRange (this) : new TextSegment (Caret.Offset, 0);
+				return !MainSelection.IsEmpty ? MainSelection.GetSelectionRange (this) : new TextSegment (Caret.Offset, 0);
 			}
 			set {
 				if (SelectionRange != value) {
 					OnSelectionChanging (EventArgs.Empty);
 					if (value.IsEmpty) {
-						MainSelection = null;
+						MainSelection = Selection.Empty;
 					} else {
 						DocumentLocation loc1 = document.OffsetToLocation (value.Offset);
 						DocumentLocation loc2 = document.OffsetToLocation (value.EndOffset);
-						if (MainSelection == null) {
+						if (MainSelection.IsEmpty) {
 							MainSelection = new Selection (loc1, loc2);
 						} else {
 							if (MainSelection.Anchor == loc1) {
-								MainSelection.Lead = loc2;
+								MainSelection = MainSelection.WithLead (loc2);
 							} else if (MainSelection.Anchor == loc2) {
-								MainSelection.Lead = loc1;
+								MainSelection = MainSelection.WithLead (loc1);
 							} else {
 								MainSelection = new Selection (loc1, loc2);
 							}
 						}
 						
 					}
-					OnSelectionChanged (EventArgs.Empty);
 				}
 			}
 		}
@@ -767,15 +824,16 @@ namespace Mono.TextEditor
 		{
 			if (!IsSomethingSelected)
 				return;
-			MainSelection = null;
-			OnSelectionChanged (EventArgs.Empty);
+			MainSelection = Selection.Empty;
 		}
 		
 		public void ExtendSelectionTo (DocumentLocation location)
 		{
-			if (MainSelection == null)
+			if (MainSelection.IsEmpty) {
 				MainSelection = new Selection (location, location);
-			MainSelection.Lead = location;
+			} else {
+				MainSelection = MainSelection.WithLead (location);
+			}
 		}
 		
 		public void SetSelection (int anchorOffset, int leadOffset)
@@ -808,8 +866,8 @@ namespace Mono.TextEditor
 
 		internal void DeleteSelection (Selection selection)
 		{
-			if (selection == null)
-				throw new ArgumentNullException ("selection");
+			if (selection.IsEmpty)
+				throw new ArgumentNullException ("selection was empty.");
 			switch (selection.SelectionMode) {
 			case SelectionMode.Normal:
 				var segment = selection.GetSelectionRange (this);
@@ -840,9 +898,10 @@ namespace Mono.TextEditor
 						Caret.Column = col1 + 1;
 				}
 				int column = System.Math.Min (selection.Anchor.Column, selection.Lead.Column);
-				selection.Anchor = new DocumentLocation (selection.Anchor.Line, column);
-				selection.Lead = new DocumentLocation (selection.Lead.Line, column);
-				Caret.Column = column;
+				MainSelection = selection.WithRange (
+					new DocumentLocation (selection.Anchor.Line, column),
+					new DocumentLocation (selection.Lead.Line, column)
+				);
 				Caret.PreserveSelection = preserve;
 				break;
 			}
@@ -862,6 +921,8 @@ namespace Mono.TextEditor
 			using (var undo = OpenUndoGroup ()) {
 				EnsureCaretIsNotVirtual ();
 				foreach (Selection selection in Selections) {
+					EnsureIsNotVirtual (selection.Anchor);
+					EnsureIsNotVirtual (selection.Lead);
 					var segment = selection.GetSelectionRange (this);
 					needUpdate |= Document.OffsetToLineNumber (segment.Offset) != Document.OffsetToLineNumber (segment.EndOffset);
 					DeleteSelection (selection);
@@ -999,24 +1060,7 @@ namespace Mono.TextEditor
 		
 		public int SearchReplaceAll (string withPattern)
 		{
-			int result = 0;
-			using (var undo = OpenUndoGroup ()) {
-				int offset = 0;
-				if (!SearchRequest.SearchRegion.IsInvalid)
-					offset = SearchRequest.SearchRegion.Offset;
-				SearchResult searchResult; 
-				while (true) {
-					searchResult = SearchForward (offset);
-					if (searchResult == null || searchResult.SearchWrapped)
-						break;
-					searchEngine.Replace (searchResult, withPattern);
-					offset = searchResult.Offset + withPattern.Length;
-					result++;
-				}
-				if (result > 0)
-					ClearSelection ();
-			}
-			return result;
+			return searchEngine.ReplaceAll (withPattern);
 		}
 		#endregion
 		
@@ -1074,23 +1118,42 @@ namespace Mono.TextEditor
 		/// That method should always be called in an undo group.
 		/// </summary>
 		public int EnsureCaretIsNotVirtual ()
+		{
+			return EnsureIsNotVirtual (Caret.Location);
+		}
+
+		public bool IsCaretInVirtualLocation {
+			get {
+				DocumentLine documentLine = Document.GetLine (Caret.Line);
+				if (documentLine == null)
+					return true;
+				return Caret.Column > documentLine.Length + 1;
+			}
+		}
+
+		int EnsureIsNotVirtual (DocumentLocation loc)
+		{
+			return EnsureIsNotVirtual (loc.Line, loc.Column);
+		}
+
+		int EnsureIsNotVirtual (int line, int column)
 		{
 			Debug.Assert (document.IsInAtomicUndo);
-			DocumentLine line = Document.GetLine (Caret.Line);
-			if (line == null)
+			DocumentLine documentLine = Document.GetLine (line);
+			if (documentLine == null)
 				return 0;
-			if (Caret.Column > line.Length + 1) {
+			if (column > documentLine.Length + 1) {
 				string virtualSpace;
-				if (HasIndentationTracker && line.Length == 0) {
-					virtualSpace = GetIndentationString (Caret.Location);
+				if (HasIndentationTracker && documentLine.Length == 0) {
+					virtualSpace = GetIndentationString (line, column);
 				} else {
-					virtualSpace = new string (' ', Caret.Column - 1 - line.Length);
+					virtualSpace = new string (' ', column - 1 - documentLine.Length);
 				}
 				var oldPreserve = Caret.PreserveSelection;
 				Caret.PreserveSelection = true;
-				Insert (Caret.Offset, virtualSpace);
+				Insert (documentLine.Offset, virtualSpace);
 				Caret.PreserveSelection = oldPreserve;
-			
+				
 				// No need to reposition the caret, because it's already at the correct position
 				// The only difference is that the position is not virtual anymore.
 				return virtualSpace.Length;
@@ -1127,6 +1190,40 @@ namespace Mono.TextEditor
 		}
 		public event EventHandler RecenterEditor;
 
+		#region Text Paste
+		/// <summary>
+		/// Gets or sets the text paste handler.
+		/// </summary>
+		public ITextPasteHandler TextPasteHandler {
+			get;
+			set;
+		}
+
+		public int PasteText (int offset, string text, byte[] copyData, ref IDisposable undoGroup)
+		{
+			if (TextPasteHandler != null) {
+				var newText = TextPasteHandler.FormatPlainText (offset, text, copyData);
+				if (newText != text) {
+					var inserted = Insert (offset, text);
+					undoGroup.Dispose ();
+					undoGroup = OpenUndoGroup ();
+					var result = Replace (offset, inserted, newText);
+					if (Paste != null)
+						Paste (offset, text, result);
+					return result;
+				}
+			}
+			var insertedChars = Insert (offset, text);
+			if (Paste != null)
+				Paste (offset, text, insertedChars);
+			return insertedChars;
+		}
+
+		public delegate void PasteCallback (int insertionOffset, string text, int insertedChars);
+		
+		public event PasteCallback Paste;
+		#endregion
+
 		#region Document delegation
 		public int Length {
 			get {
@@ -1212,7 +1309,7 @@ namespace Mono.TextEditor
 		
 		public int LineCount {
 			get {
-				return Document.LineCount;
+				return Document != null ? Document.LineCount : 0;
 			}
 		}
 		
@@ -1263,12 +1360,7 @@ namespace Mono.TextEditor
 		#endregion
 		
 		#region Parent functions
-		public bool HasFocus {
-			get {
-				return Parent != null ? Parent.HasFocus : false;
-			}
-		}
-		
+
 		public void ScrollToCaret ()
 		{
 			if (Parent != null)
