@@ -106,8 +106,15 @@ namespace MonoDevelop.Core.Assemblies
 		}
 
 		public bool HasMultitargetingMcs { get; private set; }
-		
+
 		public override IEnumerable<FilePath> GetReferenceFrameworkDirectories ()
+		{
+			//during initializion, only return the global directory once (for the running runtime) so that it doesn't
+			//get scanned multiple times
+			return GetReferenceFrameworkDirectories (IsInitialized || IsRunning);
+		}
+
+		internal IEnumerable<FilePath> GetReferenceFrameworkDirectories (bool includeMacGlobalDir)
 		{
 			//duplicate xbuild's framework folders path logic
 			//see xbuild man page
@@ -116,8 +123,11 @@ namespace MonoDevelop.Core.Assemblies
 				foreach (var dir in env.Split (new char[] { Path.PathSeparator }, StringSplitOptions.RemoveEmptyEntries))
 					yield return (FilePath) dir;
 			}
-			if (Platform.IsMac)
+
+			if (Platform.IsMac && true) {
 				yield return "/Library/Frameworks/Mono.framework/External/xbuild-frameworks";
+			}
+
 			//can't return $(TargetFrameworkRoot) MSBuild var, since that's per-project
 			yield return Path.Combine (monoDir, "xbuild-frameworks");
 		}
@@ -150,6 +160,12 @@ namespace MonoDevelop.Core.Assemblies
 			pinfo.FileName = Path.Combine (Path.Combine (MonoRuntimeInfo.Prefix, "bin"), "mono");
 		}
 
+		public override string GetToolPath (TargetFramework fx, string toolName)
+		{
+			if (fx.ClrVersion == ClrVersion.Net_2_0 && toolName == "al")
+				toolName = "al2";
+			return base.GetToolPath (fx, toolName);
+		}
 		
 		internal protected override IEnumerable<string> GetGacDirectories ()
 		{
@@ -206,7 +222,7 @@ namespace MonoDevelop.Core.Assemblies
 				return;
 			foreach (string pcfile in GetAllPkgConfigFiles ()) {
 				try {
-					ParsePCFile (pcfile);
+					ParsePCFile (FileService.ResolveFullPath (pcfile));
 					if (ShuttingDown)
 						return;
 				}
@@ -278,9 +294,9 @@ namespace MonoDevelop.Core.Assemblies
 					if (newfx == null)
 						inconsistentFrameworks = true;
 					else {
-						if (newfx.IsCompatibleWithFramework (commonFramework.Id))
+						if (newfx.CanReferenceAssembliesTargetingFramework (commonFramework))
 							commonFramework = newfx;
-						else if (!commonFramework.IsCompatibleWithFramework (newfx.Id))
+						else if (!commonFramework.CanReferenceAssembliesTargetingFramework (newfx))
 							inconsistentFrameworks = true;
 					}
 				}
