@@ -45,6 +45,9 @@ namespace MonoDevelop.Core.Assemblies
 		
 		public virtual bool IsInstalled {
 			get {
+				if (framework.Assemblies.Length == 0)
+					return false;
+
 				foreach (string dir in runtime.GetFrameworkFolders (framework)) {
 					if (Directory.Exists (dir)) {
 						string firstAsm = Path.Combine (dir, framework.Assemblies [0].Name) + ".dll";
@@ -67,7 +70,7 @@ namespace MonoDevelop.Core.Assemblies
 		{
 			foreach (string path in runtime.GetToolsPaths (framework)) {
 				string toolPath = Path.Combine (path, toolName);
-				if (PropertyService.IsWindows) {
+				if (Platform.IsWindows) {
 					if (File.Exists (toolPath + ".bat"))
 						return toolPath + ".bat";
 				}
@@ -99,15 +102,15 @@ namespace MonoDevelop.Core.Assemblies
 		
 		public virtual SystemPackageInfo GetFrameworkPackageInfo (string packageName)
 		{
+			string name = !string.IsNullOrEmpty (packageName)? packageName : framework.Name;
 			SystemPackageInfo info = new SystemPackageInfo ();
-			info.Name = string.IsNullOrEmpty (packageName) ? runtime.DisplayRuntimeName : packageName;
-			info.Description = string.IsNullOrEmpty (packageName) ? framework.Name : packageName;
+			info.Name = name;
+			info.Description = name;
 			info.IsFrameworkPackage = true;
 			info.IsCorePackage = true;
 			info.IsGacPackage = true;
-			info.Version = framework.Id;
+			info.Version = framework.Id.Version;
 			info.TargetFramework = framework.Id;
-			info.IsBaseCorePackage = framework.Id == framework.BaseCoreFramework;
 			return info;
 		}
 	}
