@@ -26,12 +26,14 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+using MonoDevelop.Core.Logging;
+
 namespace MonoDevelop.Core.Execution
 {
 	public interface IProcessHostController
 	{
+		ILogger GetLogger ();
 		void RegisterHost (IProcessHost processHost);
 		void WaitForExit ();
-		void ReleaseInstance (RemoteProcessObject proc);
 	}
 }
