@@ -63,10 +63,13 @@ namespace MonoDevelop.Core.FileSystem
 			GetNextForPath (file, false).RenameFile (file, newName);
 		}
 		
+		public virtual FilePath ResolveFullPath (FilePath path)
+		{
+			return GetNextForPath (path, true).ResolveFullPath (path);
+		}
+		
 		public virtual void MoveFile (FilePath source, FilePath dest)
 		{
-			GetNextForPath (source, false).MoveFile (source, dest);
-			
 			FileSystemExtension srcExt = GetNextForPath (source, false);
 			FileSystemExtension dstExt = GetNextForPath (dest, false);
 			
@@ -123,10 +126,24 @@ namespace MonoDevelop.Core.FileSystem
 		{
 			GetNextForPath (path, true).DeleteDirectory (path);
 		}
-		
+
+		public virtual void RequestFileEdit (IEnumerable<FilePath> files)
+		{
+#pragma warning disable 618
+			foreach (var f in files) {
+				if (!RequestFileEdit (f))
+					throw new UserException (GettextCatalog.GetString ("File '{0}' can't be modified", f.FileName));
+			}
+#pragma warning restore 618
+
+			foreach (var fg in files.GroupBy (f => GetNextForPath (f, false)))
+				fg.Key.RequestFileEdit (fg);
+		}
+
+		[Obsolete ("This will be removed. Override RequestFileEdit (IEnumerable<FilePath>) instead")]
 		public virtual bool RequestFileEdit (FilePath file)
 		{
-			return GetNextForPath (file, false).RequestFileEdit (file);
+			return true;
 		}
 		
 		public virtual void NotifyFilesChanged (IEnumerable<FilePath> files)
@@ -135,4 +152,12 @@ namespace MonoDevelop.Core.FileSystem
 				fsFiles.Key.NotifyFilesChanged (files);
 		}
 	}
+
+	class DummyFileSystemExtension: FileSystemExtension
+	{
+		public override bool CanHandlePath (FilePath path, bool isDirectory)
+		{
+			return false;
+		}
+	}
 }
