@@ -34,23 +34,35 @@ namespace MonoDevelop.Core
 		public FileEventArgs ()
 		{
 		}
-		
+
 		public FileEventArgs (FilePath fileName, bool isDirectory)
 		{
-			Add (new FileEventInfo (fileName, isDirectory));
+			Add (new FileEventInfo (fileName, isDirectory, false));
 		}
 		
+		public FileEventArgs (FilePath fileName, bool isDirectory, bool autoReload)
+		{
+			Add (new FileEventInfo (fileName, isDirectory, autoReload));
+		}
+
 		public FileEventArgs (IEnumerable<FilePath> files, bool isDirectory)
 		{
 			foreach (var f in files)
-				Add (new FileEventInfo (f, isDirectory));
+				Add (new FileEventInfo (f, isDirectory, false));
+		}
+
+		public FileEventArgs (IEnumerable<FilePath> files, bool isDirectory, bool autoReload)
+		{
+			foreach (var f in files)
+				Add (new FileEventInfo (f, isDirectory, autoReload));
 		}
 	}
 	
 	public class FileEventInfo
 	{
 		FilePath fileName;
-		bool   isDirectory;
+		bool isDirectory;
+		bool autoReload;
 		
 		public FilePath FileName {
 			get {
@@ -63,11 +75,18 @@ namespace MonoDevelop.Core
 				return isDirectory;
 			}
 		}
+
+		public bool AutoReload {
+			get {
+				return autoReload;
+			}
+		}
 		
-		public FileEventInfo (FilePath fileName, bool isDirectory)
+		public FileEventInfo (FilePath fileName, bool isDirectory, bool autoReload)
 		{
 			this.fileName = fileName;
 			this.isDirectory = isDirectory;
+			this.autoReload = autoReload;
 		}
 	}
 	
