@@ -73,6 +73,12 @@ namespace MonoDevelop.Core
 				return new FilePath (!string.IsNullOrEmpty (fileName) ? Path.GetFullPath (fileName) : "");
 			}
 		}
+
+		public bool IsDirectory {
+			get {
+				return File.GetAttributes (FullPath).HasFlag(FileAttributes.Directory);
+			}
+		}
 		
 		/// <summary>
 		/// Returns a path in standard form, which can be used to be compared
@@ -81,6 +87,8 @@ namespace MonoDevelop.Core
 		/// </summary>
 		public FilePath CanonicalPath {
 			get {
+				if (string.IsNullOrEmpty (fileName))
+					return FilePath.Empty;
 				string fp = Path.GetFullPath (fileName);
 				if (fp.Length > 0 && fp[fp.Length - 1] == Path.DirectorySeparatorChar)
 					return fp.TrimEnd (Path.DirectorySeparatorChar);
