@@ -41,31 +41,31 @@ namespace MonoDevelop.Core
 			//variable can be used to override where Gettext looks for the catalogues
 			string catalog = System.Environment.GetEnvironmentVariable ("MONODEVELOP_LOCALE_PATH");
 
-            // Set the user defined language
+			// Set the user defined language
 			string lang = PropertyService.Get ("MonoDevelop.Ide.UserInterfaceLanguage", "");
 			if (!string.IsNullOrEmpty (lang)) {
-                if (PropertyService.IsWindows) {
-                    lang = lang.Replace("_", "-");
-                    CultureInfo ci = CultureInfo.GetCultureInfo(lang);
-                    if (ci.IsNeutralCulture) {
-                        // We need a neutral culture
-                        foreach (CultureInfo c in CultureInfo.GetCultures (CultureTypes.AllCultures & ~CultureTypes.NeutralCultures))
-                            if (c.Parent != null && c.Parent.Name == ci.Name) {
-                                ci = c;
-                                break;
-                            }
-                    }
-                    if (!ci.IsNeutralCulture)
-                        System.Threading.Thread.CurrentThread.CurrentCulture = ci;
-                }
-                else
-    				Environment.SetEnvironmentVariable ("LANGUAGE", lang);
-            }
+				if (Platform.IsWindows) {
+					lang = lang.Replace("_", "-");
+					CultureInfo ci = CultureInfo.GetCultureInfo(lang);
+					if (ci.IsNeutralCulture) {
+						// We need a neutral culture
+						foreach (CultureInfo c in CultureInfo.GetCultures (CultureTypes.AllCultures & ~CultureTypes.NeutralCultures))
+							if (c.Parent != null && c.Parent.Name == ci.Name) {
+								ci = c;
+								break;
+							}
+					}
+					if (!ci.IsNeutralCulture)
+						System.Threading.Thread.CurrentThread.CurrentCulture = ci;
+				}
+				else
+					Environment.SetEnvironmentVariable ("LANGUAGE", lang);
+			}
 			
 			if (string.IsNullOrEmpty (catalog)) {
 				string location = System.Reflection.Assembly.GetExecutingAssembly ().Location;
 				location = Path.GetDirectoryName (location);
-				if (PropertyService.IsWindows) {
+				if (Platform.IsWindows) {
 					// On windows, load the catalog from a child dir
 					catalog = Path.Combine (location, "locale");
 				}
@@ -134,7 +134,7 @@ namespace MonoDevelop.Core
 		}
 		
 		public static string GetPluralString (string singular, string plural, int number, 
-		                                      object arg0, object arg1, object arg2)
+			object arg0, object arg1, object arg2)
 		{
 			return string.Format (Catalog.GetPluralString (singular, plural, number), arg0, arg1, arg2);
 		}
