@@ -29,6 +29,7 @@ using System.Text;
 using System.IO;
 using System.Reflection;
 using Mono.Addins;
+using System.Collections.Generic;
 
 namespace MonoDevelop.Core
 {
@@ -47,6 +48,10 @@ namespace MonoDevelop.Core
 		public static string SessionUuid {
 			get; private set;
 		}
+
+		internal SystemInformation ()
+		{
+		}
 		
 		static SystemInformation ()
 		{
@@ -60,75 +65,91 @@ namespace MonoDevelop.Core
 			SessionUuid = DateTime.UtcNow.Ticks.ToString ();
 		}
 		
-		protected abstract void AppendOperatingSystem (StringBuilder sb);
+		internal abstract void AppendOperatingSystem (StringBuilder sb);
 		
-		public override string ToString ()
+		IEnumerable<ISystemInformationProvider> InternalGetDescription ()
 		{
-			var sb = new StringBuilder ();
-			
 			foreach (var info in AddinManager.GetExtensionObjects<ISystemInformationProvider> ("/MonoDevelop/Core/SystemInformation", false)) {
-				try {
-					sb.AppendLine (info.Description);
-				} catch (Exception ex) {
-					LoggingService.LogError ("Error getting about information: ", ex);
-				}
+				yield return info;
 			}
-			
-			var biFile = ((FilePath)Assembly.GetCallingAssembly ().Location).ParentDirectory.Combine ("buildinfo");
+
+			var sb = new StringBuilder ();
+			// First append the MonoDevelop build information
+			var biFile = ((FilePath)Assembly.GetEntryAssembly ().Location).ParentDirectory.Combine ("buildinfo");
 			if (File.Exists (biFile)) {
-				sb.AppendLine ("Build information:");
-				foreach (var line in File.ReadAllLines (biFile)){
-					if (!string.IsNullOrWhiteSpace (line)) {
-						sb.Append ("\t");
-						sb.AppendLine (line.Trim ());
+				var lines = File.ReadAllLines (biFile)
+					.Select (l => l.Trim ())
+					.Where (l => !string.IsNullOrEmpty (l))
+					.ToArray ();
+				if (lines.Length > 0) {
+					foreach (var line in lines) {
+						sb.AppendLine (line);
 					}
 				}
-			} else {
-				sb.AppendLine ("No build info");
 			}
-			
-			sb.AppendLine ("Operating System:");
+
+			// Then append the Xamarin Addins information if it exists
+			biFile = ((FilePath) Assembly.GetEntryAssembly ().Location).ParentDirectory.Combine ("buildinfo_xamarin");
+			if (File.Exists (biFile)) {
+				var lines = File.ReadAllLines (biFile)
+					.Select (l => l.Trim ())
+					.Where (l => !string.IsNullOrEmpty (l))
+					.ToArray ();
+				if (lines.Length > 0) {
+					sb.Append ("Xamarin addins: ");
+					foreach (var line in lines)
+						sb.AppendLine (line);
+				}
+			}
+
+			if (sb.Length == 0)
+				sb.AppendLine ("Build information unavailable");
+
+			yield return new SystemInformationSection () {
+				Title = "Build Information",
+				Description = sb.ToString ()
+			};
+
+			sb.Clear ();
 			AppendOperatingSystem (sb);
-			
-			return sb.ToString ();
+
+			yield return new SystemInformationSection () {
+				Title = "Operating System",
+				Description = sb.ToString ()
+			};
 		}
 
-		public static string ToText ()
+		public static string GetReleaseId ()
 		{
-			return Instance.ToString ();
+			var biFile = ((FilePath)Assembly.GetEntryAssembly ().Location).ParentDirectory.Combine ("buildinfo");
+			if (File.Exists (biFile)) {
+				var line = File.ReadAllLines (biFile).Select (l => l.Split (':')).Where (a => a.Length > 1 && a [0].Trim () == "Release ID").FirstOrDefault ();
+				if (line != null)
+					return line [1].Trim ();
+			}
+			return null;
 		}
 
-		public static string GetLoadedAssemblies ()
+		public static IEnumerable<ISystemInformationProvider> GetDescription ()
 		{
-			var sb = new StringBuilder ();
+			return Instance.InternalGetDescription ();
+		}
 
-			int nameLength = 0;
-			int versionLength = 0;
-			sb.AppendLine ("Loaded assemblies:");
-
-			foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies ()) {
-				try {
-					if (assembly.IsDynamic)
-						continue;
-					var assemblyName = assembly.GetName ();
-					nameLength = Math.Max (nameLength, assemblyName.Name.Length);
-					versionLength = Math.Max (versionLength, assemblyName.Version.ToString ().Length);
-				} catch {
-				}
+		public static string GetTextDescription ()
+		{
+			StringBuilder sb = new StringBuilder ();
+			foreach (var info in GetDescription ()) {
+				sb.Append ("=== ").Append (info.Title.Trim ()).Append (" ===\n\n");
+				sb.Append (info.Description.Trim ());
+				sb.Append ("\n\n");
 			}
-			nameLength++;
-			versionLength++;
-			foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies ()) {
-				try {
-					if (assembly.IsDynamic)
-						continue;
-					var assemblyName = assembly.GetName ();
-					sb.AppendLine (assemblyName.Name.PadRight (nameLength) + assemblyName.Version.ToString ().PadRight (versionLength) + System.IO.Path.GetFullPath (assembly.Location));
-				} catch {
-				}
-			}
-
 			return sb.ToString ();
 		}
 	}
-}
\ No newline at end of file
+
+	class SystemInformationSection: ISystemInformationProvider
+	{
+		public string Title { get; set; }
+		public string Description { get; set; }
+	}
+}
