@@ -82,9 +82,10 @@ namespace MonoDevelop.Core
 				case XmlNodeType.EndElement:
 					if (endNodes.Contains (reader.LocalName)) 
 						return;
+					IXmlLineInfo xmlInfo = (IXmlLineInfo)reader;
 					LoggingService.LogWarning (
-						"Encountered end node '{0}' when expecting one of '{1}'. Location {2}",
-						reader.LocalName, ConcatString (endNodes), new System.Diagnostics.StackTrace ());
+						"Encountered end node '{0}' when expecting one of '{1}'. Location ln:{2} col: {3}. Stack Trace:{4}",
+						reader.LocalName, ConcatString (endNodes), xmlInfo.LineNumber, xmlInfo.LinePosition, new System.Diagnostics.StackTrace ());
 					break;
 				case XmlNodeType.Element:
 					if (!didReadStartNode && endNodes.Contains (reader.LocalName)) {
