@@ -75,6 +75,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return frameworkVersions.Contains (fx.Id) || supportsMonikers;
 		}
 
+		internal virtual bool SupportsSlnVersion (string version)
+		{
+			return version == slnVersion;
+		}
+
 		public FilePath GetValidFormatName (object obj, FilePath fileName)
 		{
 			if (slnFileFormat.CanWriteFile (obj, this))
@@ -257,6 +262,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				return new MSBuildFileFormatVS08 ();
 			case "4.0":
 				return new MSBuildFileFormatVS10 ();
+			case "4.5":
+				return new MSBuildFileFormatVS12 ();
 			}
 			throw new Exception ("Unknown ToolsVersion '" + toolsVersion + "'");
 		}
@@ -318,6 +325,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			TargetFrameworkMoniker.NET_3_0,
 			TargetFrameworkMoniker.NET_3_5,
 			TargetFrameworkMoniker.NET_4_0,
+			TargetFrameworkMoniker.SL_2_0,
+			TargetFrameworkMoniker.SL_3_0,
+			TargetFrameworkMoniker.SL_4_0,
+			TargetFrameworkMoniker.MONOTOUCH_1_0,
+			TargetFrameworkMoniker.PORTABLE_4_0
 		};
 		const bool supportsMonikers = true;
 		
@@ -329,4 +341,39 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			get { return "MSBuild10"; }
 		}
 	}
+
+	class MSBuildFileFormatVS12: MSBuildFileFormat
+	{
+		public const string Version = "12.0.0";
+		const string toolsVersion = "4.0";
+		const string slnVersion = "12.00";
+		const string productComment = "Visual Studio 2012";
+		static TargetFrameworkMoniker[] frameworkVersions = {
+			TargetFrameworkMoniker.NET_2_0,
+			TargetFrameworkMoniker.NET_3_0,
+			TargetFrameworkMoniker.NET_3_5,
+			TargetFrameworkMoniker.NET_4_0,
+			TargetFrameworkMoniker.NET_4_5,
+			TargetFrameworkMoniker.SL_2_0,
+			TargetFrameworkMoniker.SL_3_0,
+			TargetFrameworkMoniker.SL_4_0,
+			TargetFrameworkMoniker.MONOTOUCH_1_0,
+			TargetFrameworkMoniker.PORTABLE_4_0
+		};
+		const bool supportsMonikers = true;
+		
+		public MSBuildFileFormatVS12 (): base (Version, toolsVersion, slnVersion, productComment, frameworkVersions, supportsMonikers)
+		{
+		}
+		
+		public override string Id {
+			get { return "MSBuild12"; }
+		}
+
+		internal override bool SupportsSlnVersion (string version)
+		{
+			return version == "11.00" || version == "12.00";
+		}
+	}
+
 }
