@@ -146,14 +146,14 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			ProjectWriter sw = new ProjectWriter (bom);
 			sw.NewLine = newLine;
 			doc.Save (sw);
-			
+
 			string content = sw.ToString ();
 			if (endsWithEmptyLine && !content.EndsWith (newLine))
 				content += newLine;
-			
+
 			TextFile.WriteFile (fileName, content, bom, true);
 		}
-		
+
 		public string DefaultTargets {
 			get { return doc.DocumentElement.GetAttribute ("DefaultTargets"); }
 			set { doc.DocumentElement.SetAttribute ("DefaultTargets", value); }
@@ -443,7 +443,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 	{
 		MSBuildProperty GetProperty (string name);
 		IEnumerable<MSBuildProperty> Properties { get; }
-		void SetPropertyValue (string name, string value);
+		MSBuildProperty SetPropertyValue (string name, string value, bool preserveExistingCase);
 		string GetPropertyValue (string name);
 		bool RemoveProperty (string name);
 		void RemoveAllProperties ();
@@ -476,13 +476,16 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return null;
 		}
 
-		public void SetPropertyValue (string name, string value)
+		public MSBuildProperty SetPropertyValue (string name, string value, bool preserveExistingCase)
 		{
 			MSBuildProperty p = GetProperty (name);
-			if (p != null)
-				p.Value = value;
-			else
-				groups [0].SetPropertyValue (name, value);
+			if (p != null) {
+				if (!preserveExistingCase || !string.Equals (value, p.Value, StringComparison.OrdinalIgnoreCase)) {
+					p.Value = value;
+				}
+				return p;
+			}
+			return groups [0].SetPropertyValue (name, value, preserveExistingCase);
 		}
 
 		public string GetPropertyValue (string name)
@@ -585,15 +588,18 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 		}
 		
-		public void SetPropertyValue (string name, string value)
+		public MSBuildProperty SetPropertyValue (string name, string value, bool preserveExistingCase)
 		{
 			MSBuildProperty prop = GetProperty (name);
 			if (prop == null) {
 				XmlElement pelem = AddChildElement (name);
 				prop = new MSBuildProperty (pelem);
 				properties [name] = prop;
+				prop.Value = value;
+			} else if (!preserveExistingCase || !string.Equals (value, prop.Value, StringComparison.OrdinalIgnoreCase)) {
+				prop.Value = value;
 			}
-			prop.Value = value;
+			return prop;
 		}
 		
 		public string GetPropertyValue (string name)
@@ -669,21 +675,19 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return Element [name, MSBuildProject.Schema] != null;
 		}
 		
-		public void SetMetadata (string name, string value)
-		{
-			SetMetadata (name, value, true);
-		}
-		
-		public void SetMetadata (string name, string value, bool isLiteral)
+		public void SetMetadata (string name, string value, bool isXml = false)
 		{
 			XmlElement elem = Element [name, MSBuildProject.Schema];
 			if (elem == null) {
 				elem = AddChildElement (name);
 				Element.AppendChild (elem);
 			}
-			elem.InnerXml = value;
+			if (isXml)
+				elem.InnerXml = value;
+			else
+				elem.InnerText = value;
 		}
-		
+
 		public void UnsetMetadata (string name)
 		{
 			XmlElement elem = Element [name, MSBuildProject.Schema];
@@ -694,15 +698,15 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 		}
 		
-		public string GetMetadata (string name)
+		public string GetMetadata (string name, bool isXml = false)
 		{
 			XmlElement elem = Element [name, MSBuildProject.Schema];
 			if (elem != null)
-				return elem.InnerXml;
+				return isXml ? elem.InnerXml : elem.InnerText;
 			else
 				return null;
 		}
-		
+
 		public bool GetMetadataIsFalse (string name)
 		{
 			return String.Compare (GetMetadata (name), "False", StringComparison.OrdinalIgnoreCase) == 0;
@@ -712,7 +716,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		{
 			foreach (XmlNode node in Element.ChildNodes) {
 				if (node is XmlElement)
-					SetMetadata (node.LocalName, node.InnerXml);
+					SetMetadata (node.LocalName, node.InnerXml, true);
 			}
 		}
 	}
