@@ -56,8 +56,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		string lastBuildRuntime;
 		string lastFileName;
 		ITimeTracker timer;
-		bool useXBuild;
-		MSBuildVerbosity verbosity;
+		bool forceUseMSBuild;
 
 		struct ItemInfo {
 			public MSBuildItem Item;
@@ -87,7 +86,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				return subtypeGuids;
 			}
 		}
-		
+
 		public MSBuildProjectHandler ()
 		{
 		}
@@ -104,10 +103,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				this.targetImports.AddRange (import.Split (':'));
 			
 			Runtime.SystemAssemblyService.DefaultRuntimeChanged += OnDefaultRuntimeChanged;
-			
-			//FIXME: Update these when the properties change
-			useXBuild = PropertyService.Get ("MonoDevelop.Ide.BuildWithMSBuild", false);
-			verbosity = PropertyService.Get ("MonoDevelop.Ide.MSBuildVerbosity", MSBuildVerbosity.Normal);
 		}
 		
 		void OnDefaultRuntimeChanged (object o, EventArgs args)
@@ -145,14 +140,19 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 			return projectBuilder;
 		}
-		
-		public override void Dispose ()
+
+		void CleanupProjectBuilder ()
 		{
-			base.Dispose ();
 			if (projectBuilder != null) {
 				projectBuilder.Dispose ();
 				projectBuilder = null;
 			}
+		}
+
+		public override void Dispose ()
+		{
+			base.Dispose ();
+			CleanupProjectBuilder ();
 			Runtime.SystemAssemblyService.DefaultRuntimeChanged -= OnDefaultRuntimeChanged;
 		}
 		
@@ -165,18 +165,40 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 			return configObject.Platform;
 		}
+
+		ProjectConfigurationInfo[] GetConfigurations (SolutionEntityItem item, ConfigurationSelector configuration)
+		{
+			// Returns a list of project/configuration information for the provided item and all its references
+			List<ProjectConfigurationInfo> configs = new List<ProjectConfigurationInfo> ();
+			var c = item.GetConfiguration (configuration);
+			configs.Add (new ProjectConfigurationInfo () {
+				ProjectFile = item.FileName,
+				Configuration = c.Name,
+				Platform = GetExplicitPlatform (c)
+			});
+			foreach (var refProject in item.GetReferencedItems (configuration).OfType<Project> ()) {
+				var refConfig = refProject.GetConfiguration (configuration);
+				configs.Add (new ProjectConfigurationInfo () {
+					ProjectFile = refProject.FileName,
+					Configuration = refConfig.Name,
+					Platform = GetExplicitPlatform (refConfig)
+				});
+			}
+			return configs.ToArray ();
+		}
 		
 		IEnumerable<string> IAssemblyReferenceHandler.GetAssemblyReferences (ConfigurationSelector configuration)
 		{
-			if (useXBuild) {
+			if (UseMSBuildEngine) {
 				// Get the references list from the msbuild project
 				SolutionEntityItem item = (SolutionEntityItem) Item;
 				RemoteProjectBuilder builder = GetProjectBuilder ();
-				SolutionItemConfiguration configObject = item.GetConfiguration (configuration);
-				foreach (string s in builder.GetAssemblyReferences (configObject.Name, GetExplicitPlatform (configObject)))
+				var configs = GetConfigurations (item, configuration);
+				foreach (string s in builder.GetAssemblyReferences (configs))
 					yield return s;
 			}
 			else {
+				CleanupProjectBuilder ();
 				DotNetProject item = Item as DotNetProject;
 				if (item == null)
 					yield break;
@@ -189,16 +211,14 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		
 		public override BuildResult RunTarget (IProgressMonitor monitor, string target, ConfigurationSelector configuration)
 		{
-			if (useXBuild) {
+			if (UseMSBuildEngine) {
 				SolutionEntityItem item = Item as SolutionEntityItem;
 				if (item != null) {
 					
-					SolutionItemConfiguration configObject = item.GetConfiguration (configuration);
-				
 					LogWriter logWriter = new LogWriter (monitor.Log);
 					RemoteProjectBuilder builder = GetProjectBuilder ();
-					MSBuildResult[] results = builder.RunTarget (target, configObject.Name, GetExplicitPlatform (configObject),
-						logWriter, verbosity);
+					var configs = GetConfigurations (item, configuration);
+					MSBuildResult[] results = builder.RunTarget (target, configs, logWriter, MSBuildProjectService.DefaultMSBuildVerbosity);
 					System.Runtime.Remoting.RemotingServices.Disconnect (logWriter);
 					
 					BuildResult br = new BuildResult ();
@@ -212,6 +232,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				}
 			}
 			else {
+				CleanupProjectBuilder ();
 				if (Item is DotNetProject) {
 					MD1DotNetProjectHandler handler = new MD1DotNetProjectHandler ((DotNetProject)Item);
 					return handler.RunTarget (monitor, target, configuration);
@@ -274,6 +295,10 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				globalGroup = p.AddNewPropertyGroup (false);
 			
 			string itemGuid = globalGroup.GetPropertyValue ("ProjectGuid");
+			if (itemGuid == null)
+				throw new UserException ("Project file doesn't have a valid ProjectGuid");
+
+			itemGuid = itemGuid.ToUpper ();
 			string projectTypeGuids = globalGroup.GetPropertyValue ("ProjectTypeGuids");
 			string itemType = globalGroup.GetPropertyValue ("ItemType");
 
@@ -285,6 +310,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 						subtypeGuids.Add (guid);
 				}
 			}
+			// Enable xbuild by default only for standard .NET projects - not for subtypes
+			//ForceUseMSBuild = subtypeGuids.Count == 0;
 			
 			try {
 				timer.Trace ("Create item instance");
@@ -309,10 +336,14 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				timer.End ();
 			}
 		}
+
+		internal bool UseMSBuildEngine {
+			get { return forceUseMSBuild || MSBuildProjectService.DefaultBuildWithMSBuild; }
+		}
 		
-		internal bool UseXbuild {
-			get { return useXBuild; }
-			set { useXBuild = value; }
+		internal bool ForceUseMSBuild {
+			get { return forceUseMSBuild; }
+			set { forceUseMSBuild = value; }
 		}
 		
 		// All of the last 4 parameters are optional, but at least one must be provided.
@@ -324,14 +355,13 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			if (!string.IsNullOrEmpty (typeGuids)) {
 				DotNetProjectSubtypeNode st = MSBuildProjectService.GetDotNetProjectSubtype (typeGuids);
 				if (st != null) {
-					item = st.CreateInstance (language);
-					useXBuild = useXBuild || st.UseXBuild;
-					if (st.IsMigration) {
-						MigrateProject (monitor, st, p, fileName, language);
-						
+					forceUseMSBuild = st.UseXBuild;
+					Type migratedType = null;
+
+					if (st.IsMigration && (migratedType = MigrateProject (monitor, st, p, fileName, language)) != null) {
 						var oldSt = st;
-						st = MSBuildProjectService.GetItemSubtypeNodes ()
-							.Where (t => t.CanHandleItem ((SolutionEntityItem)item)).Last ();
+						st = MSBuildProjectService.GetItemSubtypeNodes ().Last (t => t.CanHandleType (migratedType));
+
 						for (int i = 0; i < subtypeGuids.Count; i++) {
 							if (string.Equals (subtypeGuids[i], oldSt.Guid, StringComparison.OrdinalIgnoreCase)) {
 								subtypeGuids[i] = st.Guid;
@@ -339,16 +369,21 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 								break;
 							}
 						}
+
 						if (oldSt != null)
 							throw new Exception ("Unable to correct flavor GUID");
+
 						var gg = string.Join (";", subtypeGuids) + ";" + TypeGuid;
-						p.GetGlobalPropertyGroup ().SetPropertyValue ("ProjectTypeGuids", gg.ToUpper ());
+						p.GetGlobalPropertyGroup ().SetPropertyValue ("ProjectTypeGuids", gg.ToUpper (), true);
 						p.Save (fileName);
 					}
+
+					item = st.CreateInstance (language);
 					st.UpdateImports ((SolutionEntityItem)item, targetImports);
 				} else
 					throw new UnknownSolutionItemTypeException (typeGuids);
 			}
+
 			if (item == null && itemClass != null)
 				item = (SolutionItem) Activator.CreateInstance (itemClass);
 			
@@ -365,22 +400,33 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					
 				item = (SolutionItem) Activator.CreateInstance (dt.ValueType);
 			}
+
 			return item;
 		}
 
-		void MigrateProject (IProgressMonitor monitor, DotNetProjectSubtypeNode st, MSBuildProject p, string fileName, string language)
+		Type MigrateProject (IProgressMonitor monitor, DotNetProjectSubtypeNode st, MSBuildProject p, string fileName, string language)
 		{
 			var projectLoadMonitor = monitor as IProjectLoadProgressMonitor;
 			if (projectLoadMonitor == null) {
+				// projectLoadMonitor will be null when running through md-tool, but
+				// this is not fatal if migration is not required, so just ignore it. --abock
+				if (!st.IsMigrationRequired)
+					return null;
+
 				LoggingService.LogError (Environment.StackTrace);
 				monitor.ReportError ("Could not open unmigrated project and no migrator was supplied", null);
 				throw new Exception ("Could not open unmigrated project and no migrator was supplied");
 			}
 			
-			var migrationType = projectLoadMonitor.ShouldMigrateProject ();
+			var migrationType = st.MigrationHandler.CanPromptForMigration
+				? st.MigrationHandler.PromptForMigration (projectLoadMonitor, p, fileName, language)
+				: projectLoadMonitor.ShouldMigrateProject ();
 			if (migrationType == MigrationType.Ignore) {
-				monitor.ReportError (string.Format ("MonoDevelop cannot open the project '{0}' unless it is migrated.", Path.GetFileName (fileName)), null);
-				throw new Exception ("The user choose not to migrate the project");
+				if (st.IsMigrationRequired) {
+					monitor.ReportError (string.Format ("{1} cannot open the project '{0}' unless it is migrated.", Path.GetFileName (fileName), BrandingService.ApplicationName), null);
+					throw new Exception ("The user choose not to migrate the project");
+				} else
+					return null;
 			}
 			
 			var baseDir = (FilePath) Path.GetDirectoryName (fileName);
@@ -399,8 +445,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					File.Copy (file, Path.Combine (backupDir, Path.GetFileName (file)));
 			}
 			
-			if (!st.MigrationHandler.Migrate (projectLoadMonitor, p, fileName, language))
+			var type = st.MigrationHandler.Migrate (projectLoadMonitor, p, fileName, language);
+			if (type == null)
 				throw new Exception ("Could not migrate the project");
+
+			return type;
 		}
 		
 		FileFormat GetFileFormat ()
@@ -495,9 +544,25 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			
 			foreach (MSBuildItem buildItem in msproject.GetAllItems ()) {
 				ProjectItem it = ReadItem (ser, buildItem);
-				if (it != null) {
-					EntityItem.Items.Add (it);
+
+				if (it == null) continue;
+
+				if (it is ProjectFile) {
+					var file = (ProjectFile)it;
+
+					if (file.Name.IndexOf ('*') > -1)  {
+						// Thanks to IsOriginatedFromWildcard, these expanded items will not be saved back to disk.
+						foreach (var expandedItem in ResolveWildcardItems (file))
+							EntityItem.Items.Add (expandedItem);
+
+						// Add to wildcard items (so it can be re-saved) instead of Items (where tools will 
+						// try to compile and display these nonstandard items
+						EntityItem.WildcardItems.Add (it);
+						continue;
+					}
 				}
+
+				EntityItem.Items.Add (it);
 			}
 			
 			timer.Trace ("Read configurations");
@@ -629,6 +694,66 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			Item.NeedsReload = false;
 		}
 
+		const string RecursiveDirectoryWildcard = "**";
+		static readonly char[] directorySeparators = new [] {
+			Path.DirectorySeparatorChar,
+			Path.AltDirectorySeparatorChar
+		};
+
+		static string GetWildcardDirectoryName (string path)
+		{
+			int indexOfLast = path.LastIndexOfAny (directorySeparators);
+			if (indexOfLast < 0)
+				return String.Empty;
+			return path.Substring (0, indexOfLast);
+		}
+
+		static string GetWildcardFileName (string path)
+		{
+			int indexOfLast = path.LastIndexOfAny (directorySeparators);
+			if (indexOfLast < 0)
+				return path;
+			if (indexOfLast == path.Length)
+				return String.Empty;
+			return path.Substring (indexOfLast + 1, path.Length - (indexOfLast + 1));
+		}
+
+		static IEnumerable<string> ExpandWildcardFilePath (string filePath)
+		{
+			if (String.IsNullOrWhiteSpace (filePath))
+				throw new ArgumentException ("Not a wildcard path");
+
+			string dir = GetWildcardDirectoryName (filePath);
+			string file = GetWildcardFileName (filePath);
+
+			if (String.IsNullOrEmpty (dir) || String.IsNullOrEmpty (file))
+				return null;
+
+			SearchOption searchOption = SearchOption.TopDirectoryOnly;
+			if (dir.EndsWith (RecursiveDirectoryWildcard, StringComparison.Ordinal)) {
+				dir = dir.Substring (0, dir.Length - RecursiveDirectoryWildcard.Length);
+				searchOption = SearchOption.AllDirectories;
+			}
+
+			if (!Directory.Exists (dir))
+				return null;
+
+			return Directory.GetFiles (dir, file, searchOption);
+		}
+
+		static IEnumerable<ProjectFile> ResolveWildcardItems (ProjectFile wildcardFile)
+		{
+			var paths = ExpandWildcardFilePath (wildcardFile.Name);
+			if (paths == null)
+				yield break;
+			foreach (var resolvedFilePath in paths) {
+				var projectFile = (ProjectFile)wildcardFile.Clone ();
+				projectFile.Name = resolvedFilePath;
+				projectFile.IsOriginatedFromWildcard = true;
+				yield return projectFile;
+			}
+		}
+
 		MSBuildPropertyGroup ExtractMergedtoprojectProperties (MSBuildSerializer ser, MSBuildPropertySet pgroup, SolutionItemConfiguration ob)
 		{
 			XmlDocument doc = new XmlDocument ();
@@ -640,27 +765,44 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			ClassDataType dt = (ClassDataType) ser.DataContext.GetConfigurationDataType (ob.GetType ());
 			foreach (ItemProperty prop in dt.GetProperties (ser.SerializationContext, ob)) {
 				MSBuildProperty bp = pgroup.GetProperty (prop.Name);
-				if (bp != null)
-					res.SetPropertyValue (bp.Name, bp.Value);
+				if (bp != null) {
+					var preserveCase = prop.DataType is MSBuildBoolDataType;
+					res.SetPropertyValue (bp.Name, bp.Value, preserveCase);
+				}
 			}
 			if (ob is DotNetProjectConfiguration) {
 				object cparams = ((DotNetProjectConfiguration)ob).CompilationParameters;
 				dt = (ClassDataType) ser.DataContext.GetConfigurationDataType (cparams.GetType ());
 				foreach (ItemProperty prop in dt.GetProperties (ser.SerializationContext, cparams)) {
 					MSBuildProperty bp = pgroup.GetProperty (prop.Name);
-					if (bp != null)
-						res.SetPropertyValue (bp.Name, bp.Value);
+					if (bp != null) {
+						var preserveCase = prop.DataType is MSBuildBoolDataType;
+						res.SetPropertyValue (bp.Name, bp.Value, preserveCase);
+					}
 				}
 			}
 			return res;
 		}
 		
-		IEnumerable<string> GetMergeToProjectProperties (MSBuildSerializer ser, object ob)
+		IEnumerable<MergedProperty> GetMergeToProjectProperties (MSBuildSerializer ser, object ob)
 		{
 			ClassDataType dt = (ClassDataType) ser.DataContext.GetConfigurationDataType (ob.GetType ());
 			foreach (ItemProperty prop in dt.GetProperties (ser.SerializationContext, ob)) {
-				if (IsMergeToProjectProperty (prop))
-					yield return prop.Name;
+				if (IsMergeToProjectProperty (prop)) {
+					yield return new MergedProperty (prop.Name, prop.DataType is MSBuildBoolDataType);
+				}
+			}
+		}
+
+		struct MergedProperty
+		{
+			public readonly string Name;
+			public readonly bool PreserveExistingCase;
+
+			public MergedProperty (string name, bool preserveExistingCase)
+			{
+				this.Name = name;
+				this.PreserveExistingCase = preserveExistingCase;
 			}
 		}
 
@@ -852,17 +994,17 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			if (eitem.Configurations.Count > 0) {
 				ItemConfiguration conf = eitem.Configurations.FirstOrDefault<ItemConfiguration> (c => c.Name == "Debug");
 				if (conf == null) conf = eitem.Configurations [0];
-				MSBuildProperty bprop = SetGroupProperty (globalGroup, "Configuration", conf.Name, false);
+				MSBuildProperty bprop = globalGroup.SetPropertyValue ("Configuration", conf.Name, false);
 				bprop.Condition = " '$(Configuration)' == '' ";
 				
 				string platform = conf.Platform.Length == 0 ? "AnyCPU" : conf.Platform;
-				bprop = SetGroupProperty (globalGroup, "Platform", platform, false);
+				bprop = globalGroup.SetPropertyValue ("Platform", platform, false);
 				bprop.Condition = " '$(Platform)' == '' ";
 			}
 			
 			if (TypeGuid == MSBuildProjectService.GenericItemGuid) {
 				DataType dt = MSBuildProjectService.DataContext.GetConfigurationDataType (Item.GetType ());
-				SetGroupProperty (globalGroup, "ItemType", dt.Name, false);
+				globalGroup.SetPropertyValue ("ItemType", dt.Name, false);
 			}
 
 			Item.ExtendedProperties ["ProjectGuid"] = Item.ItemId;
@@ -918,18 +1060,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			// Convert debug property
 			
 			foreach (SolutionItemConfiguration conf in eitem.Configurations) {
-				DotNetProjectConfiguration cp = conf as MonoDevelop.Projects.DotNetProjectConfiguration;
-				if (cp != null) {
-					if (newProject)
-						cp.ExtendedProperties ["ErrorReport"] = "prompt";
-					
-					string debugType = (string) cp.ExtendedProperties ["DebugType"];
-					if (cp.DebugMode) {
-						if (debugType != "full" && debugType != "pdbonly")
-							cp.ExtendedProperties ["DebugType"] = "full";
-					}
-					else if (debugType != "none" && debugType != "pdbonly")
-						cp.ExtendedProperties ["DebugType"] = "none";
+				if (newProject && conf is DotNetProjectConfiguration) {
+					conf.ExtendedProperties ["ErrorReport"] = "prompt";
 				}
 			}
 			
@@ -937,10 +1069,10 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 
 			if (eitem.Configurations.Count > 0) {
 				List<ConfigData> configData = GetConfigData (msproject, true);
-				
-				Dictionary<string,string> mergeToProjectProperties = new Dictionary<string,string> ();
-				HashSet<string> mergeToProjectPropertyNames = new HashSet<string> (GetMergeToProjectProperties ( ser, eitem.Configurations [0]));
-				HashSet<string> mergeToProjectPropertyNamesCopy = new HashSet<string> (mergeToProjectPropertyNames);
+
+				var mergeToProjectPropertyValues = new Dictionary<string,MergedPropertyValue> ();
+				var mergeToProjectProperties = new HashSet<MergedProperty> (GetMergeToProjectProperties (ser, eitem.Configurations [0]));
+				var mergeToProjectPropertyNames = new HashSet<string> (mergeToProjectProperties.Select (p => p.Name));
 				
 				foreach (SolutionItemConfiguration conf in eitem.Configurations) {
 					bool newConf = false;
@@ -988,23 +1120,23 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					
 					WritePropertyGroupMetadata (propGroup, ditem.ItemData, ser, conf, netConfig != null ? netConfig.CompilationParameters : null);
 					
-					CollectMergetoprojectProperties (propGroup, mergeToProjectPropertyNames, mergeToProjectProperties);
+					CollectMergetoprojectProperties (propGroup, mergeToProjectProperties, mergeToProjectPropertyValues);
 					
 					if (baseGroup != null)
-						propGroup.UnMerge (baseGroup, mergeToProjectPropertyNamesCopy);
+						propGroup.UnMerge (baseGroup, mergeToProjectPropertyNames);
 				}
 				
 				// Move properties with common values from configurations to the main
 				// property group
-				foreach (KeyValuePair<string,string> prop in mergeToProjectProperties)
-					globalGroup.SetPropertyValue (prop.Key, prop.Value);
-				foreach (string prop in mergeToProjectPropertyNamesCopy) {
-					if (!mergeToProjectProperties.ContainsKey (prop))
+				foreach (KeyValuePair<string,MergedPropertyValue> prop in mergeToProjectPropertyValues)
+					globalGroup.SetPropertyValue (prop.Key, prop.Value.Value, prop.Value.PreserveExistingCase);
+				foreach (string prop in mergeToProjectPropertyNames) {
+					if (!mergeToProjectPropertyValues.ContainsKey (prop))
 						globalGroup.RemoveProperty (prop);
 				}
 				foreach (SolutionItemConfiguration conf in eitem.Configurations) {
 					MSBuildPropertyGroup propGroup = FindPropertyGroup (configData, conf).Group;
-					foreach (string mp in mergeToProjectProperties.Keys)
+					foreach (string mp in mergeToProjectPropertyValues.Keys)
 						propGroup.RemoveProperty (mp);
 				}
 				
@@ -1022,9 +1154,9 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				oldItems [item.Name + "<" + item.Include] = new ItemInfo () { Item=item };
 			
 			// Add the new items
-			foreach (object ob in ((SolutionEntityItem)Item).Items)
+			foreach (object ob in ((SolutionEntityItem)Item).Items.Concat (((SolutionEntityItem)Item).WildcardItems))
 				SaveItem (monitor, ser, msproject, ob, oldItems);
-			
+
 			foreach (ItemInfo itemInfo in oldItems.Values) {
 				if (!itemInfo.Added)
 					msproject.RemoveItem (itemInfo.Item);
@@ -1034,26 +1166,16 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				var moniker = dotNetProject.TargetFramework.Id;
 				bool supportsMultipleFrameworks = TargetFormat.FrameworkVersions.Length > 0;
 				var def = dotNetProject.GetDefaultTargetFrameworkForFormat (GetFileFormat ());
-				bool isDefaultIdentifier = def.Identifier == moniker.Identifier;
-				bool isDefaultVersion = isDefaultIdentifier && def.Version == moniker.Version;
-				bool isDefaultProfile = isDefaultVersion && def.Profile == moniker.Profile;
 
-				// If the format only supports one fx version, or the version is the default, there is no need to store it
-				if (!isDefaultVersion && supportsMultipleFrameworks)
-					SetGroupProperty (globalGroup, "TargetFrameworkVersion", "v" + moniker.Version, false);
-				else
-					globalGroup.RemoveProperty ("TargetFrameworkVersion");
+				// If the format only supports one fx version, or the version is the default, there is no need to store it.
+				// However, is there is already a value set, do not remove it.
+				if (supportsMultipleFrameworks) {
+					SetIfPresentOrNotDefaultValue (globalGroup, "TargetFrameworkVersion", "v" + moniker.Version, "v" + def.Version);
+				}
 				
 				if (TargetFormat.SupportsMonikers) {
-					if (!isDefaultIdentifier && def.Identifier != moniker.Identifier)
-						SetGroupProperty (globalGroup, "TargetFrameworkIdentifier", moniker.Identifier, false);
-					else
-						globalGroup.RemoveProperty ("TargetFrameworkIdentifier");
-					
-					if (!isDefaultProfile && def.Profile != moniker.Profile)
-						SetGroupProperty (globalGroup, "TargetFrameworkProfile", moniker.Profile, false);
-					else
-						globalGroup.RemoveProperty ("TargetFrameworkProfile");
+					SetIfPresentOrNotDefaultValue (globalGroup, "TargetFrameworkIdentifier", moniker.Identifier, def.Identifier);
+					SetIfPresentOrNotDefaultValue (globalGroup, "TargetFrameworkProfile", moniker.Profile, def.Profile);
 				}
 			}
 
@@ -1084,13 +1206,33 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				msproject.SetProjectExtensions ("MonoDevelop", sw.ToString ());
 			} else
 				msproject.RemoveProjectExtensions ("MonoDevelop");
-			
+
 			// Don't save the file to disk if the content did not change
 			msproject.Save (eitem.FileName);
-			
+
 			if (projectBuilder != null)
 				projectBuilder.Refresh ();
 		}
+
+		void SetIfPresentOrNotDefaultValue (MSBuildPropertySet propGroup, string name, string value, string defaultValue)
+		{
+			bool hasDefaultValue = string.IsNullOrEmpty (value) || value == defaultValue;
+			var prop = propGroup.GetProperty (name);
+			if (prop != null) {
+				//if the value is default or empty, only remove the element if it was not already the default or empty
+				//to avoid unnecessary project file churn
+				if (hasDefaultValue) {
+					var existing = prop.Value;
+					bool alreadyHadDefaultValue = string.IsNullOrEmpty (existing) || existing == defaultValue;
+					if (!alreadyHadDefaultValue)
+						propGroup.RemoveProperty (name);
+				} else {
+					prop.Value = value;
+				}
+			} else if (!hasDefaultValue) {
+				propGroup.SetPropertyValue (name, value, false);
+			}
+		}
 		
 		void ForceDefaultValueSerialization (MSBuildSerializer ser, MSBuildPropertySet baseGroup, object ob)
 		{
@@ -1101,35 +1243,46 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 		}
 
-		void CollectMergetoprojectProperties (MSBuildPropertyGroup pgroup, HashSet<string> propertyNames, Dictionary<string,string> mergeToProjectProperties)
+		void CollectMergetoprojectProperties (MSBuildPropertyGroup pgroup, HashSet<MergedProperty> properties, Dictionary<string,MergedPropertyValue> mergeToProjectProperties)
 		{
 			// This method checks every property in pgroup which has the MergeToProject flag.
 			// If the value of this property is the same as the one stored in mergeToProjectProperties
 			// it means that the property can be merged to the main project property group (so far).
 			
-			foreach (string pname in new List<String> (propertyNames)) {
-				MSBuildProperty prop = pgroup.GetProperty (pname);
+			foreach (var pinfo in new List<MergedProperty> (properties)) {
+				MSBuildProperty prop = pgroup.GetProperty (pinfo.Name);
 				
-				string mvalue;
-				if (!mergeToProjectProperties.TryGetValue (pname, out mvalue)) {
+				MergedPropertyValue mvalue;
+				if (!mergeToProjectProperties.TryGetValue (pinfo.Name, out mvalue)) {
 					if (prop != null) {
 						// This is the first time the value is checked. Just assign it.
-						mergeToProjectProperties.Add (pname, prop.Value);
+						mergeToProjectProperties.Add (pinfo.Name, new MergedPropertyValue (prop.Value, pinfo.PreserveExistingCase));
 						continue;
 					}
 					// If there is no value, it can't be merged
 				}
-				else if (prop != null && prop.Value.Equals (mvalue, StringComparison.CurrentCultureIgnoreCase))
+				else if (prop != null && string.Equals (prop.Value, mvalue.Value, StringComparison.OrdinalIgnoreCase))
 					// Same value. It can be merged.
 					continue;
 
 				// The property can't be merged because different configurations have different
 				// values for it. Remove it from the list.
-				propertyNames.Remove (pname);
-				mergeToProjectProperties.Remove (pname);
+				properties.Remove (pinfo);
+				mergeToProjectProperties.Remove (pinfo.Name);
 			}
 		}
 
+		struct MergedPropertyValue
+		{
+			public readonly string Value;
+			public readonly bool PreserveExistingCase;
+
+			public MergedPropertyValue (string value, bool preserveExistingCase)
+			{
+				this.Value = value;
+				this.PreserveExistingCase = preserveExistingCase;
+			}
+		}
 		
 		void SaveItem (MonoDevelop.Core.IProgressMonitor monitor, MSBuildSerializer ser, MSBuildProject msproject, object ob, Dictionary<string,ItemInfo> oldItems)
 		{
@@ -1154,6 +1307,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		
 		void SaveProjectFile (MSBuildSerializer ser, MSBuildProject msproject, ProjectFile file, Dictionary<string,ItemInfo> oldItems)
 		{
+			if (file.IsOriginatedFromWildcard) return;
+
 			string itemName = (file.Subtype == Subtype.Directory)? "Folder" : file.BuildAction;
 
 			string path = MSBuildProjectService.ToMSBuildPath (Item.ItemDirectory, file.FilePath);
@@ -1348,7 +1503,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				if (name == "Include")
 					ditem.ItemData.Add (new DataValue ("Include", buildItem.Include));
 				else if (buildItem.HasMetadata (name)) {
-					string data = buildItem.GetMetadata (name);
+					string data = buildItem.GetMetadata (name, !prop.DataType.IsSimpleType);
 					ditem.ItemData.Add (GetDataNode (prop, data));
 				}
 			}
@@ -1430,7 +1585,21 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			foreach (DataNode node in itemData) {
 				notWrittenProps.Remove (node.Name);
 				ConvertToMsbuildFormat (node);
-				SetGroupProperty (propGroup, node.Name, GetXmlString (node), node is DataItem);
+
+				// In the other msbuild contexts (metadata, solution properties, etc) we TitleCase by default, so the node.Value is TitleCase.
+				// However, for property value, we lowercase by default and preserve existing case to reduce noise on VS-created files.
+				var boolNode = node as MSBuildBoolDataValue;
+				string value;
+				bool preserveExistingCase;
+				if (boolNode != null) {
+					value = boolNode.RawValue? "true" : "false";
+					preserveExistingCase = true;
+				} else {
+					value = GetXmlString (node);
+					preserveExistingCase = false;
+				}
+
+				propGroup.SetPropertyValue (node.Name, value, preserveExistingCase);
 			}
 			foreach (string prop in notWrittenProps)
 				propGroup.RemoveProperty (prop);
@@ -1473,12 +1642,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return configData;
 		}
 		
-		MSBuildProperty SetGroupProperty (MSBuildPropertySet propGroup, string name, string value, bool isLiteral)
-		{
-			propGroup.SetPropertyValue (name, value);
-			return propGroup.GetProperty (name);
-		}
-		
 		ConfigData FindPropertyGroup (List<ConfigData> configData, SolutionItemConfiguration config)
 		{
 			foreach (ConfigData data in configData) {
@@ -1606,7 +1769,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		string GetXmlString (DataNode node)
 		{
 			if (node is DataValue)
-				return EscapeText (((DataValue)node).Value);
+				return ((DataValue)node).Value;
 			else {
 				StringWriter sw = new StringWriter ();
 				XmlTextWriter xw = new XmlTextWriter (sw);
@@ -1646,7 +1809,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			new ItemMember (typeof(SolutionEntityItem), "SchemaVersion"),
 			new ItemMember (typeof(SolutionEntityItem), "ProjectGuid"),
 			new ItemMember (typeof(SolutionEntityItem), "ProjectTypeGuids"),
-			new ItemMember (typeof(DotNetProjectConfiguration), "DebugType"),
 			new ItemMember (typeof(DotNetProjectConfiguration), "ErrorReport"),
 			new ItemMember (typeof(DotNetProjectConfiguration), "TargetFrameworkVersion", new object[] { new MergeToProjectAttribute () }),
 			new ItemMember (typeof(ProjectReference), "RequiredTargetFramework"),
@@ -1659,49 +1821,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			"BootstrapperFile", "AppDesigner", "WebReferences", "WebReferenceUrl", "Service"
 		};
 		
-		public static string EscapeText (string text)
-		{
-			var result = new StringBuilder ();
-			foreach (char ch in text) {
-				switch (ch) {
-				case '<':
-					result.Append ("&lt;");
-					break;
-				case '>':
-					result.Append ("&gt;");
-					break;
-				case '&':
-					result.Append ("&amp;");
-					break;
-				case '\'':
-					result.Append ("&apos;");
-					break;
-				case '"':
-					result.Append ("&quot;");
-					break;
-				default:
-					int charValue = (int)ch;
-					if (IsSpecialChar (charValue)) {
-						result.AppendFormat ("&#x{0:X};", charValue);
-					} else {
-						result.Append (ch);
-					}
-					break;
-				}
-			}
-			return result.ToString ();
-		}
-		
-		static bool IsSpecialChar (int charValue)
-		{
-			return 
-				0x01 <= charValue && charValue <= 0x08 ||
-				0x0B <= charValue && charValue <= 0x0C ||
-				0x0E <= charValue && charValue <= 0x1F ||
-				0x7F <= charValue && charValue <= 0x84 ||
-				0x86 <= charValue && charValue <= 0x9F;
-		}
-		
 		public static string UnescapeText (string text)
 		{
 			var sb = new StringBuilder ();
@@ -1772,7 +1891,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			if (instance is ProjectFile)
 				return prop.IsExtendedProperty (typeof(ProjectFile));
 			if (instance is ProjectReference)
-				return prop.IsExtendedProperty (typeof(ProjectReference)) || prop.Name == "Package";
+				return prop.IsExtendedProperty (typeof(ProjectReference)) || prop.Name == "Package" || prop.Name == "Aliases";
 			if (instance is DotNetProjectConfiguration)
 				if (prop.Name == "CodeGeneration")
 					return false;
