@@ -26,14 +26,14 @@
 //
 
 using System;
+using System.Linq;
 using System.Diagnostics;
 using System.Collections.Generic;
 using System.IO;
-using System.Xml;
 using System.Globalization;
 using System.Runtime.Serialization.Formatters.Binary;
 using Mono.Addins;
-using MonoDevelop.Core.AddIns;
+using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Extensions;
 using MonoDevelop.Core.Serialization;
@@ -46,14 +46,13 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 	public static class MSBuildProjectService
 	{
 		const string ItemTypesExtensionPath = "/MonoDevelop/ProjectModel/MSBuildItemTypes";
-		public const string GenericItemGuid = "{9344bdbb-3e7f-41fc-a0dd-8665d75ee146}";
+		public const string GenericItemGuid = "{9344BDBB-3E7F-41FC-A0DD-8665D75EE146}";
 		public const string FolderTypeGuid = "{2150E333-8FDC-42A3-9474-1A3956D46DE8}";
 		
 		//NOTE: default toolsversion should match the default format.
 		// remember to update the builder process' app.config too
 		public const string DefaultFormat = "MSBuild10";
-		const string REFERENCED_MSBUILD_TOOLS = "4.0";
-		internal const string DefaultToolsVersion = REFERENCED_MSBUILD_TOOLS;
+		internal const string DefaultToolsVersion = "4.0";
 		
 		static DataContext dataContext;
 		
@@ -82,7 +81,23 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			Services.ProjectService.DataContextChanged += delegate {
 				dataContext = null;
 			};
+
+			PropertyService.PropertyChanged += HandlePropertyChanged;
+			DefaultBuildWithMSBuild = PropertyService.Get ("MonoDevelop.Ide.BuildWithMSBuild", false);
+			DefaultMSBuildVerbosity = PropertyService.Get ("MonoDevelop.Ide.MSBuildVerbosity", MSBuildVerbosity.Normal);
 		}
+
+		static void HandlePropertyChanged (object sender, PropertyChangedEventArgs e)
+		{
+			if (e.Key == "MonoDevelop.Ide.BuildWithMSBuild") {
+				DefaultBuildWithMSBuild = (bool) e.NewValue;
+			} else if (e.Key == "MonoDevelop.Ide.MSBuildVerbosity") {
+				DefaultMSBuildVerbosity = (MSBuildVerbosity) e.NewValue;
+			}
+		}
+
+		internal static bool DefaultBuildWithMSBuild { get; private set; }
+		internal static MSBuildVerbosity DefaultMSBuildVerbosity { get; private set; }
 		
 		public static SolutionEntityItem LoadItem (IProgressMonitor monitor, string fileName, MSBuildFileFormat expectedFormat, string typeGuid, string itemGuid)
 		{
@@ -152,6 +167,22 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 		}
 		
+		public static bool SupportsProjectType (string projectFile)
+		{
+			if (!string.IsNullOrWhiteSpace (projectFile)) {
+				// If we have a project file, try to load it.
+				try {
+					using (var monitor = new ConsoleProgressMonitor ()) {
+						return MSBuildProjectService.LoadItem (monitor, projectFile, null, null, null) != null;
+					}
+				} catch {
+					return false;
+				}
+			}
+
+			return false;
+		}
+
 		internal static DotNetProjectSubtypeNode GetDotNetProjectSubtype (string typeGuids)
 		{
 			if (!string.IsNullOrEmpty (typeGuids)) {
@@ -269,12 +300,10 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		
 		public static string ToMSBuildPath (string baseDirectory, string absPath)
 		{
-			absPath = EscapeString (absPath);
 			if (baseDirectory != null) {
-				absPath = FileService.NormalizeRelativePath (FileService.AbsoluteToRelativePath (
-				         baseDirectory, absPath));
+				absPath = FileService.NormalizeRelativePath (FileService.AbsoluteToRelativePath (baseDirectory, absPath));
 			}
-			return absPath.Replace ('/', '\\');
+			return EscapeString (absPath).Replace ('/', '\\');
 		}
 		
 		internal static string ToMSBuildPathRelative (string baseDirectory, string absPath)
@@ -338,7 +367,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			
 			// If we're on Windows, don't need to fix file casing.
 			if (Platform.IsWindows) {
-				resultPath = Path.GetFullPath (path);
+				resultPath = FileService.GetFullPath (path);
 				return true;
 			}
 			
@@ -449,7 +478,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 
 				//always start the remote process explicitly, even if it's using the current runtime and fx
 				//else it won't pick up the assembly redirects from the builder exe
-				var exe = GetExeLocation (toolsVersion);
+				var exe = GetExeLocation (runtime, toolsVersion);
 				MonoDevelop.Core.Execution.RemotingService.RegisterRemotingChannel ();
 				var pinfo = new ProcessStartInfo (exe) {
 					UseShellExecute = false,
@@ -483,13 +512,14 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 		}
 		
-		static string GetExeLocation (string toolsVersion)
+		static string GetExeLocation (TargetRuntime runtime, string toolsVersion)
 		{
-			FilePath sourceExe = typeof(ProjectBuilder).Assembly.Location;
-			if (toolsVersion == REFERENCED_MSBUILD_TOOLS)
-				return sourceExe;
-			
-			var exe = sourceExe.ParentDirectory.Combine ("MSBuild", toolsVersion, sourceExe.FileName);
+			FilePath sourceExe = typeof(MSBuildProjectService).Assembly.Location;
+
+			if ((runtime is MsNetTargetRuntime) && int.Parse (toolsVersion.Split ('.')[0]) >= 4)
+				toolsVersion = "dotnet." + toolsVersion;
+
+			var exe = sourceExe.ParentDirectory.Combine ("MSBuild", toolsVersion, "MonoDevelop.Projects.Formats.MSBuild.exe");
 			if (File.Exists (exe))
 				return exe;
 			
@@ -564,6 +594,13 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 
 			return globalGroup.GetPropertyValue ("ProjectTypeGuids");
 		}
+
+		internal static UnknownProjectTypeNode GetUnknownProjectTypeInfo (string[] guids)
+		{
+			var nodes = AddinManager.GetExtensionNodes<UnknownProjectTypeNode> ("/MonoDevelop/ProjectModel/UnknownMSBuildProjectTypes")
+				.Where (p => guids.Any (p.MatchesGuid)).ToList ();
+			return nodes.FirstOrDefault (n => !n.IsSolvable) ?? nodes.FirstOrDefault (n => n.IsSolvable);
+		}
 	}
 	
 	class MSBuildDataContext: DataContext
@@ -572,6 +609,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		{
 			if (type == typeof(bool))
 				return new MSBuildBoolDataType ();
+			else if (type == typeof(bool?))
+				return new MSBuildNullableBoolDataType ();
 			else
 				return base.CreateConfigurationDataType (type);
 		}
@@ -585,7 +624,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		
 		internal protected override DataNode OnSerialize (SerializationContext serCtx, object mapData, object value)
 		{
-			return new DataValue (Name, (bool)value ? "true" : "false");
+			return new MSBuildBoolDataValue (Name, (bool) value);
 		}
 		
 		internal protected override object OnDeserialize (SerializationContext serCtx, object mapData, DataNode data)
@@ -593,6 +632,48 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return String.Equals (((DataValue)data).Value, "true", StringComparison.OrdinalIgnoreCase);
 		}
 	}
+
+	class MSBuildBoolDataValue : DataValue
+	{
+		public MSBuildBoolDataValue (string name, bool value)
+			: base (name, value ? "True" : "False")
+		{
+			RawValue = value;
+		}
+
+		public bool RawValue { get; private set; }
+	}
+
+	class MSBuildNullableBoolDataType: PrimitiveDataType
+	{
+		public MSBuildNullableBoolDataType (): base (typeof(bool))
+		{
+		}
+
+		internal protected override DataNode OnSerialize (SerializationContext serCtx, object mapData, object value)
+		{
+			return new MSBuildNullableBoolDataValue (Name, (bool?) value);
+		}
+
+		internal protected override object OnDeserialize (SerializationContext serCtx, object mapData, DataNode data)
+		{
+			var d = (DataValue)data;
+			if (string.IsNullOrEmpty (d.Value))
+				return (bool?) null;
+			return (bool?) String.Equals (d.Value, "true", StringComparison.OrdinalIgnoreCase);
+		}
+	}
+
+	class MSBuildNullableBoolDataValue : DataValue
+	{
+		public MSBuildNullableBoolDataValue (string name, bool? value)
+			: base (name, value.HasValue? (value.Value? "True" : "False") : null)
+		{
+			RawValue = value;
+		}
+
+		public bool? RawValue { get; private set; }
+	}
 	
 	public class MSBuildResourceHandler: IResourceHandler
 	{
