@@ -41,10 +41,12 @@ using MonoDevelop.Core.Serialization;
 using MonoDevelop.Projects.Extensions;
 using MonoDevelop.Core;
 using MonoDevelop.Core.ProgressMonitoring;
+using System.Reflection;
+using System.Linq;
 
 namespace MonoDevelop.Projects.Formats.MSBuild
 {
-	internal class SlnFileFormat
+	public class SlnFileFormat
 	{
 		public string GetValidFormatName (object obj, string fileName, MSBuildFileFormat format)
 		{
@@ -56,7 +58,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			if (String.Compare (Path.GetExtension (file), ".sln", true) == 0) {
 				string tmp;
 				string version = GetSlnFileVersion (file, out tmp);
-				return version == format.SlnVersion;
+				return format.SupportsSlnVersion (version);
 			}
 			return false;
 		}
@@ -241,6 +243,16 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 						WriteDataItem (writer, data);
 						writer.WriteLine ("\tEndProjectSection");
 					}
+					if (item.ItemDependencies.Count > 0) {
+						writer.WriteLine ("\tProjectSection(ProjectDependencies) = postProject");
+						foreach (var dep in item.ItemDependencies)
+							writer.WriteLine ("\t\t{0} = {0}", dep.ItemId);
+						if (handler.UnresolvedProjectDependencies != null) {
+							foreach (var dep in handler.UnresolvedProjectDependencies)
+								writer.WriteLine ("\t\t{0} = {0}", dep);
+						}
+						writer.WriteLine ("\tEndProjectSection");
+					}
 				} else if (ce is SolutionFolder) {
 					//Solution
 					SlnData slnData = GetSlnData (ce);
@@ -307,12 +319,23 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					if (p is UnknownSolutionItem)
 						continue;
 					
-					list.Add (String.Format (
-						"\t\t{0}.{1}.ActiveCfg = {2}", p.ItemId, ToSlnConfigurationId (cc), ToSlnConfigurationId (cce.ItemConfiguration)));
+                    // <ProjectGuid>...</ProjectGuid> in some Visual Studio generated F# project files 
+                    // are missing "{"..."}" in their guid. This is not generally a problem since it
+                    // is a valid GUID format. However the solution file format requires that these are present. 
+                    string itemGuid = p.ItemId;
+                    if (!itemGuid.StartsWith("{") && !itemGuid.EndsWith("}")) 
+                        itemGuid = "{" + itemGuid + "}";
+
+                    list.Add (String.Format (
+                        "\t\t{0}.{1}.ActiveCfg = {2}", itemGuid, ToSlnConfigurationId (cc), ToSlnConfigurationId (cce.ItemConfiguration)));
 
 					if (cce.Build)
 						list.Add (String.Format (
-							"\t\t{0}.{1}.Build.0 = {2}", p.ItemId, ToSlnConfigurationId (cc), ToSlnConfigurationId (cce.ItemConfiguration)));
+                            "\t\t{0}.{1}.Build.0 = {2}", itemGuid, ToSlnConfigurationId (cc), ToSlnConfigurationId (cce.ItemConfiguration)));
+					
+					if (cce.Deploy)
+						list.Add (String.Format (
+							"\t\t{0}.{1}.Deploy.0 = {2}", itemGuid, ToSlnConfigurationId (cc), ToSlnConfigurationId (cce.ItemConfiguration)));
 				}
 			}
 			
@@ -333,7 +356,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		{
 			// Find a project section of type MonoDevelopProperties
 			int start, end;
-			if (!FindSection (lines, "MonoDevelopProperties", out start, out end))
+			if (!FindSection (lines, "MonoDevelopProperties", true, out start, out end))
 				return null;
 			
 			// Deserialize the object
@@ -344,13 +367,33 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return it;
 		}
 		
+		List<string> ReadSolutionItemDependencies (List<string> lines)
+		{
+			// Find a project section of type MonoDevelopProperties
+			int start, end;
+			if (!FindSection (lines, "ProjectDependencies", false, out start, out end))
+				return null;
+
+			var ids = new List<string> ();
+			for (int n=start + 1; n < end; n++) {
+				string line = lines [n];
+				int i = line.IndexOf ('=');
+				if (i != -1)
+					ids.Add (line.Substring (0, i).Trim ());
+			}
+
+			// Remove the lines, since they have already been preocessed
+			lines.RemoveRange (start, end - start + 1);
+			return ids;
+		}
+
 		List<string> ReadFolderFiles (List<string> lines)
 		{
 			// Find a solution item section of type SolutionItems
 
 			List<string> list = new List<string> ();
 			int start, end;
-			if (!FindSection (lines, "SolutionItems", out start, out end))
+			if (!FindSection (lines, "SolutionItems", true, out start, out end))
 				return list;
 			
 			for (int n=start + 1; n < end; n++) {
@@ -368,14 +411,17 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return list;
 		}
 		
-		bool FindSection (List<string> lines, string name, out int start, out int end)
+		bool FindSection (List<string> lines, string name, bool preProject, out int start, out int end)
 		{
 			start = -1;
 			end = -1;
+
+			string prePost = preProject ? "preProject" : "postProject";
+			var sectionLine = "ProjectSection(" + name + ")=" + prePost;
 			
 			for (int n=0; n<lines.Count && start == -1; n++) {
 				string line = lines [n].Replace ("\t","").Replace (" ", "");
-				if (line == "ProjectSection(" + name + ")=preProject")
+				if (line == sectionLine)
 					start = n;
 			}
 			if (start == -1)
@@ -595,6 +641,9 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				ProjectExtensionUtil.BeginLoadOperation ();
 				sol = new Solution ();
 				monitor.BeginTask (string.Format (GettextCatalog.GetString ("Loading solution: {0}"), fileName), 1);
+				var projectLoadMonitor = monitor as IProjectLoadProgressMonitor;
+				if (projectLoadMonitor != null)
+					projectLoadMonitor.CurrentSolution = sol;
 				LoadSolution (sol, fileName, format, monitor);
 			} catch (Exception ex) {
 				monitor.ReportError (GettextCatalog.GetString ("Could not load solution: {0}", fileName), ex);
@@ -693,7 +742,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				string projTypeGuid = match.Groups [1].Value.ToUpper ();
 				string projectName = match.Groups [2].Value;
 				string projectPath = match.Groups [3].Value;
-				string projectGuid = match.Groups [4].Value;
+				string projectGuid = match.Groups [4].Value.ToUpper ();
 
 				if (projTypeGuid == MSBuildProjectService.FolderTypeGuid) {
 					//Solution folder
@@ -741,7 +790,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				
 				try {
 					item = ProjectExtensionUtil.LoadSolutionItem (monitor, projectPath, delegate {
-						return MSBuildProjectService.LoadItem (monitor, projectPath, projTypeGuid, projectGuid);
+						return MSBuildProjectService.LoadItem (monitor, projectPath, format, projTypeGuid, projectGuid);
 					});
 					
 					if (item == null) {
@@ -755,21 +804,43 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 						continue;
 					}
 
-					MSBuildProjectHandler handler = (MSBuildProjectHandler) item.ItemHandler;
+					MSBuildHandler handler = (MSBuildHandler) item.ItemHandler;
 					List<string> projLines = lines.GetRange (sec.Start + 1, sec.Count - 2);
 					DataItem it = GetSolutionItemData (projLines);
+
+					handler.UnresolvedProjectDependencies = ReadSolutionItemDependencies (projLines);
 					handler.SlnProjectContent = projLines.ToArray ();
 					handler.ReadSlnData (it);
 					
 				} catch (Exception e) {
+					// If we get a TargetInvocationException from using Activator.CreateInstance we
+					// need to unwrap the real exception
+					while (e is TargetInvocationException)
+						e = ((TargetInvocationException) e).InnerException;
+					
 					if (e is UnknownSolutionItemTypeException) {
 						var name = ((UnknownSolutionItemTypeException)e).TypeName;
-						LoggingService.LogWarning (!string.IsNullOrEmpty (name)?
-							  string.Format ("Could not load project '{0}' with unknown item type '{1}'", projectPath, name)
-							: string.Format ("Could not load project '{0}' with unknown item type", projectPath));
-						monitor.ReportWarning (!string.IsNullOrEmpty (name)?
-							  GettextCatalog.GetString ("Could not load project '{0}' with unknown item type '{1}'", projectPath, name)
-							: GettextCatalog.GetString ("Could not load project '{0}' with unknown item type", projectPath));
+
+						var relPath = new FilePath (path).ToRelative (sol.BaseDirectory);
+						if (!string.IsNullOrEmpty (name)) {
+							var guids = name.Split (';');
+							var projectInfo = MSBuildProjectService.GetUnknownProjectTypeInfo (guids);
+							if (projectInfo != null) {
+								LoggingService.LogWarning (string.Format ("Could not load {0} project '{1}'. {2}", projectInfo.Name, relPath, projectInfo.GetInstructions ()));
+								monitor.ReportWarning (GettextCatalog.GetString ("Could not load {0} project '{1}'. {2}", projectInfo.Name, relPath, projectInfo.GetInstructions ()));
+							} else {
+								LoggingService.LogWarning (string.Format ("Could not load project '{0}' with unknown item type '{1}'", relPath, name));
+								monitor.ReportWarning (GettextCatalog.GetString ("Could not load project '{0}' with unknown item type '{1}'", relPath, name));
+							}
+						} else {
+							LoggingService.LogWarning (string.Format ("Could not load project '{0}' with unknown item type", relPath));
+							monitor.ReportWarning (GettextCatalog.GetString ("Could not load project '{0}' with unknown item type", relPath));
+						}
+
+					} else if (e is UserException) {
+						var ex = (UserException) e;
+						LoggingService.LogError ("{0}: {1}", ex.Message, ex.Details);
+						monitor.ReportError (string.Format ("{0}{1}{1}{2}", ex.Message, Environment.NewLine, ex.Details), null);
 					} else {
 						LoggingService.LogError (string.Format ("Error while trying to load the project {0}", projectPath), e);
 						monitor.ReportWarning (GettextCatalog.GetString (
@@ -802,6 +873,22 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				globals.Remove ("NestedProjects");
 			}
 
+			// Resolve project dependencies
+			foreach (var it in items.OfType<SolutionEntityItem> ()) {
+				MSBuildHandler handler = (MSBuildHandler) it.ItemHandler;
+				if (handler.UnresolvedProjectDependencies != null) {
+					foreach (var id in handler.UnresolvedProjectDependencies.ToArray ()) {
+						SolutionItem dep;
+						if (items.TryGetValue (id, out dep) && dep is SolutionEntityItem) {
+							handler.UnresolvedProjectDependencies.Remove (id);
+							it.ItemDependencies.Add ((SolutionEntityItem)dep);
+						}
+					}
+					if (handler.UnresolvedProjectDependencies.Count == 0)
+						handler.UnresolvedProjectDependencies = null;
+				}
+			}
+
 			//Add top level folders and projects to the main folder
 			foreach (SolutionItem ce in sortedList) {
 				if (ce.ParentFolder == null)
@@ -894,6 +981,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				//Format:
 				// {projectGuid}.SolutionConfigName|SolutionPlatform.ActiveCfg = ProjConfigName|ProjPlatform
 				// {projectGuid}.SolutionConfigName|SolutionPlatform.Build.0 = ProjConfigName|ProjPlatform
+				// {projectGuid}.SolutionConfigName|SolutionPlatform.Deploy.0 = ProjConfigName|ProjPlatform
 
 				string [] parts = s.Split (new char [] {'='}, 2);
 				if (parts.Length < 2) {
@@ -911,8 +999,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				} else if (left.EndsWith (".Build.0")) {
 					action = "Build.0";
 					left = left.Substring (0, left.Length - 8);
+				} else if (left.EndsWith (".Deploy.0")) {
+					action = "Deploy.0";
+					left = left.Substring (0, left.Length - 9);
 				} else { 
-					LoggingService.LogWarning (GettextCatalog.GetString ("{0} ({1}) : Unknown action. Only ActiveCfg & Build.0 supported.",
+					LoggingService.LogWarning (GettextCatalog.GetString ("{0} ({1}) : Unknown action. Only ActiveCfg, Build.0 and Deploy.0 supported.",
 						sln.FileName, lineNum + 1));
 					continue;
 				}
@@ -924,7 +1015,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					continue;
 				}
 
-				string projGuid = t [0];
+				string projGuid = t [0].ToUpper ();
 				string slnConfig = t [1];
 
 				if (!slnData.ItemsByGuid.ContainsKey (projGuid)) {
@@ -969,6 +1060,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 						combineConfigEntry.ItemConfiguration = FromSlnConfigurationId (projConfig);
 					} else if (action == "Build.0") {
 						combineConfigEntry.Build = true;
+					} else if (action == "Deploy.0") {
+						combineConfigEntry.Deploy = true;
 					}
 				}
 				extras.RemoveAt (extras.Count - 1);
@@ -1050,7 +1143,9 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				return;
 
 			for (int i = 0; i < sec.Count - 2; i ++) {
+				// Guids should be upper case for VS compatibility
 				KeyValuePair<string, string> pair = SplitKeyValue (lines [i + sec.Start + 1].Trim ());
+				pair = new KeyValuePair<string, string> (pair.Key.ToUpper (), pair.Value.ToUpper ());
 
 				SolutionItem folderItem;
 				SolutionItem item;
