@@ -329,9 +329,8 @@ namespace MonoDevelop.Projects
 				file = format.GetValidFileName (item, file);
 			}
 			
-			if (!FileService.RequestFileEdit (file))
-				throw new UserException (GettextCatalog.GetString ("The project could not be saved"), GettextCatalog.GetString ("Write permission has not been granted for file '{0}'", file));
-			
+			FileService.RequestFileEdit (file);
+
 			format.Format.WriteFile (file, item, monitor);
 			return file;
 		}
@@ -644,6 +643,7 @@ namespace MonoDevelop.Projects
 		
 		public override void Save (IProgressMonitor monitor, SolutionEntityItem entry)
 		{
+			FileService.RequestFileEdit (entry.GetItemFiles (false));
 			entry.OnSave (monitor);
 		}
 		
@@ -731,6 +731,17 @@ namespace MonoDevelop.Projects
 				throw new InvalidOperationException ("Unknown item type: " + item);
 		}
 		
+		public override IEnumerable<ExecutionTarget> GetExecutionTargets (IBuildTarget item, ConfigurationSelector configuration)
+		{
+			if (item is WorkspaceItem) {
+				return ((WorkspaceItem)item).OnGetExecutionTargets (configuration);
+			}
+			else if (item is SolutionItem)
+				return ((SolutionItem)item).OnGetExecutionTargets (configuration);
+			else
+				throw new InvalidOperationException ("Unknown item type: " + item);
+		}
+
 		public override bool GetNeedsBuilding (IBuildTarget item, ConfigurationSelector configuration)
 		{
 			if (item is SolutionItem) {
@@ -794,16 +805,6 @@ namespace MonoDevelop.Projects
 		public static TimerCounter BuildWorkspaceItemTimer = InstrumentationService.CreateTimerCounter ("Workspace item built", "Project Model");
 		public static TimerCounter NeedsBuildingTimer = InstrumentationService.CreateTimerCounter ("Needs building checked", "Project Model");
 		
-		public static Counter TypeIndexEntries = InstrumentationService.CreateCounter ("Type index entries", "Parser Service");
-		public static Counter LiveTypeObjects = InstrumentationService.CreateCounter ("Live type objects", "Parser Service");
-		public static Counter LiveDatabases = InstrumentationService.CreateCounter ("Parser databases", "Parser Service");
-		public static Counter LiveAssemblyDatabases = InstrumentationService.CreateCounter ("Assembly databases", "Parser Service");
-		public static Counter LiveProjectDatabases = InstrumentationService.CreateCounter ("Project databases", "Parser Service");
-		public static TimerCounter DatabasesRead = InstrumentationService.CreateTimerCounter ("Parser database read", "Parser Service");
-		public static TimerCounter DatabasesWritten = InstrumentationService.CreateTimerCounter ("Parser database written", "Parser Service");
-		public static TimerCounter FileParse = InstrumentationService.CreateTimerCounter ("File parsed", "Parser Service");
-		public static TimerCounter AssemblyParseTime = InstrumentationService.CreateTimerCounter ("Assembly parsed", "Parser Service");
-		
 		public static TimerCounter HelpServiceInitialization = InstrumentationService.CreateTimerCounter ("Help Service initialization", "IDE");
 		public static TimerCounter ParserServiceInitialization = InstrumentationService.CreateTimerCounter ("Parser Service initialization", "IDE");
 	}
