@@ -49,6 +49,7 @@ namespace MonoDevelop.Components.Commands
 		DateTime lastUserInteraction;
 		KeyboardShortcut[] chords;
 		string chord;
+		internal const int SlowCommandWarningTime = 50;
 		
 		Dictionary<object,Command> cmds = new Dictionary<object,Command> ();
 		Hashtable handlerInfo = new Hashtable ();
@@ -58,7 +59,7 @@ namespace MonoDevelop.Components.Commands
 		ArrayList visitors = new ArrayList ();
 		Dictionary<Gtk.Window,Gtk.Window> topLevelWindows = new Dictionary<Gtk.Window,Gtk.Window> ();
 		Stack delegatorStack = new Stack ();
-		
+
 		HashSet<object> visitedTargets = new HashSet<object> ();
 		
 		bool disposed;
@@ -74,7 +75,8 @@ namespace MonoDevelop.Components.Commands
 		internal static readonly object CommandRouteTerminator = new object ();
 		
 		internal bool handlerFoundInMulticast;
-		
+		Gtk.Widget lastActiveWidget;
+
 		public CommandManager (): this (null)
 		{
 		}
@@ -266,7 +268,13 @@ namespace MonoDevelop.Components.Commands
 				isEnabled = value;
 			}
 		}
-		
+
+
+		/// <summary>
+		/// The command currently being executed or for which the status is being checked
+		/// </summary>
+		public Command CurrentCommand { get; private set; }
+
 		bool CanUseBinding (KeyboardShortcut[] chords, KeyboardShortcut[] accels, out KeyBinding binding, out bool isChord)
 		{
 			if (chords != null) {
@@ -996,7 +1004,8 @@ namespace MonoDevelop.Components.Commands
 				cmd = GetActionCommand (commandId);
 				if (cmd == null)
 					return false;
-				
+
+				CurrentCommand = cmd;
 				CommandTargetRoute targetRoute = new CommandTargetRoute (initialTarget);
 				object cmdTarget = GetFirstCommandTarget (targetRoute);
 				CommandInfo info = new CommandInfo (cmd);
@@ -1074,6 +1083,9 @@ namespace MonoDevelop.Components.Commands
 				name = name.Replace ("_","");
 				ReportError (commandId, "Error while executing command: " + name, ex);
 			}
+			finally {
+				CurrentCommand = null;
+			}
 			return false;
 		}
 		
@@ -1156,11 +1168,13 @@ namespace MonoDevelop.Components.Commands
 
 			NotifyCommandTargetScanStarted ();
 			CommandInfo info = new CommandInfo (cmd);
-			
+
 			try {
 				bool multiCastEnabled = true;
 				bool multiCastVisible = false;
-				
+
+				CurrentCommand = cmd;
+
 				object cmdTarget = GetFirstCommandTarget (targetRoute);
 				
 				while (cmdTarget != null)
@@ -1231,6 +1245,7 @@ namespace MonoDevelop.Components.Commands
 				info.Visible = true;
 			} finally {
 				NotifyCommandTargetScanFinished ();
+				CurrentCommand = null;
 			}
 
 			if (guiLock > 0)
@@ -1273,16 +1288,22 @@ namespace MonoDevelop.Components.Commands
 		{
 			CommandTargetRoute targetRoute = new CommandTargetRoute (initialTarget);
 			object cmdTarget = GetFirstCommandTarget (targetRoute);
-			
-			while (cmdTarget != null)
-			{
-				if (visitor.Visit (cmdTarget))
-					return cmdTarget;
 
-				cmdTarget = GetNextCommandTarget (targetRoute, cmdTarget);
+			visitor.Start ();
+
+			try {
+				while (cmdTarget != null)
+				{
+					if (visitor.Visit (cmdTarget))
+						return cmdTarget;
+
+					cmdTarget = GetNextCommandTarget (targetRoute, cmdTarget);
+				}
+			} catch (Exception ex) {
+				LoggingService.LogError ("Error while visiting command targets", ex);
+			} finally {
+				visitor.End ();
 			}
-			
-			visitor.Visit (null);
 			return null;
 		}
 		
@@ -1509,12 +1530,22 @@ namespace MonoDevelop.Components.Commands
 			return cmdTarget;
 		}
 		
-		object GetNextCommandTarget (CommandTargetRoute targetRoute, object cmdTarget)
+		object GetNextCommandTarget (CommandTargetRoute targetRoute, object cmdTarget, bool ignoreDelegator = false)
 		{
 			if (cmdTarget is IMultiCastCommandRouter) 
 				cmdTarget = new MultiCastDelegator (this, (IMultiCastCommandRouter)cmdTarget, targetRoute);
-			
-			if (cmdTarget is ICommandDelegatorRouter) {
+
+			if (!ignoreDelegator && cmdTarget is ICommandDelegator) {
+				if (cmdTarget is ICommandDelegatorRouter)
+					throw new InvalidOperationException ("A type can't implement both ICommandDelegator and ICommandDelegatorRouter");
+				object oldCmdTarget = cmdTarget;
+				cmdTarget = ((ICommandDelegator)oldCmdTarget).GetDelegatedCommandTarget ();
+				if (cmdTarget != null)
+					delegatorStack.Push (oldCmdTarget);
+				else
+					cmdTarget = GetNextCommandTarget (targetRoute, oldCmdTarget, true);
+			}
+			else if (cmdTarget is ICommandDelegatorRouter) {
 				object oldCmdTarget = cmdTarget;
 				cmdTarget = ((ICommandDelegatorRouter)oldCmdTarget).GetDelegatedCommandTarget ();
 				if (cmdTarget != null)
@@ -1531,8 +1562,11 @@ namespace MonoDevelop.Components.Commands
 			
 			if (cmdTarget == null || !visitedTargets.Add (cmdTarget)) {
 				if (delegatorStack.Count > 0) {
-					ICommandDelegatorRouter del = (ICommandDelegatorRouter) delegatorStack.Pop ();
-					cmdTarget = del.GetNextCommandTarget ();
+					var del = delegatorStack.Pop ();
+					if (del is ICommandDelegatorRouter)
+						cmdTarget = ((ICommandDelegatorRouter)del).GetNextCommandTarget ();
+					else
+						cmdTarget = GetNextCommandTarget (targetRoute, del, true);
 					if (cmdTarget == CommandManager.CommandRouteTerminator)
 						return null;
 					if (cmdTarget != null)
@@ -1569,8 +1603,8 @@ namespace MonoDevelop.Components.Commands
 						newFocused = w;
 					}
 					if (w.IsActive && w.Type == Gtk.WindowType.Toplevel && !(w is Gtk.Dialog)) {
-						win = w;
-						break;
+						if (win == null)
+							win = w;
 					}
 					if (lastFocused == w) {
 						lastFocusedExists = true;
@@ -1592,8 +1626,8 @@ namespace MonoDevelop.Components.Commands
 		Gtk.Widget GetActiveWidget (Gtk.Window win)
 		{
 			win = GetActiveWindow (win);
+			Gtk.Widget widget = win;
 			if (win != null) {
-				Gtk.Widget widget = win;
 				while (widget is Gtk.Container) {
 					Gtk.Widget child = ((Gtk.Container)widget).FocusChild;
 					if (child != null)
@@ -1601,11 +1635,15 @@ namespace MonoDevelop.Components.Commands
 					else
 						break;
 				}
-				return widget;
 			}
-			return win;
+			if (widget != lastActiveWidget) {
+				if (ActiveWidgetChanged != null)
+					ActiveWidgetChanged (this, new ActiveWidgetEventArgs () { OldActiveWidget = lastActiveWidget, NewActiveWidget = widget });
+				lastActiveWidget = widget;
+			}
+			return widget;
 		}
-		
+
 		bool UpdateStatus ()
 		{
 			if (!disposed && toolbarUpdaterRunning)
@@ -1731,26 +1769,29 @@ namespace MonoDevelop.Components.Commands
 			if (this.disposed)
 				return;
 			
-			object activeWidget = GetActiveWidget (rootWidget);
+			var activeWidget = GetActiveWidget (rootWidget);
 			foreach (ICommandBar toolbar in toolbars) {
 				toolbar.Update (activeWidget);
 			}
 			foreach (ICommandTargetVisitor v in visitors)
 				VisitCommandTargets (v, null);
 		}
-		
+
 		void UpdateAppFocusStatus (bool hasFocus, bool lastFocusedExists)
 		{
 			if (hasFocus != appHasFocus) {
 				// The last focused window has been destroyed. Wait a few ms since another app's window
 				// may gain focus again
+
 				DateTime now = DateTime.Now;
-				if (now < focusCheckDelayTimeout)
-					return;
-				if (!hasFocus && !lastFocusedExists) {
+				if (focusCheckDelayTimeout == DateTime.MinValue) {
 					focusCheckDelayTimeout = now.AddMilliseconds (100);
 					return;
 				}
+
+				if (now < focusCheckDelayTimeout)
+					return;
+
 				focusCheckDelayTimeout = DateTime.MinValue;
 				
 				appHasFocus = hasFocus;
@@ -1761,7 +1802,8 @@ namespace MonoDevelop.Components.Commands
 					if (ApplicationFocusOut != null)
 						ApplicationFocusOut (this, EventArgs.Empty);
 				}
-			}
+			} else
+				focusCheckDelayTimeout = DateTime.MinValue;
 		}
 		
 		public void ReportError (object commandId, string message, Exception ex)
@@ -1794,6 +1836,10 @@ namespace MonoDevelop.Components.Commands
 			if (CommandTargetScanFinished != null)
 				CommandTargetScanFinished (this, EventArgs.Empty);
 		}
+
+		internal bool ApplicationHasFocus {
+			get { return appHasFocus; }
+		}
 		
 		/// <summary>
 		/// Raised when there is an exception while executing or updating the status of a command
@@ -1813,12 +1859,12 @@ namespace MonoDevelop.Components.Commands
 		/// <summary>
 		/// Fired when the application gets the focus
 		/// </summary>
-		public event EventHandler ApplicationFocusIn;
+		internal event EventHandler ApplicationFocusIn;
 		
 		/// <summary>
 		/// Fired when the application loses the focus
 		/// </summary>
-		public event EventHandler ApplicationFocusOut;
+		internal event EventHandler ApplicationFocusOut;
 		
 		/// <summary>
 		/// Fired when the command route scan starts
@@ -1834,8 +1880,20 @@ namespace MonoDevelop.Components.Commands
 		/// Fired when a key is pressed
 		/// </summary>
 		public event EventHandler<KeyPressArgs> KeyPressed;
+
+		/// <summary>
+		/// Occurs when active widget (the current command target) changes
+		/// </summary>
+		public event EventHandler<ActiveWidgetEventArgs> ActiveWidgetChanged;
 	}
-	
+
+
+	public class ActiveWidgetEventArgs: EventArgs
+	{
+		public Gtk.Widget OldActiveWidget { get; internal set; }
+		public Gtk.Widget NewActiveWidget { get; internal set; }
+	}
+
 	internal class HandlerTypeInfo
 	{
 		public CommandHandlerInfo[] CommandHandlers;
@@ -1967,13 +2025,25 @@ namespace MonoDevelop.Components.Commands
 		{
 			if (customHandlerChain != null) {
 				info.UpdateHandlerData = Method;
+
+				DateTime t = DateTime.Now;
 				customHandlerChain.CommandUpdate (cmdTarget, info);
+				var time = DateTime.Now - t;
+				if (time.TotalMilliseconds > CommandManager.SlowCommandWarningTime)
+					LoggingService.LogWarning ("Slow command update ({0}ms): Command:{1}, CustomUpdater:{2}", (int)time.TotalMilliseconds, CommandId, customHandlerChain);
 			} else {
 				if (Method == null)
 					throw new InvalidOperationException ("Invalid custom update handler. An implementation of ICommandUpdateHandler was expected.");
 				if (isArray)
 					throw new InvalidOperationException ("Invalid signature for command update handler: " + Method.DeclaringType + "." + Method.Name + "()");
+
+				DateTime t = DateTime.Now;
+
 				Method.Invoke (cmdTarget, new object[] {info} );
+
+				var time = DateTime.Now - t;
+				if (time.TotalMilliseconds > CommandManager.SlowCommandWarningTime)
+					LoggingService.LogWarning ("Slow command update ({0}ms): Command:{1}, Method:{2}", (int)time.TotalMilliseconds, CommandId, Method.DeclaringType + "." + Method.Name);
 			}
 		}
 		
@@ -1987,7 +2057,14 @@ namespace MonoDevelop.Components.Commands
 					throw new InvalidOperationException ("Invalid custom update handler. An implementation of ICommandArrayUpdateHandler was expected.");
 				if (!isArray)
 					throw new InvalidOperationException ("Invalid signature for command update handler: " + Method.DeclaringType + "." + Method.Name + "()");
+
+				DateTime t = DateTime.Now;
+
 				Method.Invoke (cmdTarget, new object[] {info} );
+				
+				var time = DateTime.Now - t;
+				if (time.TotalMilliseconds > CommandManager.SlowCommandWarningTime)
+					LoggingService.LogWarning ("Slow command update ({0}ms): Command:{1}, Method:{2}", (int)time.TotalMilliseconds, CommandId, Method.DeclaringType + "." + Method.Name);
 			}
 		}
 	}
