@@ -35,6 +35,25 @@ namespace MonoDevelop.Components.Commands
 	/// </summary>
 	public interface ICommandTargetVisitor
 	{
+		/// <summary>
+		/// Called when a visit operation is started
+		/// </summary>
+		void Start ();
+
+		/// <summary>
+		/// Called for every object in the hierarchy
+		/// </summary>
+		/// <param name='ob'>
+		/// Visited object
+		/// </param>
+		/// <returns>
+		/// True if the visit operation has to be stopped
+		/// </returns>
 		bool Visit (object ob);
+
+		/// <summary>
+		/// Called when the visit operation ends
+		/// </summary>
+		void End ();
 	}
 }
