@@ -45,9 +45,6 @@ namespace MonoDevelop.Components
 		
 		public Action<Gdk.EventButton> DoPopupMenu { get; set; }
 		
-		//HACK: work around "Bug 2157 - Context menus flaky near left edge of screen" by triggering on ButtonRelease
-		static bool workaroundBug2157 = MonoDevelop.Core.Platform.IsMac;
-		
 		protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
 		{
 			if (!evnt.TriggersContextMenu ()) {
@@ -62,8 +59,7 @@ namespace MonoDevelop.Components
 			}
 			
 			if (DoPopupMenu != null) {
-				if (!workaroundBug2157)
-					DoPopupMenu (evnt);
+				DoPopupMenu (evnt);
 				return true;
 			}
 			
@@ -75,8 +71,6 @@ namespace MonoDevelop.Components
 			bool res = base.OnButtonReleaseEvent (evnt);
 			
 			if (DoPopupMenu != null && evnt.IsContextMenuButton ()) {
-				if (workaroundBug2157)
-					DoPopupMenu (evnt);
 				return true;
 			}
 			
@@ -95,15 +89,15 @@ namespace MonoDevelop.Components
 		bool IsClickedNodeSelected (int x, int y)
 		{
 			Gtk.TreePath path;
-			if (this.GetPathAtPos (x, y, out path))
-				return this.Selection.PathIsSelected (path);
-			else
-				return false;
+			if (GetPathAtPos (x, y, out path))
+				return Selection.PathIsSelected (path);
+
+			return false;
 		}
 		
 		bool MultipleNodesSelected ()
 		{
-			return this.Selection.GetSelectedRows ().Length > 1;
+			return Selection.GetSelectedRows ().Length > 1;
 		}
 	}
 }
\ No newline at end of file
