@@ -28,8 +28,10 @@
 
 using System;
 
+using MonoDevelop.Ide;
 using Gtk;
 using Gdk;
+using Mono.TextEditor.PopupWindow;
 
 namespace MonoDevelop.Components
 {
@@ -120,19 +122,19 @@ namespace MonoDevelop.Components
 //					LimitWidth (fittedWidth);
 //				}
 				
-				Gdk.Rectangle geometry = Screen.GetMonitorGeometry (Screen.GetMonitorAtPoint (x, y));
+				Gdk.Rectangle geometry = DesktopService.GetUsableMonitorGeometry (Screen, Screen.GetMonitorAtPoint (x, y));
 				if (nudgeHorizontal) {
-					if (allocation.Width <= geometry.Width && x + allocation.Width >= geometry.Width - edgeGap)
-						x = geometry.Left + (geometry.Width - allocation.Height - edgeGap);
-					if (x <= geometry.Left)
-						x = geometry.Left;
+					if (allocation.Width <= geometry.Width && x + allocation.Width >= geometry.Left + geometry.Width - edgeGap)
+						x = geometry.Left + (geometry.Width - allocation.Width - edgeGap);
+					if (x <= geometry.Left + edgeGap)
+						x = geometry.Left + edgeGap;
 				}
 				
 				if (nudgeVertical) {
-					if (allocation.Height <= geometry.Height && y + allocation.Height >= geometry.Height - edgeGap)
+					if (allocation.Height <= geometry.Height && y + allocation.Height >= geometry.Top + geometry.Height - edgeGap)
 						y = geometry.Top + (geometry.Height - allocation.Height - edgeGap);
-					if (y <= geometry.Top)
-						y = geometry.Top;
+					if (y <= geometry.Top + edgeGap)
+						y = geometry.Top + edgeGap;
 				}
 				
 				if (y != oldY || x != oldX)
