@@ -36,7 +36,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 		
 		public static bool IsVisible {
 			get {
-				return wnd != null /*&& wnd.Visible*/;
+				return wnd != null && wnd.Visible;
 			}
 		}
 		
@@ -104,6 +104,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 					if (ForceSuggestionMode)
 						wnd.AutoSelect = false;
 					wnd.Show ();
+					DesktopService.RemoveWindowShadow (wnd);
 					OnWindowShown (EventArgs.Empty);
 					return true;
 				} catch (Exception ex) {
@@ -137,15 +138,43 @@ namespace MonoDevelop.Ide.CodeCompletion
 		{
 			if (!IsVisible)
 				return false;
+			if (keyChar != '\0') {
+				wnd.EndOffset = wnd.StartOffset + wnd.CurrentPartialWord.Length + 1;
+			}
 			return wnd.PreProcessKeyEvent (key, keyChar, modifier);
 		}
-		
+
+		public static void UpdateCursorPosition ()
+		{
+			if (!IsVisible)
+				return;
+
+			var caretOffset = wnd.CompletionWidget.CaretOffset;
+			if (caretOffset < wnd.StartOffset || caretOffset > wnd.EndOffset)
+				HideWindow ();
+		}
+
+		public static void UpdateWordSelection (string text)
+		{
+			if (IsVisible) {
+				wnd.List.CompletionString = text;
+				wnd.UpdateWordSelection ();
+			}
+		}
+
 		public static void PostProcessKeyEvent (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
 		{
 			if (!IsVisible)
 				return;
 			wnd.PostProcessKeyEvent (key, keyChar, modifier);
 		}
+
+		public static void RepositionWindow ()
+		{
+			if (!IsVisible)
+				return;
+			wnd.RepositionWindow ();
+		}
 		
 		public static void HideWindow ()
 		{
@@ -154,9 +183,9 @@ namespace MonoDevelop.Ide.CodeCompletion
 			ParameterInformationWindowManager.UpdateWindow (wnd.Extension, wnd.CompletionWidget);
 			if (wnd.Extension != null)
 				wnd.Extension.document.Editor.FixVirtualIndentation ();
-//			wnd.HideWindow ();
-//			OnWindowClosed (EventArgs.Empty);
-			DestroyWindow ();
+			wnd.HideWindow ();
+			OnWindowClosed (EventArgs.Empty);
+			//DestroyWindow ();
 		}
 		
 		
