@@ -34,7 +34,12 @@ using System.Linq;
 using System.Text;
 using System.Collections.Generic;
 using MonoDevelop.Core.Text;
+using ICSharpCode.NRefactory.Completion;
 using Mono.TextEditor;
+using MonoDevelop.Ide.Gui.Content;
+using MonoDevelop.Components;
+using Mono.TextEditor.Highlighting;
+using MonoDevelop.Core;
 
 namespace MonoDevelop.Ide.CodeCompletion
 {
@@ -48,50 +53,47 @@ namespace MonoDevelop.Ide.CodeCompletion
 		Complete = 8
 	}
 
-	public class ListWindow : Gtk.Window
+	public class ListWindow : PopoverWindow
 	{
-		internal VScrollbar scrollbar;
+		const int WindowWidth = 300;
+
 		ListWidget list;
 		Widget footer;
-		VBox vbox;
-
-		StringBuilder word = new StringBuilder();
-		int curPos;
+		protected VBox vbox;
+		
+		public CompletionTextEditorExtension Extension {
+			get;
+			set;
+		}		
 		
 		public List<int> FilteredItems {
 			get {
 				return list.filteredItems;
 			}
 		}
+
+		internal ScrolledWindow scrollbar;
 		
-		public ListWindow () : base(Gtk.WindowType.Popup)
+		public ListWindow (Gtk.WindowType type) : base(type)
 		{
 			vbox = new VBox ();
-			HBox box = new HBox ();
 			list = new ListWidget (this);
 			list.SelectionChanged += new EventHandler (OnSelectionChanged);
 			list.ScrollEvent += new ScrollEventHandler (OnScrolled);
-			
-			box.PackStart (list, true, true, 0);
-			this.BorderWidth = 1;
 
-			scrollbar = new VScrollbar (null);
-			scrollbar.ValueChanged += new EventHandler (OnScrollChanged);
-			box.PackStart (scrollbar, false, false, 0);
+			scrollbar = new MonoDevelop.Components.CompactScrolledWindow ();
+			scrollbar.Child = list;
 			list.ButtonPressEvent += delegate(object o, ButtonPressEventArgs args) {
 				if (args.Event.Button == 1 && args.Event.Type == Gdk.EventType.TwoButtonPress)
 					DoubleClick ();
 			};
-			list.WordsFiltered += delegate {
-				UpdateScrollBar ();
-			};
-			list.SizeAllocated += delegate {
-				UpdateScrollBar ();
-			};
-			vbox.PackStart (box, true, true, 0);
-			Add (vbox);
+			vbox.PackEnd (scrollbar, true, true, 0);
+			ContentBox.Add (vbox);
 			this.AutoSelect = true;
 			this.TypeHint = WindowTypeHint.Menu;
+			Theme.CornerRadius = 4;
+			var style = SyntaxModeService.GetColorStyle (IdeApp.Preferences.ColorScheme);
+			Theme.SetFlatColor (style.CompletionText.Background);
 		}
 
 		protected virtual void DoubleClick ()
@@ -114,16 +116,15 @@ namespace MonoDevelop.Ide.CodeCompletion
 			}
 		}
 
-		protected void Reset (bool clearWord)
+		/// <summary>
+		/// This method is used to set the completion window to it's inital state.
+		/// This is required for re-using the window object.
+		/// </summary>
+		protected virtual void ResetState ()
 		{
-			if (clearWord) {
-				word = new StringBuilder ();
-				curPos = 0;
-			}
-
-			list.Reset ();
-			if (DataProvider != null)
-				ResetSizes ();
+			HideWhenWordDeleted = false;
+			lastCommitCharEndoffset = -1;
+			list.ResetState ();
 		}
 		
 		protected int curXPos, curYPos;
@@ -132,47 +133,35 @@ namespace MonoDevelop.Ide.CodeCompletion
 		{
 			list.CompletionString = PartialWord;
 			
-			if (IsRealized && !Visible)
+			var allocWidth = Allocation.Width;
+			if (IsRealized && !Visible) {
+				allocWidth = list.WidthRequest = WindowWidth;
 				Show ();
-			
-			int width = list.WidthRequest;
-			int height = list.HeightRequest + (footer != null ? footer.Allocation.Height : 0);
-			
+			}
+
+			int width = Math.Max (allocWidth, list.WidthRequest + Theme.CornerRadius * 2);
+			int height = Math.Max (Allocation.Height, list.HeightRequest + 2 + (footer != null ? footer.Allocation.Height : 0) + Theme.CornerRadius * 2);
 			SetSizeRequest (width, height);
 			if (IsRealized) 
 				Resize (width, height);
 		}
 
-		void UpdateScrollBar ()
-		{
-			double pageSize = Math.Max (0, list.VisibleRows);
-			double upper = Math.Max (0, list.filteredItems.Count - 1);
-			scrollbar.Adjustment.SetBounds (0, upper, 1, pageSize, pageSize);
-			if (pageSize >= upper) {
-				this.scrollbar.Value = -1;
-				this.scrollbar.Visible = false;
-			} else {
-				this.scrollbar.Value = list.Page;
-				this.scrollbar.Visible = true;
-			}
-		}
 
-
-		public IListDataProvider DataProvider {
+		internal IListDataProvider DataProvider {
 			get;
 			set;
 		}
 
 		public string CurrentCompletionText {
 			get {
-				if (list.SelectionIndex != -1 && list.AutoSelect)
-					return DataProvider.GetCompletionText (list.SelectionIndex);
+				if (list.SelectedItem != -1 && list.AutoSelect)
+					return DataProvider.GetCompletionText (list.SelectedItem);
 				return null;
 			}
 		}
 
-		public int SelectionIndex {
-			get { return list.SelectionIndex; }
+		public int SelectedItem {
+			get { return list.SelectedItem; }
 		}
 		
 		public bool AutoSelect {
@@ -189,6 +178,11 @@ namespace MonoDevelop.Ide.CodeCompletion
 			set { list.AutoCompleteEmptyMatch = value; }
 		}
 		
+		public bool AutoCompleteEmptyMatchOnCurlyBrace {
+			get { return list.AutoCompleteEmptyMatchOnCurlyBrace; }
+			set { list.AutoCompleteEmptyMatchOnCurlyBrace = value; }
+		}
+		
 		public string DefaultCompletionString {
 			get {
 				return list.DefaultCompletionString;
@@ -198,18 +192,48 @@ namespace MonoDevelop.Ide.CodeCompletion
 			}
 		}
 		
-		public string PartialWord {
-			get { return word.ToString (); }
-			set {
-				string newword = value;
-				if (newword.Trim ().Length == 0)
-					return;
-				if (word.ToString () != newword) {
-					word = new StringBuilder (newword);
-					curPos = newword.Length;
-					UpdateWordSelection ();
-				}
+		public bool CloseOnSquareBrackets {
+			get {
+				return list.CloseOnSquareBrackets;
 			}
+			set {
+				list.CloseOnSquareBrackets = value;
+			}
+		}
+		
+		int startOffset;
+		internal int StartOffset {
+			get {
+				return startOffset;
+			}
+			set {
+				startOffset = value;
+				EndOffset = value + 1;
+			}
+		}
+
+		public int EndOffset {
+			get;
+			set;
+		}
+
+		public ICompletionWidget CompletionWidget {
+			get {
+				return list.CompletionWidget;
+			}
+			set {
+				list.CompletionWidget = value;
+			}
+		}
+		
+		int lastCommitCharEndoffset = -1;
+		public virtual string PartialWord {
+			get {
+				if (CompletionWidget == null)
+					return "";
+				return CompletionWidget.GetText (StartOffset, Math.Max (StartOffset, lastCommitCharEndoffset > 0 ? lastCommitCharEndoffset : CompletionWidget.CaretOffset)); 
+			}
+			
 		}
 	
 		public string CurrentPartialWord {
@@ -220,12 +244,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 
 		public bool IsUniqueMatch {
 			get {
-/*				int pos = list.Selection + 1;
-				if (DataProvider.ItemCount > pos && 
-					DataProvider.GetText (pos).ToLower ().StartsWith (CurrentPartialWord.ToLower ()) || 
-					!(DataProvider.GetText (list.Selection).ToLower ().StartsWith (CurrentPartialWord.ToLower ())))
-					return false;
-				*/
+				list.FilterWords ();
 				return list.filteredItems.Count == 1;
 			}
 		}
@@ -234,11 +253,6 @@ namespace MonoDevelop.Ide.CodeCompletion
 			get { return list; }
 		}
 
-		public bool CompleteWithSpaceOrPunctuation {
-			get;
-			set;
-		}
-		
 		/// <summary>
 		/// Gets or sets a value indicating that shift was pressed during enter.
 		/// </summary>
@@ -250,18 +264,70 @@ namespace MonoDevelop.Ide.CodeCompletion
 			private set;
 		}
 
-		public KeyActions ProcessKey (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
+		public KeyActions PostProcessKey (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
+		{
+			if (StartOffset > CompletionWidget.CaretOffset)
+				return KeyActions.CloseWindow | KeyActions.Process;
+
+			if (HideWhenWordDeleted && StartOffset >= CompletionWidget.CaretOffset)
+				return KeyActions.CloseWindow | KeyActions.Process;
+			switch (key) {
+			case Gdk.Key.BackSpace:
+				ResetSizes ();
+				UpdateWordSelection ();
+				return KeyActions.Process;
+			}
+			
+			const string commitChars = " <>()[]{}=+-*/%~&|!";
+			if (keyChar == '[' && CloseOnSquareBrackets)
+				return KeyActions.Process | KeyActions.CloseWindow;
+			
+			if (char.IsLetterOrDigit (keyChar) || keyChar == '_') {
+				ResetSizes ();
+				UpdateWordSelection ();
+				return KeyActions.Process;
+			}
+			
+			if (char.IsPunctuation (keyChar) || commitChars.Contains (keyChar)) {
+				bool hasMismatches;
+				var curword = PartialWord;
+				int match = FindMatchedEntry (curword, out hasMismatches);
+				if (match >= 0 && System.Char.IsPunctuation (keyChar)) {
+					string text = DataProvider.GetCompletionText (FilteredItems [match]);
+					if (!text.ToUpper ().StartsWith (curword.ToUpper (), StringComparison.Ordinal))
+						match = -1;	 
+				}
+				if (match >= 0 && !hasMismatches && keyChar != '<') {
+					ResetSizes ();
+					UpdateWordSelection ();
+					return KeyActions.Process;
+				}
+				if (keyChar == '.')
+					list.AutoSelect = list.AutoCompleteEmptyMatch = true;
+				lastCommitCharEndoffset = CompletionWidget.CaretOffset - 1;
+				if (list.SelectionEnabled) {
+					if (keyChar == '{' && !list.AutoCompleteEmptyMatchOnCurlyBrace && string.IsNullOrEmpty (list.CompletionString))
+					    return KeyActions.CloseWindow | KeyActions.Process;
+					return KeyActions.Complete | KeyActions.Process | KeyActions.CloseWindow;
+				}
+			    return KeyActions.CloseWindow | KeyActions.Process;
+			}
+			
+			return KeyActions.Process;
+		}
+		
+		public KeyActions PreProcessKey (Gdk.Key key, char keyChar, Gdk.ModifierType modifier)
 		{
 			switch (key) {
 			case Gdk.Key.Home:
 				if ((modifier & ModifierType.ShiftMask) == ModifierType.ShiftMask)
 					return KeyActions.Process;
-				List.Selection = 0;
+				List.SelectionFilterIndex = 0;
 				return KeyActions.Ignore;
 			case Gdk.Key.End:
 				if ((modifier & ModifierType.ShiftMask) == ModifierType.ShiftMask)
 					return KeyActions.Process;
-				List.Selection = List.filteredItems.Count - 1;
+				List.SelectionFilterIndex = List.filteredItems.Count - 1;
 				return KeyActions.Ignore;
 				
 			case Gdk.Key.Up:
@@ -275,7 +341,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 					List.MoveToCategory (-1);
 					return KeyActions.Ignore;
 				}
-				if (SelectionEnabled && list.filteredItems.Count < 2)
+				if (SelectionEnabled && list.filteredItems.Count < 1)
 					return KeyActions.CloseWindow | KeyActions.Process;
 				if (!SelectionEnabled /*&& !CompletionWindowManager.ForceSuggestionMode*/) {
 					AutoCompleteEmptyMatch = AutoSelect = true;
@@ -284,6 +350,19 @@ namespace MonoDevelop.Ide.CodeCompletion
 				}
 				return KeyActions.Ignore;
 
+			case Gdk.Key.Tab:
+				//tab always completes current item even if selection is disabled
+				if (!AutoSelect)
+					AutoSelect = true;
+				goto case Gdk.Key.Return;
+
+			case Gdk.Key.Return:
+			case Gdk.Key.ISO_Enter:
+			case Gdk.Key.Key_3270_Enter:
+			case Gdk.Key.KP_Enter:
+				lastCommitCharEndoffset = CompletionWidget.CaretOffset;
+				WasShiftPressed = (modifier & ModifierType.ShiftMask) == ModifierType.ShiftMask;
+				return KeyActions.Complete | KeyActions.Ignore | KeyActions.CloseWindow;
 			case Gdk.Key.Down:
 				if ((modifier & Gdk.ModifierType.ShiftMask) == Gdk.ModifierType.ShiftMask) {
 					if (!SelectionEnabled /*&& !CompletionWindowManager.ForceSuggestionMode*/)
@@ -295,7 +374,7 @@ namespace MonoDevelop.Ide.CodeCompletion
 					List.MoveToCategory (1);
 					return KeyActions.Ignore;
 				}
-				if (SelectionEnabled && list.filteredItems.Count < 2)
+				if (SelectionEnabled && list.filteredItems.Count < 1)
 					return KeyActions.CloseWindow | KeyActions.Process;
 				
 				if (!SelectionEnabled /*&& !CompletionWindowManager.ForceSuggestionMode*/) {
@@ -310,7 +389,8 @@ namespace MonoDevelop.Ide.CodeCompletion
 					return KeyActions.Process;
 				if (list.filteredItems.Count < 2)
 					return KeyActions.CloseWindow | KeyActions.Process;
-				list.MoveCursor (-(list.VisibleRows - 1));
+				scrollbar.Vadjustment.Value = Math.Max (0, scrollbar.Vadjustment.Value - scrollbar.Vadjustment.PageSize);
+				list.MoveCursor (-8);
 				return KeyActions.Ignore;
 
 			case Gdk.Key.Page_Down:
@@ -318,7 +398,8 @@ namespace MonoDevelop.Ide.CodeCompletion
 					return KeyActions.Process;
 				if (list.filteredItems.Count < 2)
 					return KeyActions.CloseWindow | KeyActions.Process;
-				list.MoveCursor (list.VisibleRows - 1);
+				scrollbar.Vadjustment.Value = Math.Max (0, Math.Min (scrollbar.Vadjustment.Upper - scrollbar.Vadjustment.PageSize, scrollbar.Vadjustment.Value + scrollbar.Vadjustment.PageSize));
+				list.MoveCursor (8);
 				return KeyActions.Ignore;
 
 			case Gdk.Key.Left:
@@ -326,17 +407,6 @@ namespace MonoDevelop.Ide.CodeCompletion
 				//curPos--;
 				return KeyActions.Process;
 
-			case Gdk.Key.BackSpace:
-				if (curPos == 0 || (modifier & Gdk.ModifierType.ControlMask) != 0)
-					return KeyActions.CloseWindow | KeyActions.Process;
-				curPos--;
-				word.Remove (curPos, 1);
-				ResetSizes ();
-				UpdateWordSelection ();
-				if (HideWhenWordDeleted && word.Length == 0)
-					return KeyActions.CloseWindow | KeyActions.Process;
-				return KeyActions.Process;
-
 			case Gdk.Key.Right:
 				//if (curPos == word.Length) return KeyActions.CloseWindow | KeyActions.Process;
 				//curPos++;
@@ -347,22 +417,6 @@ namespace MonoDevelop.Ide.CodeCompletion
 			case Gdk.Key.Scroll_Lock:
 				return KeyActions.Ignore;
 
-			case Gdk.Key.Tab:
-				//tab always completes current item even if selection is disabled
-				if (!AutoSelect)
-					AutoSelect = true;
-				goto case Gdk.Key.Return;
-
-			case Gdk.Key.Return:
-			case Gdk.Key.ISO_Enter:
-			case Gdk.Key.Key_3270_Enter:
-			case Gdk.Key.KP_Enter:
-				WasShiftPressed = (modifier & ModifierType.ShiftMask) == ModifierType.ShiftMask;
-				return (!list.AutoSelect ? KeyActions.Process : (KeyActions.Complete | KeyActions.Ignore)) | KeyActions.CloseWindow;
-
-			case Gdk.Key.Escape:
-				return KeyActions.CloseWindow | KeyActions.Ignore;
-
 			case Gdk.Key.Control_L:
 			case Gdk.Key.Control_R:
 			case Gdk.Key.Alt_L:
@@ -378,8 +432,20 @@ namespace MonoDevelop.Ide.CodeCompletion
 
 			if (keyChar == ' ' && (modifier & ModifierType.ShiftMask) == ModifierType.ShiftMask)
 				return KeyActions.CloseWindow | KeyActions.Process;
-			
-			//don't input letters/punctuation etc when non-shift modifiers are active
+
+			// special case end with punctuation like 'param:' -> don't input double punctuation, otherwise we would end up with 'param::'
+			if (char.IsPunctuation (keyChar) && keyChar != '_') {
+				foreach (var item in FilteredItems) {
+					if (DataProvider.GetText (item).EndsWith (keyChar.ToString (), StringComparison.Ordinal)) {
+						list.SelectedItem = item;
+						return KeyActions.Complete | KeyActions.CloseWindow | KeyActions.Ignore;
+					}
+				}
+			}
+
+
+
+	/*		//don't input letters/punctuation etc when non-shift modifiers are active
 			bool nonShiftModifierActive = ((Gdk.ModifierType.ControlMask | Gdk.ModifierType.MetaMask
 				| Gdk.ModifierType.Mod1Mask | Gdk.ModifierType.SuperMask)
 				& modifier) != 0;
@@ -387,40 +453,10 @@ namespace MonoDevelop.Ide.CodeCompletion
 				if (modifier.HasFlag (Gdk.ModifierType.ControlMask) && char.IsLetterOrDigit ((char)key))
 					return KeyActions.Process | KeyActions.CloseWindow;
 				return KeyActions.Ignore;
-			}
-			const string commitChars = " <>()[]{}=+-*/%~&|!";
-			if (System.Char.IsLetterOrDigit (keyChar) || keyChar == '_') {
-				word.Insert (curPos, keyChar);
-				ResetSizes ();
-				UpdateWordSelection ();
-				curPos++;
-				return KeyActions.Process;
-			} else if (System.Char.IsPunctuation (keyChar) || commitChars.Contains (keyChar)) {
-				//punctuation is only accepted if it actually matches an item in the list
-				word.Insert (curPos, keyChar);
+			}*/
+			
 
-				bool hasMismatches;
-				int match = FindMatchedEntry (CurrentPartialWord, out hasMismatches);
-				if (match >= 0 && System.Char.IsPunctuation (keyChar)) {
-					string text = DataProvider.GetCompletionText (FilteredItems [match]);
-					if (!text.ToUpper ().StartsWith (word.ToString ().ToUpper ()))
-						match =-1;	 
-				}
-				if (match >= 0 && !hasMismatches && keyChar != '<') {
-					ResetSizes ();
-					UpdateWordSelection ();
-					curPos++;
-					return KeyActions.Process;
-				} else {
-					word.Remove (curPos, 1);
-				}
-				
-				if (CompleteWithSpaceOrPunctuation && list.SelectionEnabled)
-					return KeyActions.Complete | KeyActions.Process | KeyActions.CloseWindow;
-				return KeyActions.CloseWindow | KeyActions.Process;
-			}
-
-			return KeyActions.CloseWindow | KeyActions.Process;
+			return KeyActions.Process;
 		}
 		
 		protected bool HideWhenWordDeleted {
@@ -494,15 +530,13 @@ namespace MonoDevelop.Ide.CodeCompletion
 			
 			int idx = -1;
 			var matcher = CompletionMatcher.CreateCompletionMatcher (partialWord);
-			
 			string bestWord = null;
 			int bestRank = int.MinValue;
 			int bestIndex = 0;
-			
 			if (!string.IsNullOrEmpty (partialWord)) {
 				for (int i = 0; i < list.filteredItems.Count; i++) {
 					int index = list.filteredItems[i];
-					string text = DataProvider.GetCompletionText (index);
+					string text = DataProvider.GetText (index);
 					int rank;
 					if (!matcher.CalcMatchRank (text, out rank))
 						continue;
@@ -513,7 +547,6 @@ namespace MonoDevelop.Ide.CodeCompletion
 					}
 				}
 			}
-			
 			if (bestWord != null) {
 				idx = bestIndex;
 				hasMismatches = false;
@@ -566,13 +599,13 @@ namespace MonoDevelop.Ide.CodeCompletion
 		void SelectEntry (int n)
 		{
 			if (n >= 0)
-				list.Selection = n;
+				list.SelectionFilterIndex = n;
 		}
 
 		public virtual void SelectEntry (string s)
 		{
-			list.FilterWords ();
-			/* // disable this, because we select now the last selected entry by default (word history mode)
+			/*list.FilterWords ();
+			 // disable this, because we select now the last selected entry by default (word history mode)
 			//when the list is empty, disable the selection or users get annoyed by it accepting
 			//the top entry automatically
 			if (string.IsNullOrEmpty (s)) {
@@ -583,21 +616,16 @@ namespace MonoDevelop.Ide.CodeCompletion
 			bool hasMismatches;
 			
 			int matchedIndex = FindMatchedEntry (s, out hasMismatches);
-			ResetSizes ();
+//			ResetSizes ();
 			SelectEntry (matchedIndex);
 		}
 
-		void OnScrollChanged (object o, EventArgs args)
-		{
-			list.Page = (int)scrollbar.Value;
-		}
-
 		void OnScrolled (object o, ScrollEventArgs args)
 		{
 			if (!scrollbar.Visible)
 				return;
 			
-			var adj = scrollbar.Adjustment;
+			var adj = scrollbar.Vadjustment;
 			var alloc = Allocation;
 			
 			//This widget is a special case because it's always aligned to items as it scrolls.
@@ -621,30 +649,29 @@ namespace MonoDevelop.Ide.CodeCompletion
 
 		void OnSelectionChanged (object o, EventArgs args)
 		{
-			scrollbar.Value = list.Page;
 			OnSelectionChanged ();
 		}
 
 		protected virtual void OnSelectionChanged ()
 		{
 		}
-		
+		/*
 		protected override bool OnExposeEvent (Gdk.EventExpose args)
 		{
 			base.OnExposeEvent (args);
-			
+
 			int winWidth, winHeight;
 			this.GetSize (out winWidth, out winHeight);
 			this.GdkWindow.DrawRectangle (this.Style.ForegroundGC (StateType.Insensitive), false, 0, 0, winWidth - 1, winHeight - 1);
 			return true;
-		}
+		}*/
 		
 		public int TextOffset {
-			get { return list.TextOffset + (int)this.BorderWidth; }
+			get { return list.TextOffset + (int)Theme.CornerRadius; }
 		}
 	}
 
-	public interface IListDataProvider
+	interface IListDataProvider
 	{
 		int ItemCount { get; }
 		string GetText (int n);
@@ -652,8 +679,10 @@ namespace MonoDevelop.Ide.CodeCompletion
 		CompletionCategory GetCompletionCategory (int n);
 		bool HasMarkup (int n);
 		string GetCompletionText (int n);
-		string GetDescription (int n);
+		string GetDescription (int n, bool isSelected);
+		string GetRightSideDescription (int n, bool isSelected);
 		Gdk.Pixbuf GetIcon (int n);
+		int CompareTo (int n, int m);
 	}
 }
 
