@@ -26,19 +26,12 @@
 //
 
 
-using System;
-using MonoDevelop.Core.Gui.Dialogs;
-using MonoDevelop.Core;
-using Mono.Addins;
 using MonoDevelop.Components.Commands;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Projects;
-using MonoDevelop.Ide.Gui.Dialogs;
-using MonoDevelop.Ide.Gui.Content;
-using MonoDevelop.Core.Gui;
-using System.IO;
-using Gtk;
+using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
+using MonoDevelop.Ide.Gui;
+using System;
+using MonoDevelop.Ide.Updater;
 
 namespace MonoDevelop.Ide.Commands
 {
@@ -53,7 +46,7 @@ namespace MonoDevelop.Ide.Commands
 	{
 		protected override void Run ()
 		{
-			AddinUpdateHandler.ShowManager ();
+			AddinsUpdateHandler.ShowManager ();
 		}
 	}
 
@@ -77,7 +70,7 @@ namespace MonoDevelop.Ide.Commands
 		{
 			ExternalTools.ExternalTool tool = (ExternalTools.ExternalTool)dataItem;
 			
-			string argumentsTool = StringParserService.Parse (tool.Arguments);
+			string argumentsTool = StringParserService.Parse (tool.Arguments, IdeApp.Workbench.GetStringTagModel ());
 			
 			//Save current file checkbox
 			if (tool.SaveCurrentFile && IdeApp.Workbench.ActiveDocument != null)
@@ -86,7 +79,7 @@ namespace MonoDevelop.Ide.Commands
 			if (tool.PromptForArguments) {
 				string customerArguments = MessageService.GetTextResponse (GettextCatalog.GetString ("Enter any arguments you want to use while launching tool, {0}:", tool.MenuCommand), GettextCatalog.GetString ("Command Arguments for {0}", tool.MenuCommand), "");
 				if (customerArguments != String.Empty)
-					argumentsTool = StringParserService.Parse (customerArguments);
+					argumentsTool = StringParserService.Parse (customerArguments, IdeApp.Workbench.GetStringTagModel ());
 			}
 
 			DispatchService.BackgroundDispatch (delegate {
@@ -96,8 +89,8 @@ namespace MonoDevelop.Ide.Commands
 
 		void RunExternalTool (ExternalTools.ExternalTool tool, string argumentsTool)
 		{
-			string commandTool = StringParserService.Parse (tool.Command);
-			string initialDirectoryTool = StringParserService.Parse (tool.InitialDirectory);
+			string commandTool = StringParserService.Parse (tool.Command, IdeApp.Workbench.GetStringTagModel ());
+			string initialDirectoryTool = StringParserService.Parse (tool.InitialDirectory, IdeApp.Workbench.GetStringTagModel ());
 
 			//Execute tool
 			IProgressMonitor progressMonitor = IdeApp.Workbench.ProgressMonitors.GetRunProgressMonitor ();
@@ -137,9 +130,7 @@ namespace MonoDevelop.Ide.Commands
 	{
 		protected override void Run ()
 		{
-			InstrumentationViewerDialog dlg = new InstrumentationViewerDialog ();
-			dlg.TransientFor = IdeApp.Workbench.RootWindow;
-			dlg.Show ();
+			MonoDevelop.Core.Instrumentation.InstrumentationService.StartMonitor ();
 		}
 		
 		protected override void Update (CommandInfo info)
