@@ -80,8 +80,14 @@ namespace MonoDevelop.Ide.CustomTools
 		static ISingleFileCustomTool GetGenerator (ProjectFile file)
 		{
 			CustomToolExtensionNode node;
-			if (!string.IsNullOrEmpty (file.Generator) && nodes.TryGetValue (file.Generator, out node))
-				return node.Tool;
+			if (!string.IsNullOrEmpty (file.Generator) && nodes.TryGetValue (file.Generator, out node)) {
+				try {
+					return node.Tool;
+				} catch (Exception ex) {
+					LoggingService.LogError ("Error loading generator '" + file.Generator + "'", ex);
+					nodes.Remove (file.Generator);
+				}
+			}
 			return null;
 		}
 		
@@ -214,7 +220,7 @@ namespace MonoDevelop.Ide.CustomTools
 			}
 		}
 		
-		public static void HandleRename (ProjectFileRenamedEventArgs e)
+		static void HandleRename (ProjectFileRenamedEventArgs e)
 		{
 			foreach (ProjectFileEventInfo args in e) {
 				var file = args.ProjectFile;
@@ -223,6 +229,17 @@ namespace MonoDevelop.Ide.CustomTools
 					continue;
 			}
 		}
+
+		public static string GetFileNamespace (ProjectFile file, string outputFile)
+		{
+			string ns = file.CustomToolNamespace;
+			if (string.IsNullOrEmpty (ns) && !string.IsNullOrEmpty (outputFile)) {
+				var dnp = file.Project as DotNetProject;
+				if (dnp != null)
+					ns = dnp.GetDefaultNamespace (outputFile);
+			}
+			return ns;
+		}
 	}
 }
 
