@@ -28,6 +28,7 @@
 
 using System;
 using System.IO;
+using System.Linq;
 using System.Collections;
 using System.Collections.Generic;
 using System.Diagnostics;
@@ -38,6 +39,7 @@ using MonoDevelop.Core;
 using Mono.Unix;
 using MonoDevelop.Ide.Extensions;
 using MonoDevelop.Core.Execution;
+using MonoDevelop.Components.MainToolbar;
 
 
 namespace MonoDevelop.Ide.Desktop
@@ -74,23 +76,31 @@ namespace MonoDevelop.Ide.Desktop
 			Process.Start (url);
 		}
 
+		/// <summary>
+		/// Loads the XWT toolkit backend for the native toolkit (Cocoa on Mac, WPF on Windows)
+		/// </summary>
+		/// <returns>The native toolkit.</returns>
+		public virtual Xwt.Toolkit LoadNativeToolkit ()
+		{
+			return Xwt.Toolkit.CurrentEngine;
+		}
+
 		public string GetMimeTypeForUri (string uri)
 		{
 			if (!String.IsNullOrEmpty (uri)) {
-// Creating file infos is expensive, should be avoided 
-//				FileInfo file = new FileInfo (uri);
-//				MimeTypeNode mt = FindMimeTypeForFile (file.Name);
 				MimeTypeNode mt = FindMimeTypeForFile (uri);
 				if (mt != null)
 					return mt.Id;
 			}
-			return OnGetMimeTypeForUri (uri) ?? "text/plain";
+			return OnGetMimeTypeForUri (uri) ?? "application/octet-stream";
 		}
-		
+
 		public string GetMimeTypeDescription (string mimeType)
 		{
 			if (mimeType == "text/plain")
 				return GettextCatalog.GetString ("Text file");
+			if (mimeType == "application/octet-stream")
+				return GettextCatalog.GetString ("Unknown");
 			MimeTypeNode mt = FindMimeType (mimeType);
 			if (mt != null && mt.Description != null)
 				return mt.Description;
@@ -115,14 +125,14 @@ namespace MonoDevelop.Ide.Desktop
 		{
 			yield return mimeType;
 			
-			while (mimeType != null && mimeType != "text/plain") {
+			while (mimeType != null && mimeType != "text/plain" && mimeType != "application/octet-stream") {
 				MimeTypeNode mt = FindMimeType (mimeType);
 				if (mt != null && !string.IsNullOrEmpty (mt.BaseType))
 					mimeType = mt.BaseType;
 				else {
-					if (mimeType.EndsWith ("+xml"))
+					if (mimeType.EndsWith ("+xml", StringComparison.Ordinal))
 						mimeType = "application/xml";
-					else if (mimeType.StartsWith ("text") || OnGetMimeTypeIsText (mimeType))
+					else if (mimeType.StartsWith ("text/", StringComparison.Ordinal) || OnGetMimeTypeIsText (mimeType))
 						mimeType = "text/plain";
 					else
 						break;
@@ -222,10 +232,32 @@ namespace MonoDevelop.Ide.Desktop
 			else
 				return OnGetIconForType (type);
 		}
-		
+
+		static List<MimeTypeNode> mimeTypeNodes = new List<MimeTypeNode> ();
+		static PlatformService ()
+		{
+			if (AddinManager.IsInitialized) {
+				AddinManager.AddExtensionNodeHandler ("/MonoDevelop/Core/MimeTypes", delegate (object sender, ExtensionNodeEventArgs args) {
+					var newList = new List<MimeTypeNode> (mimeTypeNodes);
+					var mimeTypeNode = (MimeTypeNode)args.ExtensionNode;
+					switch (args.Change) {
+					case ExtensionChange.Add:
+						// initialize child nodes.
+						mimeTypeNode.ChildNodes.GetEnumerator ();
+						newList.Add (mimeTypeNode);
+						break;
+					case ExtensionChange.Remove:
+						newList.Remove (mimeTypeNode);
+						break;
+					}
+					mimeTypeNodes = newList;
+				});
+			}
+		}
+
 		MimeTypeNode FindMimeTypeForFile (string fileName)
 		{
-			foreach (MimeTypeNode mt in AddinManager.GetExtensionNodes ("/MonoDevelop/Core/MimeTypes")) {
+			foreach (MimeTypeNode mt in mimeTypeNodes) {
 				if (mt.SupportsFile (fileName))
 					return mt;
 			}
@@ -234,13 +266,13 @@ namespace MonoDevelop.Ide.Desktop
 		
 		MimeTypeNode FindMimeType (string type)
 		{
-			foreach (MimeTypeNode mt in AddinManager.GetExtensionNodes ("/MonoDevelop/Core/MimeTypes")) {
+			foreach (MimeTypeNode mt in mimeTypeNodes) {
 				if (mt.Id == type)
 					return mt;
 			}
 			return null;
 		}
-		
+
 		protected virtual string OnGetMimeTypeForUri (string uri)
 		{
 			return null;
@@ -285,7 +317,8 @@ namespace MonoDevelop.Ide.Desktop
 			return null;
 		}
 		
-		public virtual bool SetGlobalMenu (MonoDevelop.Components.Commands.CommandManager commandManager, string commandMenuAddinPath)
+		public virtual bool SetGlobalMenu (MonoDevelop.Components.Commands.CommandManager commandManager,
+			string commandMenuAddinPath, string appMenuAddinPath)
 		{
 			return false;
 		}
@@ -347,6 +380,7 @@ namespace MonoDevelop.Ide.Desktop
 		{
 			return new string[0];
 		}
+
 		
 		/// <summary>
 		/// Starts the installer.
@@ -381,5 +415,40 @@ namespace MonoDevelop.Ide.Desktop
 		{
 			window.Present ();
 		}
+
+		internal virtual void RemoveWindowShadow (Gtk.Window window)
+		{
+		}
+
+		internal virtual void SetMainWindowDecorations (Gtk.Window window)
+		{
+		}
+
+		internal virtual MainToolbar CreateMainToolbar (Gtk.Window window)
+		{
+			return new MainToolbar ();
+		}
+
+		public virtual bool GetIsFullscreen (Gtk.Window window)
+		{
+			return ((bool?) window.Data ["isFullScreen"]) ?? false;
+		}
+
+		public virtual bool IsModalDialogRunning ()
+		{
+			var windows = Gtk.Window.ListToplevels ();
+			return windows.Any (w => w.Modal && w.Visible);
+		}
+
+		public virtual void SetIsFullscreen (Gtk.Window window, bool isFullscreen)
+		{
+			window.Data ["isFullScreen"] = isFullscreen;
+			if (isFullscreen) {
+				window.Fullscreen ();
+			} else {
+				window.Unfullscreen ();
+				SetMainWindowDecorations (window);
+			}
+		}
 	}
 }
\ No newline at end of file
