@@ -61,7 +61,7 @@ namespace MonoDevelop.Ide.Desktop
 			remove { recentFiles.RecentFilesChanged -= value; }
 		}
 		
-		public override IList<RecentFile> GetProjects ()
+		protected override IList<RecentFile> OnGetProjects ()
 		{
 			try {
 				return Get (projGroup);
@@ -71,7 +71,7 @@ namespace MonoDevelop.Ide.Desktop
 			}
 		}
 		
-		public override IList<RecentFile> GetFiles ()
+		protected override IList<RecentFile> OnGetFiles ()
 		{
 			try {
 				return Get (fileGroup);
@@ -160,8 +160,47 @@ namespace MonoDevelop.Ide.Desktop
 		
 	public abstract class RecentFiles
 	{
-		public abstract IList<RecentFile> GetFiles ();
-		public abstract IList<RecentFile> GetProjects ();
+		List<string> favoriteFiles;
+		const string FavoritesConfigKey = "FavoriteRecentFiles";
+
+		public RecentFiles ()
+		{
+			favoriteFiles = PropertyService.Get (FavoritesConfigKey, new List<string> ());
+		}
+
+		public IList<RecentFile> GetProjects ()
+		{
+			var projects = OnGetProjects ();
+			List<RecentFile> result = new List<RecentFile> ();
+			List<string> toRemove = null;
+			foreach (var f in favoriteFiles) {
+				if (!File.Exists (f)) {
+					if (toRemove == null)
+						toRemove = new List<string> ();
+					toRemove.Add (f);
+					continue;
+				}
+				var entry = projects.FirstOrDefault (p => f == p.FileName);
+				if (entry != null)
+					result.Add (entry);
+				else
+					result.Add (new RecentFile (f, Path.GetFileNameWithoutExtension (f), DateTime.Now));
+			}
+			if (toRemove != null) {
+				foreach (var f in toRemove)
+					favoriteFiles.Remove (f);
+			}
+			foreach (var e in projects)
+				if (!result.Contains (e))
+					result.Add (e);
+			return result;
+		}
+
+		public IList<RecentFile> GetFiles ()
+		{
+			return OnGetFiles ();
+		}
+
 		public abstract event EventHandler Changed;
 		public abstract void ClearProjects ();
 		public abstract void ClearFiles ();
@@ -170,6 +209,9 @@ namespace MonoDevelop.Ide.Desktop
 		public abstract void NotifyFileRemoved (string filename);
 		public abstract void NotifyFileRenamed (string oldName, string newName);
 		
+		protected abstract IList<RecentFile> OnGetProjects ();
+		protected abstract IList<RecentFile> OnGetFiles ();
+
 		public void AddFile (string fileName, MonoDevelop.Projects.Project project)
 		{
 			var projectName = project != null? project.Name : null;
@@ -178,13 +220,29 @@ namespace MonoDevelop.Ide.Desktop
 				: Path.GetFileName (fileName);
 			AddFile (fileName, displayName);
 		}
+
+		internal void SetFavoriteFile (FilePath file, bool favorite)
+		{
+			if (favorite)
+				favoriteFiles.Add (file);
+			else
+				favoriteFiles.Remove (file);
+
+			PropertyService.Set (FavoritesConfigKey, favoriteFiles);
+			PropertyService.SaveProperties ();
+		}
+
+		internal bool IsFavoriteFile (FilePath file)
+		{
+			return favoriteFiles.Contains (file);
+		}
 	}
 	
 	public class RecentFile
 	{
 		string displayName, fileName;
 		DateTime timestamp;
-		
+
 		public RecentFile (string fileName, string displayName, DateTime timestamp)
 		{
 			this.fileName = fileName;
@@ -192,6 +250,15 @@ namespace MonoDevelop.Ide.Desktop
 			this.timestamp = timestamp;
 		}
 
+		public bool IsFavorite {
+			get {
+				return DesktopService.RecentFiles.IsFavoriteFile (fileName);
+			}
+			set {
+				DesktopService.RecentFiles.SetFavoriteFile (fileName, value);
+			}
+		}
+
 		public string FileName { get { return fileName; } }
 		public string DisplayName {
 			get {
