@@ -43,7 +43,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		AlertButton[] buttons;
 		
 		Gtk.HBox  hbox  = new HBox ();
-		Gtk.Image image = new Image ();
+		Gtk.Image image;
 		Gtk.Label label = new Label ();
 		VBox labelsBox = new VBox (false, 6);
 		
@@ -58,7 +58,6 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		void Init ()
 		{
 			VBox.PackStart (hbox);
-			hbox.PackStart (image, false, false, 0);
 			hbox.PackStart (labelsBox, true, true, 0);
 			labelsBox.PackStart (label, true, true, 0);
 				
@@ -75,11 +74,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			// Table 3.3
 			this.hbox.Spacing     = 12;
 			this.hbox.BorderWidth = 6;
-			
-			// Table 3.4
-			this.image.Yalign   = 0.00f;
-			//this.image.IconSize = Gtk.IconSize.Dialog;
-			
+
 			// Table 3.5
 			this.label.UseMarkup = true;
 			this.label.Wrap      = true;
@@ -103,7 +98,13 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				secondaryText = message.SecondaryText;
 			}
 			
-			image.Pixbuf = ImageService.GetPixbuf (message.Icon, IconSize.Dialog);
+			if (!string.IsNullOrEmpty (message.Icon)) {
+				image = new Image ();
+				image.Yalign   = 0.00f;
+				image.Pixbuf = ImageService.GetPixbuf (message.Icon, IconSize.Dialog);
+				hbox.PackStart (image, false, false, 0);
+				hbox.ReorderChild (image, 0);
+			}
 			
 			StringBuilder markup = new StringBuilder (@"<span weight=""bold"" size=""larger"">");
 			markup.Append (GLib.Markup.EscapeText (primaryText));
@@ -157,8 +158,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				buttonNumber = ActionArea.Children.Length - 1;
 			ActionArea.Children[buttonNumber].GrabFocus ();
 		}
-			
-		
+
 		void ButtonClicked (object sender, EventArgs e) 
 		{
 			Gtk.Button clickButton = (Gtk.Button)sender;
