@@ -55,6 +55,12 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		List<PolicySet> setsInCombo = new List<PolicySet> ();
 		bool synchingPoliciesCombo;
 		bool selectingPolicy;
+
+		public List<IMimeTypePolicyOptionsPanel> Panels {
+			get {
+				return panels;
+			}
+		}
 		
 		public MimeTypePolicyOptionsSection ()
 		{
@@ -142,7 +148,10 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				notebook.AppendPage (align, tlabel);
 				panel.LoadCurrentPolicy ();
 			}
-			
+			notebook.SwitchPage += delegate(object o, SwitchPageArgs args) {
+				if (notebook.Page >= 0 && notebook.Page < this.panels.Count)
+					this.panels[notebook.Page].PanelSelected ();
+			};
 			notebook.Show ();
 			vbox.PackEnd (notebook, true, true, 0);
 			
@@ -370,6 +379,21 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				}
 			}
 		}
+
+		public void SetUseParentPolicy (bool useParentPolicy, System.Type policyType, string scope)
+		{
+			if (useParentPolicy == this.useParentPolicy) 
+				return;
+			this.useParentPolicy = useParentPolicy;
+			if (useParentPolicy) {
+				foreach (IMimeTypePolicyOptionsPanel panel in Panels) {
+					if (panel.HandlesPolicyType (policyType, scope))
+						panel.LoadParentPolicy ();
+				}
+			}
+			if (SectionLoaded)
+				SectionPanel.UpdateSelectedNamedPolicy ();
+		}
 		
 		public void AssignPolicies (PolicySet pset)
 		{
