@@ -29,6 +29,7 @@
 using System;
 using MonoDevelop.Core;
 using MonoDevelop.Ide.Gui.Dialogs;
+using MonoDevelop.Projects;
 
 namespace MonoDevelop.Ide.Gui.OptionPanels
 {
@@ -57,6 +58,8 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			nameEntry.Text = AuthorInformation.Default.Name ?? "";
 			emailEntry.Text = AuthorInformation.Default.Email ?? "";
 			copyrightEntry.Text = AuthorInformation.Default.Copyright ?? "";
+			companyEntry.Text = AuthorInformation.Default.Company ?? "";
+			trademarkEntry.Text = AuthorInformation.Default.Trademark ?? "";
 		}
 		
 		public void Save ()
@@ -64,6 +67,8 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			PropertyService.Set ("Author.Name", nameEntry.Text);
 			PropertyService.Set ("Author.Email", emailEntry.Text);
 			PropertyService.Set ("Author.Copyright", copyrightEntry.Text);
+			PropertyService.Set ("Author.Company", companyEntry.Text);
+			PropertyService.Set ("Author.Trademark", trademarkEntry.Text);
 		}
 	}
 }
