@@ -50,7 +50,7 @@ using MonoDevelop.Ide.Projects;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 {
-	public abstract class FolderNodeBuilder: TypeNodeBuilder
+	abstract class FolderNodeBuilder: TypeNodeBuilder
 	{
 		public override void GetNodeAttributes (ITreeNavigator treeNavigator, object dataObject, ref NodeAttributes attributes)
 		{
@@ -123,37 +123,56 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			if (project.Files.Count > 500)
 				return true;
 
-			ProjectFileCollection files;
-			ArrayList folders;
-			
-			string path = GetFolderPath (dataObject);
-			
-			GetFolderContent (project, path, out files, out folders);
+			var folder = ((ProjectFolder) dataObject).Path;
+
+			foreach (var file in project.Files) {
+				FilePath path;
+
+				if (file.Subtype != Subtype.Directory)
+					path = file.IsLink ? project.BaseDirectory.Combine (file.ProjectVirtualPath) : file.FilePath;
+				else
+					path = file.FilePath;
+
+				if (path.IsChildPathOf (folder))
+					return true;
+			}
 
-			if (files.Count > 0 || folders.Count > 0) return true;
-			
 			return false;
 		}
 	}
 	
-	public abstract class FolderCommandHandler: NodeCommandHandler
+	abstract class FolderCommandHandler: NodeCommandHandler
 	{
+		// CommandHandlers are constantly re-created so it's not possible to cache data inside the instance
+		// Since 'AddExistingFolder' can only be run from the UI thread anyway we can safely just make this static.
+		static FilePath PreviousFolderPath {
+			get; set;
+		}
+
 		public abstract string GetFolderPath (object dataObject);
 
 		public override bool CanDropNode (object dataObject, DragOperation operation)
 		{
-			string targetPath = GetFolderPath (CurrentNode.DataItem);
+			string targetDirectory = GetFolderPath (CurrentNode.DataItem);
 			
 			if (dataObject is ProjectFile) {
 				ProjectFile file = (ProjectFile) dataObject;
 				var srcDir = (file.Project != null && file.IsLink)
 					? file.Project.BaseDirectory.Combine (file.ProjectVirtualPath)
 					: file.FilePath.ParentDirectory;
-				
-				return (srcDir != targetPath || operation == DragOperation.Copy) && file.DependsOnFile == null;
+
+				switch (operation) {
+				case DragOperation.Move:
+					// allow grouped files to be unlinked from their parent
+					return srcDir != targetDirectory || file.DependsOnFile != null;
+				case DragOperation.Copy:
+					return true;
+				default:
+					return false;
+				}
 			}
 			else if (dataObject is ProjectFolder) {
-				return ((ProjectFolder)dataObject).Path != targetPath || operation == DragOperation.Copy;
+				return ((ProjectFolder)dataObject).Path != targetDirectory || operation == DragOperation.Copy;
 			}
 			else if (dataObject is Gtk.SelectionData) {
 				SelectionData data = (SelectionData) dataObject;
@@ -165,20 +184,28 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		
 		public override void OnMultipleNodeDrop (object[] dataObjects, DragOperation operation)
 		{
-			Set<SolutionEntityItem> toSave = new Set<SolutionEntityItem> ();
+			var projectsToSave = new HashSet<SolutionEntityItem> ();
+			var groupedFiles = new HashSet<ProjectFile> ();
+
+			foreach (var pf in dataObjects.OfType<ProjectFile> ()) {
+				foreach (var child in pf.DependentChildren)
+					groupedFiles.Add (child);
+			}
+
 			foreach (object dataObject in dataObjects)
-				DropNode (toSave, dataObject, operation);
-			IdeApp.ProjectOperations.Save (toSave);
+				DropNode (projectsToSave, dataObject, groupedFiles, operation);
+
+			IdeApp.ProjectOperations.Save (projectsToSave);
 		}
 		
-		void DropNode (Set<SolutionEntityItem> projectsToSave, object dataObject, DragOperation operation)
+		void DropNode (HashSet<SolutionEntityItem> projectsToSave, object dataObject, HashSet<ProjectFile> groupedFiles, DragOperation operation)
 		{
-			FilePath targetPath = GetFolderPath (CurrentNode.DataItem);
+			FilePath targetDirectory = GetFolderPath (CurrentNode.DataItem);
 			FilePath source;
 			string what;
 			Project targetProject = (Project) CurrentNode.GetParentDataItem (typeof(Project), true);
 			Project sourceProject;
-			System.Collections.Generic.IEnumerable<ProjectFile> groupedChildren = null;
+			IEnumerable<ProjectFile> groupedChildren = null;
 			
 			if (dataObject is ProjectFolder) {
 				source = ((ProjectFolder) dataObject).Path;
@@ -187,6 +214,22 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 			else if (dataObject is ProjectFile) {
 				ProjectFile file = (ProjectFile) dataObject;
+
+				// if this ProjectFile is one of the grouped files being pulled in by a parent being copied/moved, ignore it
+				if (groupedFiles.Contains (file))
+					return;
+
+				if (file.DependsOnFile != null && operation == DragOperation.Move) {
+					// unlink this file from its parent (since its parent is not being moved)
+					file.DependsOn = null;
+
+					// if moving a linked file into its containing folder, simply unlink it from its parent
+					if (file.FilePath.ParentDirectory == targetDirectory) {
+						projectsToSave.Add (targetProject);
+						return;
+					}
+				}
+
 				sourceProject = file.Project;
 				if (sourceProject != null && file.IsLink) {
 					source = sourceProject.BaseDirectory.Combine (file.ProjectVirtualPath);
@@ -201,6 +244,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				if (data.Type != "text/uri-list")
 					return;
 				string sources = System.Text.Encoding.UTF8.GetString (data.Data);
+				Console.WriteLine ("text/uri-list:\n{0}", sources);
 				string[] files = sources.Split (new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
 				for (int n=0; n<files.Length; n++) {
 					Uri uri = new Uri (files[n]);
@@ -211,25 +255,34 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 					files[n] = uri.LocalPath;
 				}
 				
-				IdeApp.ProjectOperations.AddFilesToProject (targetProject, files, targetPath);
+				IdeApp.ProjectOperations.AddFilesToProject (targetProject, files, targetDirectory);
 				projectsToSave.Add (targetProject);
 				return;
 			}
 			else
 				return;
 
-			targetPath = targetPath.Combine (source.FileName);
+			var targetPath = targetDirectory.Combine (source.FileName);
 			// If copying to the same directory, make a copy with a different name
 			if (targetPath == source)
 				targetPath = ProjectOperations.GetTargetCopyName (targetPath, dataObject is ProjectFolder);
-			
+
+			var targetChildPaths = groupedChildren != null ? groupedChildren.Select (child => {
+				var targetChildPath = targetDirectory.Combine (child.FilePath.FileName);
+
+				if (targetChildPath == child.FilePath)
+					targetChildPath = ProjectOperations.GetTargetCopyName (targetChildPath, false);
+
+				return targetChildPath;
+			}).ToList () : null;
+
 			if (dataObject is ProjectFolder) {
 				string q;
 				if (operation == DragOperation.Move) {
 					if (targetPath.ParentDirectory == targetProject.BaseDirectory)
 						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the root folder of project '{1}'?", what, targetProject.Name);
 					else
-						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the folder '{1}'?", what, targetPath.ParentDirectory.FileName);
+						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the folder '{1}'?", what, targetDirectory.FileName);
 					if (!MessageService.Confirm (q, AlertButton.Move))
 						return;
 				}
@@ -237,17 +290,19 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 					if (targetPath.ParentDirectory == targetProject.BaseDirectory)
 						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the root folder of project '{1}'?", what, targetProject.Name);
 					else
-						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the folder '{1}'?", what, targetPath.ParentDirectory.FileName);
+						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the folder '{1}'?", what, targetDirectory.FileName);
 					if (!MessageService.Confirm (q, AlertButton.Copy))
 						return;
 				}
 			} else if (dataObject is ProjectFile) {
-				if (File.Exists (targetPath))
-					if (!MessageService.Confirm (GettextCatalog.GetString ("The file '{0}' already exists. Do you want to overwrite it?", targetPath.FileName), AlertButton.OverwriteFile))
-						return;
+				foreach (var file in new FilePath[] { targetPath }.Concat (targetChildPaths)) {
+					if (File.Exists (file))
+						if (!MessageService.Confirm (GettextCatalog.GetString ("The file '{0}' already exists. Do you want to overwrite it?", file.FileName), AlertButton.OverwriteFile))
+							return;
+				}
 			}
 			
-			ArrayList filesToSave = new ArrayList ();
+			var filesToSave = new List<Document> ();
 			foreach (Document doc in IdeApp.Workbench.Documents) {
 				if (doc.IsDirty && doc.IsFile) {
 					if (doc.Name == source || doc.Name.StartsWith (source + Path.DirectorySeparatorChar)) {
@@ -284,7 +339,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				AlertButton res = MessageService.AskQuestion (question, AlertButton.Cancel, noSave, AlertButton.Save);
 				if (res == AlertButton.Cancel)
 					return;
-				else if (res == AlertButton.Save) { 
+				if (res == AlertButton.Save) {
 					try {
 						foreach (Document doc in filesToSave) {
 							doc.Save ();
@@ -300,14 +355,17 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				projectsToSave.Add (sourceProject);
 			if (targetProject != null)
 				projectsToSave.Add (targetProject);
+
+			bool move = operation == DragOperation.Move;
+			var opText = move ? GettextCatalog.GetString ("Moving files...") : GettextCatalog.GetString ("Copying files...");
 			
-			using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString("Copying files..."), MonoDevelop.Ide.Gui.Stock.CopyIcon, true))
-			{
+			using (var monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (opText, Stock.StatusSolutionOperation, true)) {
 				// If we drag and drop a node in the treeview corresponding to a directory, do not move
 				// the entire directory. We should only move the files which exist in the project. Otherwise
 				// we will need a lot of hacks all over the code to prevent us from incorrectly moving version
 				// control related files such as .svn directories
-				bool move = operation == DragOperation.Move;
+
+				// Note: if we are transferring a ProjectFile, this will copy/move the ProjectFile's DependentChildren as well.
 				IdeApp.ProjectOperations.TransferFiles (monitor, sourceProject, source, targetProject, targetPath, move, true);
 			}
 		}
@@ -316,9 +374,10 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		public void AddFilesToProject()
 		{
 			Project project = (Project) CurrentNode.GetParentDataItem (typeof(Project), true);
-			
+			var targetRoot = ((FilePath) GetFolderPath (CurrentNode.DataItem)).CanonicalPath;
+
 			AddFileDialog fdiag  = new AddFileDialog (GettextCatalog.GetString ("Add files"));
-			fdiag.CurrentFolder = GetFolderPath (CurrentNode.DataItem);
+			fdiag.CurrentFolder = !PreviousFolderPath.IsNullOrEmpty ? PreviousFolderPath : targetRoot;
 			fdiag.SelectMultiple = true;
 			fdiag.TransientFor = IdeApp.Workbench.RootWindow;
 			fdiag.BuildActions = project.GetBuildActions ();	
@@ -327,7 +386,8 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			
 			if (!fdiag.Run ())
 				return;
-			
+			PreviousFolderPath = fdiag.SelectedFiles.Select (f => f.FullPath.ParentDirectory).FirstOrDefault ();
+
 			var files = fdiag.SelectedFiles;
 			overrideAction = fdiag.OverrideAction;
 			
@@ -364,11 +424,14 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			var targetRoot = ((FilePath) GetFolderPath (CurrentNode.DataItem)).CanonicalPath;
 			
 			var ofdlg = new SelectFolderDialog (GettextCatalog.GetString ("Import From Folder")) {
-				CurrentFolder = targetRoot
+				CurrentFolder = !PreviousFolderPath.IsNullOrEmpty ? PreviousFolderPath : targetRoot
 			};
 			if(!ofdlg.Run ())
 				return;
-			
+			PreviousFolderPath = ofdlg.SelectedFile.CanonicalPath;
+			if (!PreviousFolderPath.ParentDirectory.IsNullOrEmpty)
+				PreviousFolderPath = PreviousFolderPath.ParentDirectory;
+
 			var srcRoot = ofdlg.SelectedFile.CanonicalPath;
 			var foundFiles = Directory.GetFiles (srcRoot, "*", SearchOption.AllDirectories);
 			
@@ -384,7 +447,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			if (added)
 				IdeApp.ProjectOperations.Save (project);
 		}
-		
+
 		///<summary>Adds an existing folder to the current folder</summary>
 		[CommandHandler (ProjectCommands.AddExistingFolder)]
 		public void AddExistingFolder ()
@@ -393,11 +456,17 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			var selectedFolder = ((FilePath) GetFolderPath (CurrentNode.DataItem)).CanonicalPath;
 			
 			var ofdlg = new SelectFolderDialog (GettextCatalog.GetString ("Add Existing Folder")) {
-				CurrentFolder = selectedFolder
+				CurrentFolder = !PreviousFolderPath.IsNullOrEmpty ? PreviousFolderPath : selectedFolder 
 			};
 			if(!ofdlg.Run ())
 				return;
-			
+
+			// We store the parent directory of the folder the user chooses as they will not need to add the same
+			// directory twice. We can save them navigating up one directory by doing it for them
+			PreviousFolderPath = ofdlg.SelectedFile.CanonicalPath;
+			if (!PreviousFolderPath.ParentDirectory.IsNullOrEmpty)
+				PreviousFolderPath = PreviousFolderPath.ParentDirectory;
+
 			var srcRoot = ofdlg.SelectedFile.CanonicalPath;
 			var targetRoot = selectedFolder.Combine (srcRoot.FileName);
 
