@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Linq;
 using System.IO;
 using System.Collections;
 
@@ -53,6 +54,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			this.absolutePath = absolutePath.CanonicalPath;
 		}
 		
+		//FIXME: we don't track when the folder goes away if it's implicit and all its children are removed
 		public bool TrackChanges {
 			get { return trackChanges; }
 			set {
@@ -129,13 +131,12 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 
 		void OnFileRenamed (object sender, FileCopyEventArgs e)
 		{
-			if (!e.IsDirectory || e.SourceFile != absolutePath) return;
-
 			// The folder path can't be updated because we would be changing
 			// the identity of the object. Another folder object will need
 			// to be created by updating the tree.
 			
-			if (FolderRenamed != null) 
+			var e2 = new FileCopyEventArgs (e.Where (i => i.IsDirectory && i.SourceFile == absolutePath));
+			if (e2.Count > 0 && FolderRenamed != null) 
 				FolderRenamed (this, e);
 		}
 
