@@ -27,18 +27,15 @@
 //
 
 using System;
-using System.Resources;
 using System.Collections.Generic;
 
 using MonoDevelop.Projects;
-using MonoDevelop.Core;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Components;
-using System.Text;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 {
-	public class ProjectSolutionPad: SolutionPad
+	class ProjectSolutionPad: SolutionPad
 	{
 		public ProjectSolutionPad ()
 		{
@@ -59,6 +56,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				}
 				return null;
 			});
+			TreeView.ShowSelectionPopupButton = true;
 		}
 		
 		protected override void OnSelectionChanged (object sender, EventArgs args)
@@ -83,9 +81,11 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		
 		void OnWindowChanged (object ob, EventArgs args)
 		{
-			DispatchService.GuiDispatch (new MessageHandler (SelectActiveFile));
+			Gtk.Application.Invoke (delegate {
+				SelectActiveFile ();
+			});
 		}
-		
+
 		void SelectActiveFile ()
 		{
 			Document doc = IdeApp.Workbench.ActiveDocument;
