@@ -38,7 +38,7 @@ using MonoDevelop.Ide.Gui.Components;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 {
-	public class ShowAllFilesBuilderExtension: NodeBuilderExtension
+	class ShowAllFilesBuilderExtension: NodeBuilderExtension
 	{
 		ProjectFileEventHandler fileAddedHandler;
 		ProjectFileEventHandler fileRemovedHandler;
@@ -335,7 +335,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		}
 	}
 	
-	public class ShowAllFilesCommandHandler: NodeCommandHandler
+	class ShowAllFilesCommandHandler: NodeCommandHandler
 	{
 		public override bool CanDropNode (object dataObject, DragOperation operation)
 		{
@@ -355,7 +355,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			if (targetPath == source)
 				targetPath = ProjectOperations.GetTargetCopyName (targetPath, false);
 			
-			using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString("Copying files..."), Stock.CopyIcon, true))
+			using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString("Copying files..."), Stock.StatusSolutionOperation, true))
 			{
 				bool move = operation == DragOperation.Move;
 				IdeApp.ProjectOperations.TransferFiles (monitor, null, source, targetProject, targetPath, move, false);
