@@ -73,6 +73,11 @@ namespace MonoDevelop.Ide.Gui
 			get { return false; }
 		}
 
+
+		public virtual bool IsHidden {
+			get { return false; }
+		}
+
 		public virtual bool IsViewOnly { get; set; }
 
 		public virtual bool IsFile {
@@ -99,9 +104,17 @@ namespace MonoDevelop.Ide.Gui
 		{
 			throw new NotImplementedException ();
 		}
+		
+		public virtual void DiscardChanges ()
+		{
+		}
 
 		public abstract void Load (string fileName);
-
+		
+		public virtual void LoadNew (System.IO.Stream content, string mimeType)
+		{
+			throw new NotSupportedException ();
+		}
 
 		public event EventHandler ContentNameChanged;
 
