@@ -47,9 +47,11 @@ using MonoDevelop.Ide.Extensions;
 using System.Linq;
 using System.Threading;
 using MonoDevelop.Ide.TypeSystem;
+using ICSharpCode.NRefactory;
 using ICSharpCode.NRefactory.TypeSystem;
 using ICSharpCode.NRefactory.TypeSystem.Implementation;
 using System.Text;
+using System.Collections.ObjectModel;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -59,7 +61,10 @@ namespace MonoDevelop.Ide.Gui
 		
 		IWorkbenchWindow window;
 		TextEditorExtension editorExtension;
-		
+		ParsedDocument parsedDocument;
+		IProjectContent singleFileContext;
+		Mono.TextEditor.ITextEditorDataProvider provider = null;
+
 		const int ParseDelay = 600;
 
 		public IWorkbenchWindow Window {
@@ -77,19 +82,21 @@ namespace MonoDevelop.Ide.Gui
  		
 		public T GetContent<T> () where T : class
 		{
+			if (window == null)
+				return null;
 			//check whether the ViewContent can return the type directly
-			T ret = Window.ActiveViewContent.GetContent<T> ();
+			T ret = Window.ActiveViewContent.GetContent (typeof(T)) as T;
 			if (ret != null)
 				return ret;
 			
 			//check the primary viewcontent
 			//not sure if this is the right thing to do, but things depend on this behaviour
 			if (Window.ViewContent != Window.ActiveViewContent) {
-				ret = Window.ViewContent.GetContent<T> ();
+				ret = Window.ViewContent.GetContent (typeof(T)) as T;
 				if (ret != null)
 					return ret;
 			}
-			
+
 			//no, so look through the TexteditorExtensions as well
 			TextEditorExtension nextExtension = editorExtension;
 			while (nextExtension != null) {
@@ -103,14 +110,14 @@ namespace MonoDevelop.Ide.Gui
 		public IEnumerable<T> GetContents<T> () where T : class
 		{
 			//check whether the ViewContent can return the type directly
-			T ret = Window.ActiveViewContent.GetContent<T> ();
+			T ret = (T) Window.ActiveViewContent.GetContent (typeof(T));
 			if (ret != null)
 				yield return ret;
 			
 			//check the primary viewcontent
 			//not sure if this is the right thing to do, but things depend on this behaviour
 			if (Window.ViewContent != Window.ActiveViewContent) {
-				ret = Window.ViewContent.GetContent<T> ();
+				ret = (T) Window.ViewContent.GetContent (typeof(T));
 				if (ret != null)
 					yield return ret;
 			}
@@ -147,6 +154,7 @@ namespace MonoDevelop.Ide.Gui
 				IdeApp.Workspace.ItemRemovedFromSolution += OnEntryRemoved;
 			if (window.ViewContent.Project != null)
 				window.ViewContent.Project.Modified += HandleProjectModified;
+			window.ViewsChanged += HandleViewsChanged;
 		}
 
 /*		void UpdateRegisteredDom (object sender, ProjectDomEventArgs e)
@@ -176,11 +184,11 @@ namespace MonoDevelop.Ide.Gui
 		}
 		
 		public bool HasProject {
-			get { return Window.ViewContent.Project != null; }
+			get { return Window != null ? Window.ViewContent.Project != null : false; }
 		}
 		
 		public Project Project {
-			get { return Window.ViewContent.Project; }
+			get { return Window != null ? Window.ViewContent.Project : null; }
 /*			set { 
 				Window.ViewContent.Project = value; 
 				if (value != null)
@@ -201,20 +209,18 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 
-		IProjectContent singleFileContext;
-		public  virtual IProjectContent ProjectContent {
+		public  IProjectContent ProjectContent {
 			get {
 				return Project != null ? TypeSystemService.GetProjectContext (Project) : GetProjectContext ();
 			}
 		}
 		
-		public  virtual ICompilation Compilation {
+		public virtual ICompilation Compilation {
 			get {
 				return Project != null ? TypeSystemService.GetCompilation (Project) : GetProjectContext ().CreateCompilation ();
 			}
 		}
 		
-		ParsedDocument parsedDocument;
 		public virtual ParsedDocument ParsedDocument {
 			get {
 				return parsedDocument;
@@ -230,14 +236,59 @@ namespace MonoDevelop.Ide.Gui
 			window.SelectWindow ();
 		}
 		
-		public IBaseViewContent ActiveView {
-			get { return window.ActiveViewContent; }
+		public DocumentView ActiveView {
+			get {
+				LoadViews (true);
+				return WrapView (window.ActiveViewContent);
+			}
 		}
 		
-		public IViewContent PrimaryView {
-			get { return window.ViewContent; }
+		public DocumentView PrimaryView {
+			get {
+				LoadViews (true);
+				return WrapView (window.ViewContent);
+			}
 		}
-		
+
+		public ReadOnlyCollection<DocumentView> Views {
+			get {
+				LoadViews (true);
+				if (viewsRO == null)
+					viewsRO = new ReadOnlyCollection<DocumentView> (views);
+				return viewsRO;
+			}
+		}
+
+		ReadOnlyCollection<DocumentView> viewsRO;
+		List<DocumentView> views = new List<DocumentView> ();
+
+		void HandleViewsChanged (object sender, EventArgs e)
+		{
+			LoadViews (false);
+		}
+
+		void LoadViews (bool force)
+		{
+			if (!force && views == null)
+				return;
+			var newList = new List<DocumentView> ();
+			newList.Add (WrapView (window.ViewContent));
+			foreach (var v in window.SubViewContents)
+				newList.Add (WrapView (v));
+			views = newList;
+			viewsRO = null;
+		}
+
+		DocumentView WrapView (IBaseViewContent content)
+		{
+			if (content == null)
+				return null;
+			if (views != null)
+				return views.FirstOrDefault (v => v.BaseContent == content) ?? new DocumentView (this, content);
+			else
+				return new DocumentView (this, content);
+		}
+
 		public string Name {
 			get {
 				IViewContent view = Window.ViewContent;
@@ -245,19 +296,14 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 
-		Mono.TextEditor.TextEditorData data = null;
 		public Mono.TextEditor.TextEditorData Editor {
 			get {
-				if (data == null) {
-					Mono.TextEditor.ITextEditorDataProvider view = GetContent <Mono.TextEditor.ITextEditorDataProvider> ();
-					if (view != null) {
-						data = view.GetTextEditorData ();
-						if (data.IsDisposed)
-							return null;
-						data.Document.Tag = this;
-					}
+				if (provider == null) {
+					provider = GetContent <Mono.TextEditor.ITextEditorDataProvider> ();
+					if (provider == null)
+						return null;
 				}
-				return data;
+				return provider.GetTextEditorData ();
 			}
 		}
 
@@ -278,7 +324,7 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		public virtual void Save ()
+		public void Save ()
 		{
 			// suspend type service "check all file loop" since we have already a parsed document.
 			// Or at least one that updates "soon".
@@ -295,8 +341,11 @@ namespace MonoDevelop.Ide.Gui
 				if (Window.ViewContent.ContentName == null) {
 					SaveAs ();
 				} else {
-					if (!FileService.RequestFileEdit (Window.ViewContent.ContentName))
-						MessageService.ShowMessage (GettextCatalog.GetString ("The file could not be saved. Write permission has not been granted."));
+					try {
+						FileService.RequestFileEdit (Window.ViewContent.ContentName, true);
+					} catch (Exception ex) {
+						MessageService.ShowException (ex, GettextCatalog.GetString ("The file could not be saved."));
+					}
 					
 					FileAttributes attr = FileAttributes.ReadOnly | FileAttributes.Directory | FileAttributes.Offline | FileAttributes.System;
 	
@@ -383,9 +432,7 @@ namespace MonoDevelop.Ide.Gui
 				else
 					Window.ViewContent.Save (filename + "~");
 			}
-			lock (TypeSystemService.FilesSkippedInParseThread) {
-				TypeSystemService.FilesSkippedInParseThread.Remove (FileName);
-			}
+			TypeSystemService.RemoveSkippedfile (FileName);
 			// do actual save
 			if (tbuffer != null && encoding != null)
 				tbuffer.Save (filename, encoding);
@@ -399,7 +446,7 @@ namespace MonoDevelop.Ide.Gui
 			UpdateParseDocument ();
 		}
 		
-		public virtual bool IsBuildTarget
+		public bool IsBuildTarget
 		{
 			get
 			{
@@ -412,17 +459,17 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		public virtual IAsyncOperation Build ()
+		public IAsyncOperation Build ()
 		{
 			return IdeApp.ProjectOperations.BuildFile (Window.ViewContent.ContentName);
 		}
 		
-		public virtual IAsyncOperation Rebuild ()
+		public IAsyncOperation Rebuild ()
 		{
 			return Build ();
 		}
 		
-		public virtual void Clean ()
+		public void Clean ()
 		{
 		}
 		
@@ -431,27 +478,27 @@ namespace MonoDevelop.Ide.Gui
 			return Run (Runtime.ProcessService.DefaultExecutionHandler);
 		}
 
-		public virtual IAsyncOperation Run (IExecutionHandler handler)
+		public IAsyncOperation Run (IExecutionHandler handler)
 		{
 			return IdeApp.ProjectOperations.ExecuteFile (Window.ViewContent.ContentName, handler);
 		}
 
-		public virtual bool CanRun ()
+		public bool CanRun ()
 		{
 			return CanRun (Runtime.ProcessService.DefaultExecutionHandler);
 		}
 		
-		public virtual bool CanRun (IExecutionHandler handler)
+		public bool CanRun (IExecutionHandler handler)
 		{
 			return IsBuildTarget && Window.ViewContent.ContentName != null && IdeApp.ProjectOperations.CanExecuteFile (Window.ViewContent.ContentName, handler);
 		}
 		
 		public bool Close ()
 		{
-			return Window.CloseWindow (false, true, 0);
+			return ((SdiWorkspaceWindow)Window).CloseWindow (false, true);
 		}
 		
-		protected virtual void OnSaved (EventArgs args)
+		void OnSaved (EventArgs args)
 		{
 			if (Saved != null)
 				Saved (this, args);
@@ -472,33 +519,15 @@ namespace MonoDevelop.Ide.Gui
 //			TypeSystemService.DomRegistered -= UpdateRegisteredDom;
 			CancelParseTimeout ();
 			ClearTasks ();
+			TypeSystemService.RemoveSkippedfile (FileName);
+
 
-			lock (TypeSystemService.FilesSkippedInParseThread) {
-				TypeSystemService.FilesSkippedInParseThread.Remove (FileName);
-			}
-			if (window is SdiWorkspaceWindow)
-				((SdiWorkspaceWindow)window).DetachFromPathedDocument ();
-			window.Closed -= OnClosed;
-			window.ActiveViewContentChanged -= OnActiveViewContentChanged;
-			if (IdeApp.Workspace != null)
-				IdeApp.Workspace.ItemRemovedFromSolution -= OnEntryRemoved;
-			
 			try {
 				OnClosed (a);
 			} catch (Exception ex) {
 				LoggingService.LogError ("Exception while calling OnClosed.", ex);
 			}
-			
-			while (editorExtension != null) {
-				try {
-					editorExtension.Dispose ();
-				} catch (Exception ex) {
-					LoggingService.LogError ("Exception while disposing extension:" + editorExtension, ex);
-				}
-				editorExtension = editorExtension.Next as TextEditorExtension;
-			}
-			editorExtension = null;
-			
+
 			// Parse the file when the document is closed. In this way if the document
 			// is closed without saving the changes, the saved compilation unit
 			// information will be restored
@@ -512,10 +541,32 @@ namespace MonoDevelop.Ide.Gui
 				dom = null;
 			}*/
 			
-			parsedDocument = null;
-			data = null;
 			Counters.OpenDocuments--;
 		}
+
+		internal void DisposeDocument ()
+		{
+			DetachExtensionChain ();
+			ClearAnnotations ();
+			if (window is SdiWorkspaceWindow)
+				((SdiWorkspaceWindow)window).DetachFromPathedDocument ();
+			window.Closed -= OnClosed;
+			window.ActiveViewContentChanged -= OnActiveViewContentChanged;
+			if (IdeApp.Workspace != null)
+				IdeApp.Workspace.ItemRemovedFromSolution -= OnEntryRemoved;
+
+			// Unsubscribe project events
+			if (window.ViewContent.Project != null)
+				window.ViewContent.Project.Modified -= HandleProjectModified;
+			window.ViewsChanged += HandleViewsChanged;
+			window = null;
+
+			parsedDocument = null;
+			singleFileContext = null;
+			provider = null;
+			views = null;
+			viewsRO = null;
+		}
 #region document tasks
 		object lockObj = new object ();
 		
@@ -548,20 +599,58 @@ namespace MonoDevelop.Ide.Gui
 			OnViewChanged (args);
 		}
 		
-		protected virtual void OnClosed (EventArgs args)
+		void OnClosed (EventArgs args)
 		{
 			if (Closed != null)
 				Closed (this, args);
 		}
 		
-		protected virtual void OnViewChanged (EventArgs args)
+		void OnViewChanged (EventArgs args)
 		{
 			if (ViewChanged != null)
 				ViewChanged (this, args);
 		}
 		
 		bool wasEdited;
-		
+
+		void InitializeExtensionChain ()
+		{
+			DetachExtensionChain ();
+			var editor = GetContent<IExtensibleTextEditor> ();
+
+			ExtensionNodeList extensions = window.ExtensionContext.GetExtensionNodes ("/MonoDevelop/Ide/TextEditorExtensions", typeof(TextEditorExtensionNode));
+			editorExtension = null;
+			TextEditorExtension last = null;
+			foreach (TextEditorExtensionNode extNode in extensions) {
+				if (!extNode.Supports (FileName))
+					continue;
+				TextEditorExtension ext = (TextEditorExtension)extNode.CreateInstance ();
+				if (ext.ExtendsEditor (this, editor)) {
+					if (editorExtension == null)
+						editorExtension = ext;
+					if (last != null)
+						last.Next = ext;
+					last = ext;
+					ext.Initialize (this);
+				}
+			}
+			if (editorExtension != null)
+				last.Next = editor.AttachExtension (editorExtension);
+		}
+
+		void DetachExtensionChain ()
+		{
+			while (editorExtension != null) {
+				try {
+					editorExtension.Dispose ();
+				} catch (Exception ex) {
+					LoggingService.LogError ("Exception while disposing extension:" + editorExtension, ex);
+				}
+				editorExtension = editorExtension.Next as TextEditorExtension;
+			}
+			editorExtension = null;
+		}
+
 		void InitializeEditor (IExtensibleTextEditor editor)
 		{
 			Editor.Document.TextReplaced += (o, a) => {
@@ -585,31 +674,8 @@ namespace MonoDevelop.Ide.Gui
 			};
 			Editor.Document.Undone += (o, a) => StartReparseThread ();
 			Editor.Document.Redone += (o, a) => StartReparseThread ();
-			
-			// If the new document is a text editor, attach the extensions
-			
-			ExtensionNodeList extensions = window.ExtensionContext.GetExtensionNodes ("/MonoDevelop/Ide/TextEditorExtensions", typeof(TextEditorExtensionNode));
-			editorExtension = null;
-			TextEditorExtension last = null;
-			foreach (TextEditorExtensionNode extNode in extensions) {
-				if (!extNode.Supports (FileName))
-					continue;
-				
-				TextEditorExtension ext = (TextEditorExtension)extNode.CreateInstance ();
-				if (ext.ExtendsEditor (this, editor)) {
-					if (editorExtension == null)
-						editorExtension = ext;
-					
-					if (last != null)
-						last.Next = ext;
-					
-					last = ext;
-					ext.Initialize (this);
-				}
-			}
-			
-			if (editorExtension != null)
-				last.Next = editor.AttachExtension (editorExtension);
+
+			InitializeExtensionChain ();
 		}
 		
 		internal void OnDocumentAttached ()
@@ -617,7 +683,7 @@ namespace MonoDevelop.Ide.Gui
 			IExtensibleTextEditor editor = GetContent<IExtensibleTextEditor> ();
 			if (editor != null) {
 				InitializeEditor (editor);
-				RunWhenLoaded (() => ReparseDocument ());
+				RunWhenLoaded (ReparseDocument);
 			}
 			
 			window.Document = this;
@@ -641,28 +707,41 @@ namespace MonoDevelop.Ide.Gui
 			}
 			e.Document.RunWhenLoaded (action);
 		}
-		
+
+		TypeSystemService.ProjectContentWrapper oldWrapper;
 		internal void SetProject (Project project)
 		{
 			if (Window.ViewContent.Project == project)
 				return;
-			while (editorExtension != null) {
-				try {
-					editorExtension.Dispose ();
-				} catch (Exception ex) {
-					LoggingService.LogError ("Exception while disposing extension:" + editorExtension, ex);
-				}
-				editorExtension = editorExtension.Next as TextEditorExtension;
-			}
-			editorExtension = null;
+			DetachExtensionChain ();
 			ISupportsProjectReload pr = GetContent<ISupportsProjectReload> ();
 			if (pr != null) {
+				// Unsubscribe project events
+				if (Window.ViewContent.Project != null)
+					Window.ViewContent.Project.Modified -= HandleProjectModified;
 				Window.ViewContent.Project = project;
 				pr.Update (project);
 			}
 			if (project != null)
 				project.Modified += HandleProjectModified;
-			OnDocumentAttached ();
+			InitializeExtensionChain ();
+			StartReparseThread ();
+
+			if (oldWrapper != null) {
+				oldWrapper.InLoadChanged -= HandleInLoadChanged;
+				oldWrapper = null;
+			}
+
+			if (project != null) {
+				var wrapper = TypeSystemService.GetProjectContentWrapper (project);
+				wrapper.InLoadChanged += HandleInLoadChanged;
+				oldWrapper = wrapper;
+			}
+		}
+
+		void HandleInLoadChanged (object sender, EventArgs e)
+		{
+			StartReparseThread ();
 		}
 
 		void HandleProjectModified (object sender, SolutionItemModifiedEventArgs e)
@@ -688,28 +767,35 @@ namespace MonoDevelop.Ide.Gui
 				var editor = Editor;
 				if (editor == null)
 					return null;
-				lock (TypeSystemService.FilesSkippedInParseThread) {
-					TypeSystemService.FilesSkippedInParseThread.Add (currentParseFile);
-				}
+				TypeSystemService.AddSkippedFile (currentParseFile);
 				string currentParseText = editor.Text;
 				this.parsedDocument = TypeSystemService.ParseFile (Project, currentParseFile, editor.Document.MimeType, currentParseText);
 				if (Project == null && this.parsedDocument != null) {
-					singleFileContext = GetProjectContext ().UpdateProjectContent (singleFileContext.GetFile (currentParseFile), parsedDocument.ParsedFile);
+					singleFileContext = GetProjectContext ().AddOrUpdateFiles (parsedDocument.ParsedFile);
 				}
 			} finally {
+
 				OnDocumentParsed (EventArgs.Empty);
 			}
 			return this.parsedDocument;
 		}
 
-		static readonly Lazy<IUnresolvedAssembly> mscorlib = new Lazy<IUnresolvedAssembly> ( () => new CecilLoader ().LoadAssemblyFile (typeof (object).Assembly.Location));
-		static readonly Lazy<IUnresolvedAssembly> systemCore = new Lazy<IUnresolvedAssembly>( () => new CecilLoader ().LoadAssemblyFile (typeof (System.Linq.Enumerable).Assembly.Location));
-		static readonly Lazy<IUnresolvedAssembly> system = new Lazy<IUnresolvedAssembly>( () => new CecilLoader ().LoadAssemblyFile (typeof (System.Uri).Assembly.Location));
+		static readonly Lazy<IUnresolvedAssembly> mscorlib = new Lazy<IUnresolvedAssembly> ( () => new IkvmLoader ().LoadAssemblyFile (typeof (object).Assembly.Location));
+		static readonly Lazy<IUnresolvedAssembly> systemCore = new Lazy<IUnresolvedAssembly>( () => new IkvmLoader ().LoadAssemblyFile (typeof (System.Linq.Enumerable).Assembly.Location));
+		static readonly Lazy<IUnresolvedAssembly> system = new Lazy<IUnresolvedAssembly>( () => new IkvmLoader ().LoadAssemblyFile (typeof (System.Uri).Assembly.Location));
 
 		static IUnresolvedAssembly Mscorlib { get { return mscorlib.Value; } }
 		static IUnresolvedAssembly SystemCore { get { return systemCore.Value; } }
 		static IUnresolvedAssembly System { get { return system.Value; } }
 
+		public bool IsProjectContextInUpdate {
+			get {
+				if (Project == null)
+					return false;
+				return TypeSystemService.GetProjectContentWrapper (Project).InLoad;
+			}
+		}
+
 		public virtual IProjectContent GetProjectContext ()
 		{
 			if (Project == null) {
@@ -718,7 +804,7 @@ namespace MonoDevelop.Ide.Gui
 					singleFileContext = singleFileContext.AddAssemblyReferences (new [] { Mscorlib, System, SystemCore });
 				}
 				if (parsedDocument != null)
-					return singleFileContext.UpdateProjectContent (null, parsedDocument.ParsedFile);
+					return singleFileContext.AddOrUpdateFiles (parsedDocument.ParsedFile);
 				return singleFileContext;
 			}
 			
@@ -732,6 +818,8 @@ namespace MonoDevelop.Ide.Gui
 			// very inefficient. Do it after a small delay instead, so several changes can
 			// be parsed at the same time.
 			string currentParseFile = FileName;
+			if (string.IsNullOrEmpty (currentParseFile))
+				return;
 			CancelParseTimeout ();
 			
 			parseTimeout = GLib.Timeout.Add (ParseDelay, delegate {
@@ -741,9 +829,7 @@ namespace MonoDevelop.Ide.Gui
 				string currentParseText = editor.Text;
 				string mimeType = editor.Document.MimeType;
 				ThreadPool.QueueUserWorkItem (delegate {
-					lock (TypeSystemService.FilesSkippedInParseThread) {
-						TypeSystemService.FilesSkippedInParseThread.Add (currentParseFile);
-					}
+					TypeSystemService.AddSkippedFile (currentParseFile);
 					var currentParsedDocument = TypeSystemService.ParseFile (Project, currentParseFile, mimeType, currentParseText);
 					Application.Invoke (delegate {
 						// this may be called after the document has closed, in that case the OnDocumentParsed event shouldn't be invoked.
@@ -782,7 +868,7 @@ namespace MonoDevelop.Ide.Gui
 				window.ViewContent.Project = null;
 		}
 		
-		protected virtual void OnDocumentParsed (EventArgs e)
+		void OnDocumentParsed (EventArgs e)
 		{
 			EventHandler handler = this.DocumentParsed;
 			if (handler != null)
@@ -797,7 +883,10 @@ namespace MonoDevelop.Ide.Gui
 		
 		public string[] CommentTags {
 			get {
-				return GetCommentTags (FileName);
+				if (IsFile)
+					return GetCommentTags (FileName);
+				else
+					return null;
 			}
 		}
 		
@@ -836,7 +925,6 @@ namespace MonoDevelop.Ide.Gui
 //			return MonoDevelop.Projects.CodeGeneration.CodeGenerator.CreateGenerator (Editor.Document.MimeType, 
 //				Editor.Options.TabsToSpaces, Editor.Options.TabSize, Editor.EolMarker);
 //		}
-
 	}
 	
 	
