@@ -46,12 +46,7 @@ namespace MonoDevelop.Ide.Projects.OptionPanels
 	internal class RuntimeOptionsPanel : ItemOptionsPanel
 	{
 		RuntimeOptionsPanelWidget widget;
-		
-		public override bool IsVisible ()
-		{
-			return ConfiguredProject is DotNetProject;
-		}
-		
+
 		public override Widget CreatePanelWidget()
 		{
 			return (widget = new RuntimeOptionsPanelWidget ((DotNetProject)ConfiguredProject, ItemConfigurations));
@@ -115,7 +110,7 @@ namespace MonoDevelop.Ide.Projects.OptionPanels
 		}
 
 		public void Store ()
-		{	
+		{
 			if (project == null || runtimeVersionCombo.Active == -1)
 				return;
 			project.TargetFramework = frameworks [runtimeVersionCombo.Active];
