@@ -36,65 +36,56 @@ using System.Xml;
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide.Gui.Content;
 
 namespace MonoDevelop.Ide.StandardHeader
 {
 	public static class StandardHeaderService
 	{
-	
-		static string[] GetComment (string language)
+		public static string GetHeader (SolutionItem policyParent, string fileName, bool newFile)
 		{
-			ILanguageBinding binding = LanguageBindingService.GetBindingPerLanguageName (language);
-			if (binding != null) {
-				if (!String.IsNullOrEmpty (binding.SingleLineCommentTag))
-					return new string[] { binding.SingleLineCommentTag };
-				if (!String.IsNullOrEmpty (binding.BlockCommentStartTag) && !String.IsNullOrEmpty (binding.BlockCommentEndTag))
-					return new string[] { binding.BlockCommentStartTag, binding.BlockCommentEndTag };
-			}
-			return null;
-		}
-		
-		public static string GetHeader (SolutionItem policyParent, string language, string fileName, bool newFile)
-		{
-			StandardHeaderPolicy policy = policyParent != null
-				? policyParent.Policies.Get<StandardHeaderPolicy> ()
-				: MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<StandardHeaderPolicy> ();
-			AuthorInformation authorInfo = IdeApp.Workspace.GetAuthorInformation (policyParent);
+			StandardHeaderPolicy headerPolicy = policyParent != null ? policyParent.Policies.Get<StandardHeaderPolicy> () : MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<StandardHeaderPolicy> ();
+			TextStylePolicy textPolicy = policyParent != null ? policyParent.Policies.Get<TextStylePolicy> ("text/plain") : MonoDevelop.Projects.Policies.PolicyService.GetDefaultPolicy<TextStylePolicy> ("text/plain");
+			AuthorInformation authorInfo = policyParent != null ? policyParent.AuthorInformation : AuthorInformation.Default;
 			
-			return GetHeader (authorInfo, policy, language, fileName, newFile);
+			return GetHeader (authorInfo, headerPolicy, textPolicy, fileName, newFile);
 		}
 		
-		public static string GetHeader (AuthorInformation authorInfo, StandardHeaderPolicy policy,
-		                                string language, string fileName, bool newFile)
+		public static string GetHeader (AuthorInformation authorInfo, StandardHeaderPolicy policy, TextStylePolicy textPolicy,
+		                                string fileName, bool newFile)
 		{
-			string[] comment = GetComment (language);
+			string[] comment = Document.GetCommentTags (fileName);
 			if (comment == null)
 				return "";
-				
+			
 			if (string.IsNullOrEmpty (policy.Text) || (newFile && !policy.IncludeInNewFiles))
 				return "";
 			
 			string result;
+			string eolMarker = TextStylePolicy.GetEolMarker (textPolicy.EolMarker);
 			
 			if (comment.Length == 1) {
 				string cmt = comment[0];
 				//make sure there's a space between the comment char and the license text
-				if (!char.IsWhiteSpace (cmt[cmt.Length -1]))
+				if (!char.IsWhiteSpace (cmt[cmt.Length - 1]))
 					cmt = cmt + " ";
-			
+				
 				StringBuilder sb = new StringBuilder (policy.Text.Length);
 				string[] lines = policy.Text.Split ('\n');
 				foreach (string line in lines) {
+					if (string.IsNullOrWhiteSpace (line)) {
+						sb.Append (cmt.TrimEnd ());
+						sb.Append (eolMarker);
+						continue;
+					}
 					sb.Append (cmt);
 					sb.Append (line);
-					// the text editor should take care of conversions to preferred newline char
-					sb.Append ('\n');
+					sb.Append (eolMarker);
 				}
 				result = sb.ToString ();
 			} else {
 				//multiline comment
 				result = String.Concat (comment[0], "\n", policy.Text, "\n", comment[1], "\n");
-				
 			}
 			
 			return StringParserService.Parse (result, new string[,] { 
