@@ -192,23 +192,25 @@ namespace MonoDevelop.Ide.Tasks
 		void LoadSolutionContents (Solution sln)
 		{
 			loadedSlns.Add (sln);
-			
-			// Load all tags that are stored in pidb files
-			foreach (Project p in sln.GetAllProjects ()) {
-				var pContext = TypeSystemService.GetProjectContentWrapper (p);
-				if (pContext == null)
-					continue;
-				var tags = pContext.GetExtensionObject<ProjectCommentTags> ();
-				if (tags == null) {
-					tags = new ProjectCommentTags ();
-					pContext.UpdateExtensionObject (tags);
-					tags.Update (pContext.Project);
-				} else {
-					foreach (var kv in tags.Tags) {
-						UpdateCommentTags (sln, kv.Key, kv.Value);
+			System.Threading.ThreadPool.QueueUserWorkItem (delegate {
+				// Load all tags that are stored in pidb files
+				foreach (Project p in sln.GetAllProjects ()) {
+					var pContext = TypeSystemService.GetProjectContentWrapper (p);
+					if (pContext == null) {
+						continue;
+					}
+					var tags = pContext.GetExtensionObject<ProjectCommentTags> ();
+					if (tags == null) {
+						tags = new ProjectCommentTags ();
+						pContext.UpdateExtensionObject (tags);
+						tags.Update (pContext.Project);
+					} else {
+						foreach (var kv in tags.Tags) {
+							UpdateCommentTags (sln, kv.Key, kv.Value);
+						}
 					}
 				}
-			}
+			});
 		}
 		
 		
@@ -254,22 +256,26 @@ namespace MonoDevelop.Ide.Tasks
 			List<Task> newTasks = new List<Task> ();
 			if (tagComments != null) {  
 				foreach (Tag tag in tagComments) {
-					if (!priorities.ContainsKey (tag.Key))
-						continue;
-					
-					//prepend the tag if it's not already there
+					TaskPriority priority;
 					string desc = tag.Text.Trim ();
-					if (!desc.StartsWith (tag.Key)) {
-						if (desc.StartsWith (":"))
-							desc = tag.Key + desc;
-						else if (tag.Key.EndsWith (":"))
-							desc = tag.Key + " " + desc;
-						else
-							desc = tag.Key + ": " + desc;
+
+					if (!priorities.TryGetValue (tag.Key, out priority)) {
+						if (!Enum.TryParse (tag.Key, out priority))
+							priority = TaskPriority.High;
+					} else {
+						//prepend the tag if it's not already there
+						if (!desc.StartsWith (tag.Key, StringComparison.Ordinal)) {
+							if (desc.StartsWith (":", StringComparison.Ordinal))
+								desc = tag.Key + desc;
+							else if (tag.Key.EndsWith (":", StringComparison.Ordinal))
+								desc = tag.Key + " " + desc;
+							else
+								desc = tag.Key + ": " + desc;
+						}
 					}
 					
 					Task t = new Task (fileName, desc, tag.Region.BeginColumn, tag.Region.BeginLine,
-					                   TaskSeverity.Information, priorities[tag.Key], wob);
+					                   TaskSeverity.Information, priority, wob);
 					newTasks.Add (t);
 				}
 			}
