@@ -38,6 +38,7 @@ using ICSharpCode.NRefactory;
 using MonoDevelop.Ide.TypeSystem;
 using MonoDevelop.Ide;
 using ICSharpCode.NRefactory.CSharp.TypeSystem;
+using System.Threading.Tasks;
 
 namespace MonoDevelop.Ide.TypeSystem
 {
@@ -66,8 +67,10 @@ namespace MonoDevelop.Ide.TypeSystem
 			if (newMember is CodeMemberMethod)
 				options.BracingStyle = "C";
 			generator.GenerateCodeFromMember (newMember, sw, options);
-			
-			suitableInsertionPoint.Insert (data, sw.ToString ());
+
+			var code = sw.ToString ();
+			if (!string.IsNullOrEmpty (code))
+				suitableInsertionPoint.Insert (data, code);
 			if (!isOpen) {
 				try {
 					File.WriteAllText (type.Region.FileName, data.Text);
@@ -104,6 +107,62 @@ namespace MonoDevelop.Ide.TypeSystem
 				}
 			}
 		}
+
+		public static Task<bool> InsertMemberWithCursor (
+			string operation, ITypeDefinition parentType, IUnresolvedTypeDefinition part,
+			IUnresolvedMember newMember, bool implementExplicit = false)
+		{
+			var tcs = new TaskCompletionSource<bool>();
+			if (parentType == null)
+				return tcs.Task;
+			part = part ?? parentType.Parts.FirstOrDefault ();
+			if (part == null)
+				return tcs.Task;
+
+			var loadedDocument = IdeApp.Workbench.OpenDocument (part.Region.FileName);
+			loadedDocument.RunWhenLoaded (delegate {
+				var editor = loadedDocument.Editor;
+				var loc = part.Region.Begin;
+				var parsedDocument = loadedDocument.UpdateParseDocument ();
+				var declaringType = parsedDocument.GetInnermostTypeDefinition (loc);
+				var mode = new InsertionCursorEditMode (
+					editor.Parent,
+					CodeGenerationService.GetInsertionPoints (loadedDocument, declaringType));
+				if (mode.InsertionPoints.Count == 0) {
+					MessageService.ShowError (
+						GettextCatalog.GetString ("No valid insertion point can be found in type '{0}'.", declaringType.Name)
+						);
+					return;
+				}
+				var suitableInsertionPoint = GetSuitableInsertionPoint (mode.InsertionPoints, part, newMember);
+				if (suitableInsertionPoint != null)
+					mode.CurIndex = mode.InsertionPoints.IndexOf (suitableInsertionPoint);
+				else
+					mode.CurIndex = 0;
+
+				var helpWindow = new Mono.TextEditor.PopupWindow.InsertionCursorLayoutModeHelpWindow () {
+					TransientFor = IdeApp.Workbench.RootWindow,
+					TitleText = operation
+				};
+				helpWindow.Shown += (s, a) => DesktopService.RemoveWindowShadow (helpWindow);
+				mode.HelpWindow = helpWindow;
+
+				mode.StartMode ();
+				mode.Exited += delegate(object s, InsertionCursorEventArgs iCArgs) {
+					if (!iCArgs.Success) {
+						tcs.SetResult (false);
+						return;
+					}
+					var generator = CreateCodeGenerator (editor, parentType.Compilation);
+					generator.IndentLevel = CalculateBodyIndentLevel (declaringType);
+					var generatedCode = generator.CreateMemberImplementation (parentType, part, newMember, implementExplicit);
+					mode.InsertionPoints[mode.CurIndex].Insert (editor, generatedCode.Code);
+					tcs.SetResult (true);
+				};
+			});
+
+			return tcs.Task;
+		}
 		
 		public static int CalculateBodyIndentLevel (IUnresolvedTypeDefinition declaringType)
 		{
@@ -114,7 +173,7 @@ namespace MonoDevelop.Ide.TypeSystem
 				indentLevel++;
 				declaringType = declaringType.DeclaringTypeDefinition;
 			}
-			var file = declaringType.ParsedFile as CSharpParsedFile;
+			var file = declaringType.UnresolvedFile as CSharpUnresolvedFile;
 			if (file == null)
 				return indentLevel;
 			var scope = file.GetUsingScope (declaringType.Region.Begin);
@@ -249,7 +308,7 @@ namespace MonoDevelop.Ide.TypeSystem
 				if (line != null) {
 					var lineOffset = line.Offset;
 					col = Math.Min (line.Length, col);
-					while (lineOffset + col - 2 >= 0 && char.IsWhiteSpace (data.GetCharAt (lineOffset + col - 2)))
+					while (lineOffset + col - 2 >= 0 && col > 1 && char.IsWhiteSpace (data.GetCharAt (lineOffset + col - 2)))
 						col--;
 				}
 				result.Add (new InsertionPoint (new DocumentLocation (type.BodyRegion.EndLine, col), insertLine, NewLineInsertion.Eol));
@@ -262,6 +321,8 @@ namespace MonoDevelop.Ide.TypeSystem
 				result.Add (new InsertionPoint (new DocumentLocation (region.Region.EndLine + 1, 1), NewLineInsertion.Eol, NewLineInsertion.Eol));
 			}
 			result.Sort ((left, right) => left.Location.CompareTo (right.Location));
+//			foreach (var res in result)
+//				Console.WriteLine (res);
 			return result;
 		}
 
@@ -313,34 +374,45 @@ namespace MonoDevelop.Ide.TypeSystem
 			if (nextLine == null) // check for 1 line case.
 				return new InsertionPoint (new DocumentLocation (line, column + 1), NewLineInsertion.BlankLine, NewLineInsertion.BlankLine);
 			
-			for (int i = nextLine.Offset; i < nextLine.Offset + nextLine.Length; i++) {
+			for (int i = nextLine.Offset; i < nextLine.EndOffset; i++) {
 				char ch = doc.GetCharAt (i);
 				if (!char.IsWhiteSpace (ch)) {
 					// case2: next line contains non ws chars.
 					return new InsertionPoint (new DocumentLocation (line + 1, 1), NewLineInsertion.Eol, NewLineInsertion.BlankLine);
 				}
 			}
-			// case3: whitespace line
+
+			DocumentLine nextLine2 = doc.GetLine (line + 2);
+			if (nextLine2 != null) {
+				for (int i = nextLine2.Offset; i < nextLine2.EndOffset; i++) {
+					char ch = doc.GetCharAt (i);
+					if (!char.IsWhiteSpace (ch)) {
+						// case3: one blank line
+						return new InsertionPoint (new DocumentLocation (line + 1, 1), NewLineInsertion.Eol, NewLineInsertion.Eol);
+					}
+				}
+			}
+			// case4: more than 1 blank line
 			return new InsertionPoint (new DocumentLocation (line + 1, 1), NewLineInsertion.Eol, NewLineInsertion.None);
 		}
 		
 		static InsertionPoint GetSuitableInsertionPoint (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls, IUnresolvedMember member)
 		{
 			var mainPart = cls;
-			switch (member.EntityType) {
-			case EntityType.Field:
+			switch (member.SymbolKind) {
+			case SymbolKind.Field:
 				return GetNewFieldPosition (points, mainPart);
-			case EntityType.Method:
-			case EntityType.Constructor:
-			case EntityType.Destructor:
-			case EntityType.Operator:
+			case SymbolKind.Method:
+			case SymbolKind.Constructor:
+			case SymbolKind.Destructor:
+			case SymbolKind.Operator:
 				return GetNewMethodPosition (points, mainPart);
-			case EntityType.Event:
+			case SymbolKind.Event:
 				return GetNewEventPosition (points, mainPart);
-			case EntityType.Property:
+			case SymbolKind.Property:
 				return GetNewPropertyPosition (points, mainPart);
 			}
-			throw new InvalidOperationException ("Invalid member type: " + member.EntityType);
+			throw new InvalidOperationException ("Invalid member type: " + member.SymbolKind);
 		}
 		
 		static InsertionPoint GetSuitableInsertionPoint (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls, CodeTypeMember mem)
