@@ -35,17 +35,25 @@ namespace MonoDevelop.Ide.Updater
 {
 	public static class UpdateService
 	{
+		static readonly TimeSpan AutoUpdateSpan = TimeSpan.FromDays (3);
+
 		static UpdateService ()
 		{
 			NotifyAddinUpdates = true;
+			ScheduleUpdateRun ();
+		}
+
+		static void ScheduleUpdateRun ()
+		{
+			new Timer (_ => CheckForUpdates (true), null, AutoUpdateSpan, AutoUpdateSpan);
 		}
 		
 		public static bool AutoCheckForUpdates {
 			get {
-				return PropertyService.Get ("MonoDevelop.Ide.AddinUpdater.CkeckForUpdates", true);
+				return PropertyService.Get ("MonoDevelop.Ide.AddinUpdater.CheckForUpdates", true);
 			}
 			set {
-				PropertyService.Set ("MonoDevelop.Ide.AddinUpdater.CkeckForUpdates", value);
+				PropertyService.Set ("MonoDevelop.Ide.AddinUpdater.CheckForUpdates", value);
 			}
 		}
 		
