@@ -34,28 +34,38 @@ using System.Xml.Linq;
 
 namespace MonoDevelop.Ide.WelcomePage
 {
-	class WelcomePageRecentProjectsList : VBox
+	public class WelcomePageRecentProjectsList : WelcomePageSection
 	{
 		bool destroyed;
 		EventHandler recentChangesHandler;
-		
+		VBox box;
 		int itemCount = 10;
+		Gdk.Pixbuf openProjectIcon;
+		Gdk.Pixbuf newProjectIcon;
 		
-		public WelcomePageRecentProjectsList (XElement el)
+		public WelcomePageRecentProjectsList (string title = null, int count = 10): base (title)
 		{
-			var countAtt = el.Attribute ("count");
-			if (countAtt != null)
-				itemCount = (int) countAtt;
+			openProjectIcon = Gdk.Pixbuf.LoadFromResource ("open_solution.png");
+			newProjectIcon = Gdk.Pixbuf.LoadFromResource ("new_solution.png");
+
+			box = new VBox ();
+
+			itemCount = count;
 			
 			recentChangesHandler = DispatchService.GuiDispatch (new EventHandler (RecentFilesChanged));
 			DesktopService.RecentFiles.Changed += recentChangesHandler;
 			RecentFilesChanged (null, null);
+
+			SetContent (box);
+			TitleAlignment.BottomPadding = Styles.WelcomeScreen.Pad.Solutions.LargeTitleMarginBottom;
+			ContentAlignment.LeftPadding = 0;
+			ContentAlignment.RightPadding = 0;
 		}
 		
-		public override void Destroy ()
+		protected override void OnDestroyed ()
 		{
 			destroyed = true;
-			base.Destroy ();
+			base.OnDestroyed ();
 			DesktopService.RecentFiles.Changed -= recentChangesHandler;
 		}
 
@@ -65,17 +75,35 @@ namespace MonoDevelop.Ide.WelcomePage
 			if (destroyed)
 				return;
 			
-			foreach (var c in Children) {
-				this.Remove (c);
+			foreach (var c in box.Children) {
+				box.Remove (c);
 				c.Destroy ();
 			}
+
+			Gtk.HBox hbox = new HBox ();
+			var btn = new WelcomePageListButton (GettextCatalog.GetString ("New..."), null, newProjectIcon, "monodevelop://MonoDevelop.Ide.Commands.FileCommands.NewProject");
+			btn.WidthRequest = (int) (Styles.WelcomeScreen.Pad.Solutions.SolutionTile.Width / 2.3);
+			btn.BorderPadding = 6;
+			btn.LeftTextPadding = 24;
+			hbox.PackStart (btn, false, false, 0);
+
+			btn = new WelcomePageListButton (GettextCatalog.GetString ("Open..."), null, openProjectIcon, "monodevelop://MonoDevelop.Ide.Commands.FileCommands.OpenFile");
+			btn.WidthRequest = (int) (Styles.WelcomeScreen.Pad.Solutions.SolutionTile.Width / 2.3);
+			btn.BorderPadding = 6;
+			btn.LeftTextPadding = 24;
+			hbox.PackStart (btn, false, false, 0);
+
+			box.PackStart (hbox, false, false, 0);
 			
 			//TODO: pinned files
 			foreach (var recent in DesktopService.RecentFiles.GetProjects ().Take (itemCount)) {
 				var filename = recent.FileName;
 				var accessed = recent.TimeStamp;
-				var button = new WelcomePageLinkButton (recent.DisplayName, "project://" + filename);
-				button.Ellipsize = Pango.EllipsizeMode.Middle;
+				var pixbuf = ImageService.GetPixbuf (GetIcon (filename), IconSize.Dnd);
+				var button = new WelcomePageListButton (recent.DisplayName, System.IO.Path.GetDirectoryName (filename), pixbuf, "project://" + filename);
+				button.BorderPadding = 2;
+				button.AllowPinning = true;
+				button.Pinned = recent.IsFavorite;
 				//FIXME: update times as needed. currently QueryTooltip causes crashes on Windows
 				//button.QueryTooltip += delegate (object o, QueryTooltipArgs args) {
 				//	args.Tooltip.Text = filename + "\n" + TimeSinceEdited (accessed);
@@ -83,10 +111,14 @@ namespace MonoDevelop.Ide.WelcomePage
 				//};
 				//button.HasTooltip = true;
 				button.TooltipText = filename + "\n" + TimeSinceEdited (accessed);
-				button.Icon = GetIcon (filename);
-				this.PackStart (button, false, false, 0);
+				box.PackStart (button, false, false, 0);
+				button.PinClicked += delegate {
+					DesktopService.RecentFiles.SetFavoriteFile (filename, button.Pinned);
+				};
 			}
 			
+
+
 			this.ShowAll ();
 		}
 		
