@@ -51,6 +51,7 @@ using MonoDevelop.Core.Instrumentation;
 using System.Diagnostics;
 using MonoDevelop.Projects;
 using System.Collections.Generic;
+using MonoDevelop.Core.LogReporting;
 
 namespace MonoDevelop.Ide
 {
@@ -74,13 +75,19 @@ namespace MonoDevelop.Ide
 		{
 			Counters.Initialization.BeginTiming ();
 			
-			if (options.LogCounters) {
-				string logFile = Path.Combine (Environment.CurrentDirectory, "monodevelop.clog");
+			if (options.PerfLog) {
+				string logFile = Path.Combine (Environment.CurrentDirectory, "monodevelop.perf-log");
 				LoggingService.LogInfo ("Logging instrumentation service data to file: " + logFile);
 				InstrumentationService.StartAutoSave (logFile, 1000);
 			}
+
+			//ensure native libs initialized before we hit anything that p/invokes
+			Platform.Initialize ();
 			
 			Counters.Initialization.Trace ("Initializing GTK");
+			if (Platform.IsWindows) {
+				CheckWindowsGtk ();
+			}
 			SetupExceptionManager ();
 			
 			try {
@@ -88,11 +95,16 @@ namespace MonoDevelop.Ide
 			} catch (Exception ex) {
 				LoggingService.LogError ("Error initialising GLib logging.", ex);
 			}
-			
-			//OSXFIXME
+
+			SetupTheme ();
+
 			var args = options.RemainingArgs.ToArray ();
-			Gtk.Application.Init ("monodevelop", ref args);
-			
+			Gtk.Application.Init (BrandingService.ApplicationName, ref args);
+
+			FilePath p = typeof(IdeStartup).Assembly.Location;
+			Assembly.LoadFrom (p.ParentDirectory.Combine ("Xwt.Gtk.dll"));
+			Xwt.Application.InitializeAsGuest (Xwt.ToolkitType.Gtk);
+
 			//default to Windows IME on Windows
 			if (Platform.IsWindows && Mono.TextEditor.GtkWorkarounds.GtkMinorVersion >= 16) {
 				var settings = Gtk.Settings.Default;
@@ -116,7 +128,7 @@ namespace MonoDevelop.Ide
 			
 			// If a combine was specified, force --newwindow.
 			
-			if(!options.NewWindow && startupInfo.HasFiles) {
+			if (!options.NewWindow && startupInfo.HasFiles) {
 				Counters.Initialization.Trace ("Pre-Initializing Runtime to load files in existing window");
 				Runtime.Initialize (true);
 				foreach (var file in startupInfo.RequestedFileList) {
@@ -127,29 +139,39 @@ namespace MonoDevelop.Ide
 				}
 			}
 			
+			Counters.Initialization.Trace ("Initializing Runtime");
+			Runtime.Initialize (true);
+
+			Counters.Initialization.Trace ("Initializing theme and splash window");
+
 			DefaultTheme = Gtk.Settings.Default.ThemeName;
-			if (!string.IsNullOrEmpty (IdeApp.Preferences.UserInterfaceTheme))
-				Gtk.Settings.Default.ThemeName = IdeApp.Preferences.UserInterfaceTheme;
+			if (!string.IsNullOrEmpty (IdeApp.Preferences.UserInterfaceTheme)) {
+				string theme;
+				if (!ValidateGtkTheme (IdeApp.Preferences.UserInterfaceTheme, out theme))
+					return 1;
+				Gtk.Settings.Default.ThemeName = theme;
+			}
 			
 			//don't show the splash screen on the Mac, so instead we get the expected "Dock bounce" effect
 			//this also enables the Mac platform service to subscribe to open document events before the GUI loop starts.
 			if (Platform.IsMac)
-				options.NoLogo = true;
+				options.NoSplash = true;
 			
-			IProgressMonitor monitor;
-			
-			if (options.NoLogo) {
+			IProgressMonitor monitor = null;
+			if (!options.NoSplash) {
+				try {
+					monitor = new SplashScreenForm ();
+					((SplashScreenForm)monitor).ShowAll ();
+				} catch (Exception ex) {
+					LoggingService.LogError ("Failed to create splash screen", ex);
+				}
+			}
+			if (monitor == null) {
 				monitor = new MonoDevelop.Core.ProgressMonitoring.ConsoleProgressMonitor ();
-			} else {
-				monitor = SplashScreenForm.SplashScreen;
-				SplashScreenForm.SplashScreen.ShowAll ();
 			}
 			
-			Counters.Initialization.Trace ("Initializing Runtime");
-			monitor.BeginTask (GettextCatalog.GetString ("Starting " + BrandingService.ApplicationName), 3);
-			monitor.Step (1);
-			Runtime.Initialize (true);
-			
+			monitor.BeginTask (GettextCatalog.GetString ("Starting {0}", BrandingService.ApplicationName), 2);
+
 			//make sure that the platform service is initialised so that the Mac platform can subscribe to open-document events
 			Counters.Initialization.Trace ("Initializing Platform Service");
 			DesktopService.Initialize ();
@@ -189,13 +211,6 @@ namespace MonoDevelop.Ide
 				version += "." + Assembly.GetEntryAssembly ().GetName ().Version.Build;
 			if (Assembly.GetEntryAssembly ().GetName ().Version.Revision != 0)
 				version += "." + Assembly.GetEntryAssembly ().GetName ().Version.Revision;
-			
-			// System checks
-			if (!CheckBug77135 ())
-				return 1;
-			
-			if (!CheckQtCurve ())
-				return 1;
 
 			CheckFileWatcher ();
 			
@@ -209,12 +224,12 @@ namespace MonoDevelop.Ide
 				
 				if (errorsList.Count > 0) {
 					if (monitor is SplashScreenForm)
-						SplashScreenForm.SplashScreen.Hide ();
+						((SplashScreenForm)monitor).Hide ();
 					AddinLoadErrorDialog dlg = new AddinLoadErrorDialog ((AddinError[]) errorsList.ToArray (typeof(AddinError)), false);
 					if (!dlg.Run ())
 						return 1;
 					if (monitor is SplashScreenForm)
-						SplashScreenForm.SplashScreen.Show ();
+						((SplashScreenForm)monitor).Show ();
 					reportedFailures = errorsList.Count;
 				}
 				
@@ -239,7 +254,7 @@ namespace MonoDevelop.Ide
 			if (error != null) {
 				LoggingService.LogFatalError (null, error);
 				MessageService.ShowException (error,
-				                              GettextCatalog.GetString ("MonoDevelop failed to start. The following error has been reported: ") + error.Message);
+				                              BrandingService.BrandApplicationName (GettextCatalog.GetString ("MonoDevelop failed to start. The following error has been reported: ") + error.Message));
 				return 1;
 			}
 
@@ -268,10 +283,6 @@ namespace MonoDevelop.Ide
 				
 			AddinManager.AddExtensionNodeHandler("/MonoDevelop/Ide/InitCompleteHandlers", OnExtensionChanged);
 			
-			string logAgentEnabled = Environment.GetEnvironmentVariable ("MONODEVELOP_LOG_AGENT_ENABLED");
-			if (string.Equals (logAgentEnabled, "true", StringComparison.OrdinalIgnoreCase))
-				LaunchCrashMonitoringService ();
-			
 			IdeApp.Run ();
 			
 			// unloading services
@@ -283,6 +294,46 @@ namespace MonoDevelop.Ide
 			
 			return 0;
 		}
+
+		void SetupTheme ()
+		{
+			// Use the bundled gtkrc only if the Xamarin theme is installed
+			if (File.Exists (Path.Combine (Gtk.Rc.ModuleDir, "libxamarin.so")) || File.Exists (Path.Combine (Gtk.Rc.ModuleDir, "libxamarin.dll"))) {
+				var gtkrc = "gtkrc";
+				if (Platform.IsWindows) {
+					gtkrc += ".win32";
+				} else if (Platform.IsMac) {
+					gtkrc += ".mac";
+				}
+				Environment.SetEnvironmentVariable ("GTK2_RC_FILES", PropertyService.EntryAssemblyPath.Combine (gtkrc));
+			}
+		}
+
+		[System.Runtime.InteropServices.DllImport("kernel32.dll", CharSet = System.Runtime.InteropServices.CharSet.Unicode, SetLastError = true)]
+		[return: System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.Bool)]
+		static extern bool SetDllDirectory (string lpPathName);
+
+		static void CheckWindowsGtk ()
+		{
+			string location = null;
+			using (var key = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Xamarin\GtkSharp\InstallFolder")) {
+				if (key != null) {
+					location = key.GetValue (null) as string;
+				}
+			}
+			if (location == null || !File.Exists (Path.Combine (location, "bin", "libgtk-win32-2.0-0.dll"))) {
+				LoggingService.LogError ("Did not find registered GTK# installation");
+				return;
+			}
+			var path = Path.Combine (location, @"bin");
+			try {
+				if (SetDllDirectory (path)) {
+					return;
+				}
+			} catch (EntryPointNotFoundException) {
+			}
+			LoggingService.LogError ("Unable to set GTK# dll directory");
+		}
 		
 		public bool Initialized {
 			get { return initialized; }
@@ -308,45 +359,6 @@ namespace MonoDevelop.Ide
 				errorsList.Add (new AddinError (args.AddinId, args.Message, args.Exception, false));
 		}
 		
-		void LaunchCrashMonitoringService ()
-		{
-			string enabledKey = "MonoDevelop.CrashMonitoring.Enabled";
-			
-			if (Platform.IsMac) {
-				var crashmonitor = Path.Combine (PropertyService.EntryAssemblyPath, "MonoDevelopLogAgent.app");
-				var pid = Process.GetCurrentProcess ().Id;
-				var logPath = UserProfile.Current.LogDir.Combine ("LogAgent");
-				var email = FeedbackService.ReporterEMail;
-				var logOnly = "";
-				
-				var fileInfo = new FileInfo (Path.Combine (logPath, "crashlogs.xml"));
-				if (!PropertyService.HasValue (enabledKey) && fileInfo.Exists && fileInfo.Length > 0) {
-					var result = MessageService.AskQuestion ("A crash has been detected",
-						"MonoDevelop has crashed recently. Details of this crash along with your configured " +
-						"email address can be uploaded to Xamarin to help diagnose the issue. This information " +
-						"will be used to help diagnose the crash and notify you of potential workarounds " +
-						"or fixes. Do you wish to upload this information?",
-						AlertButton.Yes, AlertButton.No);
-					PropertyService.Set (enabledKey, result == AlertButton.Yes);
-				}
-				
-				if (string.IsNullOrEmpty (email))
-					email = AuthorInformation.Default.Email;
-				if (string.IsNullOrEmpty (email))
-					email = "unknown@email.com";
-				if (!PropertyService.Get<bool> (enabledKey))
-					logOnly = "-logonly";
-
-				var psi = new ProcessStartInfo ("open", string.Format ("-a {0} -n --args -p {1} -l {2} -email {3} {4}", crashmonitor, pid, logPath, email, logOnly)) {
-					UseShellExecute = false,
-				};
-				Process.Start (psi);
-			}
-			//else {
-			//	LoggingService.LogError ("Could not launch crash reporter process. MonoDevelop will not be able to automatically report any crash information.");
-			//}
-		}
-
 		void ListenCallback (IAsyncResult state)
 		{
 			Socket sock = (Socket)state.AsyncState;
@@ -397,17 +409,46 @@ namespace MonoDevelop.Ide
 			IdeApp.Workbench.Present ();
 			return false;
 		}
-		
-		bool CheckQtCurve ()
+
+		internal readonly static string[] FailingGtkThemes = new string[] {
+			"QtCurve",
+			"oxygen-gtk"
+		};
+
+		internal static string[] gtkThemeFallbacks = new string[] {
+			"Gilouche", // SUSE
+			"Mint-X", // MINT
+			"Radiance", // Ubuntu 'light' theme (MD looks better with the light theme in 4.0 - if that changes switch this one)
+			"Clearlooks" // GTK theme
+		};
+
+		bool ValidateGtkTheme (string requestedTheme, out string validTheme)
 		{
-			if (Gtk.Settings.Default.ThemeName == "QtCurve") {
-				string msg = "QtCurve theme not supported";
-				string desc = "Your system is using the QtCurve GTK+ theme. This theme is known to cause stability issues in MonoDevelop. Please select another theme in the GTK+ Theme Selector.\n\nIf you click on Proceed, MonoDevelop will switch to the default GTK+ theme.";
-				AlertButton res = MessageService.GenericAlert (Gtk.Stock.DialogWarning, msg, desc, AlertButton.Cancel, AlertButton.Proceed);
-				if (res == AlertButton.Cancel)
-					return false;
-				Gtk.Settings.Default.ThemeName = "Gilouche";
+			foreach (var theme in FailingGtkThemes) {
+				if (string.Equals (requestedTheme, theme, StringComparison.OrdinalIgnoreCase)) {
+					string msg = theme +" theme not supported";
+					string desc = "Your system is using the " + theme + " GTK+ theme. This theme is known to cause stability issues in MonoDevelop. Please select another theme in the GTK+ Theme Selector.\n\nIf you click on Proceed, MonoDevelop will switch to the default GTK+ theme.";
+					AlertButton res = MessageService.GenericAlert (Gtk.Stock.DialogWarning, msg, desc, AlertButton.Cancel, AlertButton.Proceed);
+					if (res == AlertButton.Cancel) {
+						validTheme = null;
+						return false;
+					}
+					var themes = MonoDevelop.Ide.Gui.OptionPanels.IDEStyleOptionsPanelWidget.InstalledThemes;
+					string fallback = null;
+					foreach (string fb in gtkThemeFallbacks) {
+						var foundTheme = themes.FirstOrDefault (t => string.Compare (fb, t, StringComparison.OrdinalIgnoreCase) == 0);
+						if (foundTheme != null) {
+							fallback = foundTheme;
+							break;
+						}
+					}
+
+					validTheme = fallback ?? themes.FirstOrDefault () ?? requestedTheme;
+					return validTheme != null;
+				}
 			}
+
+			validTheme = requestedTheme;
 			return true;
 		}
 		
@@ -422,7 +463,7 @@ namespace MonoDevelop.Ide
 						string msg = "Inotify watch limit is too low (" + n + ").\n";
 						msg += "MonoDevelop will switch to managed file watching.\n";
 						msg += "See http://monodevelop.com/Inotify_Watches_Limit for more info.";
-						LoggingService.LogWarning (msg);
+						LoggingService.LogWarning (BrandingService.BrandApplicationName (msg));
 						Runtime.ProcessService.EnvironmentVariableOverrides["MONO_MANAGED_WATCHER"] = 
 							Environment.GetEnvironmentVariable ("MONO_MANAGED_WATCHER");
 						Environment.SetEnvironmentVariable ("MONO_MANAGED_WATCHER", "1");
@@ -432,83 +473,31 @@ namespace MonoDevelop.Ide
 				LoggingService.LogWarning ("There was a problem checking whether to use managed file watching", e);
 			}
 		}
-		
-		bool CheckBug77135 ()
-		{
-			try {
-				// Check for bug 77135. Some versions of gnome-vfs2 and libgda
-				// make MD crash in the file open dialog or in FileIconLoader.
-				// Only in Suse.
-				
-				string path = "/etc/SuSE-release";
-				if (!File.Exists (path))
-					return true;
-					
-				// Only run the check for SUSE 10
-				StreamReader sr = File.OpenText (path);
-				string txt = sr.ReadToEnd ();
-				sr.Close ();
-				
-				if (txt.IndexOf ("SUSE LINUX 10") == -1)
-					return true;
-					
-				string current_libgda;
-				string current_gnomevfs;
-				string required_libgda = "1.3.91.5.4";
-				string required_gnomevfs = "2.12.0.9.2";
-				
-				StringWriter sw = new StringWriter ();
-				ProcessWrapper pw = Runtime.ProcessService.StartProcess ("rpm", "--qf %{version}.%{release} -q libgda", null, sw, null, null);
-				pw.WaitForOutput ();
-				current_libgda = sw.ToString ().Trim (' ','\n');
-				
-				sw = new StringWriter ();
-				pw = Runtime.ProcessService.StartProcess ("rpm", "--qf %{version}.%{release} -q gnome-vfs2", null, sw, null, null);
-				pw.WaitForOutput ();
-				current_gnomevfs = sw.ToString ().Trim (' ','\n');
-				
-				bool fail1 = Addin.CompareVersions (current_libgda, required_libgda) == 1;
-				bool fail2 = Addin.CompareVersions (current_gnomevfs, required_gnomevfs) == 1;
-				
-				if (fail1 || fail2) {
-					string msg = GettextCatalog.GetString ("Some packages installed in your system are not compatible with MonoDevelop:\n");
-					if (fail1)
-						msg += "\nlibgda " + current_libgda + " ("+ GettextCatalog.GetString ("version required: {0}", required_libgda) + ")";
-					if (fail2)
-						msg += "\ngnome-vfs2 " + current_gnomevfs + " ("+ GettextCatalog.GetString ("version required: {0}", required_gnomevfs) + ")";
-					msg += "\n\n";
-					msg += GettextCatalog.GetString ("You need to upgrade the previous packages to start using MonoDevelop.");
-					
-					SplashScreenForm.SplashScreen.Hide ();
-					Gtk.MessageDialog dlg = new Gtk.MessageDialog (null, Gtk.DialogFlags.Modal, Gtk.MessageType.Error, Gtk.ButtonsType.Ok, msg);
-					dlg.Run ();
-					dlg.Destroy ();
-					
-					return false;
-				} else
-					return true;
-			}
-			catch (Exception ex)
-			{
-				// Just ignore for now.
-				Console.WriteLine (ex);
-				return true;
-			}
-		}
-		
+
 		void SetupExceptionManager ()
 		{
+			System.Threading.Tasks.TaskScheduler.UnobservedTaskException += (sender, e) => {
+				HandleException (e.Exception.Flatten (), false);
+				e.SetObserved ();
+			};
 			GLib.ExceptionManager.UnhandledException += delegate (GLib.UnhandledExceptionArgs args) {
-				var ex = (Exception)args.ExceptionObject;
-				LoggingService.LogError ("Unhandled Exception", ex);
-				MessageService.ShowException (ex, "Unhandled Exception");
+				HandleException ((Exception)args.ExceptionObject, args.IsTerminating);
 			};
 			AppDomain.CurrentDomain.UnhandledException += delegate (object sender, UnhandledExceptionEventArgs args) {
-				//FIXME: try to save all open files, since we can't prevent the runtime from terminating
-				var ex = (Exception)args.ExceptionObject;
-				LoggingService.LogFatalError ("Unhandled Exception", ex);
-				MessageService.ShowException (ex, "Unhandled Exception. MonoDevelop will now close.");
+				HandleException ((Exception)args.ExceptionObject, args.IsTerminating);
 			};
+			Xwt.Application.UnhandledException += (sender, e) => {
+				HandleException (e.ErrorException, false);
+			};
+		}
+		
+		void HandleException (Exception ex, bool willShutdown)
+		{
+			// Log the crash to the MonoDevelop.log file first:
+			LoggingService.LogError (string.Format ("An unhandled exception has occured. Terminating MonoDevelop? {0}", willShutdown), ex);
+			
+			// Pass it off to the reporting service now.
+			LogReportingService.ReportUnhandledException (ex, willShutdown);
 		}
 		
 		/// <summary>SDBM-style hash, bounded to a range of 1000.</summary>
@@ -530,25 +519,32 @@ namespace MonoDevelop.Ide
 			if (options.ShowHelp || options.Error != null)
 				return options.Error != null? -1 : 0;
 			
-			if (Platform.IsWindows || options.RedirectOutput)
-				RedirectOutputToLogFile ();
+			LoggingService.Initialize (options.RedirectOutput);
 			
 			int ret = -1;
 			bool retry = false;
 			do {
 				try {
-					Runtime.SetProcessName (BrandingService.ApplicationName);
+					var exename = Path.GetFileNameWithoutExtension (Assembly.GetEntryAssembly ().Location);
+					if (!Platform.IsMac && !Platform.IsWindows)
+						exename = exename.ToLower ();
+					Runtime.SetProcessName (exename);
 					var app = new IdeStartup ();
 					ret = app.Run (options);
 					break;
 				} catch (Exception ex) {
 					if (!retry && AddinManager.IsInitialized) {
-						LoggingService.LogWarning (BrandingService.ApplicationName + " failed to start. Rebuilding addins registry.");
+						LoggingService.LogWarning (BrandingService.ApplicationName + " failed to start. Rebuilding addins registry.", ex);
 						AddinManager.Registry.Rebuild (new Mono.Addins.ConsoleProgressStatus (true));
-						LoggingService.LogInfo ("Addin registry rebuilt. Restarting MonoDevelop.");
+						LoggingService.LogInfo ("Addin registry rebuilt. Restarting {0}.", BrandingService.ApplicationName);
 						retry = true;
 					} else {
-						LoggingService.LogFatalError (BrandingService.ApplicationName + " failed to start. Some of the assemblies required to run MonoDevelop (for example gtk-sharp, gnome-sharp or gtkhtml-sharp) may not be properly installed in the GAC.", ex);
+						LoggingService.LogFatalError (
+							string.Format (
+								"{0} failed to start. Some of the assemblies required to run {0} (for example gtk-sharp)" +
+								"may not be properly installed in the GAC.",
+								BrandingService.ApplicationName
+							), ex);
 						retry = false;
 					}
 				} finally {
@@ -557,79 +553,10 @@ namespace MonoDevelop.Ide
 			}
 			while (retry);
 
-			CloseOutputLogFile ();
+			LoggingService.Shutdown ();
 
 			return ret;
 		}
-
-		static void RedirectOutputToLogFile ()
-		{
-			FilePath logDir = UserProfile.Current.LogDir;
-			if (!Directory.Exists (logDir))
-				Directory.CreateDirectory (logDir);
-			
-			//TODO: log rotation
-			string file = logDir.Combine ("MonoDevelop.log");
-			try {
-				if (Platform.IsWindows) {
-					//TODO: redirect the file descriptors on Windows, just plugging in a textwriter won't get everything
-					RedirectOutputToFileWindows (file);
-				} else {
-					RedirectOutputToFileUnix (file);
-				}
-			} catch {
-			}
-		}
-
-		static StreamWriter logFile;
-		static int logFd = -1;
-		
-		static void CloseOutputLogFile ()
-		{
-			if (logFile != null) {
-				logFile.Dispose ();
-				logFile = null;
-			}
-			if (logFd > -1) {
-				Mono.Unix.Native.Syscall.close (logFd);
-				logFd = -1;
-			}
-		}
-		
-		static void RedirectOutputToFileWindows (string file)
-		{
-			logFile = new StreamWriter (file);
-			logFile.AutoFlush = true;
-			Console.SetOut (logFile);
-			Console.SetError (logFile);
-		}
-		
-		static void RedirectOutputToFileUnix (string file)
-		{
-			const int STDOUT_FILENO = 1;
-			const int STDERR_FILENO = 2;
-			
-			Mono.Unix.Native.OpenFlags flags = Mono.Unix.Native.OpenFlags.O_WRONLY
-				| Mono.Unix.Native.OpenFlags.O_CREAT | Mono.Unix.Native.OpenFlags.O_TRUNC;
-			var mode = Mono.Unix.Native.FilePermissions.S_IFREG
-				| Mono.Unix.Native.FilePermissions.S_IRUSR | Mono.Unix.Native.FilePermissions.S_IWUSR
-				| Mono.Unix.Native.FilePermissions.S_IRGRP | Mono.Unix.Native.FilePermissions.S_IWGRP;
-			
-			int fd = Mono.Unix.Native.Syscall.open (file, flags, mode);
-			if (fd < 0)
-				//error
-				return;
-			
-			int res = Mono.Unix.Native.Syscall.dup2 (fd, STDOUT_FILENO);
-			if (res < 0)
-				//error
-				return;
-			
-			res = Mono.Unix.Native.Syscall.dup2 (fd, STDERR_FILENO);
-			if (res < 0)
-				//error
-				return;
-		}
 	}
 	
 	public class MonoDevelopOptions
@@ -637,18 +564,18 @@ namespace MonoDevelop.Ide
 		MonoDevelopOptions ()
 		{
 			IpcTcp = (PlatformID.Unix != Environment.OSVersion.Platform);
+			RedirectOutput = true;
 		}
 		
 		Mono.Options.OptionSet GetOptionSet ()
 		{
 			return new Mono.Options.OptionSet () {
-				{ "nologo", "Do not display splash screen.", s => NoLogo = true },
+				{ "no-splash", "Do not display splash screen.", s => NoSplash = true },
 				{ "ipc-tcp", "Use the Tcp channel for inter-process comunication.", s => IpcTcp = true },
-				{ "newwindow", "Do not open in an existing instance of " + BrandingService.ApplicationName, s => NewWindow = true },
+				{ "new-window", "Do not open in an existing instance of " + BrandingService.ApplicationName, s => NewWindow = true },
 				{ "h|?|help", "Show help", s => ShowHelp = true },
-				{ "clog", "Log internal counter data", s => LogCounters = true },
-				{ "clog-interval=", "Interval between counter logs (in milliseconds)", (int i) => LogCountersInterval = i },
-				{ "redirect-output", "Whether to redirect stdout/stderr to a log file", s => RedirectOutput = true },
+				{ "perf-log", "Enable performance counter logging", s => PerfLog = true },
+				{ "no-redirect", "Disable redirection of stdout/stderr to a log file", s => RedirectOutput = false },
 			};
 		}
 		
@@ -677,12 +604,11 @@ namespace MonoDevelop.Ide
 			return opt;
 		}
 		
-		public bool NoLogo { get; set; }
+		public bool NoSplash { get; set; }
 		public bool IpcTcp { get; set; }
 		public bool NewWindow { get; set; }
 		public bool ShowHelp { get; set; }
-		public bool LogCounters { get; set; }
-		public int LogCountersInterval { get; set; }
+		public bool PerfLog { get; set; }
 		public bool RedirectOutput { get; set; }
 		public string Error { get; set; }
 		public IList<string> RemainingArgs { get; set; }
