@@ -29,7 +29,7 @@ using System.Reflection;
 
 namespace MonoDevelop.Ide
 {
-	public class IdeVersionInfo : ISystemInformationProvider
+	class IdeVersionInfo : ISystemInformationProvider
 	{
 		static bool IsMono ()
 		{
@@ -86,10 +86,14 @@ namespace MonoDevelop.Ide
 
 		}
 		
+		string ISystemInformationProvider.Title {
+			get { return BrandingService.ApplicationName; }
+		}
+
 		string ISystemInformationProvider.Description {
 			get {
 				var sb = new System.Text.StringBuilder ();
-				sb.Append ("MonoDevelop ");
+				sb.Append ("Version ");
 				sb.AppendLine (MonoDevelopVersion);
 				
 				sb.Append ("Installation UUID: ");
@@ -124,10 +128,20 @@ namespace MonoDevelop.Ide
 			}
 		}
 		
-		public string MonoDevelopVersion {
-			get { return BuildVariables.PackageVersion == BuildVariables.PackageVersionLabel
-					? BuildVariables.PackageVersionLabel
-					: string.Format ("{0} ({1})", BuildVariables.PackageVersionLabel, BuildVariables.PackageVersion);
+		public static string MonoDevelopVersion {
+			get {
+				string v = "";
+				if (BuildVariables.PackageVersion != BuildVariables.PackageVersionLabel)
+					v += BuildVariables.PackageVersion;
+				if (IdeApp.Version.Revision >= 0) {
+					if (v.Length > 0)
+						v += " ";
+					v += "build " + IdeApp.Version.Revision;
+				}
+				if (v.Length == 0)
+					return BuildVariables.PackageVersionLabel;
+				else
+					return BuildVariables.PackageVersionLabel + " (" + v + ")";
 			}
 		}
 	}
