@@ -30,9 +30,17 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 {
 	public interface IProjectBuilder
 	{
-		MSBuildResult[] RunTarget (string target, string configuration, string platform, ILogWriter logWriter,
+		MSBuildResult[] RunTarget (string target, ProjectConfigurationInfo[] configurations, ILogWriter logWriter,
 			MSBuildVerbosity verbosity);
-		string[] GetAssemblyReferences (string configuration, string platform);
+		string[] GetAssemblyReferences (ProjectConfigurationInfo[] configurations);
 		void Refresh ();
 	}
+
+	[Serializable]
+	public class ProjectConfigurationInfo
+	{
+		public string ProjectFile;
+		public string Configuration;
+		public string Platform;
+	}
 }
