@@ -196,9 +196,9 @@ namespace MonoDevelop.CSharpBinding.Tests
 		
 		public List<object> Contents = new List<object> ();
 		
-		public override T GetContent<T> () 
+		public override object GetContent (Type type) 
 		{
-			return Contents.OfType<T> ().FirstOrDefault () ??  base.GetContent<T> ();
+			return Contents.FirstOrDefault (o => type.IsInstanceOfType (type)) ??  base.GetContent (type);
 		}
 		
 		public IDisposable OpenUndoGroup ()
