@@ -33,6 +33,8 @@ using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Dom.Parser;
 using NUnit.Framework;
 using UnitTests;
+using Mono.CSharp;
+using System.Linq;
 
 namespace MonoDevelop.Projects
 {
@@ -74,7 +76,7 @@ namespace MonoDevelop.Projects
 			string tfile = mainProject.Project.GetAbsoluteChildPath (targetRelativePath);
 			string sfile = mainProject.Project.GetAbsoluteChildPath (sourceRelativePath);
 			File.Copy (sfile, tfile, true);
-			ProjectDomService.Parse (tfile, null, null);
+			ProjectDomService.Parse (tfile, null);
 		}
 
 		[Test]
@@ -261,7 +263,9 @@ namespace MonoDevelop.Projects
 			Assert.IsTrue (types.Contains ("Library1.CExtraContainerSub"));
 			Assert.IsTrue (types.Contains ("Library1.CExtraContainerInnerSub"));
 			Assert.IsTrue (types.Contains ("Library1.ISimple"));
-			Assert.AreEqual (7, types.Count);
+			Assert.IsTrue (types.Contains ("Library1.TestAttribute"));
+			
+			Assert.AreEqual (8, types.Count);
 		}
 		
 		[Test]
@@ -295,6 +299,32 @@ namespace MonoDevelop.Projects
 			Assert.AreEqual (5, types.Count);
 		}
 		
+			
+		[Test]
+		public void GetInheritanceTreeForEnumsAndStructs ()
+		{
+			IType type = mainProject.GetType ("CompletionDbTest.TestEnum", false);
+			
+			List<string> types = new List<string> ();
+			foreach (IType t in mainProject.GetInheritanceTree (type)) {
+				Console.WriteLine (t.FullName);
+				types.Add (t.FullName);
+			}
+			
+			Assert.IsTrue (types.Contains ("CompletionDbTest.TestEnum"));
+			Assert.IsTrue (types.Contains ("System.Enum"));
+			Assert.IsTrue (types.Contains ("System.Object"));
+			
+			type = mainProject.GetType ("CompletionDbTest.TestStruct", false);
+			
+			types = new List<string> ();
+			foreach (IType t in mainProject.GetInheritanceTree (type))
+				types.Add (t.FullName);
+			
+			Assert.IsTrue (types.Contains ("CompletionDbTest.TestStruct"));
+			Assert.IsTrue (types.Contains ("System.ValueType"));
+			Assert.IsTrue (types.Contains ("System.Object"));
+		}
 		[Test]
 		public void GetNamespaceContents ()
 		{
@@ -790,5 +820,70 @@ namespace MonoDevelop.Projects
 			type = mainProject.GetType ("CompletionDbTest.PartialTest");
 			Assert.IsNull (type);
 		}
+		
+		[Test]
+		public void NamespaceExistsTest ()
+		{
+			Assert.IsTrue (mainProject.NamespaceExists ("Level1"), "Level1 doesn't exist.");
+			Assert.IsTrue (mainProject.NamespaceExists ("Level1.Level2"), "Level1.Level2 doesn't exist.");
+			Assert.IsTrue (mainProject.NamespaceExists ("Level1.Level2.Level3"), "Level1.Level2.Level3 doesn't exist.");
+			Assert.IsTrue (mainProject.NamespaceExists ("Level1.Level2.Level3.Level4"), "Level1.Level2.Level3.Level4 doesn't exist.");
+			Assert.IsFalse (mainProject.NamespaceExists ("Level1.Level2.Level3.Level4.Level5"), "Level5 shouldn't exist.");
+			Assert.IsFalse (mainProject.NamespaceExists ("Level1.Level3"), "level1.level3 shouldn't exist.");
+		}
+		
+		[Test]
+		public void ClassAttributeTest ()
+		{
+			// Simple get
+			IType type = mainProject.GetType ("CompletionDbTest.AttributeTest");
+			Assert.IsNotNull (type);
+			Assert.AreEqual (1, type.Attributes.Count ());
+			Assert.AreEqual ("Serializable", type.Attributes.First ().Name);
+		}
+		
+		[Test]
+		public void MemberAttributeTest ()
+		{
+			// Simple get
+			IType type = mainProject.GetType ("CompletionDbTest.AttributeTest2");
+			Assert.IsNotNull (type);
+			
+			var prop = type.Properties.First ();
+			Assert.AreEqual (1, prop.Attributes.Count ());
+			Assert.AreEqual ("Obsolete", prop.Attributes.First ().Name);
+			
+			var method = type.Methods.First ();
+			Assert.AreEqual (1, method.Attributes.Count ());
+			Assert.AreEqual ("Obsolete", method.Attributes.First ().Name);
+		}
+		
+		[Test]
+		public void CustomAttributeTest ()
+		{
+			// Simple get
+			IType type = mainProject.GetType ("CompletionDbTest.AttributeTest3");
+			Assert.IsNotNull (type);
+			Assert.AreEqual (1, type.Attributes.Count ());
+			
+			var att = type.Attributes.First ();
+			Assert.AreEqual ("Library1.TestAttribute", att.AttributeType.FullName);
+			Assert.AreEqual (2, att.PositionalArguments.Count);
+			
+			var expr1 = att.PositionalArguments[0] as System.CodeDom.CodePrimitiveExpression;
+			Assert.IsNotNull (expr1);
+			Assert.AreEqual ("str1", expr1.Value);
+			
+			var expr2 = att.PositionalArguments[1] as System.CodeDom.CodePrimitiveExpression;
+			Assert.IsNotNull (expr2);
+			Assert.AreEqual (5, expr2.Value);
+			
+			Assert.AreEqual (1, att.NamedArguments.Count);
+			Assert.IsTrue (att.NamedArguments.ContainsKey ("Blah"));
+			var expr3 = att.NamedArguments["Blah"] as System.CodeDom.CodePrimitiveExpression;
+			Assert.IsNotNull (expr3);
+			Assert.AreEqual ("str2", expr3.Value);
+		}
+		
 	}
 }
