@@ -229,6 +229,7 @@ namespace MonoDevelop.Projects
 			
 			// Projects inherit the file format from the parent solution
 			Solution sol = new Solution ();
+			sol.ConvertToFormat (Util.FileFormatMSBuild05, true);
 			sol.RootFolder.Items.Add (prj);
 			Assert.AreEqual ("test4", prj.Name);
 			Assert.AreEqual (Path.Combine (Path.GetTempPath (), "test4.csproj"), (string) prj.FileName);
@@ -366,10 +367,7 @@ namespace MonoDevelop.Projects
 			Assert.AreEqual (0, res.ErrorCount);
 			Assert.AreEqual (0, res.WarningCount);
 			Assert.AreEqual (3, res.BuildCount);
-			Assert.IsFalse (p.NeedsBuilding (config));
-			Assert.IsFalse (lib1.NeedsBuilding (config));
-			Assert.IsFalse (lib2.NeedsBuilding (config));
-			
+
 			Assert.IsTrue (File.Exists (Util.Combine (p.BaseDirectory, "bin", "Debug", "console-with-libs-mdp.exe")));
 			Assert.IsTrue (File.Exists (Util.Combine (p.BaseDirectory, "bin", "Debug", GetMdb ("console-with-libs-mdp.exe"))));
 			Assert.IsTrue (File.Exists (Util.Combine (lib1.BaseDirectory, "bin", "Debug", "library1.dll")));
@@ -379,20 +377,10 @@ namespace MonoDevelop.Projects
 			
 			// Build the project, but not the references
 			
-			p.SetNeedsBuilding (true);
-			lib1.SetNeedsBuilding (true);
-			lib2.SetNeedsBuilding (true);
-			Assert.IsTrue (p.NeedsBuilding (config));
-			Assert.IsTrue (lib1.NeedsBuilding (config));
-			Assert.IsTrue (lib2.NeedsBuilding (config));
-			
 			res = p.Build (Util.GetMonitor (), config, false);
 			Assert.AreEqual (0, res.ErrorCount);
 			Assert.AreEqual (0, res.WarningCount);
 			Assert.AreEqual (1, res.BuildCount);
-			Assert.IsTrue (p.NeedsBuilding (config));   // True because references require building
-			Assert.IsTrue (lib1.NeedsBuilding (config));
-			Assert.IsTrue (lib2.NeedsBuilding (config));
 		}
 		
 		[Test()]
@@ -491,8 +479,8 @@ namespace MonoDevelop.Projects
 			Solution sol = TestProjectsChecks.CreateConsoleSolution ("reloading");
 			Project p = (Project) sol.Items [0];
 			
-			Assert.AreEqual (Ide.IdeApp.Services.ProjectService.DefaultFileFormatId, sol.FileFormat.Id);
-			Assert.AreEqual (Ide.IdeApp.Services.ProjectService.DefaultFileFormatId, p.FileFormat.Id);
+			Assert.AreEqual (Services.ProjectService.DefaultFileFormatId, sol.FileFormat.Id);
+			Assert.AreEqual (Services.ProjectService.DefaultFileFormatId, p.FileFormat.Id);
 			
 			// Change solution format of unsaved solution
 			
@@ -521,7 +509,7 @@ namespace MonoDevelop.Projects
 			// Add new project
 			
 			Project newp = new DotNetAssemblyProject ("C#");
-			Assert.AreEqual ("MSBuild05", newp.FileFormat.Id);
+			Assert.AreEqual ("MSBuild10", newp.FileFormat.Id);
 			
 			sol.RootFolder.Items.Add (newp);
 			Assert.AreEqual ("MD1", newp.FileFormat.Id);
